from base64 import b64decode as license64

from BroTools import __version__
import os
import sys
PY3 = (sys.version_info[0] >= 3)
if (PY3 == True):
    import builtins
    builtins.unicode = str
realpath = os.path.dirname(os.path.realpath(__file__))
if (PY3 is True):
    packages_path = os.path.join(realpath, 'packages_3_3')
else:
    packages_path = os.path.join(realpath, 'packages_2_7')
sys.path.append(packages_path)
import ntpath
import sys
import json
import platform
import re
from subprocess import check_output
from maya import cmds
from BroTools.core.config_parser import BroConfig
from BroTools.core import utils
from BroTools.core.utils import sluggify

def crossplatform_normpath(filepath):
    return ntpath.normpath(filepath).replace('\\', os.path.sep).replace('/', os.path.sep)

def get_dev_version():
    plugin_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'plug-ins', 'BroTools.py')
    with open(plugin_path, 'r') as f:
        lines = f.readlines()
    for line in lines:
        if (line.strip().startswith('__release_date__') and ('=' in line)):
            value = line.split('=')[(- 1)].strip().replace('"', '').replace("'", '').strip()
    return value
__dev_version__ = get_dev_version()
MOD_NAME = 'BroTools'
MOD_Prefix = 'Bro'
mod_p = MOD_Prefix.lower()
MOD_MenuRoot = (MOD_Prefix + 'MenuRoot')
RIGGING_TOOLS_MENU = (mod_p + '_rigging_tools_menu')
RIG_SNIPPETS_MENU = (mod_p + '_rig_snippets_menu')
COMMON_TOOLS_MENU = (mod_p + '_common_tools_menu')
ANIMATION_TOOLS_MENU = (mod_p + '_animation_tools_menu')
FILE_TOOLS_MENU = (mod_p + '_file_tools_menu')
PIPELINE_TOOLS_MENU = (mod_p + '_pipeline_tools_menu')
EXTRAS_MENU = (mod_p + '_extras_menu')
SNIPPETS_TOOLS_MENU = (mod_p + '_snippets_menu')
CUSTOM_SCRIPTS_MENU = (mod_p + '_customscripts_menu')
THIRDPARTY_TOOLS_MENU = (mod_p + '_thirdparty_menu')
optional_menu_ids = [ANIMATION_TOOLS_MENU, COMMON_TOOLS_MENU, RIGGING_TOOLS_MENU, PIPELINE_TOOLS_MENU, EXTRAS_MENU, FILE_TOOLS_MENU]
plugin_path = cmds.pluginInfo((MOD_NAME + '.py'), query=True, path=True)
plugin_dir = os.path.dirname(plugin_path)
MOD_dir_relative = os.path.join(plugin_dir, '..')
modpath = APP_DIR = os.path.abspath(MOD_dir_relative)

def win_get_documents_folder():
    try:
        print('BroTools: Attempt to find documents folder with win_get_documents_folder...')
        import ctypes.wintypes
        CSIDL_PERSONAL = 5
        SHGFP_TYPE_CURRENT = 0
        buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
        ctypes.windll.shell32.SHGetFolderPathW(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, buf)
        return buf.value
    except Exception as e:
        print('BroTools: Failed to use win_get_documents_folder!')
        return None

def get_user_home_dir():
    if (platform.system() == 'Darwin'):
        home = os.path.expanduser('~')
    elif (platform.system() == 'Windows'):
        home = win_get_documents_folder()
        if (home is None):
            home = os.path.expanduser('~')
    else:
        home = os.path.expanduser('~')
    return home
user_home = get_user_home_dir()
system = platform.system()
if (system == 'Darwin'):
    maya_user_dir = os.path.join(user_home, 'Library', 'Preferences', 'Autodesk', 'maya')
else:
    maya_user_dir = os.path.join(user_home, 'maya')
brotools_write_path = os.environ.get('BROTOOLS_CONFIG_PATH')
if (brotools_write_path == 'maya_user_dir'):
    brotools_write_path = os.path.normpath(os.path.join(maya_user_dir, 'BroTools'))
if (not brotools_write_path):
    try:
        with open(os.path.join(modpath, 'write_test_file_deleteme_.txt'), 'w+') as f:
            f.write('Hi! :)')
        brotools_write_path = modpath
    except IOError as e:
        brotools_write_path = os.path.normpath(os.path.join(maya_user_dir, 'BroTools'))
        print("// BroTools: Could not write to default write path at: {}; Write path set to: {}. You can manually change where BroTools writes files with 'BROTOOLS_CONFIG_PATH' environment variable. //".format(modpath, brotools_write_path))
        os.makedirs(brotools_write_path)
if (not os.path.exists(brotools_write_path)):
    try:
        os.makedirs(brotools_write_path)
    except Exception as e:
        cmds.warning("Can't create config path: {} - {}".format(brotools_write_path, str(e)))
if (not os.access(brotools_write_path, os.W_OK)):
    cmds.error("BroTools has no write access to the config folder, cannot proceed: {}.\nYou can change this directory by setting a 'BROTOOLS_CONFIG_PATH' environment variable.".format(brotools_write_path))
icons_path = crossplatform_normpath(os.path.join(modpath, 'icons'))
imgs_path = crossplatform_normpath(os.path.join(modpath, 'imgs'))
menu_icons_path = crossplatform_normpath(os.path.join(icons_path, 'menu'))
key_store_path = crossplatform_normpath(os.path.join(brotools_write_path, 'keys.json'))
license_file_path = crossplatform_normpath(os.path.join(brotools_write_path, 'brotools.lic'))
backups_dir = os.path.join(brotools_write_path, 'backups')
if (not os.path.exists(backups_dir)):
    try:
        os.makedirs(backups_dir)
    except Exception as e:
        cmds.warning("Can't create backups_dir: {} - {}".format(brotools_write_path, str(e)))
plugin_path = os.path.join(modpath, 'plug-ins', 'BroTools.py')
brotools_version = __version__

def create_keystore_file():
    with open(key_store_path, 'w+') as f:
        f.write('{}')
if (not os.path.exists(key_store_path)):
    create_keystore_file()
try:
    with open(key_store_path, 'r') as f:
        keystore_data = json.load(f)
except ValueError:
    r = cmds.confirmDialog(title='BroTools: Fix corrupted license key storage file?', message='It looks like your license storage key is corrupted. Would you like to regenerate it? You will have to re-add your license keys.\nYou can try checking this file and fixing it yourself.\nFilepath: {}\n\nIf left in this state some tools may fail to load or may not work properly.'.format(key_store_path), button=['Yes', 'No'], defaultButton='Yes', cancelButton='No', dismissString='No', icon='warning')
    if (r == 'Yes'):
        create_keystore_file()
configs_dir = os.path.join(brotools_write_path, 'config')
if (not os.path.exists(configs_dir)):
    try:
        os.makedirs(configs_dir)
    except:
        cmds.warning("// BroTools: Could not create configs directory, please make sure Maya has permissions to write to the directory you've set, or create those directories manually: {} //".format(configs_dir))
print('// BroTools: Configs directory is set to: {} //'.format(str(configs_dir)))
brotools_config_path = os.path.join(configs_dir, 'brotools.cfg')
brotools_config = BroConfig(brotools_config_path, defaults={'Debug': {'debug_level': 2, 'write_log_file': True}, 'Updater': {'check_updates': True}, 'MayaUtils': {'matchTransform_MatchPivot': True}})
try:
    maya_version = int(cmds.about(v=True).split(' ')[0])
except:
    maya_version = None
styles_path = os.path.join(modpath, 'core', 'broqt', 'styles')

class BroQSS(str):
    fp = os.path.join(styles_path, 'bro.qss')

    def get(self):
        with open(self.fp, 'r') as f:
            qss = f.read().replace('url(', ('url(' + icons_path.replace('\\', '/')))
        return qss

    def __str__(self):
        return self.get()

    def __repr__(self):
        return self.get()

    def __unicode__(self):
        return self.get()
bro_qss = BroQSS()
hud_qss = BroQSS()
hud_qss.fp = os.path.join(styles_path, 'hud.qss')
try:
    operating_system = '{} {} ({})'.format(platform.system(), platform.release(), platform.version())
except:
    operating_system = '?'
print('# BroTools | CONF || Building icon library...')
icon_library = {'not_found': os.path.join(APP_DIR, 'icons', 'not_found.png')}
imgs_library = {'not_found': os.path.join(APP_DIR, 'icons', 'not_found.png')}

def add_files_to_dict(dirpath, dic):
    for filename in os.listdir(dirpath):
        (name, ext) = os.path.splitext(filename)
        if (ext in ['.png', '.ico', '.xmp', '.jpg']):
            iconpath = os.path.join(dirpath, filename)
            dic[filename] = iconpath
            dic[utils.sluggify(filename)] = iconpath
for (root, dirs, files) in os.walk(APP_DIR, topdown=False):
    for dirname in dirs:
        if (dirname == 'icons'):
            dirpath = os.path.join(root, dirname)
            add_files_to_dict(dirpath, icon_library)
        elif (dirname == 'imgs'):
            dirpath = os.path.join(root, dirname)
            add_files_to_dict(dirpath, imgs_library)
print('// BroTools | CONF || Icon library: {} //'.format(icon_library))

def debug_log(*args):
    if (brotools_config.Debug.debug_level <= 0):
        namespace = sys._getframe(1).f_globals
        caller = 'debug_log'
        if ('__file__' in namespace):
            caller = os.path.basename(namespace['__file__'])
        print((((('// ' + caller) + ' / ') + ' '.join([str(arg) for arg in args])) + ' //'))

def icon(name, size=16, ext='png', fp=False, type=None):
    if (type == None):
        prefix = ('i' + str(size))
    else:
        prefix = type
    if ('_menuitem' in name):
        name = name.replace('_menuitem', '')
    if (not fp):
        icon_filename = ((((prefix + '_') + name) + '.') + ext)
        icon_filename = sluggify(icon_filename)
        if (icon_filename not in icon_library):
            debug_log('Could not find icon:', icon_filename)
            iconFilePath = icon_library['not_found']
        else:
            iconFilePath = icon_library[((((prefix + '_') + name) + '.') + ext)]
            debug_log('Found icon {} at: {}'.format(icon_filename, iconFilePath))
    else:
        iconFilePath = name
    return iconFilePath

def img(name, ext='png'):
    filename = ((name + '.') + ext)
    filename = sluggify(filename)
    if (filename in imgs_library):
        return imgs_library[filename]
    else:
        debug_log('Could not find image:', filename, name)
        return icon_library['not_found']

def modulepath_to_filepath(modpath, absolute=True):
    '\n    Convert importable python module path to filepath\n    BroTools.core.system -> C:\\BroTools\\core\\system.py\n    '
    filepath = (modpath.replace('.', os.path.sep) + '.py')
    if absolute:
        filepath = crossplatform_normpath(os.path.abspath(os.path.join(APP_DIR, '..', filepath)))
    return filepath

def filepath_to_modulepath(filepath):
    '\n    Convert filepath to importable python module path\n    C:\\BroTools\\core\\system.py -> BroTools.core.system\n    '
    modulepath = crossplatform_normpath(os.path.relpath(filepath, APP_DIR))
    (modulepath, _) = os.path.splitext(modulepath)
    modulepath = modulepath.replace(os.path.sep, '.')
    return modulepath
