from base64 import b64decode as license64

__author__ = 'Michael Davydov https://www.michaeldavydov.com'
import os, json
rp = os.path.dirname(os.path.realpath(__file__))
import maya.cmds as cmds
from maya.app.general.mayaMixin import MayaQWidgetBaseMixin
from BroTools.core.broqt.vendor.Qt import QtWidgets, QtGui, QtCore
from BroTools.configuration import modpath, bro_qss
from BroTools.core.debug import default_logger as log
from uuid import uuid4
from BroTools.core.broqt.widgets.curve_editor import CurvePreviewWidgetWithEditor, ToggleableCurveEditor

class BroSignal_Bool(QtCore.QObject):
    sig = QtCore.Signal(bool)

class BroSignal_Int(QtCore.QObject):
    sig = QtCore.Signal(int)

class BroSignal_Str(QtCore.QObject):
    sig = QtCore.Signal(str)

class BroSignal_Bytearray(QtCore.QObject):
    sig = QtCore.Signal(bytearray)

class BroWidgetBase(MayaQWidgetBaseMixin, QtWidgets.QWidget):
    '\n    Just a reminder how to do this :)\n    '

    def __init__(A):
        super(BroWidgetBase, A).__init__()

class BroDoubleSpinBox(QtWidgets.QDoubleSpinBox):

    def __init__(A, parent=None):
        QtWidgets.QDoubleSpinBox.__init__(A, parent=parent)
        A.setFixedWidth(200)

class BroSpinBox(QtWidgets.QSpinBox):

    def __init__(A, parent=None):
        QtWidgets.QSpinBox.__init__(A, parent=parent)
        A.setFixedWidth(200)

class BroObjectList(QtWidgets.QWidget):

    def __init__(A, parent=None, title='none', acceptedTypes=['']):
        B = acceptedTypes
        QtWidgets.QWidget.__init__(A, parent=parent)
        G = QtGui.QColor()
        G.setRgb(100, 0, 0, 100)
        H = QtGui.QColor()
        H.setRgb(100, 100, 0, 100)

        def K():
            B = []
            for C in range(A.list.count()):
                B.append(A.list.item(C).text())
            return B

        def C():
            L = cmds.ls(sl=True, long=True)
            for C in L:
                M = K()
                if (C not in M):
                    D = QtWidgets.QListWidgetItem(C)
                    I = cmds.objectType(C)
                    E = cmds.listRelatives(s=True)
                    F = True
                    if (E != None):
                        if (len(E) > 0):
                            for J in E:
                                if cmds.objExists(J):
                                    if ((cmds.objectType(J) not in B) and F):
                                        F = False
                    if (I not in B):
                        D.setBackground(G)
                    elif ((I in B) and (not F)):
                        D.setBackground(H)
                    A.list.addItem(D)

        def D():
            for B in A.list.selectedItems():
                A.list.takeItem(A.list.row(B))
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.setLayout(A.mainLayout)
        A.label = QtWidgets.QLabel(title)
        A.list = QtWidgets.QListWidget()
        A.list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        A.addBtn = QtWidgets.QPushButton('Add')
        A.addBtn.setIcon(QtGui.QPixmap(os.path.join(modpath, 'imgs', 'i16_add.png')))
        A.addBtn.setWhatsThis('Add selected objects to the list.')
        A.removeBtn = QtWidgets.QPushButton('Remove')
        A.removeBtn.setIcon(QtGui.QPixmap(os.path.join(modpath, 'imgs', 'i16_remove.png')))
        A.removeBtn.setWhatsThis('Remove selected objects from the list.')
        A.addBtn.clicked.connect(C)
        A.removeBtn.clicked.connect(D)
        for E in [A.label, A.list, A.addBtn, A.removeBtn]:
            A.mainLayout.addWidget(E)
        A.list.setObjectName('myObjectList_list')
        A.loadCss()

    def loadCss(A):
        try:
            A.setStyleSheet(str(bro_qss))
        except:
            log.warning('#FF0000', 'Unable to load CSS style for UI!', inv=True)

    def getAllItems(A):
        B = []
        for C in range(A.list.count()):
            B.append(A.list.item(C).text())
        return B

class BroWidgetVLayout(QtWidgets.QWidget):

    def __init__(A, parent=None, *B, **C):
        QtWidgets.QWidget.__init__(A, parent=parent)
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.setLayout(A.mainLayout)
        A.setAutoFillBackground(True)

    def addWidget(A, widget):
        A.mainLayout.addWidget(widget)

    def setBackgroundColor(A, r, g, b, a):
        A.color = QtWidgets.QColor()
        A.color.setRgb(r, g, b, a)
        A.p = A.palette()
        A.p.setColor(A.backgroundRole(), A.color)
        A.setPalette(A.p)

class BroGridWidgetLayout(QtWidgets.QWidget):

    def __init__(A, parent=None, *B, **C):
        QtWidgets.QWidget.__init__(A, parent=parent)
        A.mainLayout = QtWidgets.QGridLayout()
        A.setLayout(A.mainLayout)

    def addWidget(A, widget, column, row):
        A.mainLayout.addWidget(widget, column, row)

class BroImageButton(QtWidgets.QPushButton):

    def __init__(A, pixmap, parent=None):
        super(BroImageButton, A).__init__(parent)
        A.pixmap = pixmap

    def paintEvent(A, event):
        A.selRect = QtCore.QRect(0, 0, 10, 21)
        B = QtGui.QPainter(A)
        B.drawRect(A.selRect)
        B.drawPixmap(event.rect(), A.pixmap)

    def focusInEvent(A, *B, **C):
        pass

    def sizeHint(A):
        return A.pixmap.size()

class BroUniversalInput(MayaQWidgetBaseMixin, QtWidgets.QWidget):
    "\n    A custom universal input widget, that can adapt to it's type.\n    It provides setValue and value functions that set and return values dynamically, to avoid\n    the difference of PySide commands like setValue, setText and isChecked.\n    "

    def __init__(A, parent=None, value_type='int', options=[]):
        C = options
        B = value_type
        super(BroUniversalInput, A).__init__(parent=parent)
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.setLayout(A.mainLayout)
        if (type(B) == type):
            A.types_to_str = {float: 'float', int: 'int', str: 'str', bool: 'bool', list: 'list', dict: 'dict'}
            B = A.types_to_str[B]
        A.types = {'float': QtWidgets.QDoubleSpinBox, 'int': QtWidgets.QSpinBox, 'str': QtWidgets.QLineEdit, 'bool': QtWidgets.QCheckBox, 'list': QtWidgets.QLineEdit, 'dict': QtWidgets.QLineEdit, 'curve': ToggleableCurveEditor, 'dropdown': QtWidgets.QComboBox}
        A.type = B.lower()
        A.input = A.types[B]()
        A.mainLayout.addWidget(A.input)
        D = QtWidgets.QSizePolicy()
        D.setHorizontalPolicy(QtWidgets.QSizePolicy.Expanding)
        A.setSizePolicy(D)
        A.mainLayout.setContentsMargins(0, 0, 0, 0)
        if (A.type in ['int', 'float', 'curve']):
            A.valueChanged = A.input.valueChanged
        elif (A.type == 'bool'):
            A.valueChanged = A.input.stateChanged
        elif (A.type in ['str', 'list', 'dict']):
            A.valueChanged = A.input.textChanged
        elif (A.type in ['dropdown']):
            if (not C):
                raise Exception('Dropdown inputs require options.')
            A.valueChanged = A.input.currentIndexChanged
            A.input.addItems(C)

    def setValue(A, value):
        B = value
        if (A.type in ['int', 'float', 'curve']):
            A.input.setValue(B)
        elif (A.type == 'bool'):
            A.input.setChecked(B)
        elif (A.type == 'str'):
            A.input.setText(B)
        elif (A.type in ['list', 'dict']):
            A.input.setText(json.dumps(B))
        elif (A.type in ['dropdown']):
            A.input.setCurrentIndex(B)

    def value(A):
        if (A.type in ['int', 'float', 'curve']):
            return A.input.value()
        elif (A.type == 'bool'):
            return A.input.isChecked()
        elif (A.type == 'str'):
            return A.input.text()
        elif (A.type in ['list', 'dict']):
            return json.loads(A.input.text())
        elif (A.type in ['dropdown']):
            return A.input.currentIndex()
    text = value
    isChecked = value
    setText = setValue
    setChecked = setValue

    def setRange(A, min, max):
        if (A.type in ['int', 'float']):
            A.input.setRange(min, max)
        else:
            log.debug('BroUniversalInput of type', A.type, 'can not use setRange.')

    def setSingleStep(A, step):
        if (A.type in ['int', 'float']):
            A.input.setSingleStep(step)
        else:
            log.debug('BroUniversalInput of type', A.type, 'can not use setSingleStep.')

class BroToolTip(MayaQWidgetBaseMixin, QtWidgets.QWidget):

    def __init__(A, parent=None):
        super(BroToolTip, A).__init__(parent)
        A.mainLayout = QtWidgets.QVBoxLayout(A)
        A.setLayout(A.mainLayout)
        A.text = QtWidgets.QLabel('Im a tooltip')
        A.text.setWordWrap(True)
        A.image = QtWidgets.QLabel('')
        A.mainLayout.addWidget(A.image)
        A.mainLayout.addWidget(A.text)
        A.setWindowFlags(QtCore.Qt.ToolTip)
        try:
            A.setStyleSheet(str(bro_qss))
        except Exception as B:
            log.inViewLog('#FF0000', ('Unable to load CSS style for UI! ' + str(B)))
        A.movieFormats = ['.gif']
        A.setMaximumWidth(600)

    def setMedia(A, mediaPath):
        B = mediaPath
        (D, C) = os.path.splitext(B)
        if (C in A.movieFormats):
            A.media = QtGui.QMovie(B)
            A.image.setMovie(A.media)
            A.media.start()
        else:
            A.media = QtGui.QPixmap(B)
            A.image.setPixmap(A.media)
        A.image.show()
    setImage = setMedia
    setMovie = setMedia

    def setText(A, text):
        A.text.setText(text)
        A.text.show()

    def empty(A):
        A.text.setText('')
        A.image.setText('')
        A.image.hide()
        A.text.hide()
        A.hide()
        try:
            A.media.deleteLater()
        except Exception as B:
            log.debug(B)

class BroToolTipFilter(QtCore.QObject):

    def __init__(A, parent=None):
        B = parent
        super(BroToolTipFilter, A).__init__(B)
        A.parent = B
        A.tooltipWidget = BroToolTip(parent=B)

    def eventFilter(A, obj, event):
        C = event
        B = obj
        if (C.type() == QtCore.QEvent.Enter):
            log.debug('Mouse over', B)
            if (A.parent != None):
                if (A.parent.config.Settings.showTooltips == True):
                    if hasattr(B, 'broTooltip'):
                        log.debug('Show tooltip for', B)
                        if ('text' in B.broTooltip):
                            A.tooltipWidget.setText(B.broTooltip['text'])
                        if ('image' in B.broTooltip):
                            A.tooltipWidget.setImage(os.path.join(modpath, 'BroDynamics', 'help', 'tooltips', B.broTooltip['image']))
                        A.tooltipWidget.show()
                        D = QtGui.QCursor.pos()
                        E = A.tooltipWidget.frameGeometry()
                        F = [(D.x() + 20), ((D.y() - E.height()) - 20)]
                        log.debug('Tooltip data:', D, E, F)
                        A.tooltipWidget.move(*F)
        elif (C.type() == QtCore.QEvent.Leave):
            log.debug('Exit')
            A.tooltipWidget.empty()
            A.tooltipWidget.hide()
        if (C is not None):
            C.accept()
        return False

class BroTitle(QtWidgets.QLabel):

    def __init__(A, *B, **C):
        super(BroTitle, A).__init__(*B, **C)
        A.setStyleSheet('QLabel {color: #faa300;font-size:20px;}')
        A.setWordWrap(True)

class BroText(QtWidgets.QLabel):

    def __init__(A, *B, **C):
        super(BroText, A).__init__(*B, **C)
        A.setStyleSheet('QLabel {font-size:16px;}')
        A.setWordWrap(True)

class BroEntry(QtWidgets.QWidget):

    def __init__(A, title, text, *F, **E):
        C = text
        B = title
        G = E.pop('spacing', 0)
        D = E.pop('margin', 0)
        super(BroEntry, A).__init__(*F, **E)
        if (isinstance(B, str) or isinstance(B, unicode)):
            A.title = BroText(B)
            A.title.setStyleSheet('QLabel {font-weight:bold;}')
        else:
            A.title = B
        if (isinstance(C, str) or isinstance(C, unicode)):
            A.text = BroText(C)
        else:
            A.text = C
        A.widget = A.text
        A.layout = QtWidgets.QHBoxLayout()
        A.layout.addWidget(A.title, 1)
        A.layout.addWidget(A.text, 1)
        A.layout.setSpacing(G)
        A.layout.setContentsMargins(D, D, D, D)
        A.setLayout(A.layout)

class BroFormLayout(QtWidgets.QVBoxLayout):

    def __init__(A, *B, **C):
        super(BroFormLayout, A).__init__(*B, **C)

    def addRow(C, title, widget):
        B = widget
        A = title
        log.debug(A, B, type(A), type(B))
        D = BroEntry(A, B)
        C.addWidget(D)

class BroSlider(QtWidgets.QSlider):

    def __init__(A, onSliderPressed=None, onSliderReleased=None, onValueChanged=None):
        D = onValueChanged
        C = onSliderReleased
        B = onSliderPressed
        super(BroSlider, A).__init__()
        A.setOrientation(QtCore.Qt.Horizontal)
        A.setTickPosition(QtWidgets.QSlider.TicksBelow)
        A.hasTracking()
        A.pressed = False
        A.middle_range = True
        A.sliderReleased.connect(A.on_release)
        A.sliderPressed.connect(A.on_press)
        if (B is not None):
            A.sliderPressed.connect(B)
        if (C is not None):
            A.sliderReleased.connect(C)
        if (D is not None):
            A.valueChanged.connect(D)
        A.setObjectName(str(uuid4()))

    def on_press(A):
        A.pressed = True

    def on_release(A):
        A.pressed = False

    def paintEvent(A, event):
        super(BroSlider, A).paintEvent(event)
        if A.pressed:
            B = QtGui.QPainter()
            B.begin(A)
            B.setPen(QtGui.QPen(QtCore.Qt.NoPen))
            C = QtGui.QFont()
            C.setFamily('Tahoma')
            C.setPixelSize(14)
            C.setBold(True)
            if A.middle_range:
                F = ((A.minimum() + A.maximum()) / 2)
                D = ((A.value() - F) * 2)
            else:
                D = A.value()
            E = str(D)
            G = QtGui.QPen(QtGui.QColor(0, 0, 0, 160))
            B.setPen(G)
            B.setFont(C)
            B.drawText(2, 2, A.width(), A.height(), QtCore.Qt.AlignCenter, E)
            H = QtGui.QPen(QtGui.QColor(255, 255, 255, 240))
            B.setPen(H)
            B.drawText(0, 0, A.width(), A.height(), QtCore.Qt.AlignCenter, E)
            B.end()
        else:
            B = QtGui.QPainter()
            B.begin(A)
            B.end()

class BroDoubleSlider(BroSlider):
    doubleValueChanged = QtCore.Signal(float)

    def __init__(A, decimals=3, *B, **C):
        super(BroDoubleSlider, A).__init__(*B, **C)
        A._multi = (10 ** decimals)
        A.valueChanged.connect(A.emitDoubleValueChanged)

    def emitDoubleValueChanged(A):
        B = (float(super(BroDoubleSlider, A).value()) / A._multi)
        A.doubleValueChanged.emit(B)

    def value(A):
        return (float(super(BroDoubleSlider, A).value()) / A._multi)

    def setMinimum(A, value):
        return super(BroDoubleSlider, A).setMinimum((value * A._multi))

    def setMaximum(A, value):
        return super(BroDoubleSlider, A).setMaximum((value * A._multi))

    def setRange(A, min, max):
        return super(BroDoubleSlider, A).setRange((min * A._multi), (max * A._multi))

    def setSingleStep(A, value):
        return super(BroDoubleSlider, A).setSingleStep((value * A._multi))

    def singleStep(A):
        return (float(super(BroDoubleSlider, A).singleStep()) / A._multi)

    def setValue(A, value):
        super(BroDoubleSlider, A).setValue(int((value * A._multi)))

class QHLine(QtWidgets.QFrame):

    def __init__(A):
        super(QHLine, A).__init__()
        A.setFrameShape(QtWidgets.QFrame.HLine)
        A.setFrameShadow(QtWidgets.QFrame.Sunken)

class QVLine(QtWidgets.QFrame):

    def __init__(A):
        super(QVLine, A).__init__()
        A.setFrameShape(QtWidgets.QFrame.VLine)
        A.setFrameShadow(QtWidgets.QFrame.Sunken)
'\n# Outliner Drag and drop example\nclass HierarchyWidget(QTreeWidget):\n    """ Subclass of QTreeWidget to support Drag and Drop in Maya. """\n\n    def __init__(self, *args, **kwargs):\n        super(HierarchyWidget, self).__init__(*args, **kwargs)\n\n        self.setAcceptDrops(True)\n\n    def dragEnterEvent(self, event):\n        """ Reimplementing event to accept plain text, """\n        if event.mimeData().hasFormat(\'text/plain\'):\n            event.accept()\n        else:\n            event.ignore()\n\n    def dragMoveEvent(self, event):\n        """ Reimplementing event to accept plain text, """\n        if event.mimeData().hasFormat(\'text/plain\'):\n            event.accept()\n        else:\n            event.ignore()\n\n    def dropEvent(self, event):\n        print(event.mimeData().text())\n\n    def supportedDropActions(self):\n        """ """\n        return Qt.MoveAction | Qt.CopyAction\n        \na = HierarchyWidget()\na.show()\n'

class PythonSyntaxHighlighter(QtGui.QSyntaxHighlighter):

    def __init__(A, parent=None):
        super(PythonSyntaxHighlighter, A).__init__(parent)
        A.highlighting_rules = []
        B = QtGui.QTextCharFormat()
        B.setForeground(QtGui.QColor('#FAA300'))
        B.setFontWeight(QtGui.QFont.Bold)
        G = ['and', 'as', 'assert', 'break', 'class', 'continue', 'def', 'del', 'elif', 'else', 'except', 'False', 'finally', 'for', 'from', 'global', 'if', 'import', 'in', 'is', 'lambda', 'None', 'nonlocal', 'not', 'or', 'pass', 'raise', 'return', 'True', 'try', 'while', 'with', 'yield']
        A.add_mapping((('\\b(' + '|'.join(G)) + ')\\b'), B)
        D = QtGui.QTextCharFormat()
        D.setForeground(QtGui.QColor('#DCDCAA'))
        H = ['abs', 'all', 'any', 'ascii', 'bin', 'bool', 'bytearray', 'bytes', 'chr', 'classmethod', 'compile', 'complex', 'delattr', 'dict', 'dir', 'divmod', 'enumerate', 'eval', 'exec', 'filter', 'float', 'format', 'frozenset', 'getattr', 'globals', 'hasattr', 'hash', 'help', 'hex', 'id', 'input', 'int', 'isinstance', 'issubclass', 'iter', 'len', 'list', 'locals', 'map', 'max', 'memoryview', 'min', 'next', 'object', 'oct', 'open', 'ord', 'pow', 'print', 'property', 'range', 'repr', 'reversed', 'round', 'set', 'setattr', 'slice', 'sorted', 'staticmethod', 'str', 'sum', 'super', 'tuple', 'type', 'vars', 'zip']
        A.add_mapping((('\\b(' + '|'.join(H)) + ')\\b'), D)
        C = QtGui.QTextCharFormat()
        C.setForeground(QtGui.QColor('#78aece'))
        A.add_mapping('"[^"\\\\]*(\\\\.[^"\\\\]*)*"', C)
        A.add_mapping("'[^'\\\\]*(\\\\.[^'\\\\]*)*'", C)
        E = QtGui.QTextCharFormat()
        E.setForeground(QtGui.QColor('#B5CEA8'))
        A.add_mapping('\\b[0-9]+\\b', E)
        F = QtGui.QTextCharFormat()
        F.setForeground(QtGui.QColor('#6A9955'))
        A.add_mapping('#[^\\n]*', F)

    def add_mapping(A, pattern, format):
        A.highlighting_rules.append((QtCore.QRegExp(pattern), format))

    def highlightBlock(C, text):
        for (E, format) in C.highlighting_rules:
            B = QtCore.QRegExp(E)
            A = B.indexIn(text)
            while (A >= 0):
                D = B.matchedLength()
                C.setFormat(A, D, format)
                A = B.indexIn(text, (A + D))

class LineNumberArea(QtWidgets.QWidget):

    def __init__(A, editor):
        B = editor
        super(LineNumberArea, A).__init__(B)
        A.editor = B

    def sizeHint(A):
        return QtCore.QSize(A.editor.line_number_area_width(), 0)

    def paintEvent(A, event):
        A.editor.line_number_area_paint_event(event)

class PythonCodeEditor(QtWidgets.QPlainTextEdit):

    def __init__(A, parent=None):
        super(PythonCodeEditor, A).__init__(parent)
        A.line_number_area = LineNumberArea(A)
        A.blockCountChanged.connect(A.update_line_number_area_width)
        A.updateRequest.connect(A.update_line_number_area)
        A.update_line_number_area_width()
        A.highlighter = PythonSyntaxHighlighter(A.document())
        B = QtGui.QFont()
        B.setFamily('Courier New')
        B.setFixedPitch(True)
        B.setPointSize(10)
        A.setFont(B)
        D = QtGui.QFontMetrics(B)
        A.setTabStopWidth((4 * D.width(' ')))
        C = A.palette()
        C.setColor(QtGui.QPalette.Base, QtGui.QColor('#1E1E1E'))
        C.setColor(QtGui.QPalette.Text, QtGui.QColor('#D4D4D4'))
        A.setPalette(C)

    def line_number_area_width(A):
        B = 1
        C = max(1, A.blockCount())
        while (C >= 10):
            C //= 10
            B += 1
        D = (3 + (A.fontMetrics().width('9') * B))
        return D

    def update_line_number_area_width(A):
        A.setViewportMargins(A.line_number_area_width(), 0, 0, 0)

    def update_line_number_area(A, rect, dy):
        B = rect
        if dy:
            A.line_number_area.scroll(0, dy)
        else:
            A.line_number_area.update(0, B.y(), A.line_number_area.width(), B.height())
        if B.contains(A.viewport().rect()):
            A.update_line_number_area_width()

    def resizeEvent(A, event):
        super(PythonCodeEditor, A).resizeEvent(event)
        B = A.contentsRect()
        A.line_number_area.setGeometry(QtCore.QRect(B.left(), B.top(), A.line_number_area_width(), B.height()))

    def line_number_area_paint_event(A, event):
        D = event
        E = QtGui.QPainter(A.line_number_area)
        E.fillRect(D.rect(), QtGui.QColor('#2D2D2D'))
        B = A.firstVisibleBlock()
        G = B.blockNumber()
        H = A.contentOffset()
        C = A.blockBoundingGeometry(B).translated(H).top()
        F = (C + A.blockBoundingRect(B).height())
        while (B.isValid() and (C <= D.rect().bottom())):
            if (B.isVisible() and (F >= D.rect().top())):
                I = str((G + 1))
                E.setPen(QtGui.QColor('#858585'))
                E.drawText(0, C, (A.line_number_area.width() - 2), A.fontMetrics().height(), QtCore.Qt.AlignRight, I)
            B = B.next()
            C = F
            F = (C + A.blockBoundingRect(B).height())
            G += 1

class ObjectListWidget(QtWidgets.QWidget):
    'Custom widget for managing Maya object lists'

    def __init__(A, parent, title, object_type=None, icon_path=None, tooltip_filter=None):
        "\n        Initialize an object list widget\n        \n        Args:\n            parent: Parent widget\n            title: Title of the list\n            object_type: Type of objects to accept ('mesh' or None for any transform)\n            icon_path: Path to the icon for list items\n            tooltip_filter: Event filter for tooltips\n        "
        B = parent
        super(ObjectListWidget, A).__init__(B)
        A.parent = B
        A.title = title
        A.object_type = object_type
        A.icon_path = icon_path
        A.tooltip_filter = tooltip_filter
        A.buildUI()

    def buildUI(A):
        'Build the UI for this widget'
        A.layout = QtWidgets.QVBoxLayout(A)
        A.setLayout(A.layout)
        A.listLabel = QtWidgets.QLabel(f'{A.title}:')
        A.layout.addWidget(A.listLabel)
        A.objectsList = QtWidgets.QListWidget()
        A.objectsList.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        A.objectsList.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        A.objectsList.customContextMenuRequested.connect(A.showListContextMenu)
        A.layout.addWidget(A.objectsList)
        A.buttonsLayout = QtWidgets.QHBoxLayout()
        A.addObjectBtn = QtWidgets.QPushButton(f'Add {A.title}')
        A.addObjectBtn.clicked.connect(A.addSelectedObjects)
        if A.tooltip_filter:
            A.addObjectBtn.installEventFilter(A.tooltip_filter)
        A.buttonsLayout.addWidget(A.addObjectBtn)
        A.removeBtn = QtWidgets.QPushButton('Remove')
        B = os.path.join(modpath, 'imgs', 'i16_remove.png')
        if os.path.exists(B):
            A.removeBtn.setIcon(QtGui.QIcon(B))
        A.removeBtn.clicked.connect(A.removeSelectedObjects)
        if A.tooltip_filter:
            A.removeBtn.installEventFilter(A.tooltip_filter)
        A.buttonsLayout.addWidget(A.removeBtn)
        A.clearBtn = QtWidgets.QPushButton('Clear All')
        C = os.path.join(modpath, 'imgs', 'i16_clear.png')
        if os.path.exists(C):
            A.clearBtn.setIcon(QtGui.QIcon(C))
        A.clearBtn.clicked.connect(A.clearList)
        if A.tooltip_filter:
            A.clearBtn.installEventFilter(A.tooltip_filter)
        A.buttonsLayout.addWidget(A.clearBtn)
        A.layout.addLayout(A.buttonsLayout)

    def addSelectedObjects(A):
        'Add selected objects to the list'
        K = convertListToBroNodes(cmds.ls(sl=True))
        for B in K:
            E = B.name
            F = B.long_name
            G = True
            if (A.object_type == 'mesh'):
                H = cmds.objectType(B.long_name)
                C = False
                if (H == 'transform'):
                    I = cmds.listRelatives(B.long_name, shapes=True, fullPath=True)
                    if I:
                        for L in I:
                            if (cmds.objectType(L) == 'mesh'):
                                C = True
                                break
                elif (H == 'mesh'):
                    C = True
                G = C
                if (not C):
                    log.warning(f'Object {E} is not a mesh and cannot be used as a {A.title.lower()}.')
            if G:
                D = QtWidgets.QListWidgetItem()
                D.setText(E)
                D.setData(QtCore.Qt.UserRole, F)
                if (A.icon_path and os.path.exists(A.icon_path)):
                    D.setIcon(QtGui.QIcon(A.icon_path))
                J = False
                for M in range(A.objectsList.count()):
                    if (A.objectsList.item(M).data(QtCore.Qt.UserRole) == F):
                        J = True
                        break
                if (not J):
                    A.objectsList.addItem(D)

    def showListContextMenu(A, position):
        'Show context menu for the list'
        B = QtWidgets.QMenu()
        C = B.addAction('Remove')
        C.setIcon(QtGui.QIcon(os.path.join(modpath, 'imgs', 'i16_remove.png')))
        C.triggered.connect(A.removeSelectedObjects)
        D = B.addAction('Select in Maya')
        D.triggered.connect(A.selectObjectInMaya)
        B.exec_(A.objectsList.mapToGlobal(position))

    def removeSelectedObjects(A):
        'Remove selected objects from the list'
        for B in A.objectsList.selectedItems():
            A.objectsList.takeItem(A.objectsList.row(B))

    def clearList(A):
        'Clear all objects from the list'
        A.objectsList.clear()

    def selectObjectInMaya(B):
        'Select the object in Maya scene'
        A = []
        for C in B.objectsList.selectedItems():
            D = C.data(QtCore.Qt.UserRole)
            A.append(D)
        if A:
            cmds.select(A, replace=True)

    def getObjects(A):
        'Get all objects in the list'
        B = []
        for C in range(A.objectsList.count()):
            D = A.objectsList.item(C)
            E = D.data(QtCore.Qt.UserRole)
            B.append(E)
        return B

class GroupedComboBox(QtWidgets.QComboBox):
    '\n    A QComboBox with support for grouping items.\n    \n    Groups are visual only and don\'t affect indices or selection.\n    Groups are not selectable and are skipped when using the scroll wheel.\n    Maintains full backward compatibility with QComboBox.\n    \n    Usage:\n        combo = GroupedComboBox()\n        combo.addItem("Item 1")\n        combo.addItem("Item 2", userData="Data 2")\n        combo.addItem("Item 3", group="Group 1")\n        combo.addItem("Item 4", group="Group 1")\n        combo.addItem("Item 5", group="Group 2")\n    '

    def __init__(A, parent=None):
        super().__init__(parent)
        A._model = QtGui.QStandardItemModel(A)
        A.setModel(A._model)
        A._view = QtWidgets.QListView()
        A._view.setTextElideMode(QtCore.Qt.ElideRight)
        A.setView(A._view)
        A._real_to_visual = {}
        A._visual_to_real = {}
        A._real_count = 0
        A.view().pressed.connect(A._on_item_pressed)
        A._model.rowsInserted.connect(A._ensure_valid_selection)
        A.currentIndexChanged.connect(A._on_index_changed)

    def _on_item_pressed(A, index):
        'Prevent groups from being selected'
        B = A._model.itemFromIndex(index)
        if (B and B.data((QtCore.Qt.UserRole + 1))):
            A.view().selectionModel().clear()

    def addItem(B, *A, **I):
        '\n        Add an item to the combobox, optionally in a group.\n        \n        Supports multiple signatures for compatibility:\n        - addItem(text, userData=None, group=None)\n        - addItem(icon, text, userData=None, group=None)\n        \n        Parameters:\n            text (str): Text to display for this item\n            icon (QIcon, optional): Icon to display with this item\n            userData (any, optional): Data associated with this item\n            group (str, optional): Group name to place this item under\n        \n        Returns:\n            int: The real index of the added item\n        \n        Note: The group parameter should be passed as a keyword argument.\n        '
        F = None
        G = None
        D = None
        H = I.get('group', None)
        if (len(A) == 1):
            F = A[0]
        elif (len(A) == 2):
            if isinstance(A[0], QtGui.QIcon):
                G = A[0]
                F = A[1]
            else:
                F = A[0]
                D = A[1]
        elif (len(A) == 3):
            if isinstance(A[0], QtGui.QIcon):
                G = A[0]
                F = A[1]
                D = A[2]
            else:
                F = A[0]
                D = A[1]
        if (('userData' in I) and (D is None)):
            D = I['userData']
        if (H is not None):
            K = False
            for N in range(B._model.rowCount()):
                C = B._model.item(N)
                if (C and C.data((QtCore.Qt.UserRole + 1)) and (C.text() == H)):
                    K = True
                    break
            if (not K):
                E = QtGui.QStandardItem(H)
                E.setFlags(QtCore.Qt.NoItemFlags)
                E.setData(True, (QtCore.Qt.UserRole + 1))
                L = E.font()
                L.setBold(True)
                E.setFont(L)
                E.setBackground(QtGui.QColor(37, 37, 37))
                E.setForeground(QtGui.QColor(169, 130, 59))
                B._model.appendRow(E)
        C = QtGui.QStandardItem(F)
        if (D is not None):
            C.setData(D, QtCore.Qt.UserRole)
        if (G is not None):
            C.setIcon(G)
        C.setData(False, (QtCore.Qt.UserRole + 1))
        C.setData(H, (QtCore.Qt.UserRole + 2))
        B._model.appendRow(C)
        M = (B._model.rowCount() - 1)
        J = B._real_count
        B._real_to_visual[J] = M
        B._visual_to_real[M] = J
        B._real_count += 1
        return J

    def insertItem(A, index, *B, **C):
        '\n        Insert an item at the specified index.\n        \n        Supports multiple signatures for compatibility:\n        - insertItem(index, text, userData=None, group=None)\n        - insertItem(index, icon, text, userData=None, group=None)\n        \n        For simplicity, this implementation simply adds the item to the end.\n        '
        return A.addItem(*B, **C)

    def itemText(A, index):
        'Get text for a real index.'
        B = index
        if ((B < 0) or (B >= A._real_count)):
            return ''
        C = A._real_to_visual.get(B)
        if (C is not None):
            D = A._model.item(C)
            if D:
                return D.text()
        return ''

    def itemData(A, index, role=QtCore.Qt.UserRole):
        'Get data for a real index.'
        B = index
        if ((B < 0) or (B >= A._real_count)):
            return
        C = A._real_to_visual.get(B)
        if (C is not None):
            D = A._model.item(C)
            if D:
                return D.data(role)

    def currentIndex(A):
        'Get the current real index.'
        B = super().currentIndex()
        return A._visual_to_real.get(B, (- 1))

    def _on_index_changed(B, index):
        'Handle when the visual index changes internally.'
        A = index
        if (A >= 0):
            C = B._model.item(A)
            if (C and C.data((QtCore.Qt.UserRole + 1))):
                B._select_next_non_group_item(A)

    def _select_next_non_group_item(C, from_index):
        'Find and select the next non-group item from the given index.'
        D = from_index
        for A in range((D + 1), C._model.rowCount()):
            B = C._model.item(A)
            if (B and (not B.data((QtCore.Qt.UserRole + 1)))):
                super().setCurrentIndex(A)
                return
        for A in range((D - 1), (- 1), (- 1)):
            B = C._model.item(A)
            if (B and (not B.data((QtCore.Qt.UserRole + 1)))):
                super().setCurrentIndex(A)
                return
        super().setCurrentIndex((- 1))

    def _ensure_valid_selection(A):
        'Ensure the current selection is not a group after model changes.'
        B = super().currentIndex()
        if ((B < 0) and (A._model.rowCount() > 0)):
            A._select_next_non_group_item((- 1))
        elif (B >= 0):
            C = A._model.item(B)
            if (C and C.data((QtCore.Qt.UserRole + 1))):
                A._select_next_non_group_item(B)

    def setCurrentIndex(B, index):
        '\n        Set current index using a real index.\n        This also handles the case where external code might call\n        the original QComboBox setCurrentIndex with a visual index.\n        '
        A = index
        if ((A < 0) or (A >= B._real_count)):
            if (0 <= A < B._model.rowCount()):
                C = B._model.item(A)
                if (C and C.data((QtCore.Qt.UserRole + 1))):
                    B._select_next_non_group_item(A)
                    return
                super().setCurrentIndex(A)
            else:
                super().setCurrentIndex((- 1))
            return
        D = B._real_to_visual.get(A)
        if (D is not None):
            super().setCurrentIndex(D)

    def count(A):
        'Return the number of items (excluding groups).'
        return A._real_count

    def wheelEvent(B, event):
        'Override wheel event to skip groups when scrolling.'
        C = event
        D = super().currentIndex()
        if (C.angleDelta().y() > 0):
            A = D
            while True:
                A -= 1
                if (A < 0):
                    break
                if (A in B._visual_to_real):
                    super().setCurrentIndex(A)
                    break
        else:
            A = D
            while True:
                A += 1
                if (A >= B._model.rowCount()):
                    break
                if (A in B._visual_to_real):
                    super().setCurrentIndex(A)
                    break
        C.accept()
