from base64 import b64decode as license64

"\n------------------------------------------------------------------------------------------------------------------------\n# Maya 2017 PySide2 Docking Qt QMainWindow by Michael Davydov | https://github.com/Nixellion/\n## https://www.michaeldavydov.com | https://www.nixes.ru\n------------------------------------------------------------------------------------------------------------------------\nThis module aims to have functions and classes that you can use to create your own Maya UI, with support of Maya with\nPySide and PySide2 (2014-2018+). Primary focus on dockable windows.\n\nMajor changes have been made in Maya version 2017, when it moved from Qt4\\PySide to Qt5\\PySide2.\n\nI will update this file eventually with more functions and stuff.\n\n------------------------------------------------------------------------------------------------------------------------\nHonrable mentions of people who's work was used as part of this code, for inspiration or help:\n\n - Lior Ben Horin https://gist.github.com/liorbenhorin/ - for starting code snippet, which showed how to start working with\nworkspaceControl.\n - Guys from techart.online who helped with advice: http://discourse.techart.online/t/maya-dockable-windows-boilerplate-with-support-for-pyside2/9617/14\n\n------------------------------------------------------------------------------------------------------------------------\nTo quickly adapt this code for youserlf do the following:\n\n - Replace `log.debug` with `print`\n - Retarget Qt.py imports for your project\n\nShould work after that.\n------------------------------------------------------------------------------------------------------------------------\n"
import inspect
__author__ = 'Michael Davydov https://www.michaeldavydov.com'
import os, uuid, weakref
rp = os.path.dirname(os.path.realpath(__file__))
from maya.api import OpenMaya as om, OpenMayaUI as omui
from maya import OpenMaya as om_1
from maya import OpenMayaUI as omui_1
from maya import cmds, mel
from BroTools.core.broqt.vendor.Qt import QtCompat
from maya.app.general.mayaMixin import MayaQWidgetBaseMixin, MayaQWidgetDockableMixin
from BroTools.core.broqt.vendor.Qt import QtWidgets, QtGui, QtCore
from BroTools.configuration import brotools_config, debug_log, bro_qss, hud_qss
from BroTools.core.debug import deprecated
from BroTools.core.broqt.utils import view_to_pixmap, apply_effect_to_image, get_maya_viewport_widget, add_menu_action
from BroTools.core import callbacks
from BroTools.configuration import styles_path, modpath
from BroTools.configuration import maya_version
imgs_path = os.path.join(modpath, 'imgs')
instance_cache = []
mixinWindows = {}

class DockableBase(MayaQWidgetDockableMixin):
    '\n    Convenience class for creating dockable Maya windows.\n    '

    def __init__(A, controlName, **C):
        B = controlName
        super(DockableBase, A).__init__(**C)
        debug_log('DockableBase __init__: {}; '.format(B, A.__class__.__name__))
        A.setObjectName(B)

    def show(A, *C, **B):
        '\n        Show UI with generated uiScript argument\n        '
        debug_log('DockableBase.show({}, {}) - {}'.format(C, B, A.__class__.__name__))
        D = inspect.getmodule(A).__name__
        E = A.__class__.__name__
        B.pop('dockable', None)
        super(DockableBase, A).show(dockable=True, uiScript='import {0}; {0}.{1}._restoreUI()'.format(D, E), **B)

    @classmethod
    def _restoreUI(B):
        '\n        Internal method to restore the UI when Maya is opened.\n        '
        debug_log('DockableBase._restoreUI')
        A = B()
        C = omui_1.MQtUtil.getCurrentParent()
        D = omui_1.MQtUtil.findControl(A.objectName())
        omui_1.MQtUtil.addWidgetToMayaLayout(long(D), long(C))
        global mixinWindows
        mixinWindows[A.objectName()] = A

class BroMainWindow_Dockable(DockableBase, QtWidgets.QMainWindow):
    '\n    DO NOT use WA_DeleteOnClose  with this, other approach is needed.\n    '
    DOCK_LABEL_NAME = 'no name window'
    CONTROL_NAME = 'no_name_window'
    instances = list()

    def __init__(A):
        instance_cache.append(weakref.proxy(A))
        debug_log('Init BroMainWindow_Dockable: {}, type: {}'.format(A.DOCK_LABEL_NAME, type(A)))
        DockableBase.__init__(A, A.CONTROL_NAME)
        A.delete_instances()
        A.__class__.instances.append(weakref.proxy(A))
        if (maya_version >= 2017):
            if (cmds.window((A.CONTROL_NAME + 'WorkspaceControl'), ex=True) or cmds.workspaceControl((A.CONTROL_NAME + 'WorkspaceControl'), ex=True)):
                debug_log('Removing', (A.CONTROL_NAME + 'WorkspaceControl'))
                cmds.deleteUI((A.CONTROL_NAME + 'WorkspaceControl'))
                debug_log('Removed', (A.CONTROL_NAME + 'WorkspaceControl'))
            else:
                debug_log('Not found, no need to remove:', (A.CONTROL_NAME + 'WorkspaceControl'))
        elif cmds.window((A.CONTROL_NAME + 'DockControl'), ex=True):
            debug_log('Removing', (A.CONTROL_NAME + 'DockControl'))
            cmds.deleteUI((A.CONTROL_NAME + 'DockControl'))
            debug_log('Removed', (A.CONTROL_NAME + 'DockControl'))
        else:
            debug_log('Not found, no need to remove:', (A.CONTROL_NAME + 'DockControl'))
        A.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        A.setWindowTitle(A.DOCK_LABEL_NAME)
        A.central_widget = QtWidgets.QWidget()
        A.central_widget.setStyleSheet(str(bro_qss))
        A.setCentralWidget(A.central_widget)
        A.main_layout = QtWidgets.QVBoxLayout()
        A.central_widget.setLayout(A.main_layout)
        A.load_default_qss()

    @staticmethod
    def delete_instances():
        for A in BroMainWindow_Dockable.instances:
            try:
                debug_log('Delete {}'.format(A))
            except:
                debug_log('Window reference seems to be removed already, ignore.')
            try:
                A.setParent(None)
                A.deleteLater()
            except:
                pass
            try:
                BroMainWindow_Dockable.instances.remove(A)
                del A
            except:
                pass

    @deprecated
    def buildUI(self):
        "\n        DEPRECATED\n        This function is called at the end of window initialization and creates your actual UI.\n        Override it with your UI.\n\n        TODO Deprecation notice. Switch everything to running stuff after super().__init__()\n\n        #   When first creating it I ~was dumb~ didn't realize that I could just\n        #   run parent class' __init__ function with super().__init__() and then run my own code . Other than that his buildUI function is only\n        #   relevant if I were doing something else AFTER it, but I'm not. And instead it makes PyCharm and linters\n        #   freak out when I define new variables not in __init__, which has no apparent issues as it's still techni-\n        #   cally in init, but... ah well.\n        "
        pass

    def set_default_css(B, qss=None):
        A = qss
        if (A is None):
            A = bro_qss
        B.setStyleSheet(str(A))

    def dockCloseEventTriggered(A):
        A.delete_instances()
        super(BroMainWindow_Dockable, A).dockCloseEventTriggered()

    def closeEvent(A, evt):
        'Hide the QDockWidget and trigger the closeEventTriggered signal\n        '
        if evt.isAccepted():
            A.deleteLater()
            A.parentWidget().deleteLater()
    load_default_css = set_default_css
    load_css = set_default_css
    load_default_qss = set_default_css
    load_qss = set_default_css
global brodynamics_simple_windows
brodynamics_simple_windows = {}

class BroSimpleWindow(MayaQWidgetBaseMixin, QtWidgets.QWidget):

    def __init__(A, rootWidget=None, icon=None, qss=None, title=None, blocking=False, objectName=None, deleteExisting=False, *F, **G):
        '\n\n        :param rootWidget: Root widget\\parent widget\n        :param icon: window icon\n        :param qss: path to window qss style file\n        :param title: window title\n        :param blocking: should block access to parent window or not\n        :param deleteExisting: Removes existing windows with same name. Requires unique name.\n        '
        E = deleteExisting
        D = title
        C = rootWidget
        B = objectName
        global brodynamics_simple_windows
        instance_cache.append(weakref.proxy(A))
        if ((E is True) and (B is None)):
            print("BroSimpleWindow is setting name to '{}', to work with deleteExisting attribute.".format(type(A).__name__))
            B = type(A).__name__
        elif (B is None):
            B = str(uuid.uuid4())
        if E:
            if cmds.window(B, ex=True):
                print("Found existing '{}' window, deleting.".format(B))
                cmds.deleteUI(B, wnd=True)
            elif (B in brodynamics_simple_windows):
                print("Found existing '{}' window in dict cache, deleting.".format(B))
                try:
                    brodynamics_simple_windows[B].deleteLater()
                except:
                    pass
            brodynamics_simple_windows[B] = A
        super(BroSimpleWindow, A).__init__(*F, **G)
        if (C is not None):
            A.rootWidget = C
        else:
            H = omui_1.MQtUtil.mainWindow()
            A.rootWidget = QtCompat.wrapInstance(int(H), QtWidgets.QWidget)
        if (A.rootWidget != 'skip'):
            A.setParent(A.rootWidget)
        A.qss = qss
        if (icon != None):
            A.setWindowIcon(QtGui.QIcon(icon))
            A.setWindowFlags((((QtCore.Qt.Dialog | QtCore.Qt.WindowStaysOnTopHint) | QtCore.Qt.CustomizeWindowHint) | QtCore.Qt.WindowTitleHint))
        else:
            A.setWindowFlags(QtCore.Qt.Dialog)
        A.setAttribute(QtCore.Qt.WA_AlwaysShowToolTips)
        if blocking:
            A.setWindowModality(QtCore.Qt.ApplicationModal)
        if (D != None):
            A.setWindowTitle(D)
        else:
            A.setWindowTitle(' ')
        A.setObjectName(B)
        A.load_qss()

    @deprecated
    def buildUI(self):
        pass

    def load_qss(A):
        if (A.qss is None):
            A.qss = bro_qss
        A.setStyleSheet(str(A.qss))

    def add_script_job(A, event, command):
        try:
            cmds.scriptJob(event=[event, command], parent=A.objectName())
            debug_log('Script job added successfully to {}!'.format(A.objectName()))
        except Exception as B:
            debug_log('Script job was not added: {}'.format(B))
global viewport_widgets
viewport_widgets = []

def hide_viewport_widgets():
    debug_log('Hiding:')
    for A in viewport_widgets:
        if ((A is not None) and QtCompat.isValid(A)):
            debug_log('    {}'.format(A))
            A.hide()

def show_viewport_widgets():
    debug_log('Showing:')
    for A in viewport_widgets:
        if ((A is not None) and QtCompat.isValid(A)):
            debug_log('    {}'.format(A))
            A.show()
global playing_back
playing_back = None

def on_playingback_callback(*A, **B):
    global playing_back
    if (True in A):
        playing_back = True
        hide_viewport_widgets()
    else:
        playing_back = False
        show_viewport_widgets()
repaint_frame = 0

def repaint_viewport_widgets(*B, **C):
    global repaint_frame
    if (not playing_back):
        if (repaint_frame > 100):
            for A in viewport_widgets:
                if (A is not None):
                    A.repaint()
            repaint_frame = 0
        repaint_frame += 1

def callback_test(*A, **B):
    print('\nCallback received:\n    {}\n    {}\n---\n'.format(A, B))
callbacks.add_callback(om.MConditionMessage.addConditionCallback, 'playingBack', on_playingback_callback)
' \nMaya 2019\nfrom maya.api import OpenMaya as om\n\nfor i in om.MConditionMessage.getConditionNames():\n    print i\n    \nnewing\nplayingBack\nplayingBackAuto\nrecording\ndeleteAllCondition\ntimeChangeTemp\nSomethingSelected\nautoKeyframeState\ndelete\ndeleteUndo\ndeleteRedo\nisCurrentRenderLayerChanging\nisApplyingMemberOverride\nBatchMode\nMayaInitialized\nreadingFile\nImageUIExists\nPolyCoreExists\nBaseMayaExists\nRenderingExists\nAnimationExists\nBaseUIExists\nflushingScene\nGoButtonEnabled\nManipsExists\nMayaCreatorExists\nModelExists\nSubdivExists\nPolygonsExists\nPolyTextureExists\nDynamicsExists\nDatabaseUIExists\nKinematicsExists\nNurbsExists\nDeformersExists\nbusy\nUndoAvailable\nRedoAvailable\nopening\nwritingFile\npostSceneCallbacks\nreadingReferenceFile\nOpenMayaExists\nplayblasting\nalwaysWriteReferenced\nDimensionsExists\nClipEditorExists\nAnimationUIExists\nModelUIExists\nDynamicsUIExists\nRenderingUIExists\nExplorerExists\nDeformersUIExists\nKinematicsUIExists\nSubdivUIExists\nPolygonsUIExists\nNurbsUIExists\nSurfaceUIExists\nsoftSelectOptions\npanZoomEnabled\nplaybackIconsCondition\nhotkeyListChange\nTimeEditorExists\n'

class ViewportFloatingWidget(BroSimpleWindow):
    '\n    from BroTools.core.broqt.bro_window import ViewportFloatingWidget\n    w = ViewportFloatingWidget()\n    w.show()\n    '
    redraw_delay = 250

    def __init__(A, *B, **C):
        A.draggableHorizontally = True
        A.draggableVertically = True
        A.blur_amount = 0
        A.offset_amount = (A.blur_amount * 4)
        A.downsample_factor = 17
        A.tint = None
        D = get_maya_viewport_widget()
        viewport_widgets.append(A)
        super(ViewportFloatingWidget, A).__init__(*B, rootWidget='skip', **C)
        A.setWindowFlags(QtCore.Qt.FramelessWindowHint)
        A.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        A.setParent(D)
        A.repaint_timer = QtCore.QTimer()
        A.repaint_timer.timeout.connect(A.repaint)
        A.repaint_timer.start(A.redraw_delay)

    def load_qss(A):
        if (A.qss is None):
            A.qss = hud_qss
        A.setStyleSheet(str(A.qss))

    def mousePressEvent(B, event):
        A = event
        if (A.button() == QtCore.Qt.MiddleButton):
            B.oldPos = A.globalPos()

    def mouseMoveEvent(A, event):
        C = event
        if (C.buttons() == QtCore.Qt.MiddleButton):
            B = QtCore.QPoint((C.globalPos() - A.oldPos))
            A.move(((A.x() + B.x()) if (A.draggableHorizontally and ((A.x() + B.x()) > 0)) else A.x()), ((A.y() + B.y()) if (A.draggableVertically and ((A.y() + B.y()) > 0)) else A.y()))
            A.oldPos = C.globalPos()
            A.repaint()

    def paintBackground(A):
        C = QtGui.QPainter()
        C.setCompositionMode(QtGui.QPainter.CompositionMode_Source)
        C.begin(A)
        D = A.geometry()
        B = view_to_pixmap(as_image=True, cropRect=QtCore.QRect((D.left() - (A.offset_amount / 2)), (D.top() - (A.offset_amount / 2)), (D.width() + A.offset_amount), (D.height() + A.offset_amount)))
        (E, F) = (B.width(), B.height())
        B = B.scaled((E / A.downsample_factor), (F / A.downsample_factor), mode=QtCore.Qt.SmoothTransformation)
        B = B.scaled(E, F, mode=QtCore.Qt.SmoothTransformation)
        B = B.copy(QtCore.QRect((A.offset_amount / 2), (A.offset_amount / 2), (B.width() - A.offset_amount), (B.height() - A.offset_amount)))
        C.drawImage(QtCore.QPoint(0, 0), B)
        if (A.tint is not None):
            C.fillRect(0, 0, A.width(), A.height(), A.tint)
        C.end()

    def paintEvent(A, e):
        super(ViewportFloatingWidget, A).paintEvent(e)
        A.paintBackground()

class ViewportFloatingWindow(ViewportFloatingWidget):
    'Convenience class to allow drop-in replacement of dockable windows'
    DOCK_LABEL_NAME = 'no name window'
    CONTROL_NAME = 'no_name_window'
    instances = list()

    def __init__(A, *B, **C):
        super(ViewportFloatingWindow, A).__init__(*B, deleteExisting=True, **C)
        A.setObjectName(A.CONTROL_NAME)
        A.setWindowTitle(A.DOCK_LABEL_NAME)
        A.main_layout = QtWidgets.QVBoxLayout()
        A.setLayout(A.main_layout)
        A.central_widget = A

def get_main_window():
    '\n    Get main Maya window instance.\n    '
    A = omui_1.MQtUtil.mainWindow()
    B = QtCompat.wrapInstance(int(A), QtWidgets.QMainWindow)
    return B

def dock_window(dialog_class, width=440):
    '\n    This function is to be updates to actually dock window on creation to the right.\n    '
    A = dialog_class
    B = A()
    if (isinstance(A, BroMainWindow_Dockable) or isinstance(A, DockableBase)):
        B.show(dockable=True)
    else:
        B.show()
    return B

def show_window(dialog_class):
    '\n    Show non dockable window\n\n    :param dialog_class:\n    :return:\n    '
    A = dialog_class()
    A.show(dockable=False)
    return A

def createwindow(dialog_class):
    "\n    Create window, but don't show it\n    :param dialog_class:\n    :return:\n    "
    A = dialog_class()
    return A

def convert_to_qt(mayaName, objectType):
    '\n    Given the name of a Maya UI element of any type, return the corresponding QT Type object.\n    '
    B = mayaName
    A = omui_1.MQtUtil.findControl(B)
    if (A is None):
        A = omui_1.MQtUtil.findLayout(B)
        if (A is None):
            A = omui_1.MQtUtil.findMenuItem(B)
    if (A is not None):
        return QtCompat.wrapInstance(int(A), objectType)

def show_test():
    '\n    Show example\test window.\n    '
    ChildTestWindow().show(dockable=True)

class ChildTestWindow(BroMainWindow_Dockable):
    '\n    Example child window inheriting from main class.\n    '
    DOCK_LABEL_NAME = 'child test window'
    instances = list()
    CONTROL_NAME = 'child_test_win'

    def __init__(A):
        super(ChildTestWindow, A).__init__()

    def buildUI(A):
        A.my_label = QtWidgets.QLabel('Beam me up, Scotty!')
        A.main_layout.addWidget(A.my_label)
        A.menuBar = QtWidgets.QMenuBar()
        A.presetsMenu = A.menuBar.addMenu('&Presets')
        A.saveConfigAction = QtWidgets.QAction('&Save Settings', A)
        A.presetsMenu.addAction(A.saveConfigAction)
        A.setMenuBar(A.menuBar)
        A.statusBar = QtWidgets.QStatusBar()
        A.statusBar.showMessage('Status bar ready.')
        A.setStatusBar(A.statusBar)
        A.statusBar.setObjectName('statusBar')
        A.setStyleSheet('#statusBar {background-color:#faa300;color:#fff}')
