from base64 import b64decode as license64

from BroTools.core.broqt.vendor.Qt import QtGui
from dataclasses import dataclass
import json

@dataclass
class ColorPalette():
    'Contains all colors used in the UI. Implemented as a singleton to ensure consistent colors across all widgets.\n\n    Usage:\n        color = ColorPalette.instance().NODE_BACKGROUND\n    '
    _base_palette = {'black': '#191919', 'gray': {'50': '#fafafa', '100': '#f1f1f1', '200': '#e7e7e7', '300': '#d4d4d4', '400': '#adadad', '500': '#7f7f7f', '600': '#545454', '700': '#373737', '800': '#202020', '900': '#191919'}, 'purple': {'50': '#f9f6fe', '100': '#e6d9fa', '200': '#d3bef7', '300': '#b794f2', '400': '#a377ee', '500': '#884fe9', '600': '#732fe6', '700': '#5e24c2', '800': '#4d1d9f', '900': '#3a1677'}, 'pink': {'50': '#fef5f9', '100': '#fad8e8', '200': '#f6b7d5', '300': '#f189b9', '400': '#ec65a4', '500': '#e32a80', '600': '#c3246e', '700': '#a01d5a', '800': '#7d1746', '900': '#5d1134'}, 'red': {'50': '#fef5f5', '100': '#fad8d7', '200': '#f6b6b3', '300': '#f08885', '400': '#ed6c68', '500': '#e73c37', '600': '#c92a25', '700': '#a2221e', '800': '#8a1d19', '900': '#651512'}, 'orange': {'50': '#fefaf5', '100': '#faebd9', '200': '#f5d3ac', '300': '#edb06a', '400': '#e58e2a', '500': '#c57a24', '600': '#a6671f', '700': '#855218', '800': '#684113', '900': '#563510'}, 'yellow': {'50': '#fffefb', '100': '#fcf9e7', '200': '#f6eeb9', '300': '#f0e180', '400': '#e5cc2b', '500': '#bda823', '600': '#97861c', '700': '#766916', '800': '#584f10', '900': '#49410d'}, 'green': {'50': '#f3fef9', '100': '#bef7dd', '200': '#6bedb0', '300': '#28d986', '400': '#23be76', '500': '#1ea365', '600': '#198754', '700': '#136941', '800': '#105635', '900': '#0d472c'}, 'teal': {'50': '#eefcfd', '100': '#b3f4f6', '200': '#64e8ec', '300': '#27d2d7', '400': '#22b2b7', '500': '#1d989c', '600': '#177b7e', '700': '#126062', '800': '#0f5052', '900': '#0c4243'}, 'cyan': {'50': '#f3fbfd', '100': '#cbf0f9', '200': '#b2e8f6', '300': '#95e0f2', '400': '#37c4e7', '500': '#27b4d7', '600': '#23a2c2', '700': '#1d86a0', '800': '#186e84', '900': '#135566'}, 'blue': {'50': '#f1f7fd', '100': '#cae0f9', '200': '#a4c9f4', '300': '#78afef', '400': '#4e96e9', '500': '#297fe1', '600': '#236abd', '700': '#1a5190', '800': '#164376', '900': '#123660'}, 'primary': {'50': '#fdf7ec', '100': '#f5ddb1', '200': '#edbf6a', '300': '#d09d3e', '400': '#ba8d37', '500': '#9d762f', '600': '#846427', '700': '#6a5020', '800': '#5a441b', '900': '#413113'}}
    NODE_BACKGROUND: str = _base_palette['gray']['800']
    NODE_DEFAULT_HEADER: str = _base_palette['blue']['500']
    SOCKET_BACKGROUND: str = _base_palette['gray']['500']
    SOCKET_CONNECTED_BACKGROUND: str = _base_palette['green']['500']
    SOCKET_HOVERED: str = _base_palette['green']['300']
    SOCKET_BORDER: str = _base_palette['blue']['700']
    CONNECTION_LINE: str = SOCKET_CONNECTED_BACKGROUND
    CONNECTION_HOVERED: str = SOCKET_HOVERED
    TEXT_LIGHT: str = _base_palette['gray']['100']
    TEXT_DARK: str = _base_palette['black']
    SCENE_BACKGROUND: str = _base_palette['gray']['900']
    GRID_LINES: str = _base_palette['gray']['800']
    RED: str = _base_palette['red']['500']
    GREEN: str = _base_palette['green']['500']
    BLUE: str = _base_palette['blue']['500']
    YELLOW: str = _base_palette['yellow']['500']
    ORANGE: str = _base_palette['orange']['500']
    PURPLE: str = _base_palette['purple']['500']
    CYAN: str = _base_palette['cyan']['500']
    PINK: str = _base_palette['pink']['500']
    _instance = None

    def __new__(A):
        if (A._instance is None):
            A._instance = super().__new__(A)
            A._instance.__initialized = False
        return A._instance

    def __init__(A):
        if (not A.__initialized):
            super().__init__()
            A.__initialized = True
            for B in A.__dataclass_fields__:
                setattr(A, B, QtGui.QColor(getattr(A, B)))

    def to_json(A):
        return json.dumps({B: getattr(A, B).name() for B in A.__dataclass_fields__})

    @classmethod
    def from_json(B, json_str):
        A = B()
        C = json.loads(json_str)
        for (D, E) in C.items():
            setattr(A, D, QtGui.QColor(E))
        return A

    @classmethod
    def instance(A):
        'Get the singleton instance of ColorPalette'
        if (A._instance is None):
            A._instance = A()
        return A._instance

class Color(QtGui.QColor):

    @classmethod
    def fromString(E, text):
        '\n        :type text: str\n        '
        A = 255
        try:
            (B, C, D, A) = text.replace('rgb(', '').replace(')', '').split(',')
        except ValueError:
            (B, C, D) = text.replace('rgb(', '').replace(')', '').split(',')
        return E(int(B), int(C), int(D), int(A))

    @classmethod
    def fromColor(B, color):
        '\n        :type color: QtGui.QColor\n        '
        A = color
        A = ('rgb(%d, %d, %d, %d)' % A.getRgb())
        return B.fromString(A)

    def __eq__(B, other):
        A = other
        if (A == B):
            return True
        elif isinstance(A, Color):
            return (B.toString() == A.toString())
        else:
            return False

    def toString(A):
        '\n        :type: str\n        '
        return ('rgb(%d, %d, %d, %d)' % A.getRgb())

    def isDark(A):
        '\n        :type: bool\n        '
        return ((A.red() < 125) and (A.green() < 125) and (A.blue() < 125))
