from base64 import b64decode as license64

import ctypes
from maya.api import OpenMaya as om, OpenMayaUI as omui
from BroTools.core.broqt.vendor.Qt import QtWidgets, QtGui, QtCore
from BroTools.core.broqt.vendor.Qt import QtCompat
wrapInstance = QtCompat.wrapInstance
cb = QtWidgets.QApplication.clipboard()
try:
    long(1)
except:
    long = int

def clipboardSet(text):
    '\n    Store string in system clipboard.\n\n    :param text: Text string to store.\n    :return: None\n    '
    cb.clear(mode=cb.Clipboard)
    cb.setText(text, mode=cb.Clipboard)

def clipboardGet():
    '\n    :return: Value in system clipboard.\n    '
    return cb.text(mode=cb.Clipboard)
"\n#Import api modules\nimport maya.OpenMaya as api\nimport maya.OpenMayaUI as apiUI\n\n#Grab the last active 3d viewport\nview = apiUI.M3dView.active3dView()\n\n#read the color buffer from the view, and save the MImage to disk\nimage = api.MImage()\nview.readColorBuffer(image, True)\nimage.writeToFile('C:/test.jpg', 'jpg')\n"

def get_maya_viewport_widget(name=None):
    A = omui.M3dView()
    if name:
        omui.M3dView.getM3dViewFromModelPanel('modelPanel4', A)
    else:
        A = omui.M3dView.active3dView()
    B = wrapInstance(long(A.widget()), QtWidgets.QWidget)
    return B

def view_to_pixmap(size=None, as_image=False, cropRect=None):
    'Render currently active 3D viewport as a QPixmap'
    E = cropRect
    A = om.MImage()
    G = omui.M3dView.active3dView()
    G.readColorBuffer(A, True)
    A.verticalFlip()
    F = (size or QtCore.QSize(512, 256))
    B = A.getSize()
    D = ((ctypes.c_ubyte * B[0]) * B[1])
    D = D.from_address(long(A.pixels()))
    C = QtGui.QImage(D, B[0], B[1], QtGui.QImage.Format_RGB32).rgbSwapped()
    if E:
        C = C.copy(E)
    if as_image:
        return C
    return QtGui.QPixmap.fromImage(C).scaled(F.width(), F.height(), QtCore.Qt.KeepAspectRatio, QtCore.Qt.SmoothTransformation)

def apply_effect_to_image(src, effect):
    A = src
    C = QtWidgets.QGraphicsScene()
    B = QtWidgets.QGraphicsPixmapItem()
    B.setPixmap(QtGui.QPixmap.fromImage(A))
    C.addItem(B)
    D = QtGui.QImage(A.size(), QtGui.QImage.Format_RGB32)
    E = QtGui.QPainter(D)
    E.setCompositionMode(QtGui.QPainter.CompositionMode_Source)
    B.setGraphicsEffect(effect)
    C.render(E, QtCore.QRectF(), QtCore.QRectF(0, 0, A.width(), A.height()))
    return D

def add_menu_action(menu, title=None, checkbox=False, start_checked=False, on_triggered=None, on_changed=None, separator=False):
    '\n    Add an action to a QMenu, convenience wrapper\n\n    :param menu: QMenu instance to add menu to\n    :param title: Title of the menu item\n    :param on_triggered: Function to call on trigger\n    :param on_changed: Function to call on changed, works only if checkbox is True\n    :param checkbox: Whether this action is a checkbox\n    :return:\n    '
    C = on_changed
    B = on_triggered
    if separator:
        return menu.addSeparator()
    A = menu.addAction(title)
    if (checkbox is True):
        A.setCheckable(True)
        A.setChecked(start_checked)
        if (C is not None):
            A.changed.connect(C)
    if (B is not None):
        A.triggered.connect(B)
    return A
