from base64 import b64decode as license64

'Minimal Python 2 & 3 shim around all Qt bindings\n\nDOCUMENTATION\n    Qt.py was born in the film and visual effects industry to address\n    the growing need for the development of software capable of running\n    with more than one flavour of the Qt bindings for Python.\n\n    Supported Binding: PySide, PySide2, PySide6, PyQt4, PyQt5\n\n    1. Build for one, run with all\n    2. Explicit is better than implicit\n    3. Support co-existence\n\n    Default resolution order:\n        - PySide6\n        - PySide2\n        - PyQt5\n        - PySide\n        - PyQt4\n\n    Usage:\n        >> import sys\n        >> from Qt import QtWidgets\n        >> app = QtWidgets.QApplication(sys.argv)\n        >> button = QtWidgets.QPushButton("Hello World")\n        >> button.show()\n        >> app.exec_()\n\n    All members of PySide2 are mapped from other bindings, should they exist.\n    If no equivalent member exist, it is excluded from Qt.py and inaccessible.\n    The idea is to highlight members that exist across all supported binding,\n    and guarantee that code that runs on one binding runs on all others.\n\n    For more details, visit https://github.com/mottosso/Qt.py\n\nLICENSE\n\n    See end of file for license (MIT, BSD) information.\n\n'
import os, sys, types, shutil, importlib, json
__version__ = '1.4.1'
__all__ = []
QT_VERBOSE = bool(os.getenv('QT_VERBOSE'))
QT_PREFERRED_BINDING_JSON = os.getenv('QT_PREFERRED_BINDING_JSON', '')
QT_PREFERRED_BINDING = os.getenv('QT_PREFERRED_BINDING', '')
QT_SIP_API_HINT = os.getenv('QT_SIP_API_HINT')
Qt = sys.modules[__name__]
Qt.QtCompat = types.ModuleType('QtCompat')
try:
    long
except NameError:
    long = int
'Common members of all bindings\n\nThis is where each member of Qt.py is explicitly defined.\nIt is based on a "lowest common denominator" of all bindings;\nincluding members found in each of the 4 bindings.\n\nThe "_common_members" dictionary is generated using the\nbuild_membership.sh script.\n\n'
_common_members = {'QtCore': ['QAbstractAnimation', 'QAbstractEventDispatcher', 'QAbstractItemModel', 'QAbstractListModel', 'QAbstractTableModel', 'QAnimationGroup', 'QBasicTimer', 'QBitArray', 'QBuffer', 'QByteArray', 'QByteArrayMatcher', 'QChildEvent', 'QCoreApplication', 'QCryptographicHash', 'QDataStream', 'QDate', 'QDateTime', 'QDir', 'QDirIterator', 'QDynamicPropertyChangeEvent', 'QEasingCurve', 'QElapsedTimer', 'QEvent', 'QEventLoop', 'QFile', 'QFileInfo', 'QFileSystemWatcher', 'QGenericArgument', 'QGenericReturnArgument', 'QIODevice', 'QLibraryInfo', 'QLine', 'QLineF', 'QLocale', 'QMargins', 'QMetaClassInfo', 'QMetaEnum', 'QMetaMethod', 'QMetaObject', 'QMetaProperty', 'QMimeData', 'QModelIndex', 'QMutex', 'QMutexLocker', 'QObject', 'QParallelAnimationGroup', 'QPauseAnimation', 'QPersistentModelIndex', 'QPluginLoader', 'QPoint', 'QPointF', 'QProcess', 'QProcessEnvironment', 'QPropertyAnimation', 'QReadLocker', 'QReadWriteLock', 'QRect', 'QRectF', 'QResource', 'QRunnable', 'QSemaphore', 'QSequentialAnimationGroup', 'QSettings', 'QSignalMapper', 'QSize', 'QSizeF', 'QSocketNotifier', 'QSysInfo', 'QSystemSemaphore', 'QT_TRANSLATE_NOOP', 'QT_TR_NOOP', 'QT_TR_NOOP_UTF8', 'QTemporaryFile', 'QTextBoundaryFinder', 'QTextStream', 'QTextStreamManipulator', 'QThread', 'QThreadPool', 'QTime', 'QTimeLine', 'QTimer', 'QTimerEvent', 'QTranslator', 'QUrl', 'QVariantAnimation', 'QWaitCondition', 'QWriteLocker', 'QXmlStreamAttribute', 'QXmlStreamAttributes', 'QXmlStreamEntityDeclaration', 'QXmlStreamEntityResolver', 'QXmlStreamNamespaceDeclaration', 'QXmlStreamNotationDeclaration', 'QXmlStreamReader', 'QXmlStreamWriter', 'Qt', 'QtMsgType', 'qAbs', 'qAddPostRoutine', 'qCritical', 'qDebug', 'qFatal', 'qFuzzyCompare', 'qIsFinite', 'qIsInf', 'qIsNaN', 'qIsNull', 'qRegisterResourceData', 'qUnregisterResourceData', 'qVersion', 'qWarning'], 'QtGui': ['QAbstractTextDocumentLayout', 'QActionEvent', 'QBitmap', 'QBrush', 'QClipboard', 'QCloseEvent', 'QColor', 'QConicalGradient', 'QContextMenuEvent', 'QCursor', 'QDesktopServices', 'QDoubleValidator', 'QDrag', 'QDragEnterEvent', 'QDragLeaveEvent', 'QDragMoveEvent', 'QDropEvent', 'QFileOpenEvent', 'QFocusEvent', 'QFont', 'QFontDatabase', 'QFontInfo', 'QFontMetrics', 'QFontMetricsF', 'QGradient', 'QHelpEvent', 'QHideEvent', 'QHoverEvent', 'QIcon', 'QIconDragEvent', 'QIconEngine', 'QImage', 'QImageIOHandler', 'QImageReader', 'QImageWriter', 'QInputEvent', 'QInputMethodEvent', 'QIntValidator', 'QKeyEvent', 'QKeySequence', 'QLinearGradient', 'QMatrix2x2', 'QMatrix2x3', 'QMatrix2x4', 'QMatrix3x2', 'QMatrix3x3', 'QMatrix3x4', 'QMatrix4x2', 'QMatrix4x3', 'QMatrix4x4', 'QMouseEvent', 'QMoveEvent', 'QMovie', 'QPaintDevice', 'QPaintEngine', 'QPaintEngineState', 'QPaintEvent', 'QPainter', 'QPainterPath', 'QPainterPathStroker', 'QPalette', 'QPen', 'QPicture', 'QPixmap', 'QPixmapCache', 'QPolygon', 'QPolygonF', 'QQuaternion', 'QRadialGradient', 'QRegion', 'QResizeEvent', 'QSessionManager', 'QShortcutEvent', 'QShowEvent', 'QStandardItem', 'QStandardItemModel', 'QStatusTipEvent', 'QSyntaxHighlighter', 'QTabletEvent', 'QTextBlock', 'QTextBlockFormat', 'QTextBlockGroup', 'QTextBlockUserData', 'QTextCharFormat', 'QTextCursor', 'QTextDocument', 'QTextDocumentFragment', 'QTextFormat', 'QTextFragment', 'QTextFrame', 'QTextFrameFormat', 'QTextImageFormat', 'QTextInlineObject', 'QTextItem', 'QTextLayout', 'QTextLength', 'QTextLine', 'QTextList', 'QTextListFormat', 'QTextObject', 'QTextObjectInterface', 'QTextOption', 'QTextTable', 'QTextTableCell', 'QTextTableCellFormat', 'QTextTableFormat', 'QTouchEvent', 'QTransform', 'QValidator', 'QVector2D', 'QVector3D', 'QVector4D', 'QWhatsThisClickedEvent', 'QWheelEvent', 'QWindowStateChangeEvent', 'qAlpha', 'qBlue', 'qGray', 'qGreen', 'qIsGray', 'qRed', 'qRgb', 'qRgba'], 'QtHelp': ['QHelpContentItem', 'QHelpContentModel', 'QHelpContentWidget', 'QHelpEngine', 'QHelpEngineCore', 'QHelpIndexModel', 'QHelpIndexWidget', 'QHelpSearchEngine', 'QHelpSearchQuery', 'QHelpSearchQueryWidget', 'QHelpSearchResultWidget'], 'QtNetwork': ['QAbstractNetworkCache', 'QAbstractSocket', 'QAuthenticator', 'QHostAddress', 'QHostInfo', 'QLocalServer', 'QLocalSocket', 'QNetworkAccessManager', 'QNetworkAddressEntry', 'QNetworkCacheMetaData', 'QNetworkCookie', 'QNetworkCookieJar', 'QNetworkDiskCache', 'QNetworkInterface', 'QNetworkProxy', 'QNetworkProxyFactory', 'QNetworkProxyQuery', 'QNetworkReply', 'QNetworkRequest', 'QSsl', 'QTcpServer', 'QTcpSocket', 'QUdpSocket'], 'QtPrintSupport': ['QAbstractPrintDialog', 'QPageSetupDialog', 'QPrintDialog', 'QPrintEngine', 'QPrintPreviewDialog', 'QPrintPreviewWidget', 'QPrinter', 'QPrinterInfo'], 'QtSvg': ['QSvgGenerator', 'QSvgRenderer'], 'QtTest': ['QTest'], 'QtWidgets': ['QAbstractButton', 'QAbstractGraphicsShapeItem', 'QAbstractItemDelegate', 'QAbstractItemView', 'QAbstractScrollArea', 'QAbstractSlider', 'QAbstractSpinBox', 'QAction', 'QApplication', 'QBoxLayout', 'QButtonGroup', 'QCalendarWidget', 'QCheckBox', 'QColorDialog', 'QColumnView', 'QComboBox', 'QCommandLinkButton', 'QCommonStyle', 'QCompleter', 'QDataWidgetMapper', 'QDateEdit', 'QDateTimeEdit', 'QDial', 'QDialog', 'QDialogButtonBox', 'QDockWidget', 'QDoubleSpinBox', 'QErrorMessage', 'QFileDialog', 'QFileIconProvider', 'QFileSystemModel', 'QFocusFrame', 'QFontComboBox', 'QFontDialog', 'QFormLayout', 'QFrame', 'QGesture', 'QGestureEvent', 'QGestureRecognizer', 'QGraphicsAnchor', 'QGraphicsAnchorLayout', 'QGraphicsBlurEffect', 'QGraphicsColorizeEffect', 'QGraphicsDropShadowEffect', 'QGraphicsEffect', 'QGraphicsEllipseItem', 'QGraphicsGridLayout', 'QGraphicsItem', 'QGraphicsItemGroup', 'QGraphicsLayout', 'QGraphicsLayoutItem', 'QGraphicsLineItem', 'QGraphicsLinearLayout', 'QGraphicsObject', 'QGraphicsOpacityEffect', 'QGraphicsPathItem', 'QGraphicsPixmapItem', 'QGraphicsPolygonItem', 'QGraphicsProxyWidget', 'QGraphicsRectItem', 'QGraphicsRotation', 'QGraphicsScale', 'QGraphicsScene', 'QGraphicsSceneContextMenuEvent', 'QGraphicsSceneDragDropEvent', 'QGraphicsSceneEvent', 'QGraphicsSceneHelpEvent', 'QGraphicsSceneHoverEvent', 'QGraphicsSceneMouseEvent', 'QGraphicsSceneMoveEvent', 'QGraphicsSceneResizeEvent', 'QGraphicsSceneWheelEvent', 'QGraphicsSimpleTextItem', 'QGraphicsTextItem', 'QGraphicsTransform', 'QGraphicsView', 'QGraphicsWidget', 'QGridLayout', 'QGroupBox', 'QHBoxLayout', 'QHeaderView', 'QInputDialog', 'QItemDelegate', 'QItemEditorCreatorBase', 'QItemEditorFactory', 'QLCDNumber', 'QLabel', 'QLayout', 'QLayoutItem', 'QLineEdit', 'QListView', 'QListWidget', 'QListWidgetItem', 'QMainWindow', 'QMdiArea', 'QMdiSubWindow', 'QMenu', 'QMenuBar', 'QMessageBox', 'QPanGesture', 'QPinchGesture', 'QPlainTextDocumentLayout', 'QPlainTextEdit', 'QProgressBar', 'QProgressDialog', 'QPushButton', 'QRadioButton', 'QRubberBand', 'QScrollArea', 'QScrollBar', 'QSizeGrip', 'QSizePolicy', 'QSlider', 'QSpacerItem', 'QSpinBox', 'QSplashScreen', 'QSplitter', 'QSplitterHandle', 'QStackedLayout', 'QStackedWidget', 'QStatusBar', 'QStyle', 'QStyleFactory', 'QStyleHintReturn', 'QStyleHintReturnMask', 'QStyleHintReturnVariant', 'QStyleOption', 'QStyleOptionButton', 'QStyleOptionComboBox', 'QStyleOptionComplex', 'QStyleOptionDockWidget', 'QStyleOptionFocusRect', 'QStyleOptionFrame', 'QStyleOptionGraphicsItem', 'QStyleOptionGroupBox', 'QStyleOptionHeader', 'QStyleOptionMenuItem', 'QStyleOptionProgressBar', 'QStyleOptionRubberBand', 'QStyleOptionSizeGrip', 'QStyleOptionSlider', 'QStyleOptionSpinBox', 'QStyleOptionTab', 'QStyleOptionTabBarBase', 'QStyleOptionTabWidgetFrame', 'QStyleOptionTitleBar', 'QStyleOptionToolBar', 'QStyleOptionToolBox', 'QStyleOptionToolButton', 'QStyleOptionViewItem', 'QStylePainter', 'QStyledItemDelegate', 'QSwipeGesture', 'QSystemTrayIcon', 'QTabBar', 'QTabWidget', 'QTableView', 'QTableWidget', 'QTableWidgetItem', 'QTableWidgetSelectionRange', 'QTapAndHoldGesture', 'QTapGesture', 'QTextBrowser', 'QTextEdit', 'QTimeEdit', 'QToolBar', 'QToolBox', 'QToolButton', 'QToolTip', 'QTreeView', 'QTreeWidget', 'QTreeWidgetItem', 'QTreeWidgetItemIterator', 'QUndoView', 'QVBoxLayout', 'QWhatsThis', 'QWidget', 'QWidgetAction', 'QWidgetItem', 'QWizard', 'QWizardPage'], 'QtXml': ['QDomAttr', 'QDomCDATASection', 'QDomCharacterData', 'QDomComment', 'QDomDocument', 'QDomDocumentFragment', 'QDomDocumentType', 'QDomElement', 'QDomEntity', 'QDomEntityReference', 'QDomImplementation', 'QDomNamedNodeMap', 'QDomNode', 'QDomNodeList', 'QDomNotation', 'QDomProcessingInstruction', 'QDomText']}
' Missing members\n\nThis mapping describes members that have been deprecated\nin one or more bindings and have been left out of the\n_common_members mapping.\n\nThe member can provide an extra details string to be\nincluded in exceptions and warnings.\n'
_missing_members = {'QtGui': {'QMatrix': 'Deprecated in PyQt5'}}

def _qInstallMessageHandler(handler):
    'Install a message handler that works in all bindings\n\n    Args:\n        handler: A function that takes 3 arguments, or None\n    '
    C = handler

    def B(*A):
        if (len(A) == 3):
            (D, E, B) = A
        elif (len(A) == 2):
            (D, B) = A
            E = None
        else:
            raise TypeError('handler expected 2 or 3 arguments, got {0}'.format(len(A)))
        if isinstance(B, bytes):
            B = B.decode()
        C(D, E, B)
    A = (B if C else C)
    if (Qt.IsPySide or Qt.IsPyQt4):
        return Qt._QtCore.qInstallMsgHandler(A)
    elif (Qt.IsPySide2 or Qt.IsPyQt5 or Qt.IsPySide6):
        return Qt._QtCore.qInstallMessageHandler(A)

def _getcpppointer(object):
    if hasattr(Qt, '_shiboken6'):
        return getattr(Qt, '_shiboken6').getCppPointer(object)[0]
    elif hasattr(Qt, '_shiboken2'):
        return getattr(Qt, '_shiboken2').getCppPointer(object)[0]
    elif hasattr(Qt, '_shiboken'):
        return getattr(Qt, '_shiboken').getCppPointer(object)[0]
    elif hasattr(Qt, '_sip'):
        return getattr(Qt, '_sip').unwrapinstance(object)
    raise AttributeError("'module' has no attribute 'getCppPointer'")

def _wrapinstance(ptr, base=None):
    'Enable implicit cast of pointer to most suitable class\n\n    This behaviour is available in sip per default.\n\n    Based on http://nathanhorne.com/pyqtpyside-wrap-instance\n\n    Usage:\n        This mechanism kicks in under these circumstances.\n        1. Qt.py is using PySide 1 or 2.\n        2. A `base` argument is not provided.\n\n        See :func:`QtCompat.wrapInstance()`\n\n    Arguments:\n        ptr (long): Pointer to QObject in memory\n        base (QObject, optional): Base class to wrap with. Defaults to QObject,\n            which should handle anything.\n\n    '
    C = ptr
    A = base
    assert isinstance(C, long), "Argument 'ptr' must be of type <long>"
    assert ((A is None) or issubclass(A, Qt.QtCore.QObject)), "Argument 'base' must be of type <QObject>"
    if (Qt.IsPyQt4 or Qt.IsPyQt5):
        B = getattr(Qt, '_sip').wrapinstance
    elif Qt.IsPySide2:
        B = getattr(Qt, '_shiboken2').wrapInstance
    elif Qt.IsPySide6:
        B = getattr(Qt, '_shiboken6').wrapInstance
    elif Qt.IsPySide:
        B = getattr(Qt, '_shiboken').wrapInstance
    else:
        raise AttributeError("'module' has no attribute 'wrapInstance'")
    if (A is None):
        if (Qt.IsPyQt4 or Qt.IsPyQt5):
            A = Qt.QtCore.QObject
        else:
            F = B(long(C), Qt.QtCore.QObject)
            D = F.metaObject()
            while True:
                E = D.className()
                try:
                    A = getattr(Qt.QtWidgets, E)
                except AttributeError:
                    try:
                        A = getattr(Qt.QtCore, E)
                    except AttributeError:
                        D = D.superClass()
                        continue
                break
    return B(long(C), A)

def _isvalid(object):
    'Check if the object is valid to use in Python runtime.\n\n    Usage:\n        See :func:`QtCompat.isValid()`\n\n    Arguments:\n        object (QObject): QObject to check the validity of.\n\n    '
    if hasattr(Qt, '_shiboken6'):
        return getattr(Qt, '_shiboken6').isValid(object)
    elif hasattr(Qt, '_shiboken2'):
        return getattr(Qt, '_shiboken2').isValid(object)
    elif hasattr(Qt, '_shiboken'):
        return getattr(Qt, '_shiboken').isValid(object)
    elif hasattr(Qt, '_sip'):
        return (not getattr(Qt, '_sip').isdeleted(object))
    else:
        raise AttributeError("'module' has no attribute isValid")

def _translate(context, sourceText, *B):
    G = sourceText
    F = context
    try:
        H = Qt.QtCore.QCoreApplication
    except AttributeError:
        raise NotImplementedError('Missing QCoreApplication implementation for {}'.format(Qt.__binding__))

    def I(index):
        try:
            return B[index]
        except IndexError:
            pass
    A = (- 1)
    C = None
    if (len(B) == 3):
        (D, C, A) = B
    else:
        D = I(0)
        E = I(1)
        if isinstance(E, int):
            A = E
        else:
            C = E
    if (Qt.__binding__ in ('PySide2', 'PySide6', 'PyQt5')):
        J = [F, G, D, A]
    else:
        J = [F, G, D, (C or H.CodecForTr), A]
    return H.translate(*J)

def _loadUi(uifile, baseinstance=None):
    'Dynamically load a user interface from the given `uifile`\n\n    This function calls `uic.loadUi` if using PyQt bindings,\n    else it implements a comparable binding for PySide.\n\n    Documentation:\n        http://pyqt.sourceforge.net/Docs/PyQt5/designer.html#PyQt5.uic.loadUi\n\n    Arguments:\n        uifile (str): Absolute path to Qt Designer file.\n        baseinstance (QWidget): Instantiated QWidget or subclass thereof\n\n    Return:\n        baseinstance if `baseinstance` is not `None`. Otherwise\n        return the newly created instance of the user interface.\n\n    '
    B = baseinstance
    A = uifile
    if hasattr(Qt, '_uic'):
        return Qt._uic.loadUi(A, B)
    elif hasattr(Qt, '_QtUiTools'):

        class C(Qt._QtUiTools.QUiLoader):
            'Create the user interface in a base instance.\n\n            Unlike `Qt._QtUiTools.QUiLoader` itself this class does not\n            create a new instance of the top-level widget, but creates the user\n            interface in an existing instance of the top-level class if needed.\n\n            This mimics the behaviour of `PyQt5.uic.loadUi`.\n\n            '

            def __init__(A, baseinstance):
                B = baseinstance
                super(C, A).__init__(B)
                A.baseinstance = B
                A.custom_widgets = {}

            def _loadCustomWidgets(F, etree):
                '\n                Workaround to pyside-77 bug.\n\n                From QUiLoader doc we should use registerCustomWidget method.\n                But this causes a segfault on some platforms.\n\n                Instead we fetch from customwidgets DOM node the python class\n                objects. Then we can directly use them in createWidget method.\n                '

                def G(header):
                    '\n                    Translate a header file to python module path\n                    foo/bar.h => foo.bar\n                    '
                    A = os.path.splitext(header)[0]
                    return A.replace('/', '.').replace('\\', '.')
                A = etree.find('customwidgets')
                if (A is None):
                    return
                for B in A:
                    C = B.find('class').text
                    D = B.find('header').text
                    try:
                        E = importlib.import_module(D)
                    except ImportError:
                        E = importlib.import_module(G(D))
                    F.custom_widgets[C] = getattr(E, C)

            def load(A, uifile, *E, **F):
                B = uifile
                from xml.etree.ElementTree import ElementTree as G
                C = G()
                C.parse(B)
                A._loadCustomWidgets(C)
                D = Qt._QtUiTools.QUiLoader.load(A, B, *E, **F)
                D.parentWidget()
                return D

            def createWidget(A, class_name, parent=None, name=''):
                'Called for each widget defined in ui file\n\n                Overridden here to populate `baseinstance` instead.\n\n                '
                C = parent
                B = class_name
                if ((C is None) and A.baseinstance):
                    return A.baseinstance
                if (B in (A.availableWidgets() + ['Line'])):
                    D = Qt._QtUiTools.QUiLoader.createWidget(A, B, C, name)
                elif (B in A.custom_widgets):
                    D = A.custom_widgets[B](parent=C)
                else:
                    raise Exception(("Custom widget '%s' not supported" % B))
                if A.baseinstance:
                    setattr(A.baseinstance, name, D)
                return D
        D = C(B).load(A)
        Qt.QtCore.QMetaObject.connectSlotsByName(D)
        return D
    else:
        raise NotImplementedError('No implementation available for loadUi')
'Misplaced members\n\nThese members from the original submodule are misplaced relative PySide2\n\nNOTE: For bindings where a member is not replaced, they still\n      need to be added such that they are added to Qt.py\n\n'
_misplaced_members = {'PySide6': {'QtGui.QUndoCommand': 'QtWidgets.QUndoCommand', 'QtGui.QUndoGroup': 'QtWidgets.QUndoGroup', 'QtGui.QUndoStack': 'QtWidgets.QUndoStack', 'QtGui.QActionGroup': 'QtWidgets.QActionGroup', 'QtCore.QStringListModel': 'QtCore.QStringListModel', 'QtCore.Property': 'QtCore.Property', 'QtCore.Signal': 'QtCore.Signal', 'QtCore.Slot': 'QtCore.Slot', 'QtCore.QAbstractProxyModel': 'QtCore.QAbstractProxyModel', 'QtCore.QSortFilterProxyModel': 'QtCore.QSortFilterProxyModel', 'QtCore.QItemSelection': 'QtCore.QItemSelection', 'QtCore.QItemSelectionModel': 'QtCore.QItemSelectionModel', 'QtCore.QItemSelectionRange': 'QtCore.QItemSelectionRange', 'QtCore.QRegularExpression': 'QtCore.QRegExp', 'QtStateMachine.QStateMachine': 'QtCore.QStateMachine', 'QtStateMachine.QState': 'QtCore.QState', 'QtGui.QRegularExpressionValidator': 'QtGui.QRegExpValidator', 'QtGui.QShortcut': 'QtWidgets.QShortcut', 'QtGui.QAction': 'QtWidgets.QAction', 'QtUiTools.QUiLoader': ['QtCompat.loadUi', _loadUi], 'shiboken6.wrapInstance': ['QtCompat.wrapInstance', _wrapinstance], 'shiboken6.getCppPointer': ['QtCompat.getCppPointer', _getcpppointer], 'shiboken6.isValid': ['QtCompat.isValid', _isvalid], 'QtWidgets.qApp': 'QtWidgets.QApplication.instance()', 'QtCore.QCoreApplication.translate': ['QtCompat.translate', _translate], 'QtWidgets.QApplication.translate': ['QtCompat.translate', _translate], 'QtCore.qInstallMessageHandler': ['QtCompat.qInstallMessageHandler', _qInstallMessageHandler], 'QtWidgets.QStyleOptionViewItem': 'QtCompat.QStyleOptionViewItemV4'}, 'PySide2': {'QtWidgets.QUndoCommand': 'QtWidgets.QUndoCommand', 'QtWidgets.QUndoGroup': 'QtWidgets.QUndoGroup', 'QtWidgets.QUndoStack': 'QtWidgets.QUndoStack', 'QtWidgets.QActionGroup': 'QtWidgets.QActionGroup', 'QtCore.QStringListModel': 'QtCore.QStringListModel', 'QtGui.QStringListModel': 'QtCore.QStringListModel', 'QtCore.Property': 'QtCore.Property', 'QtCore.Signal': 'QtCore.Signal', 'QtCore.Slot': 'QtCore.Slot', 'QtCore.QRegExp': 'QtCore.QRegExp', 'QtWidgets.QShortcut': 'QtWidgets.QShortcut', 'QtGui.QRegExpValidator': 'QtGui.QRegExpValidator', 'QtCore.QAbstractProxyModel': 'QtCore.QAbstractProxyModel', 'QtCore.QSortFilterProxyModel': 'QtCore.QSortFilterProxyModel', 'QtCore.QItemSelection': 'QtCore.QItemSelection', 'QtCore.QItemSelectionModel': 'QtCore.QItemSelectionModel', 'QtCore.QItemSelectionRange': 'QtCore.QItemSelectionRange', 'QtUiTools.QUiLoader': ['QtCompat.loadUi', _loadUi], 'shiboken2.wrapInstance': ['QtCompat.wrapInstance', _wrapinstance], 'shiboken2.getCppPointer': ['QtCompat.getCppPointer', _getcpppointer], 'shiboken2.isValid': ['QtCompat.isValid', _isvalid], 'QtWidgets.qApp': 'QtWidgets.QApplication.instance()', 'QtCore.QCoreApplication.translate': ['QtCompat.translate', _translate], 'QtWidgets.QApplication.translate': ['QtCompat.translate', _translate], 'QtCore.qInstallMessageHandler': ['QtCompat.qInstallMessageHandler', _qInstallMessageHandler], 'QtWidgets.QStyleOptionViewItem': 'QtCompat.QStyleOptionViewItemV4'}, 'PyQt5': {'QtWidgets.QUndoCommand': 'QtWidgets.QUndoCommand', 'QtWidgets.QUndoGroup': 'QtWidgets.QUndoGroup', 'QtWidgets.QUndoStack': 'QtWidgets.QUndoStack', 'QtWidgets.QActionGroup': 'QtWidgets.QActionGroup', 'QtCore.pyqtProperty': 'QtCore.Property', 'QtCore.pyqtSignal': 'QtCore.Signal', 'QtCore.pyqtSlot': 'QtCore.Slot', 'QtCore.QAbstractProxyModel': 'QtCore.QAbstractProxyModel', 'QtCore.QSortFilterProxyModel': 'QtCore.QSortFilterProxyModel', 'QtCore.QStringListModel': 'QtCore.QStringListModel', 'QtCore.QItemSelection': 'QtCore.QItemSelection', 'QtCore.QItemSelectionModel': 'QtCore.QItemSelectionModel', 'QtCore.QItemSelectionRange': 'QtCore.QItemSelectionRange', 'uic.loadUi': ['QtCompat.loadUi', _loadUi], 'sip.wrapinstance': ['QtCompat.wrapInstance', _wrapinstance], 'sip.unwrapinstance': ['QtCompat.getCppPointer', _getcpppointer], 'sip.isdeleted': ['QtCompat.isValid', _isvalid], 'QtWidgets.qApp': 'QtWidgets.QApplication.instance()', 'QtGui.QRegExpValidator': 'QtGui.QRegExpValidator', 'QtCore.QRegExp': 'QtCore.QRegExp', 'QtCore.QCoreApplication.translate': ['QtCompat.translate', _translate], 'QtWidgets.QApplication.translate': ['QtCompat.translate', _translate], 'QtCore.qInstallMessageHandler': ['QtCompat.qInstallMessageHandler', _qInstallMessageHandler], 'QtWidgets.QShortcut': 'QtWidgets.QShortcut', 'QtWidgets.QStyleOptionViewItem': 'QtCompat.QStyleOptionViewItemV4'}, 'PySide': {'QtGui.QUndoCommand': 'QtWidgets.QUndoCommand', 'QtGui.QUndoGroup': 'QtWidgets.QUndoGroup', 'QtGui.QUndoStack': 'QtWidgets.QUndoStack', 'QtGui.QActionGroup': 'QtWidgets.QActionGroup', 'QtCore.Property': 'QtCore.Property', 'QtCore.Signal': 'QtCore.Signal', 'QtCore.Slot': 'QtCore.Slot', 'QtGui.QAbstractProxyModel': 'QtCore.QAbstractProxyModel', 'QtGui.QSortFilterProxyModel': 'QtCore.QSortFilterProxyModel', 'QtGui.QStringListModel': 'QtCore.QStringListModel', 'QtGui.QItemSelection': 'QtCore.QItemSelection', 'QtGui.QItemSelectionModel': 'QtCore.QItemSelectionModel', 'QtGui.QItemSelectionRange': 'QtCore.QItemSelectionRange', 'QtGui.QAbstractPrintDialog': 'QtPrintSupport.QAbstractPrintDialog', 'QtGui.QRegExpValidator': 'QtGui.QRegExpValidator', 'QtGui.QPageSetupDialog': 'QtPrintSupport.QPageSetupDialog', 'QtGui.QPrintDialog': 'QtPrintSupport.QPrintDialog', 'QtGui.QPrintEngine': 'QtPrintSupport.QPrintEngine', 'QtGui.QPrintPreviewDialog': 'QtPrintSupport.QPrintPreviewDialog', 'QtGui.QPrintPreviewWidget': 'QtPrintSupport.QPrintPreviewWidget', 'QtGui.QPrinter': 'QtPrintSupport.QPrinter', 'QtWidgets.QShortcut': 'QtWidgets.QShortcut', 'QtGui.QPrinterInfo': 'QtPrintSupport.QPrinterInfo', 'QtUiTools.QUiLoader': ['QtCompat.loadUi', _loadUi], 'shiboken.wrapInstance': ['QtCompat.wrapInstance', _wrapinstance], 'shiboken.unwrapInstance': ['QtCompat.getCppPointer', _getcpppointer], 'shiboken.isValid': ['QtCompat.isValid', _isvalid], 'QtGui.qApp': 'QtWidgets.QApplication.instance()', 'QtCore.QRegExp': 'QtCore.QRegExp', 'QtCore.QCoreApplication.translate': ['QtCompat.translate', _translate], 'QtGui.QApplication.translate': ['QtCompat.translate', _translate], 'QtCore.qInstallMsgHandler': ['QtCompat.qInstallMessageHandler', _qInstallMessageHandler], 'QtGui.QStyleOptionViewItemV4': 'QtCompat.QStyleOptionViewItemV4'}, 'PyQt4': {'QtGui.QUndoCommand': 'QtWidgets.QUndoCommand', 'QtGui.QUndoGroup': 'QtWidgets.QUndoGroup', 'QtGui.QUndoStack': 'QtWidgets.QUndoStack', 'QtGui.QActionGroup': 'QtWidgets.QActionGroup', 'QtGui.QAbstractProxyModel': 'QtCore.QAbstractProxyModel', 'QtGui.QSortFilterProxyModel': 'QtCore.QSortFilterProxyModel', 'QtGui.QItemSelection': 'QtCore.QItemSelection', 'QtGui.QStringListModel': 'QtCore.QStringListModel', 'QtGui.QItemSelectionModel': 'QtCore.QItemSelectionModel', 'QtCore.pyqtProperty': 'QtCore.Property', 'QtCore.pyqtSignal': 'QtCore.Signal', 'QtCore.pyqtSlot': 'QtCore.Slot', 'QtGui.QItemSelectionRange': 'QtCore.QItemSelectionRange', 'QtGui.QAbstractPrintDialog': 'QtPrintSupport.QAbstractPrintDialog', 'QtGui.QRegExpValidator': 'QtGui.QRegExpValidator', 'QtGui.QPageSetupDialog': 'QtPrintSupport.QPageSetupDialog', 'QtGui.QPrintDialog': 'QtPrintSupport.QPrintDialog', 'QtGui.QPrintEngine': 'QtPrintSupport.QPrintEngine', 'QtWidgets.QShortcut': 'QtWidgets.QShortcut', 'QtGui.QPrintPreviewDialog': 'QtPrintSupport.QPrintPreviewDialog', 'QtGui.QPrintPreviewWidget': 'QtPrintSupport.QPrintPreviewWidget', 'QtGui.QPrinter': 'QtPrintSupport.QPrinter', 'QtGui.QPrinterInfo': 'QtPrintSupport.QPrinterInfo', 'uic.loadUi': ['QtCompat.loadUi', _loadUi], 'sip.wrapinstance': ['QtCompat.wrapInstance', _wrapinstance], 'sip.unwrapinstance': ['QtCompat.getCppPointer', _getcpppointer], 'sip.isdeleted': ['QtCompat.isValid', _isvalid], 'QtCore.QString': 'str', 'QtGui.qApp': 'QtWidgets.QApplication.instance()', 'QtCore.QRegExp': 'QtCore.QRegExp', 'QtCore.QCoreApplication.translate': ['QtCompat.translate', _translate], 'QtGui.QApplication.translate': ['QtCompat.translate', _translate], 'QtCore.qInstallMsgHandler': ['QtCompat.qInstallMessageHandler', _qInstallMessageHandler], 'QtGui.QStyleOptionViewItemV4': 'QtCompat.QStyleOptionViewItemV4'}}
' Compatibility Members\n\nThis dictionary is used to build Qt.QtCompat objects that provide a consistent\ninterface for obsolete members, and differences in binding return values.\n\n{\n    "binding": {\n        "classname": {\n            "targetname": "binding_namespace",\n        }\n    }\n}\n'
_compatibility_members = {'PySide6': {'QWidget': {'grab': 'QtWidgets.QWidget.grab'}, 'QHeaderView': {'sectionsClickable': 'QtWidgets.QHeaderView.sectionsClickable', 'setSectionsClickable': 'QtWidgets.QHeaderView.setSectionsClickable', 'sectionResizeMode': 'QtWidgets.QHeaderView.sectionResizeMode', 'setSectionResizeMode': 'QtWidgets.QHeaderView.setSectionResizeMode', 'sectionsMovable': 'QtWidgets.QHeaderView.sectionsMovable', 'setSectionsMovable': 'QtWidgets.QHeaderView.setSectionsMovable'}, 'QFileDialog': {'getOpenFileName': 'QtWidgets.QFileDialog.getOpenFileName', 'getOpenFileNames': 'QtWidgets.QFileDialog.getOpenFileNames', 'getSaveFileName': 'QtWidgets.QFileDialog.getSaveFileName'}, 'QFont': {'setWeight': 'QtGui.QFont.setWeight'}, 'Qt': {'MidButton': 'QtCore.Qt.MiddleButton'}}, 'PySide2': {'QWidget': {'grab': 'QtWidgets.QWidget.grab'}, 'QHeaderView': {'sectionsClickable': 'QtWidgets.QHeaderView.sectionsClickable', 'setSectionsClickable': 'QtWidgets.QHeaderView.setSectionsClickable', 'sectionResizeMode': 'QtWidgets.QHeaderView.sectionResizeMode', 'setSectionResizeMode': 'QtWidgets.QHeaderView.setSectionResizeMode', 'sectionsMovable': 'QtWidgets.QHeaderView.sectionsMovable', 'setSectionsMovable': 'QtWidgets.QHeaderView.setSectionsMovable'}, 'QFileDialog': {'getOpenFileName': 'QtWidgets.QFileDialog.getOpenFileName', 'getOpenFileNames': 'QtWidgets.QFileDialog.getOpenFileNames', 'getSaveFileName': 'QtWidgets.QFileDialog.getSaveFileName'}, 'QFont': {'setWeight': 'QtGui.QFont.setWeight'}, 'Qt': {'MidButton': 'QtCore.Qt.MiddleButton'}}, 'PyQt5': {'QWidget': {'grab': 'QtWidgets.QWidget.grab'}, 'QHeaderView': {'sectionsClickable': 'QtWidgets.QHeaderView.sectionsClickable', 'setSectionsClickable': 'QtWidgets.QHeaderView.setSectionsClickable', 'sectionResizeMode': 'QtWidgets.QHeaderView.sectionResizeMode', 'setSectionResizeMode': 'QtWidgets.QHeaderView.setSectionResizeMode', 'sectionsMovable': 'QtWidgets.QHeaderView.sectionsMovable', 'setSectionsMovable': 'QtWidgets.QHeaderView.setSectionsMovable'}, 'QFileDialog': {'getOpenFileName': 'QtWidgets.QFileDialog.getOpenFileName', 'getOpenFileNames': 'QtWidgets.QFileDialog.getOpenFileNames', 'getSaveFileName': 'QtWidgets.QFileDialog.getSaveFileName'}, 'QFont': {'setWeight': 'QtGui.QFont.setWeight'}, 'Qt': {'MidButton': 'QtCore.Qt.MiddleButton'}}, 'PySide': {'QWidget': {'grab': 'QtWidgets.QPixmap.grabWidget'}, 'QHeaderView': {'sectionsClickable': 'QtWidgets.QHeaderView.isClickable', 'setSectionsClickable': 'QtWidgets.QHeaderView.setClickable', 'sectionResizeMode': 'QtWidgets.QHeaderView.resizeMode', 'setSectionResizeMode': 'QtWidgets.QHeaderView.setResizeMode', 'sectionsMovable': 'QtWidgets.QHeaderView.isMovable', 'setSectionsMovable': 'QtWidgets.QHeaderView.setMovable'}, 'QFileDialog': {'getOpenFileName': 'QtWidgets.QFileDialog.getOpenFileName', 'getOpenFileNames': 'QtWidgets.QFileDialog.getOpenFileNames', 'getSaveFileName': 'QtWidgets.QFileDialog.getSaveFileName'}, 'QFont': {'setWeight': 'QtGui.QFont.setWeight'}, 'Qt': {'MidButton': 'QtCore.Qt.MiddleButton'}}, 'PyQt4': {'QWidget': {'grab': 'QtWidgets.QPixmap.grabWidget'}, 'QHeaderView': {'sectionsClickable': 'QtWidgets.QHeaderView.isClickable', 'setSectionsClickable': 'QtWidgets.QHeaderView.setClickable', 'sectionResizeMode': 'QtWidgets.QHeaderView.resizeMode', 'setSectionResizeMode': 'QtWidgets.QHeaderView.setResizeMode', 'sectionsMovable': 'QtWidgets.QHeaderView.isMovable', 'setSectionsMovable': 'QtWidgets.QHeaderView.setMovable'}, 'QFileDialog': {'getOpenFileName': 'QtWidgets.QFileDialog.getOpenFileName', 'getOpenFileNames': 'QtWidgets.QFileDialog.getOpenFileNames', 'getSaveFileName': 'QtWidgets.QFileDialog.getSaveFileName'}, 'QFont': {'setWeight': 'QtGui.QFont.setWeight'}, 'Qt': {'MidButton': 'QtCore.Qt.MiddleButton'}}}

def _apply_site_config():
    try:
        import QtSiteConfig as A
    except ImportError:
        pass
    else:
        if hasattr(A, 'update_members'):
            A.update_members(_common_members)
        if hasattr(A, 'update_misplaced_members'):
            A.update_misplaced_members(members=_misplaced_members)
        if hasattr(A, 'update_compatibility_members'):
            A.update_compatibility_members(members=_compatibility_members)

def _new_module(name):
    return types.ModuleType(((__name__ + '.') + name))

def _import_sub_module(module, name):
    'import_sub_module will mimic the function of importlib.import_module'
    A = module
    A = __import__(((A.__name__ + '.') + name))
    for B in name.split('.'):
        A = getattr(A, B)
    return A

def _setup(module, extras):
    'Install common submodules'
    C = extras
    B = module
    Qt.__binding__ = B.__name__

    def D(exc, module):
        A = str(exc)
        if ('No module named' in A):
            return
        _warn(('ImportError(%s): %s' % (module, A)))
    for A in (list(_common_members) + C):
        try:
            E = _import_sub_module(B, A)
        except ImportError as F:
            try:
                E = __import__(A)
            except ImportError as G:
                D(F, A)
                D(G, A)
                continue
        setattr(Qt, ('_' + A), E)
        if (A not in C):
            setattr(Qt, A, _new_module(A))

def _reassign_misplaced_members(binding):
    'Apply misplaced members from `binding` to Qt.py\n\n    Arguments:\n        binding (dict): Misplaced members\n\n    '
    for (J, D) in _misplaced_members[binding].items():
        A = None
        E = J.split('.')
        K = E[0]
        C = None
        if (len(E) > 1):
            C = E[1:]
        if isinstance(D, (list, tuple)):
            (D, A) = D
        F = D.split('.')
        B = F[0]
        G = None
        if (len(F) > 1):
            G = F[1]
        if (not A):
            try:
                H = getattr(Qt, ('_' + K))
                while C:
                    L = C.pop(0)
                    H = getattr(H, L)
                A = H
            except AttributeError:
                _log('Misplaced member has no source: {0}'.format(J))
                continue
        try:
            I = getattr(Qt, B)
        except AttributeError:
            if (B not in _common_members):
                M = 'Not creating missing member module "{m}" for "{c}"'
                _log(M.format(m=B, c=G))
                continue
            setattr(Qt, B, _new_module(B))
            I = getattr(Qt, B)
            sys.modules[((__name__ + '.') + B)] = I
        if (not A):
            A = getattr(Qt, ('_' + K))
            if C:
                A = getattr(A, C)
        setattr(I, (G or B), A)

def _build_compatibility_members(binding, decorators=None):
    'Apply `binding` to QtCompat\n\n    Arguments:\n        binding (str): Top level binding in _compatibility_members.\n        decorators (dict, optional): Provides the ability to decorate the\n            original Qt methods when needed by a binding. This can be used\n            to change the returned value to a standard value. The key should\n            be the classname, the value is a dict where the keys are the\n            target method names, and the values are the decorator functions.\n\n    '
    C = binding
    A = decorators
    A = (A or dict())
    try:
        import QtSiteConfig as F
    except ImportError:
        pass
    else:
        if hasattr(F, 'update_compatibility_decorators'):
            F.update_compatibility_decorators(C, A)
    I = type('QtCompat', (object,), {})
    for (D, J) in _compatibility_members[C].items():
        G = {}
        for (E, C) in J.items():
            H = C.split('.')
            try:
                B = getattr(Qt, ('_' + H[0]))
            except AttributeError as K:
                _log(('QtCompat: AttributeError: %s' % K))
                continue
            for L in H[1:]:
                B = getattr(B, L)
            if (E in A.get(D, [])):
                B = staticmethod(A[D][E](B))
            G[E] = B
        M = type(D, (I,), G)
        setattr(Qt.QtCompat, D, M)

def _pyside6():
    'Initialise PySide6\n\n    These functions serve to test the existence of a binding\n    along with set it up in such a way that it aligns with\n    the final step; adding members from the original binding\n    to Qt.py\n\n    '
    import PySide6 as A
    B = ['QtUiTools']
    try:
        import shiboken6 as C
        B.append('shiboken6')
    except ImportError as D:
        print(('ImportError: %s' % D))
    _setup(A, B)
    Qt.__binding_version__ = A.__version__
    if hasattr(Qt, '_shiboken6'):
        Qt.QtCompat.wrapInstance = _wrapinstance
        Qt.QtCompat.getCppPointer = _getcpppointer
        Qt.QtCompat.delete = C.delete
    if hasattr(Qt, '_QtUiTools'):
        Qt.QtCompat.loadUi = _loadUi
    if hasattr(Qt, '_QtCore'):
        Qt.__qt_version__ = Qt._QtCore.qVersion()
        Qt.QtCompat.dataChanged = (lambda self, topleft, bottomright, roles=None: self.dataChanged.emit(topleft, bottomright, (roles or [])))
    if hasattr(Qt, '_QtWidgets'):
        Qt.QtCompat.setSectionResizeMode = Qt._QtWidgets.QHeaderView.setSectionResizeMode

    def E(func):
        A = func

        def B(self, weight):
            B = weight
            B = {100: Qt._QtGui.QFont.Thin, 200: Qt._QtGui.QFont.ExtraLight, 300: Qt._QtGui.QFont.Light, 400: Qt._QtGui.QFont.Normal, 500: Qt._QtGui.QFont.Medium, 600: Qt._QtGui.QFont.DemiBold, 700: Qt._QtGui.QFont.Bold, 800: Qt._QtGui.QFont.ExtraBold, 900: Qt._QtGui.QFont.Black}.get(B, Qt._QtGui.QFont.Normal)
            return A(self, B)
        B.__doc__ = A.__doc__
        B.__name__ = A.__name__
        return B
    F = {'QFont': {'setWeight': E}}
    _reassign_misplaced_members('PySide6')
    _build_compatibility_members('PySide6', F)

def _pyside2():
    'Initialise PySide2\n\n    These functions serve to test the existence of a binding\n    along with set it up in such a way that it aligns with\n    the final step; adding members from the original binding\n    to Qt.py\n\n    '
    import PySide2 as A
    B = ['QtUiTools']
    try:
        try:
            import shiboken2
        except ImportError:
            from PySide2 import shiboken2
        B.append('shiboken2')
    except ImportError:
        pass
    _setup(A, B)
    Qt.__binding_version__ = A.__version__
    if hasattr(Qt, '_shiboken2'):
        Qt.QtCompat.wrapInstance = _wrapinstance
        Qt.QtCompat.getCppPointer = _getcpppointer
        Qt.QtCompat.delete = shiboken2.delete
    if hasattr(Qt, '_QtUiTools'):
        Qt.QtCompat.loadUi = _loadUi
    if hasattr(Qt, '_QtCore'):
        Qt.__qt_version__ = Qt._QtCore.qVersion()
        Qt.QtCompat.dataChanged = (lambda self, topleft, bottomright, roles=None: self.dataChanged.emit(topleft, bottomright, (roles or [])))
    if hasattr(Qt, '_QtWidgets'):
        Qt.QtCompat.setSectionResizeMode = Qt._QtWidgets.QHeaderView.setSectionResizeMode
    _reassign_misplaced_members('PySide2')
    _build_compatibility_members('PySide2')

def _pyside():
    'Initialise PySide'
    import PySide as A
    B = ['QtUiTools']
    try:
        try:
            import shiboken
        except ImportError:
            from PySide import shiboken
        B.append('shiboken')
    except ImportError:
        pass
    _setup(A, B)
    Qt.__binding_version__ = A.__version__
    if hasattr(Qt, '_shiboken'):
        Qt.QtCompat.wrapInstance = _wrapinstance
        Qt.QtCompat.getCppPointer = _getcpppointer
        Qt.QtCompat.delete = shiboken.delete
    if hasattr(Qt, '_QtUiTools'):
        Qt.QtCompat.loadUi = _loadUi
    if hasattr(Qt, '_QtGui'):
        setattr(Qt, 'QtWidgets', _new_module('QtWidgets'))
        setattr(Qt, '_QtWidgets', Qt._QtGui)
        if hasattr(Qt._QtGui, 'QX11Info'):
            setattr(Qt, 'QtX11Extras', _new_module('QtX11Extras'))
            Qt.QtX11Extras.QX11Info = Qt._QtGui.QX11Info
        Qt.QtCompat.setSectionResizeMode = Qt._QtGui.QHeaderView.setResizeMode
    if hasattr(Qt, '_QtCore'):
        Qt.__qt_version__ = Qt._QtCore.qVersion()
        Qt.QtCompat.dataChanged = (lambda self, topleft, bottomright, roles=None: self.dataChanged.emit(topleft, bottomright))
    _reassign_misplaced_members('PySide')
    _build_compatibility_members('PySide')

def _pyqt5():
    'Initialise PyQt5'
    import PyQt5 as B
    A = ['uic']
    try:
        from PyQt5 import sip
        A += ['sip']
    except ImportError:
        try:
            import sip
            A += ['sip']
        except ImportError:
            sip = None
    _setup(B, A)
    if hasattr(Qt, '_sip'):
        Qt.QtCompat.wrapInstance = _wrapinstance
        Qt.QtCompat.getCppPointer = _getcpppointer
        Qt.QtCompat.delete = sip.delete
    if hasattr(Qt, '_uic'):
        Qt.QtCompat.loadUi = _loadUi
    if hasattr(Qt, '_QtCore'):
        Qt.__binding_version__ = Qt._QtCore.PYQT_VERSION_STR
        Qt.__qt_version__ = Qt._QtCore.QT_VERSION_STR
        Qt.QtCompat.dataChanged = (lambda self, topleft, bottomright, roles=None: self.dataChanged.emit(topleft, bottomright, (roles or [])))
    if hasattr(Qt, '_QtWidgets'):
        Qt.QtCompat.setSectionResizeMode = Qt._QtWidgets.QHeaderView.setSectionResizeMode
    _reassign_misplaced_members('PyQt5')
    _build_compatibility_members('PyQt5')

def _pyqt4():
    'Initialise PyQt4'
    import sip as A
    try:
        B = int(QT_SIP_API_HINT)
    except TypeError:
        B = None
    except ValueError:
        raise ImportError('QT_SIP_API_HINT=%s must be a 1 or 2')
    for C in ('QString', 'QVariant', 'QDate', 'QDateTime', 'QTextStream', 'QTime', 'QUrl'):
        try:
            A.setapi(C, (B or 2))
        except AttributeError:
            raise ImportError("PyQt4 < 4.6 isn't supported by Qt.py")
        except ValueError:
            E = A.getapi(C)
            if (not B):
                raise ImportError(('API version already set to %d' % E))
            else:
                sys.stderr.write(("Warning: API '%s' has already been set to %d.\n" % (C, E)))
    import PyQt4 as G
    F = ['uic']
    try:
        import sip as A
        F.append(A.__name__)
    except ImportError:
        A = None
    _setup(G, F)
    if hasattr(Qt, '_sip'):
        Qt.QtCompat.wrapInstance = _wrapinstance
        Qt.QtCompat.getCppPointer = _getcpppointer
        Qt.QtCompat.delete = A.delete
    if hasattr(Qt, '_uic'):
        Qt.QtCompat.loadUi = _loadUi
    if hasattr(Qt, '_QtGui'):
        setattr(Qt, 'QtWidgets', _new_module('QtWidgets'))
        setattr(Qt, '_QtWidgets', Qt._QtGui)
        if hasattr(Qt._QtGui, 'QX11Info'):
            setattr(Qt, 'QtX11Extras', _new_module('QtX11Extras'))
            Qt.QtX11Extras.QX11Info = Qt._QtGui.QX11Info
        Qt.QtCompat.setSectionResizeMode = Qt._QtGui.QHeaderView.setResizeMode
    if hasattr(Qt, '_QtCore'):
        Qt.__binding_version__ = Qt._QtCore.PYQT_VERSION_STR
        Qt.__qt_version__ = Qt._QtCore.QT_VERSION_STR
        Qt.QtCompat.dataChanged = (lambda self, topleft, bottomright, roles=None: self.dataChanged.emit(topleft, bottomright))
    _reassign_misplaced_members('PyQt4')

    def D(some_function):
        'Decorator that makes PyQt4 return conform to other bindings'
        A = some_function

        def B(*B, **C):
            D = A(*B, **C)
            return (D, '')
        B.__doc__ = A.__doc__
        B.__name__ = A.__name__
        return B
    H = {'QFileDialog': {'getOpenFileName': D, 'getOpenFileNames': D, 'getSaveFileName': D}}
    _build_compatibility_members('PyQt4', H)

def _none():
    'Internal option (used in installer)'
    A = type('Mock', (), {'__getattr__': (lambda Qt, attr: None)})
    Qt.__binding__ = 'None'
    Qt.__qt_version__ = '0.0.0'
    Qt.__binding_version__ = '0.0.0'
    Qt.QtCompat.loadUi = (lambda uifile, baseinstance=None: None)
    Qt.QtCompat.setSectionResizeMode = (lambda *A, **B: None)
    for B in _common_members.keys():
        setattr(Qt, B, A())
        setattr(Qt, ('_' + B), A())

def _log(text):
    if QT_VERBOSE:
        sys.stdout.write(('Qt.py [info]: %s\n' % text))

def _warn(text):
    try:
        sys.stderr.write(('Qt.py [warning]: %s\n' % text))
    except UnicodeDecodeError:
        import locale as A
        B = A.getpreferredencoding()
        sys.stderr.write(('Qt.py [warning]: %s\n' % text.decode(B)))

def _convert(lines):
    'Convert compiled .ui file from PySide2 to Qt.py\n\n    Arguments:\n        lines (list): Each line of of .ui file\n\n    Usage:\n        >> with open("myui.py") as f:\n        ..   lines = _convert(f.readlines())\n\n    '

    def C(line):
        A = line
        A = A.replace('from PySide2 import', 'from Qt import QtCompat,')
        A = A.replace('QtWidgets.QApplication.translate', 'QtCompat.translate')
        if ('QtCore.SIGNAL' in A):
            raise NotImplementedError('QtCore.SIGNAL is missing from PyQt5 and so Qt.py does not support it: you should avoid defining signals inside your ui files.')
        return A
    B = list()
    for A in lines:
        A = C(A)
        B.append(A)
    return B

def _cli(args):
    'Qt.py command-line interface'
    A = args
    import argparse as E
    B = E.ArgumentParser()
    B.add_argument('--convert', help='Path to compiled Python module, e.g. my_ui.py')
    B.add_argument('--compile', help='Accept raw .ui file and compile with native PySide2 compiler.')
    B.add_argument('--stdout', help='Write to stdout instead of file', action='store_true')
    B.add_argument('--stdin', help='Read from stdin instead of file', action='store_true')
    A = B.parse_args(A)
    if A.stdout:
        raise NotImplementedError('--stdout')
    if A.stdin:
        raise NotImplementedError('--stdin')
    if A.compile:
        raise NotImplementedError('--compile')
    if A.convert:
        sys.stdout.write('#\n# WARNING: --convert is an ALPHA feature.\n#\n# See https://github.com/mottosso/Qt.py/pull/132\n# for details.\n#\n')
        with open(A.convert) as C:
            F = _convert(C.readlines())
        D = ('%s_backup%s' % os.path.splitext(A.convert))
        sys.stdout.write(('Creating "%s"..\n' % D))
        shutil.copy(A.convert, D)
        with open(A.convert, 'w') as C:
            C.write(''.join(F))
        sys.stdout.write(('Successfully converted "%s"\n' % A.convert))

class MissingMember(object):
    '\n    A placeholder type for a missing Qt object not\n    included in Qt.py\n\n    Args:\n        name (str): The name of the missing type\n        details (str): An optional custom error message\n    '
    ERR_TMPL = '{} is not a common object across PySide2 and the other Qt bindings. It is not included as a common member in the Qt.py layer'

    def __init__(A, name, details=''):
        B = details
        A.__name = name
        A.__err = A.ERR_TMPL.format(name)
        if B:
            A.__err = '{}: {}'.format(A.__err, B)

    def __repr__(A):
        return '<{}: {}>'.format(A.__class__.__name__, A.__name)

    def __getattr__(A, name):
        raise NotImplementedError(A.__err)

    def __call__(A, *B, **C):
        raise NotImplementedError(A.__err)

def _install():
    I = ('PySide6', 'PySide2', 'PyQt5', 'PySide', 'PyQt4')
    C = None
    if QT_PREFERRED_BINDING_JSON:
        try:
            F = json.loads(QT_PREFERRED_BINDING_JSON)
        except ValueError:
            _warn(("Failed to parse QT_PREFERRED_BINDING_JSON='%s'" % QT_PREFERRED_BINDING_JSON))
            _warn('Falling back to default preferred order')
        else:
            C = F.get(__name__)
            if (C is None):
                C = F.get('default', None)
    if (C is None):
        C = list((A for A in QT_PREFERRED_BINDING.split(os.pathsep) if A))
    G = (C or I)
    J = {'PySide6': _pyside6, 'PySide2': _pyside2, 'PyQt5': _pyqt5, 'PySide': _pyside, 'PyQt4': _pyqt4, 'None': _none}
    _log(("Order: '%s'" % "', '".join(G)))
    _apply_site_config()
    H = False
    for A in G:
        _log(('Trying %s' % A))
        try:
            J[A]()
            H = True
            break
        except ImportError as K:
            _log(('ImportError: %s' % K))
        except KeyError:
            _log(("ImportError: Preferred binding '%s' not found." % A))
    if (not H):
        raise ImportError('No Qt binding were found.')
    for (A, E) in _common_members.items():
        try:
            L = getattr(Qt, ('_%s' % A))
        except AttributeError:
            continue
        D = getattr(Qt, A)
        __all__.append(A)
        sys.modules[((__name__ + '.') + A)] = D
        for B in E:
            try:
                M = getattr(L, B)
            except AttributeError:
                _log(("'%s.%s' was missing." % (A, B)))
                continue
            setattr(D, B, M)
    for (A, E) in _missing_members.items():
        D = getattr(Qt, A)
        for B in E:
            if hasattr(D, B):
                continue
            N = MissingMember('{}.{}'.format(A, B), details=E[B])
            setattr(D, B, N)
    sys.modules[(__name__ + '.QtCompat')] = Qt.QtCompat
    if hasattr(Qt.QtCompat, 'loadUi'):
        Qt.QtCompat.load_ui = Qt.QtCompat.loadUi
_install()
Qt.IsPySide6 = (Qt.__binding__ == 'PySide6')
Qt.IsPySide2 = (Qt.__binding__ == 'PySide2')
Qt.IsPyQt5 = (Qt.__binding__ == 'PyQt5')
Qt.IsPySide = (Qt.__binding__ == 'PySide')
Qt.IsPyQt4 = (Qt.__binding__ == 'PyQt4')
'Augment QtCompat\n\nQtCompat contains wrappers and added functionality\nto the original bindings, such as the CLI interface\nand otherwise incompatible members between bindings,\nsuch as `QHeaderView.setSectionResizeMode`.\n\n'
Qt.QtCompat._cli = _cli
Qt.QtCompat._convert = _convert
if (__name__ == '__main__'):
    _cli(sys.argv[1:])
