from base64 import b64decode as license64

import os
from BroTools.core.broqt.vendor.Qt import QtWidgets, QtGui, QtCore
from BroTools.configuration import modpath, debug_log, img, icon

class BroCollapseTab(QtWidgets.QWidget):

    def __init__(A, parent=None, title='', doResize=True, rootWidget=None, adaptiveWidth=False, *C, **D):
        B = parent
        QtWidgets.QWidget.__init__(A, parent=B)
        A.colorVis = QtGui.QColor()
        A.colorVis.setRgb(60, 60, 60, 100)
        A.title = title
        A.doResize = doResize
        A.parent = B
        A.rootWidget = rootWidget
        A.adaptiveWidth = adaptiveWidth
        A.init()

    def init(A):
        A.colorHid = QtGui.QColor()
        A.colorHid.setRgb(40, 40, 40, 100)
        A.downArrow = QtGui.QPixmap(img('downArrow'))
        A.rightArrow = QtGui.QPixmap(img('rightArrow'))
        A.container = QtWidgets.QWidget()
        A.titleButton = QtWidgets.QPushButton(A.title)
        A.titleButton.setIcon(A.downArrow)
        A.titleButton.setIconSize(QtCore.QSize(14, 14))
        A.titleButton.setObjectName('TabTitle')
        A.shadowHeader = QtWidgets.QWidget()
        A.shadowHeader.setFixedHeight(8)
        A.shadowHeader.setObjectName('TabShadow')
        A.containerLayout = A.initContainerLayout()
        A.container.setLayout(A.containerLayout)
        A.container.setObjectName('TabContainer')
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.mainLayout.setSpacing(0)
        A.mainLayout.setContentsMargins(0, 0, 0, 0)
        A.setLayout(A.mainLayout)
        A.mainLayout.addWidget(A.titleButton)
        A.mainLayout.addWidget(A.shadowHeader)
        A.mainLayout.addWidget(A.container)
        A.titleButton.clicked.connect(A.switchState)
        if A.adaptiveWidth:
            B = A.parent.frameGeometry().width()
            C = (B - (B / 10))
            debug_log('PARENT WIDTH:', B)
            debug_log('COLLAPSE TAB WIDTH', C)
            A.setMaximumWidth(C)

    def initContainerLayout(A):
        return QtWidgets.QVBoxLayout()

    def addWidget(A, item):
        A.containerLayout.addWidget(item)

    def addLayout(A, item):
        A.containerLayout.addLayout(item)

    def switchState(A):
        debug_log('switchState')
        if A.container.isHidden():
            A.shadowHeader.show()
            A.container.show()
            A.titleButton.setIcon(A.downArrow)
        else:
            A.shadowHeader.hide()
            A.container.hide()
            A.titleButton.setIcon(A.rightArrow)
        A.resizeParent()

    def closeTab(A):
        if (not A.container.isHidden()):
            A.container.hide()
            A.titleButton.setIcon(A.rightArrow)

    def openTab(A):
        if A.container.isHidden():
            A.container.show()
            A.titleButton.setIcon(A.downArrow)

    def unlockHeight(A):
        A.parent.setMaximumSize(10000, 10000)

    def lockHeight(A):
        A.parent.initHeight = A.parent.sizeHint().height()
        A.parent.setMaximumHeight(A.parent.initHeight)

    def resizeParent(A):
        if A.doResize:
            A.unlockHeight()
            A.parent.resize(0, 0)
            A.lockHeight()

class BroCollapseGridTab(BroCollapseTab):

    def __init__(A, parent=None, title='', doResize=True, rootWidget=None, adaptiveWidth=False, *C, **D):
        B = parent
        QtWidgets.QWidget.__init__(A, parent=B)
        A.colorVis = QtGui.QColor()
        A.colorVis.setRgb(60, 60, 60, 100)
        A.title = title
        A.doResize = doResize
        A.parent = B
        A.rootWidget = rootWidget
        A.adaptiveWidth = adaptiveWidth
        A.init()

    def initContainerLayout(A):
        return QtWidgets.QGridLayout()

    def addWidget(A, item, column, row):
        A.containerLayout.addWidget(item, column, row)

    def addLayout(A, item, column, row):
        A.containerLayout.addLayout(item, column, row)
