from base64 import b64decode as license64

import json, uuid
from BroTools.core.broqt.vendor.Qt import QtCore, QtGui, QtWidgets
from BroTools.core.broqt.widgets.node_editor import Node, NodeSocket, NodeConnection, NodeScene, NodeView
from BroTools.core.debug import default_logger as log
from BroTools.core.errors import catch_errors

class Node(Node):
    'Base class for all nodes in the system'
    HEADER_COLORS = {'ExampleNodeClass': '#2ecc71'}

@Node.register
class ExampleNodeClass(Node):

    def __init__(A):
        super().__init__('World')
        A.addInputSocket('any')

class AnimatterWidget(QtWidgets.QWidget):

    def __init__(A, parent=None):
        super().__init__(parent)
        A._setup_ui()
        A.add_node_callbacks = []
        A.get_dcc_selection = None

    def _setup_ui(A):
        B = QtWidgets.QVBoxLayout(A)
        B.setContentsMargins(8, 8, 8, 8)
        B.setSpacing(8)
        E = QtWidgets.QHBoxLayout()
        C = QtWidgets.QPushButton('+')
        C.setFixedSize(32, 32)
        C.setStyleSheet('\n            QPushButton {\n                background-color: #3498db;\n                border: none;\n                border-radius: 16px;\n                color: white;\n                font-size: 18px;\n                padding: 0px;\n                margin: 0px;\n                text-align: center;\n                line-height: 28px;\n            }\n            QPushButton:hover {\n                background-color: #2980b9;\n            }\n            QPushButton::menu-indicator {\n                width: 0px;\n            }\n        ')
        D = QtWidgets.QMenu(C)
        D.setStyleSheet('\n            QMenu {\n                background-color: #2A2A2A;\n                border: 1px solid #404040;\n            }\n            QMenu::item {\n                padding: 8px 20px;\n                color: #E0E0E0;\n            }\n            QMenu::item:selected {\n                background-color: #3498db;\n            }\n        ')
        J = {'World': WorldNode, 'Point': PointNode, 'Chain': ChainNode, 'MeshCollider': MeshColliderNode, 'Force': ForceNode}
        for (K, L) in J.items():
            M = D.addAction(K)
            M.triggered.connect((lambda checked=False, nc=L: A._add_node(nc)))
        C.setMenu(D)
        E.addWidget(C, alignment=QtCore.Qt.AlignRight)
        B.addLayout(E)
        A.scene = NodeScene()
        A.view = NodeView()
        A.view.setScene(A.scene)
        B.addWidget(A.view)
        F = QtWidgets.QHBoxLayout()
        G = QtWidgets.QPushButton('Preview Direct')
        H = QtWidgets.QPushButton('Preview Proxied')
        I = QtWidgets.QPushButton('Bake')
        for N in [G, H, I]:
            F.addWidget(N)
        G.clicked.connect(A._preview_direct)
        H.clicked.connect(A._preview_proxied)
        I.clicked.connect(A._bake_simulation)
        B.addLayout(F)

    def _add_node(A, node_class):
        C = node_class
        B = None
        for F in A.add_node_callbacks:
            G = F(C.__name__)
            if (not G):
                raise Exception(f'Failed to add {C.__name__}')
        if ((C == PointNode) and A.get_dcc_selection):
            D = A.get_dcc_selection()
            if D:
                E = PointNode.create_for_dcc_objects(D, A.scene)
                B = (E[0] if E else None)
            else:
                B = C()
                A.scene.addItem(B)
        elif ((C in (MeshColliderNode, ForceNode)) and A.get_dcc_selection):
            D = A.get_dcc_selection()
            if D:
                B = C.create_for_dcc_object(D[0], A.scene)
            else:
                B = C()
                A.scene.addItem(B)
        else:
            B = C()
            A.scene.addItem(B)
        if (B and isinstance(B, Node)):
            B.setPos(A.view.mapToScene(A.view.viewport().rect().center()))

    def _preview_direct(A):
        log.debug('_preview_direct')

    def _preview_proxied(A):
        log.debug('_preview_proxied')

    def _bake_simulation(A):
        log.debug('_bake_simulation')

    def save_to_json(A, filepath):
        'Save the current node network to a JSON file'
        B = A.scene.serialize()
        with open(filepath, 'w') as C:
            json.dump(B, C, indent=2)

    def load_from_json(A, filepath):
        'Load a node network from a JSON file'
        with open(filepath, 'r') as B:
            C = json.load(B)
        A.scene.deserialize(C)
