from base64 import b64decode as license64

import os
from BroTools.core.broqt.vendor.Qt import QtCore, QtGui, QtWidgets
import BroTools.core.utils
from BroTools.configuration import brotools_config, key_store_path, modpath, bro_qss, icons_path
from BroTools.core import dataio, maya_utils
from BroTools.core.broqt.bro_widgets import BroUniversalInput, BroEntry, BroTitle, BroText, BroFormLayout
from BroTools.core.broqt.bro_windows import BroSimpleWindow
from BroTools.core.config_parser import BroConfig
from BroTools.core.debug import default_logger as log
from BroTools.core.utils import split_words_in_variable

class BroConfigEditWindow(BroSimpleWindow):

    def __init__(A, rootWidget=None, config=None, title='BroTools Preferences', *C, **D):
        B = config
        super(BroConfigEditWindow, A).__init__(*C, icon=os.path.join(modpath, 'imgs', 'config.png'), qss=bro_qss, title=title, rootWidget=rootWidget, **D)
        log.info('Config', B)
        if (not issubclass(type(B), BroConfig)):
            A.config = BroConfig(B)
        else:
            A.config = B
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.setLayout(A.mainLayout)
        A.generateInputsFromConfig(A.config)
        A.setWindowFlags(QtCore.Qt.Dialog)
        A.setAttribute(QtCore.Qt.WA_AlwaysShowToolTips)
        A.setWindowIcon(QtGui.QIcon(os.path.join(icons_path, 'i16_preferences.png')))
        A.resize(480, 200)

    def generateInputsFromConfig(A, config):
        '\n        Generates a list of widget inputs from the given list, adding them to new layout\n        :param data: Input dict to generate menu items from.\n        '
        log.debug('Generating menu from config file...')
        A.inputItems = {}
        for B in config.sections:
            D = BroFormLayout()
            E = QtWidgets.QGroupBox(B['name'])
            E.setLayout(D)
            A.mainLayout.addWidget(E)
            A.inputItems[B['name']] = {}
            for (F, G) in list(B['data'].__dict__.items()):
                C = BroUniversalInput(value_type=type(G))
                C.setValue(G)
                C.valueChanged.connect(A.valueChange)
                D.addRow(split_words_in_variable(F).title(), C)
                A.inputItems[B['name']][F] = C

    def valueChange(B):
        log.debug('Saving config...')
        C = B.config
        for A in C.sections:
            if (A['name'].lower().strip() != 'internal'):
                for (D, E) in list(A['data'].__dict__.items()):
                    C.__dict__[A['name']].__dict__[D] = B.inputItems[A['name']][D].value()
        B.config.save()

class BroToolsConfigWindow(BroConfigEditWindow):

    def __init__(A, *B, **C):
        super(BroToolsConfigWindow, A).__init__(*B, config=brotools_config, title='BroTools Preferences', **C)
        A.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)

def showConfigWindow(*B, **C):
    A = BroToolsConfigWindow()
    A.show()
