from base64 import b64decode as license64

import json, os
from maya import cmds
import maya.OpenMayaUI as omui
from maya.app.general.mayaMixin import MayaQWidgetBaseMixin, MayaQWidgetDockableMixin
from BroTools.core.broqt.vendor.Qt import QtCore, QtWidgets, QtGui
from BroTools.core.broqt.vendor.Qt import QtCompat
wrapInstance = QtCompat.wrapInstance
from BroTools.configuration import bro_qss, modpath, icon, img
from BroTools.core.broqt.bro_windows import BroSimpleWindow
from BroTools.core.broqt.widgets.collapse_tab import BroCollapseTab
import uuid

class BroErrorWindow(MayaQWidgetBaseMixin, QtWidgets.QWidget):
    buttonLayout = None
    acceptBtn = None
    reportBtn = None
    mainLayout = None
    detailsField = None
    imageLabel = None
    detailsFieldLabel = None

    def __init__(A, *B, **C):
        super(BroErrorWindow, A).__init__(*B, **C)
        A.icon = icon('logo', size=32)
        D = omui.MQtUtil.mainWindow()
        A.rootWidget = wrapInstance(int(D), QtWidgets.QWidget)
        A.setParent(A.rootWidget)
        A.setWindowIcon(QtGui.QIcon(A.icon))
        A.setWindowModality(QtCore.Qt.ApplicationModal)
        A.setWindowFlags((((QtCore.Qt.Dialog | QtCore.Qt.WindowStaysOnTopHint) | QtCore.Qt.CustomizeWindowHint) | QtCore.Qt.WindowTitleHint))
        A.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        A.setAttribute(QtCore.Qt.WA_AlwaysShowToolTips)
        A.setWindowModality(QtCore.Qt.ApplicationModal)
        A.buttonLayout = QtWidgets.QHBoxLayout()
        A.acceptBtn = QtWidgets.QPushButton('Close')
        A.buttonLayout.addWidget(A.acceptBtn)
        A.acceptBtn.clicked.connect(A.accept)
        A.mainLayout = QtWidgets.QVBoxLayout()
        A.errorLayout = QtWidgets.QHBoxLayout()
        A.mainErrorText = QtWidgets.QLabel('Error text')
        A.mainErrorText.setWordWrap(True)
        A.detailsField = QtWidgets.QTextBrowser()
        A.detailsField.setText('Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas porta aliquet dui quis facilisis. Nam efficitur vulputate elit eget scelerisque. Quisque commodo sapien at malesuada tincidunt. In laoreet at dolor non ullamcorper. Donec non posuere augue. Ut ac sem egestas arcu rhoncus tristique. Donec facilisis iaculis est, pulvinar aliquam urna porttitor a. Donec et risus lacus. Cras convallis maximus massa vitae posuere. Morbi blandit, arcu quis volutpat posuere, libero ligula vehicula enim, id tristique libero ipsum ac neque. Aliquam erat volutpat. Praesent non ex consequat, congue sapien at, varius ante. Pellentesque ac erat vel nunc consectetur pretium. Ut maximus feugiat elit, vel condimentum tellus. Ut vitae elementum sem, ac dictum mi.\nDonec nec auctor quam. Proin nec pulvinar orci, ut pharetra ipsum. Aliquam vitae finibus nunc. Sed bibendum urna non risus lobortis consequat. Nam congue vel felis vel finibus. Vestibulum imperdiet, nisl a malesuada volutpat, nulla lorem tincidunt lorem, nec feugiat nulla felis a turpis. Maecenas ligula urna, ullamcorper aliquam tincidunt nec, convallis ut elit.')
        A.detailsField.setObjectName('ErrorDetailsField')
        A.detailsFieldLabel = QtWidgets.QLabel('Details')
        A.imageLabel = QtWidgets.QLabel(A)
        A.errorLayout.addWidget(A.imageLabel, 1)
        A.errorLayout.addWidget(A.mainErrorText, 3)
        A.mainErrorText.setObjectName('ErrorText')
        A.collapseTab = BroCollapseTab(parent=A, title='Details', doResize=False)
        A.collapseTab.addWidget(A.detailsField)
        A.collapseTab.closeTab()
        A.setLayout(A.mainLayout)
        A.mainLayout.addLayout(A.errorLayout)
        A.mainLayout.addWidget(A.collapseTab)
        A.mainLayout.addLayout(A.buttonLayout)
        A.setStyleSheet(str(bro_qss))
        A.setFixedWidth(800)

    def accept(A):
        print('Closing...')
        A.close()
        print('Destroying...')
        A.destroy()

    def show(A, summary=None, trace=None, image='error', icn=None, title=None, **G):
        F = image
        E = trace
        D = summary
        C = title
        B = icn
        if (C is None):
            C = D
        if ((B is None) and ('icon' in G)):
            B = G['icon']
        A.setWindowTitle('BroTools: {}'.format(C))
        if F:
            H = QtGui.QPixmap(img(F))
        elif icon:
            H = QtGui.QPixmap(icon(B))
        A.imageLabel.setPixmap(H)
        A.mainErrorText.setText(D)
        if E:
            A.detailsField.setText(E)
        else:
            A.collapseTab.hide()
        super(BroErrorWindow, A).show()
