from base64 import b64decode as license64

'\n=== module meta start ===\nDCC: Maya\n=== module meta end ===\n'
from BroTools.packages import jsonpickle
import os, codecs
modpath = APP_DIR = os.path.dirname(os.path.realpath(__file__))
MOD_PATH = os.path.abspath(os.path.join(modpath, '..'))

def verbose_log(modpath):
    '\n    Self contained verbose log check.\n    v 1.0.0\n    '
    try:
        A = os.environ.get('BROTOOLS_CONFIG_PATH')
        if (not A):
            A = modpath
        B = os.path.join(A, 'config', 'brotools.cfg')
        if os.path.exists(B):
            with codecs.open(B, 'r', encoding='utf-8') as F:
                C = F.readlines()
        else:
            C = []
            print('CANT FIND brotools.cfg FILE AT', B)
        D = False
        for E in C:
            if (('debug_level: 0' in E) or ('debug_level:0' in E)):
                D = True
                print('BROTOOLS.py VERBOSE LOG ENABLED')
        return D
    except Exception as G:
        raise Exception('{} verbose_log failure: {}'.format(__file__, G))
VERBOSE_LOG = verbose_log(MOD_PATH)

def log(*B, **D):
    if VERBOSE_LOG:
        A = ''
        for C in B:
            A += (str(C) + ' ')
        print(A)

class BroConfig_KeyError(Exception):

    def __init__(A, message, *C):
        B = message
        A.message = B
        super(BroConfig_KeyError, A).__init__(B, *C)

class BroConfig(object):
    "\n    Custom config parser.\n    Because python 2's ConfigParser s...\n    "

    def __init__(A, filepath, skip_load=False, defaults=None):
        E = filepath
        C = defaults
        A.filepath = E
        A.params = ['filepath', 'params', 'raw_data']
        if (os.path.exists(E) and (not skip_load)):
            A.load_from_file()
            if (C != None):
                for (B, F) in C.items():
                    A.add_section(B)
                    for (D, G) in F.items():
                        if (D not in A.__dict__[B].__dict__):
                            A.__dict__[B].__dict__[D] = G
                log('// BroTools : Config file {} was updated with new defaults data. Most likely due to version update. //'.format(A.filepath))
                A.save()
        if ((not os.path.exists(E)) and (C != None)):
            for (B, F) in C.items():
                A.add_section(B)
                for (D, G) in F.items():
                    A.__dict__[B].__dict__[D] = G
            A.save()

    def __getattr__(B, attr):
        A = attr
        try:
            if (A not in B.__dict__):
                B.add_section(A)
            return B.__dict__[A]
        except KeyError:
            raise BroConfig_KeyError('Could not find Section "{}"'.format(str(A)))

    def __setattr__(C, attr, value):
        B = value
        A = attr
        if (A.strip() != ''):
            log('// BroConfig set', A, B, '//')
            C.__dict__[A] = B
        else:
            log('// BroConfig Warning: attribute passed with empty name, skipped. //')

    @property
    def sections(self):
        A = []
        for (B, C) in sorted(self.__dict__.items()):
            if (B not in self.params):
                A.append({'name': B, 'data': C})
        return A

    def set(A, section_name, attr_name, value):
        B = section_name
        if (not A.has_section(B)):
            A.add_section(B)
        A.__dict__[B].__dict__[attr_name] = value

    def add_section(A, name):
        if (name not in A.__dict__):
            A.__dict__[name] = BroConfig_Section()

    def load_from_file(A):
        log('// BroConfig Load from file', A.filepath, '//')
        with codecs.open(A.filepath, 'r', encoding='utf-8') as E:
            A.raw_data = E.readlines()
        for B in A.raw_data:
            if (not (B.strip() == '')):
                if (not B.strip().startswith('>')):
                    if B.strip().startswith('#'):
                        C = B.replace('#', '').strip()
                        A.add_section(C)
                    else:
                        D = B.partition(':')
                        F = D[0].strip()
                        G = A.convert_value(D[2].strip())
                        A.__dict__[C].__dict__[F] = G

    def has_section(A, name):
        return (name in A.__dict__)

    def save(A):
        if (not os.path.exists(A.filepath)):
            E = os.path.dirname(A.filepath)
            if (not os.path.exists(E)):
                os.makedirs(E)
            with codecs.open(A.filepath, 'a+', encoding='utf-8') as C:
                C.write('')
        B = ''
        F = 0
        for (G, I) in sorted(A.__dict__.items()):
            if (G not in A.params):
                if (F > 0):
                    B += '\n'
                B += '# {}\n'.format(str(G))
                for (H, D) in sorted(I.__dict__.items()):
                    if (type(D) in [list, dict]):
                        B += '{}: {}\n'.format(str(H), jsonpickle.encode(D))
                    else:
                        B += '{}: {}\n'.format(str(H), str(D))
                F += 1
        with codecs.open(A.filepath, 'w', encoding='utf-8') as C:
            C.write(B)
    write = save

    def convert_value(B, value):
        A = value
        try:
            return jsonpickle.decode(A)
        except:
            pass
        if (A in ['True', 'true']):
            return True
        elif (A in ['False', 'false']):
            return False
        if ('.' in A):
            try:
                float(A)
                return float(A)
            except ValueError:
                return str(A)
        else:
            try:
                int(A)
                return int(A)
            except ValueError:
                return str(A)

class BroConfig_Section(object):
    '\n    Config parser section\n    '

    def __getattr__(A, attr):
        try:
            return A.__dict__[attr]
        except KeyError:
            raise BroConfig_KeyError('Could not find Item "{}"'.format(str(attr)))

    def __setattr__(A, attr, value):
        A.__dict__[attr] = value
