from base64 import b64decode as license64

'\nDebugging module of BroTools.\nThis should only contain functionality intended for debugging and reporting:\n- Logger functions and classes\n- Error catch wrappers\n- Self-contained error-report windows\n'
import logging, os, traceback
from datetime import datetime
from maya import cmds
from BroTools import __version__
from BroTools.configuration import MOD_NAME, brotools_config, __dev_version__, brotools_write_path
log_path = os.path.join(brotools_write_path, 'brotools.log')
if brotools_config.Debug.write_log_file:
    print('BroTools log file dump enabled:', log_path)
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
if brotools_config.Debug.write_log_file:
    if os.path.exists(log_path):
        try:
            os.remove(log_path)
            print('Old log file removed', log_path)
        except:
            pass
    cmdFileOutput_descriptor = cmds.cmdFileOutput(q=True, o=log_path)
    if ((- 1) != cmdFileOutput_descriptor):
        cmds.cmdFileOutput(close=cmdFileOutput_descriptor)
    cmdFileOutput_descriptor = cmds.cmdFileOutput(o=log_path)
    import sys, platform, locale
    print('\n// -----------------')
    print('BroTools Log')
    print('Version: {}.{}\n'.format(__version__, __dev_version__))
    print('-  {:<15} {}, {}'.format('Log path:', str(log_path), str(cmdFileOutput_descriptor)))
    print('-  {:<15} {}'.format('Date started:', str(datetime.now())))
    print('\nMAYA INFO')
    print('-  {:<15} {}'.format('OS:', cmds.about(q=True, os=True)))
    print('-  {:<15} {}'.format('UI Lang:', cmds.about(q=True, uil=True)))
    print('-  {:<15} {}'.format('Maya:', cmds.about(q=True, v=True)))
    print('-  {:<15} {}'.format('Maya full:', cmds.about(q=True, iv=True)))
    print('-  {:<15} {}'.format('Plugins:', cmds.pluginInfo(q=True, lsp=True)))
    print('-  {:<15} {}'.format('Python:', sys.version))
    print('\nSYSTEM INFO')
    print('-  {:<15} {}'.format('Platform:', platform.platform()))
    print('-  {:<15} {}'.format('Locale Def:', locale.getdefaultlocale()))
    print('-  {:<15} {}'.format('Locale:', locale.getlocale()))
    if cmds.about(q=True, win=True):
        import ctypes
        user32 = ctypes.windll.user32
        screensize = (user32.GetSystemMetrics(0), user32.GetSystemMetrics(1), user32.GetSystemMetrics(78))
        print('-  {:<15} {}'.format('Screen Info:', str(screensize)))
        print('-  {:<15} {}'.format('Win:', str(sys.getwindowsversion())))
    print("\nIf you wish to keep any of this information private you're free to remove\n    some lines from here, however it can make debugging process harder. One part of\n    debugging process is to recreate your error. If I can't do that in my main system\n    my next step is to start a Virtual Machine with your OS and environment as close to\n    yours as possible. That's what all the above data is used for. \n")
    print('----------------- //\n')
import inspect

class brokwargs():

    def __init__(D, labelLong, labelShort, defautValue, kwargDic):
        C = labelShort
        B = labelLong
        A = kwargDic
        if (A.get(B) and A.get(C)):
            print('Same flag used twice.')
        else:
            D.userInput = (A.get(B) or A.get(C) or defautValue)

    def get(A):
        return A.userInput

class BroLogger():
    '\n    Custom logging function.\n    TODO Switch to extending python logging.\n    '
    levels = ['DEBG', 'INFO', 'WARN', 'ERR ', 'CRIT', 'MSG ']
    colors = {'green': '00FF00', 'orange': 'FAA300', 'yellow': 'FFF607', 'red': 'FF0000'}

    def __init__(A):
        A.critical = A.crit

    def __call__(E, *F, **B):
        C = brokwargs('callframe_depth', 'cfd', 5, B).get()
        G = inspect.currentframe()
        D = inspect.getouterframes(G, 2)
        if (len(D) > C):
            A = inspect.getframeinfo(D[C][0])
        else:
            A = ['(unknown)', '(-)', '(unknown)']
        H = os.path.basename(A[0]).replace('.py', '')
        I = A[2]
        J = A[1]
        print('# !!! Deprecated use of BroLogger: {}.{}.{}'.format(H, I, J))
        E.debug(*F, **B)

    def debug(C, *B, **A):
        A.pop('lvl', None)
        A.pop('level', None)
        A['lvl'] = 0
        log(*B, **A)

    def info(C, *B, **A):
        A.pop('lvl', None)
        A.pop('level', None)
        A['level'] = 1
        log(*B, **A)

    def warning(C, *B, **A):
        A.pop('lvl', None)
        A.pop('level', None)
        A['level'] = 2
        A.pop('warn', None)
        A.pop('warning', None)
        A['warning'] = True
        log(*B, **A)

    def error(C, *B, **A):
        A.pop('lvl', None)
        A.pop('level', None)
        A.pop('err', None)
        A.pop('error', None)
        A['level'] = 3
        A['error'] = True
        log(*B, **A)

    def crit(C, *B, **A):
        A.pop('lvl', None)
        A.pop('level', None)
        A['level'] = 4
        log(*B, **A)

    def inViewLog(C, *B, **A):
        A.pop('inv', None)
        A.pop('inview', None)
        A['inview'] = True
        log(*B, **A)

    def message(C, *B, **A):
        '\n        Important message that has to be always shown, but that is, however, not a warning or an error\n        :return:\n        '
        A.pop('lvl', None)
        A.pop('level', None)
        A['level'] = 5
        log(*B, **A)
    msg = message

    def log(G, *N, **A):
        '\n        Print for maya on steroids!\n\n        Pass any number of unnamed variables, just like for "print" function.\n        And control how it\'s handled with some custom attributes.\n\n        :param inview: Display message inside the viewport, right in user\'s face. Available since Maya... probably 2015?\n        :param color: Color of the inview message, hex format. Like #FFFFFF without #.\n        :param err: (Bool) Will call cmds.error if true. Will also set color to red.\n        :param warn: (Bool) Will call cmds.warning if true. Will also set color to orange.\n        :param traceback: (Bool) Print traceback.\n        :param args: Any number of your arguments to turn into a single string separated with spaces.\n        '
        J = brokwargs('level', 'lvl', 0, A).get()
        K = brokwargs('inview', 'inv', False, A).get()
        if ((J >= brotools_config.Debug.debug_level) or K):
            B = brokwargs('color', 'clr', '00FF00', A).get()
            if (B in G.colors):
                B = G.colors[B]
            H = brokwargs('error', 'err', False, A).get()
            I = brokwargs('warning', 'warn', False, A).get()
            O = brokwargs('traceback', 'exc_info', False, A).get()
            W = brokwargs('critical', 'crit', False, A).get()
            P = brokwargs('new_line', 'nl', 0, A).get()
            L = brokwargs('callframe_depth', 'cfd', 5, A).get()
            X = brokwargs('show_error', 'sher', False, A).get()
            Y = brokwargs('exception', 'e', None, A).get()
            Z = brokwargs('traceback_text', 'tbt', None, A).get()
            Q = G.levels[J]
            a = str(datetime.now().strftime('%Y.%m.%d %H:%M:%S'))
            M = inspect.currentframe()
            try:
                D = inspect.getouterframes(M, 2)
            except:
                try:
                    D = inspect.getouterframes(M, 1)
                except:
                    D = None
            if (D == None):
                E = ['(unknown)', '(-)', '(unknown)']
            elif (len(D) > L):
                E = inspect.getframeinfo(D[L][0])
            else:
                E = ['(unknown)', '(-)', '(unknown)']
            R = os.path.basename(E[0]).replace('.py', '')
            S = E[2]
            T = E[1]
            F = ('// {} | {} || '.format(MOD_NAME, Q) + ' {} |'.format('{}.{}:{}'.format(R, S, T)))
            if P:
                C = '\n'
            else:
                C = ''
            for U in N:
                C += ' '
                C += str(U)
            F = ((F + C) + ' //')
            if K:
                if I:
                    B = 'FAA300'
                if H:
                    B = 'FF0000'
                V = '<span style="color:#{0};">{1}</span>'.format(B, C)
                cmds.inViewMessage(amg=V, pos='topCenter', fade=True, fst=1000, dk=True)
            if O:
                print('\ndebug.py traceback')
                traceback.print_exc()
                print('debug.py traceback end\n')
            if I:
                cmds.warning(F)
            if H:
                F += ' [D] //'
                cmds.warning(C)
            if ((not I) and (not H)):
                print(F)
default_logger = BroLogger()
log = default_logger

def log(*B, **A):
    '\n    Function for backwards compatibility with old code, replace once all code is reformatted to work with new BroLogger function.\n    :param args:\n    :param kwargs:\n    :return:\n    '
    A['callframe_depth'] = 3
    default_logger.log(*B, **A)
from functools import wraps

def deprecated(fn):
    'This is a decorator which can be used to mark functions\n    as deprecated. It will result in a warning being emitted\n    when the function is used.'

    @wraps(fn)
    def A(*A, **B):
        log.warning('Call to deprecated function {}.'.format(fn.__name__))
        return fn(*A, **B)
    return A

def log_wrapper(fn, fname=None):
    'This is a decorator which can be used to mark functions\n    as deprecated. It will result in a warning being emitted\n    when the function is used.'
    B = fname
    A = fn
    if (B is None):
        B = A.__name__

    @wraps(A)
    def C(*C, **D):
        print(('=' * 80))
        print('Running operation: {}'.format(os.path.basename(B)))
        if (B != A.__name__):
            print('Operation function name: {}'.format(A.__name__))
        print(('-' * 80))
        try:
            return A(*C, **D)
        except Exception as E:
            log.error(E, silent=False, exc_info=True)
        print(('=' * 80))
    return C
