from base64 import b64decode as license64

import traceback
from maya import cmds, mel
from BroTools.core.debug import default_logger
from BroTools.core.broqt.windows.error import BroErrorWindow
from BroTools.core.debug import default_logger as log
from random import choice
from functools import wraps
user_friendly_error_aliases = {'given path is not valid': 'Object does not have a transform', "no valid objects supplied to 'xform' command": 'Object does not have a transform'}

def show_error(text, title='BroTools Error', informativeText=None, image='error', icn=None, **B):
    A = icn
    if ((A is None) and ('icon' in B)):
        A = B['icon']
    BroErrorWindow().show(str(text), informativeText, image=image, icn=A, title=title)

class BroToolsError(Exception):

    def __init__(A, message='Unknown error', exception=None, fn=None, traceback_text=None, inView=False, *I, **E):
        D = traceback_text
        C = exception
        B = message
        F = E.get('silent', True)
        A.message = ''
        if ('Maya error text' not in str(B)):
            for (G, H) in user_friendly_error_aliases.items():
                if (G in str(B).lower()):
                    A.message += (H + '. ')
        A.message += str(B)
        if (C != None):
            A.message += ' [{}, '.format(type(C).__name__)
        elif (fn != None):
            A.message += ' ['
        if (fn != None):
            A.message += "'{}.{}']".format(getattr(fn, '__module__', 'unknown'), getattr(fn, '__name__', 'unknown'))
        log.error(A.message, exc_info=True)
        A.exception = C
        A.titles = ['Sorry, something went wrong!', 'Oops, mayday, we have a broblem!', "Oh, something's not right...", 'This is embarassing, something is not right.', "Sorry for this inconvenience, but there's a broblem."]
        print('\n\n# ----- {} ----- #\n'.format(choice(A.titles)))
        print('# - Summary: {} \n'.format(str(B)))
        traceback.print_exc()
        if inView:
            cmds.inViewMessage(amg=(('<span style="color:#F05A5A;">Error: </span>' + str(B)) + ' <span style="color:#FAA300;">More info in script editor.</span>'), pos='topCenter', fade=True, fst=2000, dk=True)
        if (D is None):
            D = str(traceback.format_exc())
        if (not F):
            show_error(A.message, informativeText=D)
        super(BroToolsError, A).__init__(A.message)

def catch_errors_native(fn):
    '\n    BroRig exception wrapper. Use @catch_errors above the function to wrap it.\n    Use it on functions that need to show errors on fail.\n    Args:\n        fn: function to wrap\n\n    Returns:\n        wrapped: wrapped function\n    '

    @wraps(fn)
    def A(*A, **B):
        try:
            cmds.undoInfo(openChunk=True)
            C = fn(*A, **B)
            cmds.undoInfo(closeChunk=True)
            return C
        except Exception as D:
            cmds.undoInfo(closeChunk=True)
            E = mel.eval('$tmp = $gMainProgressBar')
            cmds.progressBar(E, edit=True, endProgress=True)
            print(traceback.format_exc())
            cmds.error(str(D))
    return A

def catch_errors(fn):
    '\n    BroRig exception wrapper. Use @catch_errors above the function to wrap it.\n    Use it on functions that need to show errors on fail.\n    Args:\n        fn: function to wrap\n\n    Returns:\n        wrapped: wrapped function\n    '

    @wraps(fn)
    def A(*C, **D):
        try:
            cmds.undoInfo(openChunk=True)
            A = fn(*C, **D)
            cmds.undoInfo(closeChunk=True)
            default_logger.debug('FUNC: {}; RETURNED: {}'.format(fn.__name__, str(A)))
            return A
        except Exception as B:
            cmds.undoInfo(closeChunk=True)
            E = mel.eval('$tmp = $gMainProgressBar')
            cmds.progressBar(E, edit=True, endProgress=True)
            raise BroToolsError(message=str(B), exception=B, fn=fn, silent=False)
    return A

def catch_errors_silent(fn):
    '\n    BroRig exception wrapper. Use @catch_errors above the function to wrap it.\n    Use it on functions that need to show errors on fail.\n    Args:\n        fn: function to wrap\n\n    Returns:\n        wrapped: wrapped function\n    '

    @wraps(fn)
    def A(*C, **D):
        try:
            cmds.undoInfo(openChunk=True)
            A = fn(*C, **D)
            cmds.undoInfo(closeChunk=True)
            default_logger.debug('FUNC: {}; RETURNED: {}'.format(fn.__name__, str(A)))
            return A
        except Exception as B:
            cmds.undoInfo(closeChunk=True)
            E = mel.eval('$tmp = $gMainProgressBar')
            cmds.progressBar(E, edit=True, endProgress=True)
            raise BroToolsError(message=str(B), exception=B, fn=fn, silent=True)
    return A
