from base64 import b64decode as license64

'\nModule for file system operations\n'
import os
from BroTools.core.debug import default_logger as log
from BroTools.core.maya_utils import dict_key_with_max_val
import os, platform, subprocess

def reveal_in_explorer(path):
    if (platform.system() == 'Windows'):
        subprocess.Popen(['explorer', '/select,', path.replace('/', '\\')])
    elif (platform.system() == 'Darwin'):
        log.error('Openning folder is not supported for this OS yet.')
    else:
        log.error('Openning folder is not supported for this OS yet.')
maya_scene_file_exts = ['.ma', '.mb']

def is_maya_scene(filepath):
    '\n    Returns true if filepath is a maya scene file\n    '
    (B, A) = os.path.splitext(filepath)
    return (A.lower() in maya_scene_file_exts)

def get_maya_scenes(dirpath, recursive=False):
    '\n    From a directory of files returns only maya scenes\n    '
    A = dirpath
    B = []
    if recursive:
        raise NotImplementedError('Recursive get_maya_scenes is not yet implemented.')
    else:
        D = os.listdir(A)
        for E in D:
            C = os.path.join(A, E)
            if is_maya_scene(C):
                B.append(C)
    return B

def get_scene_version(filepath):
    '\n    From a name of type `filename.0001.ma` it extracts the `0001` part as an int number.\n    '
    B = os.path.basename(filepath)
    (C, D) = os.path.splitext(B)
    try:
        A = C.rpartition('.')[(- 1)]
        A = int(A)
    except ValueError:
        log.debug("Can't convert {} into integer to get check scene version ({}). Returning zero.".format(str(A), B))
        return 0
    return A

def get_scene_name(filepath, with_path=False):
    '\n    Return name without version string\n    '
    A = filepath
    if with_path:
        (B, C) = os.path.splitext(A)
    else:
        (B, C) = os.path.splitext(os.path.basename(A))
    return B.rpartition('.')[0]

def get_scene_with_greater_version(filepaths):
    A = {}
    for B in filepaths:
        A[B] = get_scene_version(B)
    return dict_key_with_max_val(A)

def get_maya_scenes_sorted_by_version(dirpath):
    A = {}
    for (D, G, E) in os.walk(dirpath, topdown=False):
        for F in E:
            B = os.path.join(D, F)
            if (not is_maya_scene(B)):
                continue
            C = get_scene_name(B, with_path=True)
            if (C not in A):
                A[C] = []
            A[C].append(B)
    return A

def get_latest_maya_scenes(dirpath):
    '\n    Returns an array of filepaths to latest scenes for each scene name by version from a directory.\n    '
    B = get_maya_scenes_sorted_by_version(dirpath)
    A = []
    for (D, C) in B.items():
        A.append(get_scene_with_greater_version(C))
    return A
'\nimport os\ncmds.file(prompt=False)\ndef playblast_panel_accepted(panel):\n    accepted = False\n    if panel != None:\n        if cmds.getPanel(to=panel) == "modelPanel":\n            accepted = True\n        else:\n            print(panel, "is not modelPanel")\n    else:\n        print(panel, "is None")\n    return accepted\n\njsn = r"D:\\Projects\\ShowReels\\Final_Rendersũ8_Req_ShowReel_2019_1.json"\nimport json\nwith open(jsn, "r") as f:\n    data = json.loads(f.read())   \nknown_names = []\nfor key, value in data.items():\n    n = key.rpartition(".")[0].rpartition(".")[0]\n    known_names.append(n)\nfrom BroTools.core import fileworks\nreload(fileworks)\nlatest_scenes = fileworks.get_latest_maya_scenes(r"D:\\Workũ8\\JamCity\\WT_Animation_Package\\Animation\\Gameboard_ANIM")\n\nrender_scenes = []\n\nfor s in latest_scenes:\n    if fileworks.get_scene_name(s) in known_names and fileworks.is_maya_scene(s):\n        \n        render_scenes.append(s)\n\nfrom BroTools.core import utils\nutils.progressBarStart(len(render_scenes))\nfor scene in render_scenes:\n    print "Scene", scene\n    if utils.progressBarIsCanceled():\n        break\n    cmds.file(scene, o=True, f=True)\n    if utils.progressBarIsCanceled():\n        break\n    panel = cmds.getPanel(up=True)\n    \n    \n    if not playblast_panel_accepted(panel):\n        panel = cmds.getPanel(wf=True)\n    \n    if not playblast_panel_accepted(panel):\n        visible_panels = cmds.getPanel(vis=True)\n        for p in visible_panels:\n            if playblast_panel_accepted(p):\n                panel = p\n    cmds.lookThru("Cam_A")\n    cmds.hide("imagePlane1")\n    cmds.hide("Grp_Ctrl_Master_Root")\n    cmds.setAttr("defaultRenderGlobals.currentRenderer", "mayaHardware2", type="string")\n    cmds.setAttr("defaultRenderGlobals.imageFormat", 32)\n    cmds.setAttr("hardwareRenderingGlobals.multiSampleEnable", 1)\n    cmds.setAttr("defaultRenderGlobals.animation", 1)\n    cmds.setAttr ("defaultRenderGlobals.outFormatControl", 0)\n    cmds.setAttr ("defaultRenderGlobals.animation", 1)\n    cmds.setAttr ("defaultRenderGlobals.putFrameBeforeExt", 1)\n    cmds.setAttr ("defaultRenderGlobals.extensionPadding", 4)\n    cmds.setAttr ("defaultRenderGlobals.periodInExt", 1)\n    cmds.setAttr("defaultRenderGlobals.skipExistingFrames", 1)\n    cmds.colorManagementPrefs(e=True, outputTransformEnabled=True)\n    filepath = cmds.file(q=True, sn=True)\n    filename = os.path.basename(filepath)\n    raw_name, extension = os.path.splitext(filename)\n    \n    \n    for i in range(int(cmds.playbackOptions(q=True, min=True)), int(cmds.playbackOptions(q=True, max=True))):\n        if utils.progressBarIsCanceled():\n            break\n        cmds.ogsRender(w=1920, h=1080, camera="Cam_A", f=i)\n    utils.progressBarStep()\nutils.progressBarEnd()\ncmds.file(prompt=True)\n\nprint "DONE!"\n\n'
