from base64 import b64decode as license64

import os
from functools import partial
from maya import cmds
import BroTools.core.maya_utils as utils
from BroTools.core import fileworks as fw
from BroTools.configuration import FILE_TOOLS_MENU
from BroTools.core.broqt.utils import clipboardSet

def build():
    A = {'Open newest version of this scene': openNewestVersionOfThisScene, 'Locate current scene': locateCurrentScene, 'Locate current project': locateCurrentProject, 'Copy project path': copyCurrentProjectPath, 'Copy scene path': copyCurrentScenePath, 'Find Textures': find_textures, 'Find Textures In': find_textures_in}
    for (B, C) in sorted(A.items()):
        utils.add_menu_item(label=B, parent=FILE_TOOLS_MENU, command=partial(C))

def openNewestVersionOfThisScene(*E, **F):
    A = os.path.normpath(cmds.file(q=True, sceneName=True))
    C = os.path.basename(A)
    D = os.path.dirname(A)
    for B in fw.get_latest_maya_scenes(D):
        if (os.path.normpath(B) == A):
            G = cmds.confirmDialog(title='BroTools: All good!', message='It looks like latest version of this scene is already open! :)', button=['Awesome!'], defaultButton='Awesome!', icon='info')
            return
        if (fw.get_scene_name(B) == fw.get_scene_name(C)):
            cmds.file(B, o=True, f=True)
            return

def locateCurrentScene(*A, **B):
    fw.reveal_in_explorer(cmds.file(q=True, sceneName=True))

def locateCurrentProject(*A, **B):
    fw.reveal_in_explorer(os.path.dirname(cmds.workspace(q=True, rd=True)))

def copyCurrentScenePath(*A, **B):
    clipboardSet(cmds.file(q=True, sceneName=True))

def copyCurrentProjectPath(*A, **B):
    clipboardSet(os.path.dirname(cmds.workspace(q=True, rd=True)))

def find_textures(*O, **G):
    C = G.get('custom_root', False)
    H = cmds.confirmDialog(title='Find Textures - Proceed?', message='This script will go over all file texture nodes in the scene and try to find them in {} based on their filenames. Would you like to proceed?'.format(('current project folder' if (C is False) else 'given folder')), button=['Yes', 'No'], defaultButton='Yes', cancelButton='No', dismissString='No', icon='info')
    if (H != 'Yes'):
        return
    print(('=' * 80))
    print('TEXTURE FINDER\n')
    I = cmds.ls(type='file')
    if C:
        D = cmds.fileDialog2(dialogStyle=2, fm=3, cap='Select search root folder')[0]
    else:
        D = cmds.workspace(q=True, rootDirectory=True)
    B = {}
    for (J, P, K) in os.walk(D, topdown=False):
        for L in K:
            B[L] = J
    for E in I:
        M = cmds.getAttr((E + '.fileTextureName'))
        A = os.path.basename(M)
        if (A in B):
            F = B[A]
            print('{:<25} {:<}'.format(A, F))
            N = os.path.join(F, A)
            cmds.setAttr((E + '.fileTextureName'), N, type='string')
        else:
            print('{:<25} {:<}'.format(A, 'NOT FOUND'))
    print(('=' * 80))

def find_textures_in(*A, **B):
    find_textures(custom_root=True)
