from base64 import b64decode as license64

'euclid graphics maths module\n\nDocumentation and tests are included in the file "euclid.rst", or online\nat https://github.com/ezag/pyeuclid/blob/master/euclid.rst\n'
__docformat__ = 'restructuredtext'
__version__ = '$Id$'
__revision__ = '$Revision$'
import math, operator, types
try:
    long
except NameError:
    long = int
_use_slots = True
_enable_swizzle_set = False
if _enable_swizzle_set:
    _use_slots = True

class _EuclidMetaclass(type):

    def __new__(B, name, bases, dct):
        C = bases
        A = dct
        if ('__slots__' in A):
            A['__getstate__'] = B._create_getstate(A['__slots__'])
            A['__setstate__'] = B._create_setstate(A['__slots__'])
        if _use_slots:
            return type.__new__(B, name, (C + (object,)), A)
        else:
            if ('__slots__' in A):
                del A['__slots__']
            return types.ClassType.__new__(types.ClassType, name, C, A)

    @classmethod
    def _create_getstate(A, slots):

        def __getstate__(self):
            A = {}
            for B in slots:
                A[B] = getattr(self, B)
            return A
        return __getstate__

    @classmethod
    def _create_setstate(A, slots):

        def __setstate__(self, state):
            for (A, B) in state.items():
                setattr(self, A, B)
        return __setstate__
__metaclass__ = _EuclidMetaclass

class Vector2():
    __slots__ = ['x', 'y']
    __hash__ = None

    def __init__(A, x=0, y=0):
        A.x = x
        A.y = y

    def __copy__(A):
        return A.__class__(A.x, A.y)
    copy = __copy__

    def __repr__(A):
        return ('Vector2(%.2f, %.2f)' % (A.x, A.y))

    def __eq__(B, other):
        A = other
        if isinstance(A, Vector2):
            return ((B.x == A.x) and (B.y == A.y))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 2))
            return ((B.x == A[0]) and (B.y == A[1]))

    def __ne__(A, other):
        return (not A.__eq__(other))

    def __nonzero__(A):
        return bool(((A.x != 0) or (A.y != 0)))

    def __len__(A):
        return 2

    def __getitem__(A, key):
        return (A.x, A.y)[key]

    def __setitem__(A, key, value):
        B = [A.x, A.y]
        B[key] = value
        (A.x, A.y) = B

    def __iter__(A):
        return iter((A.x, A.y))

    def __getattr__(A, name):
        try:
            return tuple([(A.x, A.y)['xy'.index(B)] for B in name])
        except ValueError:
            raise AttributeError(name)
    if _enable_swizzle_set:

        def __setattr__(A, name, value):
            C = value
            B = name
            if (len(B) == 1):
                object.__setattr__(A, B, C)
            else:
                try:
                    D = [A.x, A.y]
                    for (E, F) in map(None, B, C):
                        D['xy'.index(E)] = F
                    (A.x, A.y) = D
                except ValueError:
                    raise AttributeError(B)

    def __add__(B, other):
        A = other
        if isinstance(A, Vector2):
            if (B.__class__ is A.__class__):
                C = Vector2
            else:
                C = Point2
            return C((B.x + A.x), (B.y + A.y))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 2))
            return Vector2((B.x + A[0]), (B.y + A[1]))
    __radd__ = __add__

    def __iadd__(A, other):
        B = other
        if isinstance(B, Vector2):
            A.x += B.x
            A.y += B.y
        else:
            A.x += B[0]
            A.y += B[1]
        return A

    def __sub__(B, other):
        A = other
        if isinstance(A, Vector2):
            if (B.__class__ is A.__class__):
                C = Vector2
            else:
                C = Point2
            return C((B.x - A.x), (B.y - A.y))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 2))
            return Vector2((B.x - A[0]), (B.y - A[1]))

    def __rsub__(B, other):
        A = other
        if isinstance(A, Vector2):
            return Vector2((A.x - B.x), (A.y - B.y))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 2))
            return Vector2((A.x - B[0]), (A.y - B[1]))

    def __mul__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2((B.x * A), (B.y * A))
    __rmul__ = __mul__

    def __imul__(A, other):
        B = other
        assert (type(B) in (int, long, float))
        A.x *= B
        A.y *= B
        return A

    def __div__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.div(B.x, A), operator.div(B.y, A))

    def __rdiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.div(A, B.x), operator.div(A, B.y))

    def __floordiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.floordiv(B.x, A), operator.floordiv(B.y, A))

    def __rfloordiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.floordiv(A, B.x), operator.floordiv(A, B.y))

    def __truediv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.truediv(B.x, A), operator.truediv(B.y, A))

    def __rtruediv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector2(operator.truediv(A, B.x), operator.truediv(A, B.y))

    def __neg__(A):
        return Vector2((- A.x), (- A.y))
    __pos__ = __copy__

    def __abs__(A):
        return math.sqrt(((A.x ** 2) + (A.y ** 2)))
    magnitude = __abs__

    def magnitude_squared(A):
        return ((A.x ** 2) + (A.y ** 2))

    def normalize(A):
        B = A.magnitude()
        if B:
            A.x /= B
            A.y /= B
        return A

    def normalized(A):
        B = A.magnitude()
        if B:
            return Vector2((A.x / B), (A.y / B))
        return A.copy()

    def dot(B, other):
        A = other
        assert isinstance(A, Vector2)
        return ((B.x * A.x) + (B.y * A.y))

    def cross(A):
        return Vector2(A.y, (- A.x))

    def reflect(B, normal):
        A = normal
        assert isinstance(A, Vector2)
        C = (2 * ((B.x * A.x) + (B.y * A.y)))
        return Vector2((B.x - (C * A.x)), (B.y - (C * A.y)))

    def angle(A, other):
        'Return the angle to the vector other'
        B = other
        return math.acos((A.dot(B) / (A.magnitude() * B.magnitude())))

    def project(B, other):
        'Return one vector projected on the vector other'
        A = other.normalized()
        return (B.dot(A) * A)

class Vector3():
    __slots__ = ['x', 'y', 'z']
    __hash__ = None

    def __init__(A, x=0, y=0, z=0):
        A.x = x
        A.y = y
        A.z = z

    def __copy__(A):
        return A.__class__(A.x, A.y, A.z)
    copy = __copy__

    def __repr__(A):
        return ('Vector3(%.2f, %.2f, %.2f)' % (A.x, A.y, A.z))

    def __eq__(B, other):
        A = other
        if isinstance(A, Vector3):
            return ((B.x == A.x) and (B.y == A.y) and (B.z == A.z))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 3))
            return ((B.x == A[0]) and (B.y == A[1]) and (B.z == A[2]))

    def __ne__(A, other):
        return (not A.__eq__(other))

    def __nonzero__(A):
        return bool(((A.x != 0) or (A.y != 0) or (A.z != 0)))

    def __len__(A):
        return 3

    def __getitem__(A, key):
        return (A.x, A.y, A.z)[key]

    def __setitem__(A, key, value):
        B = [A.x, A.y, A.z]
        B[key] = value
        (A.x, A.y, A.z) = B

    def __iter__(A):
        return iter((A.x, A.y, A.z))

    def __getattr__(A, name):
        try:
            return tuple([(A.x, A.y, A.z)['xyz'.index(B)] for B in name])
        except ValueError:
            raise AttributeError(name)
    if _enable_swizzle_set:

        def __setattr__(A, name, value):
            C = value
            B = name
            if (len(B) == 1):
                object.__setattr__(A, B, C)
            else:
                try:
                    D = [A.x, A.y, A.z]
                    for (E, F) in map(None, B, C):
                        D['xyz'.index(E)] = F
                    (A.x, A.y, A.z) = D
                except ValueError:
                    raise AttributeError(B)

    def __add__(B, other):
        A = other
        if isinstance(A, Vector3):
            if (B.__class__ is A.__class__):
                C = Vector3
            else:
                C = Point3
            return C((B.x + A.x), (B.y + A.y), (B.z + A.z))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 3))
            return Vector3((B.x + A[0]), (B.y + A[1]), (B.z + A[2]))
    __radd__ = __add__

    def __iadd__(A, other):
        B = other
        if isinstance(B, Vector3):
            A.x += B.x
            A.y += B.y
            A.z += B.z
        else:
            A.x += B[0]
            A.y += B[1]
            A.z += B[2]
        return A

    def __sub__(B, other):
        A = other
        if isinstance(A, Vector3):
            if (B.__class__ is A.__class__):
                C = Vector3
            else:
                C = Point3
            return Vector3((B.x - A.x), (B.y - A.y), (B.z - A.z))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 3))
            return Vector3((B.x - A[0]), (B.y - A[1]), (B.z - A[2]))

    def __rsub__(B, other):
        A = other
        if isinstance(A, Vector3):
            return Vector3((A.x - B.x), (A.y - B.y), (A.z - B.z))
        else:
            assert (hasattr(A, '__len__') and (len(A) == 3))
            return Vector3((A.x - B[0]), (A.y - B[1]), (A.z - B[2]))

    def __mul__(B, other):
        A = other
        if isinstance(A, Vector3):
            if ((B.__class__ is Point3) or (A.__class__ is Point3)):
                C = Point3
            else:
                C = Vector3
            return C((B.x * A.x), (B.y * A.y), (B.z * A.z))
        else:
            assert (type(A) in (int, long, float))
            return Vector3((B.x * A), (B.y * A), (B.z * A))
    __rmul__ = __mul__

    def __imul__(A, other):
        B = other
        assert (type(B) in (int, long, float))
        A.x *= B
        A.y *= B
        A.z *= B
        return A

    def __div__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.div(B.x, A), operator.div(B.y, A), operator.div(B.z, A))

    def __rdiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.div(A, B.x), operator.div(A, B.y), operator.div(A, B.z))

    def __floordiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.floordiv(B.x, A), operator.floordiv(B.y, A), operator.floordiv(B.z, A))

    def __rfloordiv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.floordiv(A, B.x), operator.floordiv(A, B.y), operator.floordiv(A, B.z))

    def __truediv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.truediv(B.x, A), operator.truediv(B.y, A), operator.truediv(B.z, A))

    def __rtruediv__(B, other):
        A = other
        assert (type(A) in (int, long, float))
        return Vector3(operator.truediv(A, B.x), operator.truediv(A, B.y), operator.truediv(A, B.z))

    def __neg__(A):
        return Vector3((- A.x), (- A.y), (- A.z))
    __pos__ = __copy__

    def __abs__(A):
        return math.sqrt((((A.x ** 2) + (A.y ** 2)) + (A.z ** 2)))
    magnitude = __abs__
    length = magnitude

    def clamp_magnitude(A, max_length):
        B = max_length
        if (A.magnitude() > B):
            return (A.normalized() * B)
        return A

    def rotate_vector_by_quaternion(A, quaternion):
        '\n        Rotate a vector by a quaternion.\n    \n        Args:\n            vector (Vector3D): The vector to rotate.\n            quaternion (Quaternion): The quaternion representing the rotation.\n    \n        Returns:\n            Vector3D: The rotated vector.\n        '
        C = quaternion
        D = Quaternion(0, A.x, A.y, A.z)
        B = ((C * D) * C.conjugated())
        E = Vector3D(B.x, B.y, B.z)
        return E

    def magnitude_squared(A):
        return (((A.x ** 2) + (A.y ** 2)) + (A.z ** 2))

    def normalize(A):
        B = A.magnitude()
        if B:
            A.x /= B
            A.y /= B
            A.z /= B
        return A

    def normalized(A):
        B = A.magnitude()
        if B:
            return Vector3((A.x / B), (A.y / B), (A.z / B))
        return A.copy()

    def dot(B, other):
        A = other
        assert isinstance(A, Vector3)
        return (((B.x * A.x) + (B.y * A.y)) + (B.z * A.z))

    def cross(B, other):
        A = other
        assert isinstance(A, Vector3)
        return Vector3(((B.y * A.z) - (B.z * A.y)), (((- B.x) * A.z) + (B.z * A.x)), ((B.x * A.y) - (B.y * A.x)))

    def reflect(B, normal):
        A = normal
        assert isinstance(A, Vector3)
        C = (2 * (((B.x * A.x) + (B.y * A.y)) + (B.z * A.z)))
        return Vector3((B.x - (C * A.x)), (B.y - (C * A.y)), (B.z - (C * A.z)))

    def rotate_around(H, axis, theta):
        'Return the vector rotated around axis through angle theta. Right hand rule applies'
        L = theta
        I = axis
        (D, E, F) = (H.x, H.y, H.z)
        (A, B, C) = (I.x, I.y, I.z)
        M = (((A ** 2) + (B ** 2)) + (C ** 2))
        N = math.sqrt(M)
        G = math.cos(L)
        J = (math.sin(L) / N)
        K = (((((A * D) + (B * E)) + (C * F)) * (1 - G)) / M)
        return Vector3((((A * K) + (D * G)) + ((((- C) * E) + (B * F)) * J)), (((B * K) + (E * G)) + (((C * D) - (A * F)) * J)), (((C * K) + (F * G)) + ((((- B) * D) + (A * E)) * J)))

    def angle(A, other):
        'Return the angle to the vector other'
        B = other
        return math.acos((A.dot(B) / (A.magnitude() * B.magnitude())))

    def project(B, other):
        'Return one vector projected on the vector other'
        A = other.normalized()
        return (B.dot(A) * A)

    @staticmethod
    def findCenterPoint(vectors):
        '\n        Find center point among given vectors.\n        :param vectors: list of Vector3D \n        :return: Vector3D vector\n        '
        A = vectors
        B = Vector3(0, 0, 0)
        for C in A:
            B += C
        D = (B / len(A))
        return D
Vector3D = Vector3

class Matrix3():
    __slots__ = list('abcefgijk')

    def __init__(A):
        A.identity()

    def __copy__(B):
        A = Matrix3()
        A.a = B.a
        A.b = B.b
        A.c = B.c
        A.e = B.e
        A.f = B.f
        A.g = B.g
        A.i = B.i
        A.j = B.j
        A.k = B.k
        return A
    copy = __copy__

    def __repr__(A):
        return ('Matrix3([% 8.2f % 8.2f % 8.2f\n         % 8.2f % 8.2f % 8.2f\n         % 8.2f % 8.2f % 8.2f])' % (A.a, A.b, A.c, A.e, A.f, A.g, A.i, A.j, A.k))

    def __getitem__(A, key):
        return [A.a, A.e, A.i, A.b, A.f, A.j, A.c, A.g, A.k][key]

    def __setitem__(A, key, value):
        B = A[:]
        B[key] = value
        (A.a, A.e, A.i, A.b, A.f, A.j, A.c, A.g, A.k) = B

    def __mul__(B, other):
        A = other
        if isinstance(A, Matrix3):
            F = B.a
            G = B.b
            H = B.c
            I = B.e
            J = B.f
            K = B.g
            L = B.i
            M = B.j
            N = B.k
            O = A.a
            P = A.b
            Q = A.c
            R = A.e
            S = A.f
            T = A.g
            U = A.i
            V = A.j
            W = A.k
            D = Matrix3()
            D.a = (((F * O) + (G * R)) + (H * U))
            D.b = (((F * P) + (G * S)) + (H * V))
            D.c = (((F * Q) + (G * T)) + (H * W))
            D.e = (((I * O) + (J * R)) + (K * U))
            D.f = (((I * P) + (J * S)) + (K * V))
            D.g = (((I * Q) + (J * T)) + (K * W))
            D.i = (((L * O) + (M * R)) + (N * U))
            D.j = (((L * P) + (M * S)) + (N * V))
            D.k = (((L * Q) + (M * T)) + (N * W))
            return D
        elif isinstance(A, Point2):
            C = B
            E = A
            X = Point2(0, 0)
            X.x = (((C.a * E.x) + (C.b * E.y)) + C.c)
            X.y = (((C.e * E.x) + (C.f * E.y)) + C.g)
            return X
        elif isinstance(A, Vector2):
            C = B
            E = A
            Y = Vector2(0, 0)
            Y.x = ((C.a * E.x) + (C.b * E.y))
            Y.y = ((C.e * E.x) + (C.f * E.y))
            return Y
        else:
            A = A.copy()
            A._apply_transform(B)
            return A

    def __imul__(A, other):
        B = other
        assert isinstance(B, Matrix3)
        C = A.a
        D = A.b
        E = A.c
        F = A.e
        G = A.f
        H = A.g
        I = A.i
        J = A.j
        K = A.k
        L = B.a
        M = B.b
        N = B.c
        O = B.e
        P = B.f
        Q = B.g
        R = B.i
        S = B.j
        T = B.k
        A.a = (((C * L) + (D * O)) + (E * R))
        A.b = (((C * M) + (D * P)) + (E * S))
        A.c = (((C * N) + (D * Q)) + (E * T))
        A.e = (((F * L) + (G * O)) + (H * R))
        A.f = (((F * M) + (G * P)) + (H * S))
        A.g = (((F * N) + (G * Q)) + (H * T))
        A.i = (((I * L) + (J * O)) + (K * R))
        A.j = (((I * M) + (J * P)) + (K * S))
        A.k = (((I * N) + (J * Q)) + (K * T))
        return A

    def identity(A):
        A.a = A.f = A.k = 1.0
        A.b = A.c = A.e = A.g = A.i = A.j = 0
        return A

    def scale(A, x, y):
        A *= Matrix3.new_scale(x, y)
        return A

    def translate(A, x, y):
        A *= Matrix3.new_translate(x, y)
        return A

    def rotate(A, angle):
        A *= Matrix3.new_rotate(angle)
        return A

    def new_identity(A):
        B = A()
        return B
    new_identity = classmethod(new_identity)

    def new_scale(B, x, y):
        A = B()
        A.a = x
        A.f = y
        return A
    new_scale = classmethod(new_scale)

    def new_translate(B, x, y):
        A = B()
        A.c = x
        A.g = y
        return A
    new_translate = classmethod(new_translate)

    def new_rotate(D, angle):
        B = angle
        A = D()
        C = math.sin(B)
        E = math.cos(B)
        A.a = A.f = E
        A.b = (- C)
        A.e = C
        return A
    new_rotate = classmethod(new_rotate)

    def determinant(A):
        return (((((((A.a * A.f) * A.k) + ((A.b * A.g) * A.i)) + ((A.c * A.e) * A.j)) - ((A.a * A.g) * A.j)) - ((A.b * A.e) * A.k)) - ((A.c * A.f) * A.i))

    def inverse(A):
        C = Matrix3()
        B = A.determinant()
        if (abs(B) < 0.001):
            return C
        else:
            B = (1.0 / B)
            C.a = (B * ((A.f * A.k) - (A.g * A.j)))
            C.b = (B * ((A.c * A.j) - (A.b * A.k)))
            C.c = (B * ((A.b * A.g) - (A.c * A.f)))
            C.e = (B * ((A.g * A.i) - (A.e * A.k)))
            C.f = (B * ((A.a * A.k) - (A.c * A.i)))
            C.g = (B * ((A.c * A.e) - (A.a * A.g)))
            C.i = (B * ((A.e * A.j) - (A.f * A.i)))
            C.j = (B * ((A.b * A.i) - (A.a * A.j)))
            C.k = (B * ((A.a * A.f) - (A.b * A.e)))
            return C

class Matrix4():
    __slots__ = list('abcdefghijklmnop')

    def __init__(A):
        A.identity()

    def __copy__(B):
        A = Matrix4()
        A.a = B.a
        A.b = B.b
        A.c = B.c
        A.d = B.d
        A.e = B.e
        A.f = B.f
        A.g = B.g
        A.h = B.h
        A.i = B.i
        A.j = B.j
        A.k = B.k
        A.l = B.l
        A.m = B.m
        A.n = B.n
        A.o = B.o
        A.p = B.p
        return A
    copy = __copy__

    def __repr__(A):
        return ('Matrix4([% 8.2f % 8.2f % 8.2f % 8.2f\n         % 8.2f % 8.2f % 8.2f % 8.2f\n         % 8.2f % 8.2f % 8.2f % 8.2f\n         % 8.2f % 8.2f % 8.2f % 8.2f])' % (A.a, A.b, A.c, A.d, A.e, A.f, A.g, A.h, A.i, A.j, A.k, A.l, A.m, A.n, A.o, A.p))

    def __getitem__(A, key):
        return [A.a, A.e, A.i, A.m, A.b, A.f, A.j, A.n, A.c, A.g, A.k, A.o, A.d, A.h, A.l, A.p][key]

    def __setitem__(A, key, value):
        B = A[:]
        B[key] = value
        (A.a, A.e, A.i, A.m, A.b, A.f, A.j, A.n, A.c, A.g, A.k, A.o, A.d, A.h, A.l, A.p) = B

    def __mul__(C, other):
        A = other
        if isinstance(A, Matrix4):
            F = C.a
            G = C.b
            H = C.c
            I = C.d
            J = C.e
            K = C.f
            L = C.g
            M = C.h
            N = C.i
            O = C.j
            P = C.k
            Q = C.l
            R = C.m
            S = C.n
            T = C.o
            U = C.p
            V = A.a
            W = A.b
            X = A.c
            Y = A.d
            Z = A.e
            a = A.f
            b = A.g
            c = A.h
            d = A.i
            e = A.j
            f = A.k
            g = A.l
            h = A.m
            i = A.n
            j = A.o
            k = A.p
            E = Matrix4()
            E.a = ((((F * V) + (G * Z)) + (H * d)) + (I * h))
            E.b = ((((F * W) + (G * a)) + (H * e)) + (I * i))
            E.c = ((((F * X) + (G * b)) + (H * f)) + (I * j))
            E.d = ((((F * Y) + (G * c)) + (H * g)) + (I * k))
            E.e = ((((J * V) + (K * Z)) + (L * d)) + (M * h))
            E.f = ((((J * W) + (K * a)) + (L * e)) + (M * i))
            E.g = ((((J * X) + (K * b)) + (L * f)) + (M * j))
            E.h = ((((J * Y) + (K * c)) + (L * g)) + (M * k))
            E.i = ((((N * V) + (O * Z)) + (P * d)) + (Q * h))
            E.j = ((((N * W) + (O * a)) + (P * e)) + (Q * i))
            E.k = ((((N * X) + (O * b)) + (P * f)) + (Q * j))
            E.l = ((((N * Y) + (O * c)) + (P * g)) + (Q * k))
            E.m = ((((R * V) + (S * Z)) + (T * d)) + (U * h))
            E.n = ((((R * W) + (S * a)) + (T * e)) + (U * i))
            E.o = ((((R * X) + (S * b)) + (T * f)) + (U * j))
            E.p = ((((R * Y) + (S * c)) + (T * g)) + (U * k))
            return E
        elif isinstance(A, Point3):
            B = C
            D = A
            l = Point3(0, 0, 0)
            l.x = ((((B.a * D.x) + (B.b * D.y)) + (B.c * D.z)) + B.d)
            l.y = ((((B.e * D.x) + (B.f * D.y)) + (B.g * D.z)) + B.h)
            l.z = ((((B.i * D.x) + (B.j * D.y)) + (B.k * D.z)) + B.l)
            return l
        elif isinstance(A, Vector3):
            B = C
            D = A
            m = Vector3(0, 0, 0)
            m.x = (((B.a * D.x) + (B.b * D.y)) + (B.c * D.z))
            m.y = (((B.e * D.x) + (B.f * D.y)) + (B.g * D.z))
            m.z = (((B.i * D.x) + (B.j * D.y)) + (B.k * D.z))
            return m
        else:
            A = A.copy()
            A._apply_transform(C)
            return A

    def __imul__(A, other):
        B = other
        assert isinstance(B, Matrix4)
        C = A.a
        D = A.b
        E = A.c
        F = A.d
        G = A.e
        H = A.f
        I = A.g
        J = A.h
        K = A.i
        L = A.j
        M = A.k
        N = A.l
        O = A.m
        P = A.n
        Q = A.o
        R = A.p
        S = B.a
        T = B.b
        U = B.c
        V = B.d
        W = B.e
        X = B.f
        Y = B.g
        Z = B.h
        a = B.i
        b = B.j
        c = B.k
        d = B.l
        e = B.m
        f = B.n
        g = B.o
        h = B.p
        A.a = ((((C * S) + (D * W)) + (E * a)) + (F * e))
        A.b = ((((C * T) + (D * X)) + (E * b)) + (F * f))
        A.c = ((((C * U) + (D * Y)) + (E * c)) + (F * g))
        A.d = ((((C * V) + (D * Z)) + (E * d)) + (F * h))
        A.e = ((((G * S) + (H * W)) + (I * a)) + (J * e))
        A.f = ((((G * T) + (H * X)) + (I * b)) + (J * f))
        A.g = ((((G * U) + (H * Y)) + (I * c)) + (J * g))
        A.h = ((((G * V) + (H * Z)) + (I * d)) + (J * h))
        A.i = ((((K * S) + (L * W)) + (M * a)) + (N * e))
        A.j = ((((K * T) + (L * X)) + (M * b)) + (N * f))
        A.k = ((((K * U) + (L * Y)) + (M * c)) + (N * g))
        A.l = ((((K * V) + (L * Z)) + (M * d)) + (N * h))
        A.m = ((((O * S) + (P * W)) + (Q * a)) + (R * e))
        A.n = ((((O * T) + (P * X)) + (Q * b)) + (R * f))
        A.o = ((((O * U) + (P * Y)) + (Q * c)) + (R * g))
        A.p = ((((O * V) + (P * Z)) + (Q * d)) + (R * h))
        return A

    def transform(E, other):
        A = E
        B = other
        C = Point3(0, 0, 0)
        C.x = ((((A.a * B.x) + (A.b * B.y)) + (A.c * B.z)) + A.d)
        C.y = ((((A.e * B.x) + (A.f * B.y)) + (A.g * B.z)) + A.h)
        C.z = ((((A.i * B.x) + (A.j * B.y)) + (A.k * B.z)) + A.l)
        D = ((((A.m * B.x) + (A.n * B.y)) + (A.o * B.z)) + A.p)
        if (D != 0):
            C.x /= D
            C.y /= D
            C.z /= D
        return C

    def identity(A):
        A.a = A.f = A.k = A.p = 1.0
        A.b = A.c = A.d = A.e = A.g = A.h = A.i = A.j = A.l = A.m = A.n = A.o = 0
        return A

    def scale(A, x, y, z):
        A *= Matrix4.new_scale(x, y, z)
        return A

    def translate(A, x, y, z):
        A *= Matrix4.new_translate(x, y, z)
        return A

    def rotatex(A, angle):
        A *= Matrix4.new_rotatex(angle)
        return A

    def rotatey(A, angle):
        A *= Matrix4.new_rotatey(angle)
        return A

    def rotatez(A, angle):
        A *= Matrix4.new_rotatez(angle)
        return A

    def rotate_axis(A, angle, axis):
        A *= Matrix4.new_rotate_axis(angle, axis)
        return A

    def rotate_euler(A, heading, attitude, bank):
        A *= Matrix4.new_rotate_euler(heading, attitude, bank)
        return A

    def rotate_triple_axis(A, x, y, z):
        A *= Matrix4.new_rotate_triple_axis(x, y, z)
        return A

    def transpose(A):
        (A.a, A.e, A.i, A.m, A.b, A.f, A.j, A.n, A.c, A.g, A.k, A.o, A.d, A.h, A.l, A.p) = (A.a, A.b, A.c, A.d, A.e, A.f, A.g, A.h, A.i, A.j, A.k, A.l, A.m, A.n, A.o, A.p)

    def transposed(B):
        A = B.copy()
        A.transpose()
        return A

    def new(B, *C):
        A = B()
        A[:] = C
        return A
    new = classmethod(new)

    def new_identity(A):
        B = A()
        return B
    new_identity = classmethod(new_identity)

    def new_scale(B, x, y, z):
        A = B()
        A.a = x
        A.f = y
        A.k = z
        return A
    new_scale = classmethod(new_scale)

    def get_scale(A):
        return Vector3(A.a, A.f, A.k)

    def new_translate(B, x, y, z):
        A = B()
        A.d = x
        A.h = y
        A.l = z
        return A
    new_translate = classmethod(new_translate)

    def new_rotatex(D, angle):
        B = angle
        A = D()
        C = math.sin(B)
        E = math.cos(B)
        A.f = A.k = E
        A.g = (- C)
        A.j = C
        return A
    new_rotatex = classmethod(new_rotatex)

    def new_rotatey(D, angle):
        B = angle
        A = D()
        C = math.sin(B)
        E = math.cos(B)
        A.a = A.k = E
        A.c = C
        A.i = (- C)
        return A
    new_rotatey = classmethod(new_rotatey)

    def new_rotatez(D, angle):
        B = angle
        A = D()
        C = math.sin(B)
        E = math.cos(B)
        A.a = A.f = E
        A.b = (- C)
        A.e = C
        return A
    new_rotatez = classmethod(new_rotatez)

    def new_rotate_axis(J, angle, axis):
        I = angle
        assert isinstance(axis, Vector3)
        H = axis.normalized()
        C = H.x
        D = H.y
        E = H.z
        A = J()
        F = math.sin(I)
        G = math.cos(I)
        B = (1.0 - G)
        A.a = (((C * C) * B) + G)
        A.b = (((C * D) * B) - (E * F))
        A.c = (((C * E) * B) + (D * F))
        A.e = (((D * C) * B) + (E * F))
        A.f = (((D * D) * B) + G)
        A.g = (((D * E) * B) - (C * F))
        A.i = (((C * E) * B) - (D * F))
        A.j = (((D * E) * B) + (C * F))
        A.k = (((E * E) * B) + G)
        return A
    new_rotate_axis = classmethod(new_rotate_axis)

    def new_rotate_euler(J, heading, attitude, bank):
        I = attitude
        H = heading
        B = math.cos(H)
        C = math.sin(H)
        G = math.cos(I)
        D = math.sin(I)
        E = math.cos(bank)
        F = math.sin(bank)
        A = J()
        A.a = (B * G)
        A.b = ((C * F) - ((B * D) * E))
        A.c = (((B * D) * F) + (C * E))
        A.e = D
        A.f = (G * E)
        A.g = ((- G) * F)
        A.i = ((- C) * G)
        A.j = (((C * D) * E) + (B * F))
        A.k = ((((- C) * D) * F) + (B * E))
        return A
    new_rotate_euler = classmethod(new_rotate_euler)

    def new_rotate_triple_axis(B, x, y, z):
        A = B()
        (A.a, A.b, A.c) = (x.x, y.x, z.x)
        (A.e, A.f, A.g) = (x.y, y.y, z.y)
        (A.i, A.j, A.k) = (x.z, y.z, z.z)
        return A
    new_rotate_triple_axis = classmethod(new_rotate_triple_axis)

    def new_look_at(F, eye, at, up):
        B = eye
        C = (B - at).normalized()
        D = up.cross(C).normalized()
        E = C.cross(D)
        A = F.new_rotate_triple_axis(D, E, C)
        A.transpose()
        (A.d, A.h, A.l) = ((- D.dot(B)), (- E.dot(B)), (- C.dot(B)))
        return A
    new_look_at = classmethod(new_look_at)

    def new_perspective(E, fov_y, aspect, near, far):
        C = far
        B = near
        D = (1 / math.tan((fov_y / 2)))
        A = E()
        assert ((B != 0.0) and (B != C))
        A.a = (D / aspect)
        A.f = D
        A.k = ((C + B) / (B - C))
        A.l = (((2 * C) * B) / (B - C))
        A.o = (- 1)
        A.p = 0
        return A
    new_perspective = classmethod(new_perspective)

    def determinant(A):
        return ((((((((A.a * A.f) - (A.e * A.b)) * ((A.k * A.p) - (A.o * A.l))) - (((A.a * A.j) - (A.i * A.b)) * ((A.g * A.p) - (A.o * A.h)))) + (((A.a * A.n) - (A.m * A.b)) * ((A.g * A.l) - (A.k * A.h)))) + (((A.e * A.j) - (A.i * A.f)) * ((A.c * A.p) - (A.o * A.d)))) - (((A.e * A.n) - (A.m * A.f)) * ((A.c * A.l) - (A.k * A.d)))) + (((A.i * A.n) - (A.m * A.j)) * ((A.c * A.h) - (A.g * A.d))))

    def inverse(A):
        C = Matrix4()
        B = A.determinant()
        if (abs(B) < 0.001):
            return C
        else:
            B = (1.0 / B)
            C.a = (B * (((A.f * ((A.k * A.p) - (A.o * A.l))) + (A.j * ((A.o * A.h) - (A.g * A.p)))) + (A.n * ((A.g * A.l) - (A.k * A.h)))))
            C.e = (B * (((A.g * ((A.i * A.p) - (A.m * A.l))) + (A.k * ((A.m * A.h) - (A.e * A.p)))) + (A.o * ((A.e * A.l) - (A.i * A.h)))))
            C.i = (B * (((A.h * ((A.i * A.n) - (A.m * A.j))) + (A.l * ((A.m * A.f) - (A.e * A.n)))) + (A.p * ((A.e * A.j) - (A.i * A.f)))))
            C.m = (B * (((A.e * ((A.n * A.k) - (A.j * A.o))) + (A.i * ((A.f * A.o) - (A.n * A.g)))) + (A.m * ((A.j * A.g) - (A.f * A.k)))))
            C.b = (B * (((A.j * ((A.c * A.p) - (A.o * A.d))) + (A.n * ((A.k * A.d) - (A.c * A.l)))) + (A.b * ((A.o * A.l) - (A.k * A.p)))))
            C.f = (B * (((A.k * ((A.a * A.p) - (A.m * A.d))) + (A.o * ((A.i * A.d) - (A.a * A.l)))) + (A.c * ((A.m * A.l) - (A.i * A.p)))))
            C.j = (B * (((A.l * ((A.a * A.n) - (A.m * A.b))) + (A.p * ((A.i * A.b) - (A.a * A.j)))) + (A.d * ((A.m * A.j) - (A.i * A.n)))))
            C.n = (B * (((A.i * ((A.n * A.c) - (A.b * A.o))) + (A.m * ((A.b * A.k) - (A.j * A.c)))) + (A.a * ((A.j * A.o) - (A.n * A.k)))))
            C.c = (B * (((A.n * ((A.c * A.h) - (A.g * A.d))) + (A.b * ((A.g * A.p) - (A.o * A.h)))) + (A.f * ((A.o * A.d) - (A.c * A.p)))))
            C.g = (B * (((A.o * ((A.a * A.h) - (A.e * A.d))) + (A.c * ((A.e * A.p) - (A.m * A.h)))) + (A.g * ((A.m * A.d) - (A.a * A.p)))))
            C.k = (B * (((A.p * ((A.a * A.f) - (A.e * A.b))) + (A.d * ((A.e * A.n) - (A.m * A.f)))) + (A.h * ((A.m * A.b) - (A.a * A.n)))))
            C.o = (B * (((A.m * ((A.f * A.c) - (A.b * A.g))) + (A.a * ((A.n * A.g) - (A.f * A.o)))) + (A.e * ((A.b * A.o) - (A.n * A.c)))))
            C.d = (B * (((A.b * ((A.k * A.h) - (A.g * A.l))) + (A.f * ((A.c * A.l) - (A.k * A.d)))) + (A.j * ((A.g * A.d) - (A.c * A.h)))))
            C.h = (B * (((A.c * ((A.i * A.h) - (A.e * A.l))) + (A.g * ((A.a * A.l) - (A.i * A.d)))) + (A.k * ((A.e * A.d) - (A.a * A.h)))))
            C.l = (B * (((A.d * ((A.i * A.f) - (A.e * A.j))) + (A.h * ((A.a * A.j) - (A.i * A.b)))) + (A.l * ((A.e * A.b) - (A.a * A.f)))))
            C.p = (B * (((A.a * ((A.f * A.k) - (A.j * A.g))) + (A.e * ((A.j * A.c) - (A.b * A.k)))) + (A.i * ((A.b * A.g) - (A.f * A.c)))))
        return C

    def get_quaternion(B):
        'Returns a quaternion representing the rotation part of the matrix.\n        Taken from:\n        http://web.archive.org/web/20041029003853/http://www.j3d.org/matrix_faq/matrfaq_latest.html#Q55\n        '
        H = ((B.a + B.f) + B.k)
        if (H > 1e-08):
            C = (math.sqrt((1.0 + H)) * 2)
            D = ((B.j - B.g) / C)
            E = ((B.c - B.i) / C)
            F = ((B.e - B.b) / C)
            G = (0.25 * C)
        else:
            A = (B.a, B.b, B.c, B.d, B.e, B.f, B.g, B.h, B.i, B.j, B.k, B.l, B.m, B.n, B.o, B.p)
            if ((A[0] > A[5]) and (A[0] > A[10])):
                C = (math.sqrt((((1.0 + A[0]) - A[5]) - A[10])) * 2)
                D = (0.25 * C)
                E = ((A[4] + A[1]) / C)
                F = ((A[2] + A[8]) / C)
                G = ((A[9] - A[6]) / C)
            elif (A[5] > A[10]):
                C = (math.sqrt((((1.0 + A[5]) - A[0]) - A[10])) * 2)
                D = ((A[4] + A[1]) / C)
                E = (0.25 * C)
                F = ((A[9] + A[6]) / C)
                G = ((A[2] - A[8]) / C)
            else:
                C = (math.sqrt((((1.0 + A[10]) - A[0]) - A[5])) * 2)
                D = ((A[2] + A[8]) / C)
                E = ((A[9] + A[6]) / C)
                F = (0.25 * C)
                G = ((A[4] - A[1]) / C)
        return Quaternion(G, D, E, F)

    def get_translation(A):
        return Vector3(A.d, A.h, A.l)

    @property
    def quaternion(self):
        return self.get_quaternion()

    @property
    def translation(self):
        return self.get_translation()

class Quaternion():
    __slots__ = ['w', 'x', 'y', 'z']

    def __init__(A, w=1, x=0, y=0, z=0):
        A.w = w
        A.x = x
        A.y = y
        A.z = z

    def __copy__(B):
        A = Quaternion()
        A.w = B.w
        A.x = B.x
        A.y = B.y
        A.z = B.z
        return A
    copy = __copy__

    def __repr__(A):
        return ('Quaternion(real=%.2f, imag=<%.2f, %.2f, %.2f>)' % (A.w, A.x, A.y, A.z))

    def __mul__(B, other):
        A = other
        if isinstance(A, Quaternion):
            I = B.x
            J = B.y
            K = B.z
            L = B.w
            M = A.x
            N = A.y
            O = A.z
            P = A.w
            F = Quaternion()
            F.x = ((((I * P) + (J * O)) - (K * N)) + (L * M))
            F.y = (((((- I) * O) + (J * P)) + (K * M)) + (L * N))
            F.z = ((((I * N) - (J * M)) + (K * P)) + (L * O))
            F.w = (((((- I) * M) - (J * N)) - (K * O)) + (L * P))
            return F
        elif isinstance(A, Vector3):
            R = B.w
            Q = B.x
            G = B.y
            H = B.z
            C = A.x
            D = A.y
            E = A.z
            S = (R * R)
            T = (R * 2)
            X = (T * Q)
            Y = (T * G)
            Z = (T * H)
            U = (Q * Q)
            a = (Q * 2)
            b = (a * G)
            c = (a * H)
            V = (G * G)
            d = ((2 * G) * H)
            W = (H * H)
            return A.__class__(((((((((S * C) + (Y * E)) - (Z * D)) + (U * C)) + (b * D)) + (c * E)) - (W * C)) - (V * C)), ((((((((b * C) + (V * D)) + (d * E)) + (Z * C)) - (W * D)) + (S * D)) - (X * E)) - (U * D)), ((((((((c * C) + (d * D)) + (W * E)) - (Y * C)) - (V * E)) + (X * D)) - (U * E)) + (S * E)))
        else:
            A = A.copy()
            A._apply_transform(B)
            return A

    def __sub__(A, other):
        'Subtract another quaternion from this quaternion.'
        B = other
        return Quaternion((A.w - B.w), (A.x - B.x), (A.y - B.y), (A.z - B.z))

    def __neg__(A):
        'Negates the quaternion.'
        return Quaternion((- A.w), (- A.x), (- A.y), (- A.z))

    def __eq__(B, other):
        A = other
        if (not isinstance(A, Quaternion)):
            return False
        return ((B.w == A.w) and (B.x == A.x) and (B.y == A.y) and (B.z == A.z))

    def __imul__(A, other):
        B = other
        assert isinstance(B, Quaternion)
        C = A.x
        D = A.y
        E = A.z
        F = A.w
        G = B.x
        H = B.y
        I = B.z
        J = B.w
        A.x = ((((C * J) + (D * I)) - (E * H)) + (F * G))
        A.y = (((((- C) * I) + (D * J)) + (E * G)) + (F * H))
        A.z = ((((C * H) - (D * G)) + (E * J)) + (F * I))
        A.w = (((((- C) * G) - (D * H)) - (E * I)) + (F * J))
        return A

    def __abs__(A):
        return math.sqrt(((((A.w ** 2) + (A.x ** 2)) + (A.y ** 2)) + (A.z ** 2)))
    magnitude = __abs__

    def magnitude_squared(A):
        return ((((A.w ** 2) + (A.x ** 2)) + (A.y ** 2)) + (A.z ** 2))

    @staticmethod
    def between_vectors_with_up(v1, v2, up_hint=Vector3(0, 1, 0)):
        '\n        Create a quaternion that represents the rotation from vector v1 to vector v2,\n        using up_hint to determine rotation axis when vectors are parallel/anti-parallel.\n        '
        A = v1
        A = A.normalized()
        v2 = v2.normalized()
        C = A.dot(v2)
        if (C < (- 0.99999)):
            B = A.cross(up_hint)
            if (abs(B) < 1e-06):
                B = A.cross(Vector3(1, 0, 0))
                if (abs(B) < 1e-06):
                    B = A.cross(Vector3(0, 0, 1))
            B = B.normalized()
            return Quaternion.new_rotate_axis(math.pi, B)
        elif (C > 0.99999):
            return Quaternion()
        D = A.cross(v2)
        return Quaternion((1 + C), D.x, D.y, D.z).normalized()

    def normal(A, axis=Vector3(0, 0, 1)):
        B = (A * axis)
        return B

    @staticmethod
    def look_at(direction, forward_axis=Vector3(0, 0, 1), up_axis=Vector3(0, 1, 0), world_up=Vector3(0, 1, 0)):
        '\n        Create a quaternion that rotates forward_axis to point in direction.\n        '
        E = world_up
        D = forward_axis
        A = direction
        A = A.normalized()
        D = D.normalized()
        F = Quaternion.between_vectors_with_up(D, A, E)
        H = (F * up_axis)
        B = (E - (A * A.dot(E)))
        B = B.normalized()
        G = H.dot(B)
        G = max(min(G, 1.0), (- 1.0))
        C = math.acos(G)
        if (abs(C) > 1e-06):
            I = H.cross(B)
            if (I.dot(A) < 0):
                C = (- C)
            J = Quaternion.new_rotate_axis(C, A)
            return (J * F).normalized()
        return F

    @staticmethod
    def between_vectors(v1, v2):
        '\n        Create a quaternion that represents the rotation from vector v1 to vector v2.\n        Both vectors are assumed to be normalized.\n        '
        B = v1
        A = B.cross(v2)
        C = B.dot(v2)
        if (C < (- 0.99999)):
            D = (Vector3(1, 0, 0) if (abs(B.x) < abs(B.z)) else Vector3(0, 0, 1))
            A = B.cross(D).normalized()
            return Quaternion(0, A.x, A.y, A.z)
        elif (C > 0.99999):
            return Quaternion(1, 0, 0, 0)
        return Quaternion((1 + C), A.x, A.y, A.z).normalized()

    def identity(A):
        A.w = 1
        A.x = 0
        A.y = 0
        A.z = 0
        return A

    def rotate_axis(A, angle, axis):
        A *= Quaternion.new_rotate_axis(angle, axis)
        return A

    def rotate_euler(A, heading, attitude, bank):
        A *= Quaternion.new_rotate_euler(heading, attitude, bank)
        return A

    def rotate_matrix(A, m):
        A *= Quaternion.new_rotate_matrix(m)
        return A

    def dot(A, other):
        'Compute the dot product of this quaternion with another.'
        B = other
        return ((((A.w * B.w) + (A.x * B.x)) + (A.y * B.y)) + (A.z * B.z))

    def conjugated(B):
        A = Quaternion()
        A.w = B.w
        A.x = (- B.x)
        A.y = (- B.y)
        A.z = (- B.z)
        return A

    def normalize(A):
        B = A.magnitude()
        if (B != 0):
            A.w /= B
            A.x /= B
            A.y /= B
            A.z /= B
        return A

    def normalized(A):
        B = A.magnitude()
        if (B != 0):
            C = Quaternion()
            C.w = (A.w / B)
            C.x = (A.x / B)
            C.y = (A.y / B)
            C.z = (A.z / B)
            return C
        else:
            return A.copy()

    def get_angle_axis(A):
        if (A.w > 1):
            A = A.normalized()
        C = (2 * math.acos(A.w))
        B = math.sqrt((1 - (A.w ** 2)))
        if (B < 0.001):
            return (C, Vector3(1, 0, 0))
        else:
            return (C, Vector3((A.x / B), (A.y / B), (A.z / B)))

    def get_euler(A, as_degrees=False):
        E = ((A.x * A.y) + (A.z * A.w))
        if (E > 0.4999):
            B = (2 * math.atan2(A.x, A.w))
            C = (math.pi / 2)
            D = 0
        elif (E < (- 0.4999)):
            B = ((- 2) * math.atan2(A.x, A.w))
            C = ((- math.pi) / 2)
            D = 0
        else:
            G = (A.x ** 2)
            H = (A.y ** 2)
            F = (A.z ** 2)
            B = math.atan2((((2 * A.y) * A.w) - ((2 * A.x) * A.z)), ((1 - (2 * H)) - (2 * F)))
            C = math.asin((2 * E))
            D = math.atan2((((2 * A.x) * A.w) - ((2 * A.y) * A.z)), ((1 - (2 * G)) - (2 * F)))
        if as_degrees:
            return (math.degrees(B), math.degrees(C), math.degrees(D))
        return (B, C, D)

    def get_matrix(A):
        C = (A.x ** 2)
        D = (A.x * A.y)
        E = (A.x * A.z)
        F = (A.x * A.w)
        G = (A.y ** 2)
        H = (A.y * A.z)
        I = (A.y * A.w)
        J = (A.z ** 2)
        K = (A.z * A.w)
        B = Matrix4()
        B.a = (1 - (2 * (G + J)))
        B.b = (2 * (D - K))
        B.c = (2 * (E + I))
        B.e = (2 * (D + K))
        B.f = (1 - (2 * (C + J)))
        B.g = (2 * (H - F))
        B.i = (2 * (E - I))
        B.j = (2 * (H + F))
        B.k = (1 - (2 * (C + G)))
        return B

    def new_identity(A):
        return A()
    new_identity = classmethod(new_identity)

    def new_rotate_axis(E, angle, axis):
        D = angle
        A = axis
        assert isinstance(A, Vector3)
        A = A.normalized()
        C = math.sin((D / 2))
        B = E()
        B.w = math.cos((D / 2))
        B.x = (A.x * C)
        B.y = (A.y * C)
        B.z = (A.z * C)
        return B
    new_rotate_axis = classmethod(new_rotate_axis)

    def new_rotate_euler(J, heading, attitude, bank):
        I = attitude
        H = heading
        A = J()
        B = math.cos((H / 2))
        C = math.sin((H / 2))
        D = math.cos((I / 2))
        E = math.sin((I / 2))
        F = math.cos((bank / 2))
        G = math.sin((bank / 2))
        A.w = (((B * D) * F) - ((C * E) * G))
        A.x = (((C * E) * F) + ((B * D) * G))
        A.y = (((C * D) * F) + ((B * E) * G))
        A.z = (((B * E) * F) - ((C * D) * G))
        return A
    new_rotate_euler = classmethod(new_rotate_euler)

    def new_rotate_matrix(C, m):
        if (((m[0] + m[5]) + m[10]) > 1e-08):
            B = (((m[0] + m[5]) + m[10]) + 1.0)
            A = (0.5 / math.sqrt(B))
            return C((A * B), ((m[6] - m[9]) * A), ((m[8] - m[2]) * A), ((m[1] - m[4]) * A))
        elif ((m[0] > m[5]) and (m[0] > m[10])):
            B = (((m[0] - m[5]) - m[10]) + 1.0)
            A = (0.5 / math.sqrt(B))
            return C(((m[6] - m[9]) * A), (A * B), ((m[1] + m[4]) * A), ((m[8] + m[2]) * A))
        elif (m[5] > m[10]):
            B = ((((- m[0]) + m[5]) - m[10]) + 1.0)
            A = (0.5 / math.sqrt(B))
            return C(((m[8] - m[2]) * A), ((m[1] + m[4]) * A), (A * B), ((m[6] + m[9]) * A))
        else:
            B = ((((- m[0]) - m[5]) + m[10]) + 1.0)
            A = (0.5 / math.sqrt(B))
            return C(((m[1] - m[4]) * A), ((m[8] + m[2]) * A), ((m[6] + m[9]) * A), (A * B))
    new_rotate_matrix = classmethod(new_rotate_matrix)

    def new_interpolate(I, q1, q2, t):
        B = q1
        A = q2
        assert (isinstance(B, Quaternion) and isinstance(A, Quaternion))
        C = I()
        D = ((((B.w * A.w) + (B.x * A.x)) + (B.y * A.y)) + (B.z * A.z))
        if (D < 0.0):
            D = (- D)
            B = B.conjugated()
        elif (D > 1):
            D = 1
        G = math.acos(D)
        if (abs(G) < 0.01):
            C.w = A.w
            C.x = A.x
            C.y = A.y
            C.z = A.z
            return C
        H = math.sqrt((1.0 - (D * D)))
        if (abs(H) < 0.01):
            C.w = ((B.w + A.w) * 0.5)
            C.x = ((B.x + A.x) * 0.5)
            C.y = ((B.y + A.y) * 0.5)
            C.z = ((B.z + A.z) * 0.5)
            return C
        E = (math.sin(((1 - t) * G)) / H)
        F = (math.sin((t * G)) / H)
        C.w = ((B.w * E) + (A.w * F))
        C.x = ((B.x * E) + (A.x * F))
        C.y = ((B.y * E) + (A.y * F))
        C.z = ((B.z * E) + (A.z * F))
        return C
    new_interpolate = classmethod(new_interpolate)

    @classmethod
    def slerp(I, q1, q2, t):
        C = q1
        B = q2
        assert (isinstance(C, Quaternion) and isinstance(B, Quaternion))
        A = I()
        D = ((((C.w * B.w) + (C.x * B.x)) + (C.y * B.y)) + (C.z * B.z))
        if (D < 0):
            B = I((- B.w), (- B.x), (- B.y), (- B.z))
            D = (- D)
        D = max((- 1.0), min(1.0, D))
        H = math.acos(D)
        if (abs(H) < 0.01):
            A.w = ((C.w * (1 - t)) + (B.w * t))
            A.x = ((C.x * (1 - t)) + (B.x * t))
            A.y = ((C.y * (1 - t)) + (B.y * t))
            A.z = ((C.z * (1 - t)) + (B.z * t))
            E = math.sqrt(((((A.w * A.w) + (A.x * A.x)) + (A.y * A.y)) + (A.z * A.z)))
            A.w /= E
            A.x /= E
            A.y /= E
            A.z /= E
            return A
        J = math.sqrt((1.0 - (D * D)))
        F = (math.sin(((1 - t) * H)) / J)
        G = (math.sin((t * H)) / J)
        A.w = ((C.w * F) + (B.w * G))
        A.x = ((C.x * F) + (B.x * G))
        A.y = ((C.y * F) + (B.y * G))
        A.z = ((C.z * F) + (B.z * G))
        return A

class Geometry():

    def _connect_unimplemented(A, other):
        raise AttributeError(('Cannot connect %s to %s' % (A.__class__, other.__class__)))

    def _intersect_unimplemented(A, other):
        raise AttributeError(('Cannot intersect %s and %s' % (A.__class__, other.__class__)))
    _intersect_point2 = _intersect_unimplemented
    _intersect_line2 = _intersect_unimplemented
    _intersect_circle = _intersect_unimplemented
    _connect_point2 = _connect_unimplemented
    _connect_line2 = _connect_unimplemented
    _connect_circle = _connect_unimplemented
    _intersect_point3 = _intersect_unimplemented
    _intersect_line3 = _intersect_unimplemented
    _intersect_sphere = _intersect_unimplemented
    _intersect_plane = _intersect_unimplemented
    _connect_point3 = _connect_unimplemented
    _connect_line3 = _connect_unimplemented
    _connect_sphere = _connect_unimplemented
    _connect_plane = _connect_unimplemented

    def intersect(A, other):
        raise NotImplementedError

    def connect(A, other):
        raise NotImplementedError

    def distance(B, other):
        A = B.connect(other)
        if A:
            return A.length
        return 0.0

def _intersect_point2_circle(P, C):
    return (abs((P - C.c)) <= C.r)

def _intersect_line2_line2(A, B):
    C = ((B.v.y * A.v.x) - (B.v.x * A.v.y))
    if (C == 0):
        return
    E = (A.p.y - B.p.y)
    F = (A.p.x - B.p.x)
    D = (((B.v.x * E) - (B.v.y * F)) / C)
    if (not A._u_in(D)):
        return
    G = (((A.v.x * E) - (A.v.y * F)) / C)
    if (not B._u_in(G)):
        return
    return Point2((A.p.x + (D * A.v.x)), (A.p.y + (D * A.v.y)))

def _intersect_line2_circle(L, C):
    D = L.v.magnitude_squared()
    E = (2 * ((L.v.x * (L.p.x - C.c.x)) + (L.v.y * (L.p.y - C.c.y))))
    H = (((C.c.magnitude_squared() + L.p.magnitude_squared()) - (2 * C.c.dot(L.p))) - (C.r ** 2))
    F = ((E ** 2) - ((4 * D) * H))
    if (F < 0):
        return
    G = math.sqrt(F)
    A = (((- E) + G) / (2 * D))
    B = (((- E) - G) / (2 * D))
    if (((A * B) > 0) and (not L._u_in(A)) and (not L._u_in(B))):
        return
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    if (not L._u_in(B)):
        B = max(min(B, 1.0), 0.0)
    if (A == B):
        return Point2((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)))
    return LineSegment2(Point2((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y))), Point2((L.p.x + (B * L.v.x)), (L.p.y + (B * L.v.y))))

def _intersect_circle_circle(A, B):
    F = abs((A.c - B.c))
    G = (A.r + B.r)
    H = abs((A.r - B.r))
    if ((F > G) or (F < H)):
        return
    C = (F ** 2)
    I = (G ** 2)
    J = (H ** 2)
    K = (0.25 * math.sqrt(((I - C) * (C - J))))
    D = (((A.r ** 2) - (B.r ** 2)) / C)
    E = ((2 * K) / C)
    return (Point2(((0.5 * ((A.c.x + B.c.x) + ((B.c.x - A.c.x) * D))) + ((B.c.y - A.c.y) * E)), ((0.5 * ((A.c.y + B.c.y) + ((B.c.y - A.c.y) * D))) - ((B.c.x - A.c.x) * E))), Point2(((0.5 * ((A.c.x + B.c.x) + ((B.c.x - A.c.x) * D))) - ((B.c.y - A.c.y) * E)), ((0.5 * ((A.c.y + B.c.y) + ((B.c.y - A.c.y) * D))) + ((B.c.x - A.c.x) * E))))

def _connect_point2_line2(P, L):
    B = L.v.magnitude_squared()
    assert (B != 0)
    A = ((((P.x - L.p.x) * L.v.x) + ((P.y - L.p.y) * L.v.y)) / B)
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    return LineSegment2(P, Point2((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y))))

def _connect_point2_circle(P, C):
    A = (P - C.c)
    A.normalize()
    A *= C.r
    return LineSegment2(P, Point2((C.c.x + A.x), (C.c.y + A.y)))

def _connect_line2_line2(A, B):
    E = ((B.v.y * A.v.x) - (B.v.x * A.v.y))
    if (E == 0):
        if (isinstance(B, Ray2) or isinstance(B, LineSegment2)):
            (H, I) = _connect_point2_line2(B.p, A)
            return (I, H)
        return _connect_point2_line2(A.p, B)
    F = (A.p.y - B.p.y)
    G = (A.p.x - B.p.x)
    C = (((B.v.x * F) - (B.v.y * G)) / E)
    if (not A._u_in(C)):
        C = max(min(C, 1.0), 0.0)
    D = (((A.v.x * F) - (A.v.y * G)) / E)
    if (not B._u_in(D)):
        D = max(min(D, 1.0), 0.0)
    return LineSegment2(Point2((A.p.x + (C * A.v.x)), (A.p.y + (C * A.v.y))), Point2((B.p.x + (D * B.v.x)), (B.p.y + (D * B.v.y))))

def _connect_circle_line2(C, L):
    D = L.v.magnitude_squared()
    assert (D != 0)
    A = ((((C.c.x - L.p.x) * L.v.x) + ((C.c.y - L.p.y) * L.v.y)) / D)
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    E = Point2((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)))
    B = (E - C.c)
    B.normalize()
    B *= C.r
    return LineSegment2(Point2((C.c.x + B.x), (C.c.y + B.y)), E)

def _connect_circle_circle(A, B):
    C = (B.c - A.c)
    D = C.magnitude()
    if ((A.r >= B.r) and (D < A.r)):
        (E, F) = ((+ 1), (+ 1))
    elif ((B.r > A.r) and (D < B.r)):
        (E, F) = ((- 1), (- 1))
    elif ((D >= A.r) and (D >= B.r)):
        (E, F) = ((+ 1), (- 1))
    C.normalize()
    return LineSegment2(Point2((A.c.x + ((E * C.x) * A.r)), (A.c.y + ((E * C.y) * A.r))), Point2((B.c.x + ((F * C.x) * B.r)), (B.c.y + ((F * C.y) * B.r))))

class Point2(Vector2, Geometry):

    def __repr__(A):
        return ('Point2(%.2f, %.2f)' % (A.x, A.y))

    def intersect(A, other):
        return other._intersect_point2(A)

    def _intersect_circle(A, other):
        return _intersect_point2_circle(A, other)

    def connect(A, other):
        return other._connect_point2(A)

    def _connect_point2(A, other):
        return LineSegment2(other, A)

    def _connect_line2(B, other):
        A = _connect_point2_line2(B, other)
        if A:
            return A._swap()

    def _connect_circle(B, other):
        A = _connect_point2_circle(B, other)
        if A:
            return A._swap()

class Line2(Geometry):
    __slots__ = ['p', 'v']

    def __init__(B, *A):
        if (len(A) == 3):
            assert (isinstance(A[0], Point2) and isinstance(A[1], Vector2) and (type(A[2]) == float))
            B.p = A[0].copy()
            B.v = ((A[1] * A[2]) / abs(A[1]))
        elif (len(A) == 2):
            if (isinstance(A[0], Point2) and isinstance(A[1], Point2)):
                B.p = A[0].copy()
                B.v = (A[1] - A[0])
            elif (isinstance(A[0], Point2) and isinstance(A[1], Vector2)):
                B.p = A[0].copy()
                B.v = A[1].copy()
            else:
                raise AttributeError(('%r' % (A,)))
        elif (len(A) == 1):
            if isinstance(A[0], Line2):
                B.p = A[0].p.copy()
                B.v = A[0].v.copy()
            else:
                raise AttributeError(('%r' % (A,)))
        else:
            raise AttributeError(('%r' % (A,)))
        if (not B.v):
            raise AttributeError('Line has zero-length vector')

    def __copy__(A):
        return A.__class__(A.p, A.v)
    copy = __copy__

    def __repr__(A):
        return ('Line2(<%.2f, %.2f> + u<%.2f, %.2f>)' % (A.p.x, A.p.y, A.v.x, A.v.y))
    p1 = property((lambda self: self.p))
    p2 = property((lambda self: Point2((self.p.x + self.v.x), (self.p.y + self.v.y))))

    def _apply_transform(A, t):
        A.p = (t * A.p)
        A.v = (t * A.v)

    def _u_in(A, u):
        return True

    def intersect(A, other):
        return other._intersect_line2(A)

    def _intersect_line2(A, other):
        return _intersect_line2_line2(A, other)

    def _intersect_circle(A, other):
        return _intersect_line2_circle(A, other)

    def connect(A, other):
        return other._connect_line2(A)

    def _connect_point2(A, other):
        return _connect_point2_line2(other, A)

    def _connect_line2(A, other):
        return _connect_line2_line2(other, A)

    def _connect_circle(A, other):
        return _connect_circle_line2(other, A)

class Ray2(Line2):

    def __repr__(A):
        return ('Ray2(<%.2f, %.2f> + u<%.2f, %.2f>)' % (A.p.x, A.p.y, A.v.x, A.v.y))

    def _u_in(A, u):
        return (u >= 0.0)

class LineSegment2(Line2):

    def __repr__(A):
        return ('LineSegment2(<%.2f, %.2f> to <%.2f, %.2f>)' % (A.p.x, A.p.y, (A.p.x + A.v.x), (A.p.y + A.v.y)))

    def _u_in(A, u):
        return ((u >= 0.0) and (u <= 1.0))

    def __abs__(A):
        return abs(A.v)

    def magnitude_squared(A):
        return A.v.magnitude_squared()

    def _swap(A):
        A.p = A.p2
        A.v *= (- 1)
        return A
    length = property((lambda self: abs(self.v)))

class Circle(Geometry):
    __slots__ = ['c', 'r']

    def __init__(A, center, radius):
        C = radius
        B = center
        assert (isinstance(B, Vector2) and (type(C) == float))
        A.c = B.copy()
        A.r = C

    def __copy__(A):
        return A.__class__(A.c, A.r)
    copy = __copy__

    def __repr__(A):
        return ('Circle(<%.2f, %.2f>, radius=%.2f)' % (A.c.x, A.c.y, A.r))

    def _apply_transform(A, t):
        A.c = (t * A.c)

    def intersect(A, other):
        return other._intersect_circle(A)

    def _intersect_point2(A, other):
        return _intersect_point2_circle(other, A)

    def _intersect_line2(A, other):
        return _intersect_line2_circle(other, A)

    def _intersect_circle(A, other):
        return _intersect_circle_circle(other, A)

    def connect(A, other):
        return other._connect_circle(A)

    def _connect_point2(A, other):
        return _connect_point2_circle(other, A)

    def _connect_line2(B, other):
        A = _connect_circle_line2(B, other)
        if A:
            return A._swap()

    def _connect_circle(A, other):
        return _connect_circle_circle(other, A)

    def tangent_points(A, p):
        B = (0.5 * (A.c + p))
        return A.intersect(Circle(B, abs((p - B))))

def _connect_point3_line3(P, L):
    B = L.v.magnitude_squared()
    assert (B != 0)
    A = (((((P.x - L.p.x) * L.v.x) + ((P.y - L.p.y) * L.v.y)) + ((P.z - L.p.z) * L.v.z)) / B)
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    return LineSegment3(P, Point3((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)), (L.p.z + (A * L.v.z))))

def _connect_point3_sphere(P, S):
    A = (P - S.c)
    A.normalize()
    A *= S.r
    return LineSegment3(P, Point3((S.c.x + A.x), (S.c.y + A.y), (S.c.z + A.z)))

def _connect_point3_plane(p, plane):
    A = plane
    B = A.n.normalized()
    C = (p.dot(A.n) - A.k)
    return LineSegment3(p, Point3((p.x - (B.x * C)), (p.y - (B.y * C)), (p.z - (B.z * C))))

def _connect_line3_line3(A, B):
    assert (A.v and B.v)
    G = (A.p - B.p)
    H = G.dot(B.v)
    E = B.v.dot(A.v)
    J = G.dot(A.v)
    F = B.v.magnitude_squared()
    I = ((A.v.magnitude_squared() * F) - (E ** 2))
    if (I == 0):
        if (isinstance(B, Ray3) or isinstance(B, LineSegment3)):
            return _connect_point3_line3(B.p, A)._swap()
        return _connect_point3_line3(A.p, B)
    C = (((H * E) - (J * F)) / I)
    if (not A._u_in(C)):
        C = max(min(C, 1.0), 0.0)
    D = ((H + (E * C)) / F)
    if (not B._u_in(D)):
        D = max(min(D, 1.0), 0.0)
    return LineSegment3(Point3((A.p.x + (C * A.v.x)), (A.p.y + (C * A.v.y)), (A.p.z + (C * A.v.z))), Point3((B.p.x + (D * B.v.x)), (B.p.y + (D * B.v.y)), (B.p.z + (D * B.v.z))))

def _connect_line3_plane(L, P):
    B = P.n.dot(L.v)
    if (not B):
        return _connect_point3_plane(L.p, P)
    A = ((P.k - P.n.dot(L.p)) / B)
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
        return _connect_point3_plane(Point3((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)), (L.p.z + (A * L.v.z))), P)

def _connect_sphere_line3(S, L):
    C = L.v.magnitude_squared()
    assert (C != 0)
    A = (((((S.c.x - L.p.x) * L.v.x) + ((S.c.y - L.p.y) * L.v.y)) + ((S.c.z - L.p.z) * L.v.z)) / C)
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    D = Point3((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)), (L.p.z + (A * L.v.z)))
    B = (D - S.c)
    B.normalize()
    B *= S.r
    return LineSegment3(Point3((S.c.x + B.x), (S.c.y + B.y), (S.c.z + B.z)), D)

def _connect_sphere_sphere(A, B):
    C = (B.c - A.c)
    F = C.magnitude()
    if ((A.r >= B.r) and (F < A.r)):
        (D, E) = ((+ 1), (+ 1))
    elif ((B.r > A.r) and (F < B.r)):
        (D, E) = ((- 1), (- 1))
    elif ((F >= A.r) and (F >= B.r)):
        (D, E) = ((+ 1), (- 1))
    C.normalize()
    return LineSegment3(Point3((A.c.x + ((D * C.x) * A.r)), (A.c.y + ((D * C.y) * A.r)), (A.c.z + ((D * C.z) * A.r))), Point3((B.c.x + ((E * C.x) * B.r)), (B.c.y + ((E * C.y) * B.r)), (B.c.z + ((E * C.z) * B.r))))

def _connect_sphere_plane(S, P):
    B = _connect_point3_plane(S.c, P)
    if (not B):
        return
    C = B.p2
    A = (C - S.c)
    A.normalize()
    A *= S.r
    return LineSegment3(Point3((S.c.x + A.x), (S.c.y + A.y), (S.c.z + A.z)), C)

def _connect_plane_plane(A, B):
    if A.n.cross(B.n):
        return
    else:
        return _connect_point3_plane(A._get_point(), B)

def _intersect_point3_sphere(P, S):
    return (abs((P - S.c)) <= S.r)

def _intersect_line3_sphere(L, S):
    C = L.v.magnitude_squared()
    D = (2 * (((L.v.x * (L.p.x - S.c.x)) + (L.v.y * (L.p.y - S.c.y))) + (L.v.z * (L.p.z - S.c.z))))
    G = (((S.c.magnitude_squared() + L.p.magnitude_squared()) - (2 * S.c.dot(L.p))) - (S.r ** 2))
    E = ((D ** 2) - ((4 * C) * G))
    if (E < 0):
        return
    F = math.sqrt(E)
    A = (((- D) + F) / (2 * C))
    B = (((- D) - F) / (2 * C))
    if (not L._u_in(A)):
        A = max(min(A, 1.0), 0.0)
    if (not L._u_in(B)):
        B = max(min(B, 1.0), 0.0)
    return LineSegment3(Point3((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)), (L.p.z + (A * L.v.z))), Point3((L.p.x + (B * L.v.x)), (L.p.y + (B * L.v.y)), (L.p.z + (B * L.v.z))))

def _intersect_line3_plane(L, P):
    B = P.n.dot(L.v)
    if (not B):
        return
    A = ((P.k - P.n.dot(L.p)) / B)
    if (not L._u_in(A)):
        return
    return Point3((L.p.x + (A * L.v.x)), (L.p.y + (A * L.v.y)), (L.p.z + (A * L.v.z)))

def _intersect_plane_plane(A, B):
    G = A.n.magnitude_squared()
    H = B.n.magnitude_squared()
    C = A.n.dot(B.n)
    D = ((G * H) - (C ** 2))
    if (D == 0):
        return
    E = (((A.k * H) - (B.k * C)) / D)
    F = (((B.k * G) - (A.k * C)) / D)
    return Line3(Point3(((E * A.n.x) + (F * B.n.x)), ((E * A.n.y) + (F * B.n.y)), ((E * A.n.z) + (F * B.n.z))), A.n.cross(B.n))

class Point3(Vector3, Geometry):

    def __repr__(A):
        return ('Point3(%.2f, %.2f, %.2f)' % (A.x, A.y, A.z))

    def intersect(A, other):
        return other._intersect_point3(A)

    def _intersect_sphere(A, other):
        return _intersect_point3_sphere(A, other)

    def connect(A, other):
        return other._connect_point3(A)

    def _connect_point3(A, other):
        B = other
        if (A != B):
            return LineSegment3(B, A)

    def _connect_line3(B, other):
        A = _connect_point3_line3(B, other)
        if A:
            return A._swap()

    def _connect_sphere(B, other):
        A = _connect_point3_sphere(B, other)
        if A:
            return A._swap()

    def _connect_plane(B, other):
        A = _connect_point3_plane(B, other)
        if A:
            return A._swap()

class Line3():
    __slots__ = ['p', 'v']

    def __init__(B, *A):
        if (len(A) == 3):
            assert (isinstance(A[0], Point3) and isinstance(A[1], Vector3) and (type(A[2]) == float))
            B.p = A[0].copy()
            B.v = ((A[1] * A[2]) / abs(A[1]))
        elif (len(A) == 2):
            if (isinstance(A[0], Point3) and isinstance(A[1], Point3)):
                B.p = A[0].copy()
                B.v = (A[1] - A[0])
            elif (isinstance(A[0], Point3) and isinstance(A[1], Vector3)):
                B.p = A[0].copy()
                B.v = A[1].copy()
            else:
                raise AttributeError(('%r' % (A,)))
        elif (len(A) == 1):
            if isinstance(A[0], Line3):
                B.p = A[0].p.copy()
                B.v = A[0].v.copy()
            else:
                raise AttributeError(('%r' % (A,)))
        else:
            raise AttributeError(('%r' % (A,)))

    def __copy__(A):
        return A.__class__(A.p, A.v)
    copy = __copy__

    def __repr__(A):
        return ('Line3(<%.2f, %.2f, %.2f> + u<%.2f, %.2f, %.2f>)' % (A.p.x, A.p.y, A.p.z, A.v.x, A.v.y, A.v.z))
    p1 = property((lambda self: self.p))
    p2 = property((lambda self: Point3((self.p.x + self.v.x), (self.p.y + self.v.y), (self.p.z + self.v.z))))

    def _apply_transform(A, t):
        A.p = (t * A.p)
        A.v = (t * A.v)

    def _u_in(A, u):
        return True

    def intersect(A, other):
        return other._intersect_line3(A)

    def _intersect_sphere(A, other):
        return _intersect_line3_sphere(A, other)

    def _intersect_plane(A, other):
        return _intersect_line3_plane(A, other)

    def connect(A, other):
        return other._connect_line3(A)

    def _connect_point3(A, other):
        return _connect_point3_line3(other, A)

    def _connect_line3(A, other):
        return _connect_line3_line3(other, A)

    def _connect_sphere(A, other):
        return _connect_sphere_line3(other, A)

    def _connect_plane(B, other):
        A = _connect_line3_plane(B, other)
        if A:
            return A

class Ray3(Line3):

    def __repr__(A):
        return ('Ray3(<%.2f, %.2f, %.2f> + u<%.2f, %.2f, %.2f>)' % (A.p.x, A.p.y, A.p.z, A.v.x, A.v.y, A.v.z))

    def _u_in(A, u):
        return (u >= 0.0)

class LineSegment3(Line3):

    def __repr__(A):
        return ('LineSegment3(<%.2f, %.2f, %.2f> to <%.2f, %.2f, %.2f>)' % (A.p.x, A.p.y, A.p.z, (A.p.x + A.v.x), (A.p.y + A.v.y), (A.p.z + A.v.z)))

    def _u_in(A, u):
        return ((u >= 0.0) and (u <= 1.0))

    def __abs__(A):
        return abs(A.v)

    def magnitude_squared(A):
        return A.v.magnitude_squared()

    def _swap(A):
        A.p = A.p2
        A.v *= (- 1)
        return A
    length = property((lambda self: abs(self.v)))

class Sphere():
    __slots__ = ['c', 'r']

    def __init__(A, center, radius):
        C = radius
        B = center
        assert (isinstance(B, Vector3) and (type(C) == float))
        A.c = B.copy()
        A.r = C

    def __copy__(A):
        return A.__class__(A.c, A.r)
    copy = __copy__

    def __repr__(A):
        return ('Sphere(<%.2f, %.2f, %.2f>, radius=%.2f)' % (A.c.x, A.c.y, A.c.z, A.r))

    def _apply_transform(A, t):
        A.c = (t * A.c)

    def intersect(A, other):
        return other._intersect_sphere(A)

    def _intersect_point3(A, other):
        return _intersect_point3_sphere(other, A)

    def _intersect_line3(A, other):
        return _intersect_line3_sphere(other, A)

    def connect(A, other):
        return other._connect_sphere(A)

    def _connect_point3(A, other):
        return _connect_point3_sphere(other, A)

    def _connect_line3(B, other):
        A = _connect_sphere_line3(B, other)
        if A:
            return A._swap()

    def _connect_sphere(A, other):
        return _connect_sphere_sphere(other, A)

    def _connect_plane(B, other):
        A = _connect_sphere_plane(B, other)
        if A:
            return A

class Plane():
    __slots__ = ['n', 'k']

    def __init__(B, *A):
        if (len(A) == 3):
            assert (isinstance(A[0], Point3) and isinstance(A[1], Point3) and isinstance(A[2], Point3))
            B.n = (A[1] - A[0]).cross((A[2] - A[0]))
            B.n.normalize()
            B.k = B.n.dot(A[0])
        elif (len(A) == 2):
            if (isinstance(A[0], Point3) and isinstance(A[1], Vector3)):
                B.n = A[1].normalized()
                B.k = B.n.dot(A[0])
            elif (isinstance(A[0], Vector3) and (type(A[1]) == float)):
                B.n = A[0].normalized()
                B.k = A[1]
            else:
                raise AttributeError(('%r' % (A,)))
        else:
            raise AttributeError(('%r' % (A,)))
        if (not B.n):
            raise AttributeError('Points on plane are colinear')

    def __copy__(A):
        return A.__class__(A.n, A.k)
    copy = __copy__

    def __repr__(A):
        return ('Plane(<%.2f, %.2f, %.2f>.p = %.2f)' % (A.n.x, A.n.y, A.n.z, A.k))

    def _get_point(A):
        if A.n.z:
            return Point3(0.0, 0.0, (A.k / A.n.z))
        elif A.n.y:
            return Point3(0.0, (A.k / A.n.y), 0.0)
        else:
            return Point3((A.k / A.n.x), 0.0, 0.0)

    def _apply_transform(A, t):
        B = (t * A._get_point())
        A.n = (t * A.n)
        A.k = A.n.dot(B)

    def intersect(A, other):
        return other._intersect_plane(A)

    def _intersect_line3(A, other):
        return _intersect_line3_plane(other, A)

    def _intersect_plane(A, other):
        return _intersect_plane_plane(A, other)

    def connect(A, other):
        return other._connect_plane(A)

    def _connect_point3(A, other):
        return _connect_point3_plane(other, A)

    def _connect_line3(A, other):
        return _connect_line3_plane(other, A)

    def _connect_sphere(A, other):
        return _connect_sphere_plane(other, A)

    def _connect_plane(A, other):
        return _connect_plane_plane(other, A)
