from base64 import b64decode as license64

from maya import cmds
import BroTools.core.broqt.windows.config
from BroTools.core.debug import default_logger as log
from BroTools.configuration import MOD_MenuRoot, ANIMATION_TOOLS_MENU, COMMON_TOOLS_MENU, RIGGING_TOOLS_MENU, PIPELINE_TOOLS_MENU, MOD_Prefix, APP_DIR, EXTRAS_MENU, FILE_TOOLS_MENU
from functools import partial
import os
from BroTools import __version__
from BroTools.core.system import utilities as u
import BroTools.core.maya_utils as mu
mod_dir = APP_DIR

def cmds_menuItem(*A, **B):
    '\n    Debugging warpper\n    '
    try:
        cmds.menuItem(*A, **B)
    except Exception as C:
        log.warning('Failed running cmds.menuItem {}: {}'.format(A[0], str(C)))

def cmds_setParent(*A, **B):
    '\n    Debugging warpper\n    '
    try:
        cmds.setParent(*A, **B)
    except Exception as C:
        log.warning('Failed running cmds.setParent {}: {}'.format(A[0], str(C)))

def build():
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(divider=True, l=('BroTools v' + str(__version__)))
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(ANIMATION_TOOLS_MENU, l='Animation Tools', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(COMMON_TOOLS_MENU, l='Common Tools', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(RIGGING_TOOLS_MENU, l='Rigging Tools', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(PIPELINE_TOOLS_MENU, l='Pipeline Tools', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(FILE_TOOLS_MENU, l='File Tools', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem(EXTRAS_MENU, l='Extras', subMenu=True, to=True)
    cmds_setParent(MOD_MenuRoot, menu=True)
    log.debug('=== Main menu categories generated. === ')
    log.debug('Add core bottom menu entries', debug=True)
    import BroTools.core
    log.debug('import BroTools passed, set menu parent...', debug=True)
    cmds_setParent('{}MenuRoot'.format(MOD_Prefix), menu=True)
    log.debug('Build core menu items...', debug=True)
    cmds_menuItem('store_divider', divider=True)
    cmds_menuItem('documentation_menuItem', l='Documentation', c=partial(mu.open_web_url, '{}'.format(u.DOCUMENTATION_URL)), i=os.path.join(mod_dir, 'icons', 'i16_url.png'))
    cmds_menuItem('changelog_menuItem', l='Changelog', c=partial(mu.open_web_url, '{}/changelog'.format(u.API_MAIN_URL)), i=os.path.join(mod_dir, 'icons', 'i16_url.png'))
    cmds_menuItem('preferences_menuItem', l='Preferences', c=partial(BroTools.core.broqt.windows.config.showConfigWindow), i=os.path.join(mod_dir, 'icons', 'i16_preferences.png'))
    cmds_menuItem('update_menuItem', l='Check for updates', c=partial(u.show_updater), i=os.path.join(mod_dir, 'icons', 'i16_cloud.png'))
    cmds_menuItem('licenser_menuItem', l='License Manager', c=partial(u.Ox8r0_ZUp6TEZ0ZTI4UXlaeGFmMjl2ZzhzMVgxYWE5blNDMEdBRTZwQ0t3_ox07), i=os.path.join(mod_dir, 'icons', 'i16_key.png'))
    cmds_menuItem('store_menuItem', l='Purchase', c=partial(mu.open_web_url, '{}/store'.format(u.API_MAIN_URL)), i=os.path.join(mod_dir, 'icons', 'i16_store.png'))
    cmds_menuItem('brotools_community_menuItem', l='Community', subMenu=True, to=True, i=os.path.join(mod_dir, 'icons', 'i16_community.png'))
    cmds_setParent('brotools_community_menuItem', menu=True)
    cmds_menuItem('facebook_menuItem', l='Facebook', c=partial(mu.open_web_url, '{}/community/facebook'.format(u.API_MAIN_URL)), i=os.path.join(mod_dir, 'icons', 'i16_url.png'), p='brotools_community_menuItem')
    cmds_menuItem('discord_menuItem', l='Discord', c=partial(mu.open_web_url, '{}/community/discord'.format(u.API_MAIN_URL)), i=os.path.join(mod_dir, 'icons', 'i16_url.png'), p='brotools_community_menuItem')
    cmds_menuItem('reddit_menuItem', l='Reddit', c=partial(mu.open_web_url, '{}/community/reddit'.format(u.API_MAIN_URL)), i=os.path.join(mod_dir, 'icons', 'i16_url.png'), p='brotools_community_menuItem')
    cmds_setParent(MOD_MenuRoot, menu=True)
    cmds_menuItem('bugreport_menuItem', l='Report a Bug', c=partial(u.show_bugreport_window), i=os.path.join(mod_dir, 'icons', 'i16_bug.png'))
    cmds_menuItem('restorebackup_menuItem', l='Restore backup', c=partial(u.restore_backup), i=os.path.join(mod_dir, 'icons', 'i16_bug.png'))
    cmds_menuItem('about_menuItem', l='About', c=partial(u.show_about_window), i=os.path.join(mod_dir, 'icons', 'i16_info.png'))
    cmds_menuItem('debugging_header', l='Debugging', divider=True, boldFont=True)
    cmds_menuItem('reload_menuItem', l='Reload BroTools', c=partial(u.self_reload), i=os.path.join(mod_dir, 'icons', 'i16_reload.png'))
    cmds_menuItem('unload_menuItem', l='Unload BroTools', c=partial(u.self_unload), i=os.path.join(mod_dir, 'icons', 'i16_reload.png'))
    if u.WelcomeBackWindow.revert_available():
        cmds_menuItem('downgrade_menuItem', l='Downgrade to BroDynamics 2.x', c=partial(u.WelcomeBackWindow.revert_version), i=os.path.join(mod_dir, 'icons', 'i16_reload.png'))
    log.debug('Add universal bottom menu entries - done.', debug=True)
