from base64 import b64decode as license64

__version__ = '2024.06.14'
print('# BroTools utilities library version: ', __version__)
import os, ssl, random, math, nlib64
API_MAIN_URL = os.environ.get('BROTOOLS_API_MAIN_URL_ROOT', 'https://api.brotools.tech')
API_IGNORE_SSL = os.environ.get('BROTOOLS_API_IGNORE_SSL', False)
FLOATING_DONT_LEASE_ON_START = os.environ.get('BROTOOLS_FLOATING_DONT_LEASE_ON_START', False)
if (API_IGNORE_SSL is not False):
    print('BROTOOLS: IGNORE SSL ENABLED! Use at your own risk. This might be a security concern. Please, consider if you can find another solution to SSL problem. ')
    ssl_context = ssl.create_default_context()
    ssl_context.check_hostname = False
    ssl_context.verify_mode = ssl.CERT_NONE
else:
    ssl_context = None
DOCUMENTATION_URL = 'https://docs.brotools.tech/'
URL_CLIENT_DASHBOARD = 'https://brotools.tech/dashboard'
PING_FAIL_MESSAGE = "Can't reach server.\n\nThis could be due to server maintenance or some problems in your network. Please, check your internet access and try again in 10 minutes.\nIf you keep experiencing this problem, contact me at support@brotools.tech.\n\nSorry for the inconvenience!\n"
'\nUsing `partial` with .connect is required for it to work with Maya 2022 and Python 3 when compiling with Cython.\nIt seems to be either Cython or PySide2 specific bug.\n'
Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_VARNAME = 'BROTOOLS_FLOATING'

def decode(*args, **kwargs):
    try:
        from maya import cmds
        import sys
        cmds.error('Attempting to import decode before it was initialized by {}!'.format(sys._getframe(1).f_globals.get('__file__', '<undefined file>')))
    except:
        print('Attempting to import decode before it was initialized!')

def gethostname(*args, **kwargs):
    return socket.gethostname().encode('utf-8').decode('utf-8')
import hashlib, json, math, ntpath, re, random, shutil, socket, time, traceback, webbrowser, base64, os, platform, sys
sys.setrecursionlimit(3000)
import uuid, zlib, zipfile, struct, threading
from subprocess import check_output, CalledProcessError
from datetime import datetime, timedelta
from functools import partial, wraps

def generic_function_wrap(fn):
    '\n    Simple wrapper around a function adding args and kwargs, and debug logging\n    Args:\n        fn: function to wrap\n\n    Returns:\n        wrapped: wrapped function\n    '

    @wraps(fn)
    def wrapped(*args, **kwargs):
        log.debug('Running function: {}'.format(fn.__name__))
        result = fn(*args, **kwargs)
        log.debug('Function: {}; Returns: {}'.format(fn.__name__, str(result)))
        return result
    return wrapped
import BroTools
from BroTools.core.debug import default_logger as log
from BroTools.core.debug import log_path
from BroTools.core.errors import show_error, catch_errors, BroToolsError
from BroTools.configuration import icon, bro_qss, APP_DIR, img, brotools_config, brotools_write_path, PY3, backups_dir
from BroTools.core import callbacks

class DataCache():

    def __init__(self, fp=os.path.join(brotools_write_path, 'cache.json')):
        self.fp = fp
        if (not os.path.exists(self.fp)):
            self.write({})

    def write(self, data):
        with open(self.fp, 'w+') as f:
            return f.write(json.dumps(data, indent=4, sort_keys=True))

    def read(self):
        with open(self.fp, 'r') as f:
            return json.loads(f.read())

    def get(self, key, default=None):
        return self.read().get(key, default)

    def set(self, key, value):
        data = self.read()
        data[key] = value
        self.write(data)

    def remove(self, key):
        data = self.read()
        if (key in data):
            data.pop(key)
        self.write(data)

    def key_exists(self, key):
        data = self.read()
        return (key in data)
    pop = remove
if PY3:
    import urllib.request, urllib.parse, urllib.error, urllib.request, urllib.error, urllib.parse
    from urllib.parse import urlparse
    Request = urllib.request.Request
    urlopen = urllib.request.urlopen
    URLError = urllib.error.URLError
    HTTPError = urllib.error.HTTPError
    import builtins
    builtins.unicode = str
else:
    import urllib, urllib2
    from urlparse import urlparse
    Request = urllib2.Request
    urlopen = urllib2.urlopen
    URLError = urllib2.URLError
    HTTPError = urllib2.HTTPError
if ((Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_VARNAME in os.environ) and (os.environ[Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_VARNAME] != 'None')):
    Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 = True
    Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_FROM_UI = False
    log.message('Floating license server mode.')
elif (DataCache().get('turbofloat_server') is not None):
    Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 = True
    Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_FROM_UI = True
    log.message('Floating license server mode.')
else:
    Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 = False
    log.message('Regular license mode.')
from BroTools.core.broqt.vendor.Qt import QtCore, QtWidgets, QtGui
from BroTools.core.broqt.bro_windows import BroSimpleWindow
from BroTools.core.broqt.bro_widgets import BroTitle, BroText, BroEntry, QHLine
latest_version = BroTools.__version__
if os.path.exists(os.path.join(APP_DIR, 'dev.lock')):
    DEV_MODE = True
else:
    DEV_MODE = False
if DEV_MODE:
    print('( UTILITIES )\n888888b.   8888888b.   .d88888b. 88888888888 .d88888b.   .d88888b.  888      .d8888b.  \n888  "88b  888   Y88b d88P" "Y88b    888    d88P" "Y88b d88P" "Y88b 888     d88P  Y88b \n888  .88P  888    888 888     888    888    888     888 888     888 888     Y88b.      \n8888888K.  888   d88P 888     888    888    888     888 888     888 888      "Y888b.   \n888  "Y88b 8888888P"  888     888    888    888     888 888     888 888         "Y88b. \n888    888 888 T88b   888     888    888    888     888 888     888 888           "888 \n888   d88P 888  T88b  Y88b. .d88P    888    Y88b. .d88P Y88b. .d88P 888     Y88b  d88P \n8888888P"  888   T88b  "Y88888P"     888     "Y88888P"   "Y88888P"  88888888 "Y8888P"  \n                                                                                       \n                                                                                       \n                                                                                       \n8888888b.  8888888888 888     888      888b     d888  .d88888b.  8888888b.  8888888888 \n888  "Y88b 888        888     888      8888b   d8888 d88P" "Y88b 888  "Y88b 888        \n888    888 888        888     888      88888b.d88888 888     888 888    888 888        \n888    888 8888888    Y88b   d88P      888Y88888P888 888     888 888    888 8888888    \n888    888 888         Y88b d88P       888 Y888P 888 888     888 888    888 888        \n888    888 888          Y88o88P        888  Y8P  888 888     888 888    888 888        \n888  .d88P 888           Y888P         888   "   888 Y88b. .d88P 888  .d88P 888        \n8888888P"  8888888888     Y8P          888       888  "Y88888P"  8888888P"  8888888888 \n    ')
'\nNOTES\n\n# try:\n#     if SYSTEM == "Windows":\n#         uuid = str(check_output(\'wmic csproduct get uuid\', shell=True)).strip()\n#     elif SYSTEM == "Darwin":\n#         uuid = str(check_output(\'cat /var/lib/dbus/machine-id\', shell=True)).strip()\n#     elif SYSTEM == "Linux":\n#         uuid = str(check_output("ioreg -rd1 -c IOPlatformExpertDevice | grep -E \'(UUID)\'", shell=True)).split(\'"\')[-2].strip() # for me i got it on list value -2 if having any trouble try getting it with any alternative list element.\n#     uuid = uuid.replace("\n", "").replace("UUID", "").replace("uuid", "").strip()\n\n\n#     return uuid\n\n# except:\n'

def cleanup_deleteme_files():
    for (root, dirs, files) in os.walk(APP_DIR, topdown=False):
        for name in files:
            if ('_deleteme_' in name):
                filepath = os.path.join(root, name)
                log.info('Cleanup found old version file, removing: {}'.format(filepath))
                try:
                    os.remove(filepath)
                except Exception as e:
                    log.warning('Failed cleaning up file: {}; {}'.format(filepath, e))
cleanup_deleteme_files()
if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
    import sys
    from os import path as ospath
    from ctypes import cdll, c_uint, c_uint32, c_char_p, c_wchar_p, create_string_buffer, create_unicode_buffer, CFUNCTYPE
    is_win = (sys.platform == 'win32')
    wbuf = (create_unicode_buffer if is_win else create_string_buffer)
    wstr_type = (c_wchar_p if is_win else c_char_p)

    class wstr(wstr_type):

        def __init__(self, string):
            if ((sys.version_info > (3, 0)) and isinstance(string, str)):
                super(wstr, self).__init__((string.encode('utf-8') if (not is_win) else string))
            else:
                super(wstr, self).__init__(string)
    TF_OK = 0
    TF_FAIL = 1
    TF_E_SERVER = 2
    TF_E_NO_CALLBACK = 3
    TF_E_INET = 4
    TF_E_NO_FREE_LEASES = 5
    TF_E_LEASE_EXISTS = 6
    TF_E_WRONG_TIME = 7
    TF_E_PDETS = 8
    TF_E_INVALID_HANDLE = 9
    TF_E_NO_LEASE = 10
    TF_E_COM = 11
    TF_E_INSUFFICIENT_BUFFER = 12
    TF_E_PERMISSION = 13
    TF_E_INVALID_FLAGS = 14
    TF_E_WRONG_SERVER_PRODUCT = 15
    TF_E_INET_TIMEOUT = 16
    TF_E_UPGRADE_LIBRARY = 17
    TF_E_USERNAME_NOT_ALLOWED = 18
    TF_E_BAD_HOST_ADDRESS = 19
    TF_E_CLIENT_IPC = 20
    TF_E_SERVER_UUID_MISMATCH = 27
    TF_E_ENABLE_NETWORK_ADAPTERS = 28
    TF_E_BROKEN_WMI = 34
    TF_E_INET_TLS = 36
    TF_SYSTEM = 1
    TF_USER = 2
    TF_REQUEST_OVER_HTTPS = 4
    TF_HAS_NOT_EXPIRED = 1
    "\n    Called when the lease has expired and couldn't be renewed. You\n    should disable your app immediately when this return code is called.\n    Of course give your user a way to save the current state of their\n    data and/or request a lease renewal from the server.\n\n    In other words, don't make the user mad. Make sure you test this\n    extensively with real end-users so you get the best behavior.\n    "
    TF_CB_EXPIRED = 0
    "\n    Called when the lease has expired and couldn't be renewed due to\n    failure to connect to the TurboFloat Server.\n    "
    TF_CB_EXPIRED_INET = 1
    '\n    Called when the lease was renewed and some or all of the custom\n    license fields have since changed. If you use the TF_GetFeatureValue()\n    function then it behooves you to get the latest changed feature\n    values upon this callback status.\n    '
    TF_CB_FEATURES_CHANGED = 2
    "\n    Called when the lease has been dropped by the TurboFloat Server. You\n    should disable your app immediately when this return code is called.\n    Of course give your user a way to save the current state of their\n    data and/or request a lease renewal from the server.\n\n    In other words, don't make the user mad. Make sure you test this\n    extensively with real end-users so you get the best behavior.\n    "
    TF_CB_LEASE_DROPPED = 3
    "\n    Called when the lease has been dropped because the system has entered\n    sleep or hibernation mode. You should disable your app immediately when\n    this return code is called. Of course give your user a way to save the\n    current state of their data and/or request a lease renewal from the server.\n\n    In other words, don't make the user mad. Make sure you test this\n    extensively with real end-users so you get the best behavior.\n\n    When / if the system is resumed, TurboFloat will automatically attempt\n    to request the lease again. If it's successful the LeaseCallback\n    will be TF_CB_LEASE_REGAINED.\n    "
    TF_CB_LEASE_DROPPED_SLEEP = 4
    '\n    Called when the lease has been regained after having been\n    dropped from TF_CB_LEASE_DROPPED_SLEEP.\n\n    If the lease request is attempted and fails, this will\n    never be called.\n\n    The best real-world usage of this is when the lease\n    callback function drops the lease (or the lease expires)\n    show a user-prompt for the customer (whether via a dialog\n    box or other interactive method) and if this "TF_CB_LEASE_REGAINED"\n    is called, then hide that re-attempt dialog and re-enable your\n    '
    TF_CB_LEASE_REGAINED = 5
    LeaseCallback = CFUNCTYPE(None, c_uint32)

    def load_library(path):
        if ((sys.platform == 'win32') or (sys.platform == 'cygwin')):
            lib_path = os.path.join(path, 'TurboFloat.dll')
            log.message('Loading TurboFloat library: {}'.format(lib_path))
            return cdll.LoadLibrary(lib_path)
        elif (sys.platform == 'darwin'):
            lib_path = os.path.join(path, 'TurboFloat.dylib')
            log.message('Loading TurboFloat library: {}'.format(lib_path))
            return cdll.LoadLibrary(lib_path)
        return cdll.LoadLibrary(ospath.join(path, 'libTurboFloat.so'))

    def validate_result(return_code):
        log.debug('validate_result...')
        if (return_code == TF_OK):
            return
        if (return_code == TF_FAIL):
            raise TurboFloatFailError()
        elif (return_code == TF_E_SERVER):
            raise TurboFloatServerError()
        elif (return_code == TF_E_NO_CALLBACK):
            raise TurboFloatNoCallbackError()
        elif (return_code == TF_E_NO_FREE_LEASES):
            raise TurboFloatNoFreeLeasesError()
        elif (return_code == TF_E_LEASE_EXISTS):
            raise TurboFloatLeaseExistsError()
        elif (return_code == TF_E_WRONG_TIME):
            raise TurboFloatWrongTimeError()
        elif (return_code == TF_E_NO_LEASE):
            raise TurboFloatNoLeaseError()
        elif (return_code == TF_E_PDETS):
            raise TurboFloatDatFileError()
        elif (return_code == TF_E_INVALID_FLAGS):
            raise TurboFloatFlagsError()
        elif (return_code == TF_E_WRONG_SERVER_PRODUCT):
            raise TurboFloatWrongServerProductError()
        elif (return_code == TF_E_UPGRADE_LIBRARY):
            raise TurboFloatUpgradeLibraryError()
        elif (return_code == TF_E_USERNAME_NOT_ALLOWED):
            raise TurboFloatUsernameNotAllowedError()
        elif (return_code == TF_E_BAD_HOST_ADDRESS):
            raise TurboFloatBadHostAddressError()
        elif (return_code == TF_E_CLIENT_IPC):
            raise TurboFloatClientIPCError()
        elif (return_code == TF_E_SERVER_UUID_MISMATCH):
            raise TurboFloatServerUUIDMismatchError()
        elif (return_code == TF_E_COM):
            raise TurboFloatComError()
        elif (return_code == TF_E_INET):
            raise TurboFloatInetError()
        elif (return_code == TF_E_PERMISSION):
            raise TurboFloatPermissionError()
        elif (return_code == TF_E_INVALID_HANDLE):
            raise TurboFloatInvalidHandleError()
        elif (return_code == TF_E_ENABLE_NETWORK_ADAPTERS):
            raise TurboFloatEnableNetworkAdaptersError()
        elif (return_code == TF_E_BROKEN_WMI):
            raise TurboFloatBrokenWMIError()
        elif (return_code == TF_E_INET_TIMEOUT):
            raise TurboFloatInetTimeoutError()
        elif (return_code == TF_E_INET_TLS):
            raise TurboFloatInetTLSError()
        raise TurboFloatError(return_code)

    class TurboFloatError(Exception):
        'Generic TurboFloat error'
        pass

    class TurboFloatFailError(TurboFloatError):
        'Fail error'
        pass

    class TurboFloatInetError(TurboFloatError):
        'Connection to the server failed.'
        pass

    class TurboFloatInetTimeoutError(TurboFloatInetError):
        'The connection to the server timed out because a long period of time\n        elapsed since the last data was sent or received.'
        pass

    class TurboFloatInetTLSError(TurboFloatInetError):
        'The secure connection to the activation servers failed due to a TLS or\n        certificate error. More information here: https://wyday.com/limelm/help/faq/#internet-error'
        pass

    class TurboFloatServerError(TurboFloatError):
        "\n        There's no server specified. You must call tf.save_server() at least once to save the server.\n        "
        pass

    class TurboFloatNoCallbackError(TurboFloatError):
        "\n        You didn't specify a callback. Do so using the TF_SetLeaseCallback() or TF_SetLeaseCallbackEx() function.\n        "
        pass

    class TurboFloatNoFreeLeasesError(TurboFloatError):
        '\n        There are no more free leases available from the TurboFloat server. Either\n        increase the number of allowed floating licenses for the TurboFloat server\n        or wait for one of the other leases to expire.\n        '
        pass

    class TurboFloatLeaseExistsError(TurboFloatError):
        '\n        The lease has already been acquired. TurboFloat automatically renews the lease\n        when it needs to based on the information the TurboFloat Server provides.\n        '
        pass

    class TurboFloatWrongTimeError(TurboFloatError):
        '\n        This computer\'s system time is more than 5 minutes (before/after)\n        different from the TurboFloat Server\'s system time. Make sure\n        the server\'s Date, Time, and Timezone are set correctly and make\n        sure this computer\'s Date, Time, and Timezone are set correctly.\n\n        Note: That TurboFloat and TurboFloat Server work even if the timezones\n            of the computer/server are different. In other words, your\n            TurboFloat Server could be hosted in France and have a "client"\n            computer running in New York (a 6 hour time difference) and\n            everything will work fine provided both server and "client"\n            have their date, time, and timezones correctly configured.\n        '
        pass

    class TurboFloatComError(TurboFloatError):
        '\n        The hardware id couldn\'t be generated due to an error in the COM setup.\n        Re-enable Windows Management Instrumentation (WMI) in your group policy\n        editor or reset the local group policy to the default values. Contact\n        your system admin for more information.\n\n        This error is Windows only.\n\n        This error can also be caused by the user (or another program) disabling\n        the "Windows Management Instrumentation" service. Make sure the "Startup type"\n        is set to Automatic and then start the service.\n\n\n        To further debug WMI problems open the "Computer Management" (compmgmt.msc),\n        expand the "Services and Applications", right click "WMI Control" click\n        "Properties" and view the status of the WMI.\n        '
        pass

    class TurboFloatPermissionError(TurboFloatError):
        '\n        Insufficient system permission. Either start your process as an\n        admin / elevated user or call the function again with the\n        TF_USER flag instead of the TF_SYSTEM flag.\n        '
        pass

    class TurboFloatDatFileError(TurboFloatError):
        'The product details file "TurboActivate.dat" failed to load.'
        pass

    class TurboFloatFlagsError(TurboFloatError):
        '\n        The flags you passed to is_date_valid(...) or save_server were invalid (or missing).\n        '
        pass

    class TurboFloatInvalidHandleError(TurboFloatError):
        '\n        The handle is not valid. To get a handle use TF_GetHandle().\n        '
        pass

    class TurboFloatNoLeaseError(TurboFloatError):
        "\n        There's no lease. Your must first have a lease before you can\n        drop it or get information on it.\n        "
        pass

    class TurboFloatWrongServerProductError(TurboFloatError):
        "\n        The TurboFloat Server you're trying to contact can't give license\n        leases for this product version.\n        "
        pass

    class TurboFloatUpgradeLibraryError(TurboFloatError):
        "\n        The TurboFloat Library you're currently using cannot be used\n        to communicate with the TurboFloat Server instance. Release a new\n        version of your app with the latest version of the TurboFloat Library.\n\n        Get it here: https://wyday.com/limelm/api/#turbofloat\n        "
        pass

    class TurboFloatUsernameNotAllowedError(TurboFloatError):
        '\n        The current user cannot request a license lease from the server because\n        their username has not been added to the whitelist of approved usernames.\n\n        More information about per-seat licensing: https://wyday.com/limelm/help/licensing-types/#named-user\n        '
        pass

    class TurboFloatBadHostAddressError(TurboFloatError):
        '\n        The host_address value you passed to SaveServer(...) was invalid.\n        The value should not contain a leading protocol prefix (i.e., "http://")\n        or a trailing port number (i.e., ":443").\n        '
        pass

    class TurboFloatClientIPCError(TurboFloatError):
        '\n        An error occurred while the TurboFloat client library was trying\n        use interprocess communication facilities. These are needed to\n        coordinate between multiple client-program instances running in\n        the same operating-system session.\n        '
        pass

    class TurboFloatServerUUIDMismatchError(TurboFloatError):
        "\n        The host_address value passed to SaveServer(...) contained an invalid\n        'server' parameter. (The 'server' parameter is used with the hosted\n        version of the TurboFloatServer.)\n        "
        pass

    class TurboFloatEnableNetworkAdaptersError(TurboFloatError):
        '\n        There are network adapters on the system that are disabled and\n        TurboFloat couldn\'t read their hardware properties (even after trying\n        and failing to enable the adapters automatically). Enable the network adapters,\n        re-run the function, and TurboFloat will be able to "remember" the adapters\n        even if the adapters are disabled in the future.\n\n        Note:   The network adapters do not need an active Internet connections. They just\n                need to not be disabled. Whether they are or are not connected to the\n                internet/intranet is not important and does not affect this error code at all.\n\n\n        On Linux you\'ll get this error if you don\'t have any real network adapters attached.\n        For example if you have no "eth[x]", "wlan[x]", "en[x]", "wl[x]", "ww[x]", or "sl[x]"\n        network interface devices.\n\n        See: https://wyday.com/limelm/help/faq/#disabled-adapters\n        '
        pass

    class TurboFloatBrokenWMIError(TurboFloatError):
        '\n        The WMI repository on the computer is broken. To fix the WMI repository\n        see the instructions here:\n\n        https://wyday.com/limelm/help/faq/#fix-broken-wmi\n        '
        pass
    from ctypes import pointer, sizeof, c_uint32, c_ushort
    from BroTools.configuration import APP_DIR
    import os, sys

    class TurboFloat(object):

        def __init__(self, guid, callback, dat_file_loc='', library_folder=''):
            system_folder = os.path.join(APP_DIR, 'core', 'system')
            if (not library_folder):
                library_folder = os.path.join(system_folder)
            if (not dat_file_loc):
                dat_file_loc = os.path.join(system_folder, 'TurboActivate.dat')
            if (not os.path.exists(dat_file_loc)):
                show_error(text='Your system is set up to use Floating licensing, but TurboActivate.dat file is missing.\nPlease, put the appropriate TurboActivate.dat* file for your bundle into the root of BroTools folder.\nIt should be located at: {}\n\n* Different BroTools bundles, for example Complete Edition or BroDynamics, have their own TurboActivate.dat files.\n                    '.format(dat_file_loc), title='Could not find TurboActivate.dat file')
                raise BroToolsError('Could not find TurboActivate.dat file')
            else:
                log.message('TurboFloat.dat found at: {}'.format(dat_file_loc))
            self._lib = load_library(library_folder)
            log.message('Library loaded!')
            self._set_restype()
            log.message('self._set_restype() success.')
            try:
                self._lib.TF_PDetsFromPath(wstr(dat_file_loc))
            except TurboFloatFailError:
                log.message('The dat file is already loaded')
                pass
            self._handle = self._lib.TF_GetHandle(wstr(guid))
            if (self._handle == 0):
                raise TurboFloatDatFileError()
            self._callback = LeaseCallback(callback)
            self._lib.TF_SetLeaseCallback(self._handle, self._callback)

        def save_server(self, host_address, port, flags):
            '\n            Saves the TurboFloat server location on the disk. You must call this function at\n            least once before requesting a lease from the server. A good place to call this function\n            is from your installer (so an IT admin could set the server location once).\n\n            Using the "port" value of 443 is equivalent to passing in the "TF_REQUEST_OVER_HTTPS"\n            flag. If you want to contact a TFS instance over HTTPS and not use port 443\n            then you *must* pass in the "TF_REQUEST_OVER_HTTPS" flag. You cannot use "raw"\n            (non-HTTPS) communication on port 443.\n\n\n            Note: If you pass in the TF_SYSTEM flag and you don\'t have "admin" or "elevated"\n            permission then the call will fail.\n\n            If you call this function once in the past with the flag TF_SYSTEM and the calling\n            process was an admin process then subsequent calls with the TF_SYSTEM flag will\n            succeed even if the calling process is *not* admin/elevated.\n\n            If you want to take advantage of this behavior from an admin process\n            (e.g. an installer) but the user hasn\'t entered a product key then you can\n            call this function with a null string:\n\n                        TF_SaveServer(YourHandle, 0, 0, TF_SYSTEM);\n\n            This will set everything up so that subsequent calls with the TF_SYSTEM flag will\n            succeed even if from non-admin processes.\n            '
            self._lib.TF_SaveServer(self._handle, wstr(host_address), c_ushort(port), flags)

        def get_server(self):
            '\n            Gets the stored TurboFloat Server location.\n            '
            buf_size = self._lib.TF_GetServer(self._handle, 0, 0, 0)
            buf = wbuf(buf_size)
            port = c_ushort(0)
            self._lib.TF_GetServer(self._handle, buf, buf_size, pointer(port))
            return (buf.value, port.value)

        def request_lease(self):
            '\n            Requests a floating license lease from the TurboFloat Server. You should run\n            this at the top of your app after calling TF_SetLeaseCallback().\n            '
            log.debug('request_lease...')
            self._lib.TF_RequestLease(self._handle)
            log.debug('request lease complete')

        def drop_lease(self):
            '\n            Drops the active lease from the TurboFloat Server. This frees up the lease\n            so it can be used by another instance of your app.\n\n            We recommend calling this before your application exits.\n\n            Note: This function does *not* call the lease callback function. If you want that\n                    behavior then you can do it like this:\n\n\n                    if (TF_DropLease(tfHandle) == TF_OK)\n                    {\n                        YourLeaseCallbackFunction(TF_CB_EXPIRED);\n                    }\n            '
            self._lib.TF_DropLease(self._handle)

        def has_lease(self):
            "\n            Lets you know whether there's an active lease for the handle specified. This function\n            isn't necessary if you're tracking the responses from TF_RequestLease(), TF_DropLease(),\n            and the callback function that you set in TF_SetLeaseCallback().\n            "
            ret = self._lib.TF_HasLease(self._handle)
            if (ret == TF_OK):
                return True
            elif (ret == TF_FAIL):
                return False
            validate_result(ret)

        def has_feature(self, name):
            return (len(self.get_feature_value(name)) > 0)

        def get_feature_value(self, name):
            'Gets the value of a feature.'
            buf_size = self._lib.TF_GetFeatureValue(self._handle, wstr(name), 0, 0)
            buf = wbuf(buf_size)
            self._lib.TF_GetFeatureValue(self._handle, wstr(name), buf, buf_size)
            return buf.value

        def is_date_valid(self, date):
            '\n            Check if the date is valid\n            '
            try:
                self._lib.TF_IsDateValid(self._handle, wstr(date), TF_HAS_NOT_EXPIRED)
                return True
            except TurboFloatFlagsError as e:
                raise e
            except TurboFloatError:
                return False

        def set_custom_proxy(self, address):
            '\n            Sets the custom proxy to be used when "https" protocol is enabled (when the\n            TF_REQUEST_OVER_HTTPS flag is given to the TF_SaveServer() function).\n\n            Proxy address in the form: http://username:password@host:port/\n\n            Example 1 (just a host): http://127.0.0.1/\n            Example 2 (host and port): http://127.0.0.1:8080/\n            Example 3 (all 3): http://user:pass@127.0.0.1:8080/\n\n            If the port is not specified, TurboFloat will default to using port 1080 for proxies.\n            '
            self._lib.TF_SetCustomProxy(wstr(address))

        def cleanup(self):
            '\n            You should call this before your application exits. This frees up any\n            allocated memory for all open handles. If you have an active license\n            lease then you should call tf.DropLease() before you call TurboFloat.Cleanup().\n            '
            self._lib.TF_Cleanup()

        def get_version(self):
            "\n            Gets the version number of the currently used TurboFloat library.\n            This is a useful alternative for platforms which don't support file meta-data\n            (like Linux, FreeBSD, and other unix variants).\n\n            The version format is:  Major.Minor.Build.Revision\n            "
            major = c_uint32(0)
            minor = c_uint32(0)
            build = c_uint32(0)
            rev = c_uint32(0)
            self._lib.TF_GetVersion(pointer(major), pointer(minor), pointer(build), pointer(rev))
            return (major.value, minor.value, build.value, rev.value)

        def _set_restype(self):
            self._lib.TF_PDetsFromPath.restype = validate_result
            self._lib.TF_SetLeaseCallback.restype = validate_result
            self._lib.TF_SaveServer.restype = validate_result
            self._lib.TF_RequestLease.restype = validate_result
            self._lib.TF_DropLease.restype = validate_result
            self._lib.TF_IsDateValid.restype = validate_result
            self._lib.TF_SetCustomProxy.restype = validate_result
            self._lib.TF_Cleanup.restype = validate_result
______________UTILITIES______________ = None
SYSTEM = platform.system()
if (SYSTEM not in ['Windows', 'Linux', 'Darwin']):
    cmds.error('BroTools does not support OS: {}'.format(SYSTEM))
VERBOSE_LOG = False
MOD_NAME = 'BroTools'
from maya import cmds, mel

class brokwargs():

    def __init__(self, labelLong, labelShort, defautValue, kwargDic):
        if (kwargDic.get(labelLong) and kwargDic.get(labelShort)):
            log.warning('Same flag used twice.')
        else:
            self.userInput = (kwargDic.get(labelLong) or kwargDic.get(labelShort) or defautValue)

    def get(self):
        return self.userInput
from BroTools.core.errors import catch_errors, catch_errors_native

def file_hash(filepath):
    hash_obj = hashlib.sha1(open(filepath, 'rb').read())
    checksum = hash_obj.hexdigest()
    return checksum

def semver_compare(current_version, new_version):
    '\n    Returns True if new version is higher than current.\n    '
    current_int = int(current_version.replace('.', ''))
    new_int = int(new_version.replace('.', ''))
    return (new_int > current_int)

def self_reload(*args, **kwargs):
    mel.eval('print "Reloading BroTools...";  evalDeferred ("unloadPlugin \\"BroTools.py\\""); evalDeferred ("loadPlugin \\"BroTools.py\\"");')

def self_unload(*args, **kwargs):
    mel.eval('print "Unloading BroTools..."; unloadPlugin "BroTools.py";')
______________MATH______________ = None

def lerp(a, b, f):
    return (a + (f * (b - a)))

def lerp_color(a, b, f):
    return (lerp(a[0], b[0], f), lerp(a[1], b[1], f), lerp(a[2], b[2], f))

def encode_base(string, key=None):
    return nlib64.walk.c20e(string, key)

def decode_base(string, key=None):
    return nlib64.walk.c20e(string, (nlib64.paths.path() if (key is None) else key))

def base64encode(data):
    if (not isinstance(data, bytes)):
        data = data.encode('utf-8')
    data = base64.urlsafe_b64encode(data).rstrip(b'=')
    if isinstance(data, bytes):
        return data.decode('utf-8')
    else:
        return data

def base64decode(string):
    padding = (4 - (len(string) % 4))
    return base64.urlsafe_b64decode((string + ('=' * padding)))

def encode(string, key=None):
    if (key is None):
        key = nlib64.paths.path()
    encoded_string = string
    encoded_string = encode_base(string, (nlib64.paths.path() if (key is None) else key))
    encoded_string = zlib.compress(encoded_string)
    encoded_string = base64encode(encoded_string)
    return encoded_string

def _decode(string, key=None):
    padding = (4 - (len(string) % 4))
    decoded_string = base64decode(string)
    decoded_string = zlib.decompress(decoded_string)
    decoded_string = decode_base(decoded_string, (nlib64.paths.path() if (key is None) else key))
    if (decoded_string == string):
        log.warning('Decoding failed for: {string}'.format())
        return decoded_string
    return decoded_string

def decode(string, key=None):
    decoded_string = _decode(string, (nlib64.paths.path() if (key is None) else key))
    return decoded_string

def smart_pyopen(fp):
    "DEPRECATED\n    Open python file, automatically detect if it's encoded or not. Primarily used in menu build function\n    TODO Potential security risk"
    with open(fp, 'r') as f:
        raw = f.read()
    return raw
______________REQUESTS______________ = None
ip_reg = '\\b((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\\.|$)){4}\\b'
url_reg = '^((https?|http):\\/\\/)?(www.)?[a-z0-9]+\\.[a-z]+(\\/[a-zA-Z0-9#]+\\/?)*$'
import locale
encodings = {'ru_RU': 'cp866', 'de_DE': 'ISO8859-1'}
console_encoding = 'utf-8'
if (SYSTEM == 'Windows'):
    system_locale = locale.getdefaultlocale()[0]
    if (system_locale in encodings):
        console_encoding = encodings[system_locale]
    else:
        try:
            _CP_CODE = None
            cmd = check_output('chcp', shell=True).decode()
            result = re.search('\\d+', cmd)
            if (result is None):
                log.warning(f'Failed getting locale encoding, reverting to default: Regex returned None for: {result}')
            else:
                _CP_CODE = int(result.group())
                console_encoding = ('cp' + str(_CP_CODE))
            cmd_output = check_output('chcp', shell=True).decode()
        except Exception as e:
            log.warning(f'Failed getting locale encoding, reverting to default: {e}')
            console_encoding = locale.getpreferredencoding()
    log.message('System locale: {}'.format(system_locale))
    log.message('System preferred console encoding: {}'.format(console_encoding))

def shell_command(*args, **kwargs):
    try:
        if (SYSTEM == 'Windows'):
            log.debug('Shell command: {}'.format(str(args)))
            result = check_output(*args, encoding=console_encoding, **kwargs)
        else:
            result = check_output(*args, **kwargs).decode('utf-8')
        return result
    except CalledProcessError as e:
        log.debug(f"Command failed with return code {e.returncode}: {(e.output if hasattr(e, 'output') else 'No output')}")
        raise

def ping(host):
    '\n    Returns True if host (str) responds to a ping request.\n    Remember that some hosts may not respond to a ping request even if the host name is valid.\n    '
    log.debug("ping request for '{}'".format(host))
    host = str(host)
    if (not re.match(ip_reg, host)):
        host = urlparse(str(host)).netloc.split(':')[0]
    if (SYSTEM == 'Windows'):
        parameters = '-n 2 -w 10000'
    elif (SYSTEM == 'Darwin'):
        parameters = '-c 3'
    else:
        parameters = '-c 3'
    try:
        log.debug(((('ping ' + parameters) + ' ') + host))
        return shell_command(((('ping ' + parameters) + ' ') + host), shell=True)
    except CalledProcessError:
        return False
request_headers = {'Referer': 'BroTools for Maya'}
if (SYSTEM == 'Windows'):
    request_headers['User-Agent'] = 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.10; rv:75.0) Gecko/20100101 Firefox/75.0'
elif (SYSTEM == 'Darwin'):
    request_headers['User-Agent'] = 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.10; rv:75.0) Gecko/20100101 Firefox/75.0'
else:
    request_headers['User-Agent'] = 'Mozilla/5.0 (X11; Linux ppc64le; rv:75.0) Gecko/20100101 Firefox/75.0'

def get(url, type='json', timeout=5, headers=None):
    '\n    Simplified GET function.\n\n    :param type: request, plain, json\n    :return:\n    '
    try:
        log.debug('GET', url)
        try:
            if (not headers):
                headers = request_headers
            if (not isinstance(url, Request)):
                req = Request(url, headers=headers)
            else:
                req = url
            if (type != 'request'):
                req = urlopen(req, timeout=timeout, context=ssl_context)
            response = req.read()
            try:
                return json.loads(response)
            except ValueError as e:
                log.debug("Json decode error on request '{}', return as plain.".format(str(response)))
                return response
        except HTTPError as e:
            raise BroToolsError('Unable to GET resource: {}. Reason: {}'.format(url, e))
    except HTTPError as e:
        raise BroToolsError('Generic HTTP Exception: {} {}'.format(url, e))
    except URLError as e:
        raise BroToolsError('Generic HTTP Exception: {} {}'.format(url, e))
    except ssl.SSLError as e:
        traceback.print_exc()
        try:
            report_ssl_error(url)
        except Exception as e:
            log.debug('Failed to report SSL error: {}'.format(e))
        raise BroToolsError('SSL encryption error while connecting to the server. If you get this error, please, send an email to support@brotools.tech and attach brotools.log file. [{}]'.format(e))
    except Exception as e:
        raise BroToolsError('Generic HTTP Exception: {} {}'.format(url, e))

def post(url, data, type='json', timeout=15, headers=None):
    '\n    Simplified POST function\n    :param url:\n    :param data:\n    :param timeout:\n    :return:\n    '
    if (not headers):
        headers = request_headers
    log.debug('POST {}'.format(url))
    if ('on_startup' not in url):
        log.debug('POST', url, data)
    try:
        req = Request(url, headers=headers)
        if (type == 'json'):
            req.add_header('Content-Type', 'application/json')
            data = json.dumps(data)
            if PY3:
                data = data.encode('utf-8')
        else:
            raise TypeError
        req = urlopen(req, data=data, timeout=timeout, context=ssl_context)
        response = req.read()
        try:
            return json.loads(response)
        except ValueError as e:
            log.debug("Json decode error on request '{}', return as plain.".format(str(response)))
            return response
    except HTTPError as e:
        raise BroToolsError(('HTTPError: ' + str(e.code)), silent=True)
    except URLError as e:
        raise BroToolsError(('URLError: {}' + str(e.reason)), silent=True)
    except ssl.SSLError as e:
        traceback.print_exc()
        try:
            report_ssl_error(url, data)
        except Exception as e:
            log.debug('Failed to report SSL error: {}'.format(e))
        raise BroToolsError('SSL encryption error while connecting to the server. If you get this error, please, send an email to support@brotools.tech and attach brotools.log file. [{}]'.format(e))
    except Exception as e:
        raise BroToolsError('Generic HTTP Exception: {}; [{}]'.format(e, (data if data else 'no data')), silent=True)

def report_ssl_error(tried_url=None, post_data=None):
    try:
        data = {'data': {'Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07': (str(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data) if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07 else 'NONE'), 'hid': str(get_hid()), 'tried_url': str(tried_url), 'post_data': str(post_data), 'os': str(SYSTEM), 'platform': str(platform.platform()), 'maya_version': str(cmds.about(q=True, iv=True))}}
        report_ssl_context = ssl.create_default_context()
        report_ssl_context.check_hostname = False
        report_ssl_context.verify_mode = ssl.CERT_NONE
        req = Request('https://api.brotools.tech/v2/ssl_error_report', headers=request_headers)
        req.add_header('Content-Type', 'application/json')
        data = json.dumps(data)
        if PY3:
            data = data.encode('utf-8')
        req = urlopen(req, data=data, timeout=20, context=report_ssl_context)
        response = req.read()
        try:
            return json.loads(response)
        except ValueError as e:
            log.debug("Json decode error on request '{}', return as plain.".format(str(response)))
            return response
    except HTTPError as e:
        raise BroToolsError(('HTTPError: ' + str(e.code)), silent=True)
    except URLError as e:
        raise BroToolsError(('URLError: {}' + str(e.reason)), silent=True)
    except ssl.SSLError as e:
        traceback.print_exc()
        raise BroToolsError('SSL encryption error while connecting to the server. If you get this error, please, send an email to support@brotools.tech and attach brotools.log file. [{}]'.format(e))
    except Exception as e:
        raise BroToolsError('Generic HTTP Exception: {}; [{}]'.format(e, (data if data else 'no data')), silent=True)

def crossplatform_normpath(filepath):
    return ntpath.normpath(filepath).replace('\\', os.path.sep).replace('/', os.path.sep)

@catch_errors
def post_stream(url, data, filepath, type='json'):
    '\n    Download file as stream. Generator function.\n    :param url: Url of the file\n    :param filepath: Local path where to save the file. If file exists it will be removed and overwritten.\n    :return: Yields progress percent.\n    '
    filepath = crossplatform_normpath(filepath)
    log.debug('post_stream filepath: {}'.format(filepath))
    dirpath = os.path.dirname(filepath)
    if (not os.path.exists(dirpath)):
        os.makedirs(dirpath)
    temp_filepath = (filepath + '.brotemp')
    if os.path.exists(temp_filepath):
        os.remove(temp_filepath)
    log.debug('post_stream temp_filepath: {}'.format(temp_filepath))
    data = json.dumps(data)
    if PY3:
        data = data.encode('utf-8')
    req = Request(url, data=data, headers=request_headers)
    if (type == 'json'):
        req.add_header('Content-Type', 'application/json')
    response = urlopen(req, timeout=10, context=ssl_context)
    if PY3:
        total_size = response.headers.get('Content-Length')
    else:
        total_size = response.info().getheader('Content-Length')
        if (total_size is not None):
            total_size = total_size.strip()
    if total_size:
        total_size = int(total_size)
        blocksize = max(4096, (total_size // 100))
    else:
        blocksize = 1000000
    log.debug('Starting file download (size: {} | blocksize: {}): {}'.format(total_size, blocksize, temp_filepath))
    with open(temp_filepath, 'ab') as f:
        dl = 0
        while 1:
            chunk = response.read(blocksize)
            if (not chunk):
                break
            dl += 1
            f.write(chunk)
            (yield (dl - 1))
    smart_remove(filepath)
    shutil.move(temp_filepath, filepath)

@catch_errors
def get_stream(url, filepath):
    '\n    Download file as stream. Generator function.\n    :param url: Url of the file\n    :param filepath: Local path where to save the file. If file exists it will be removed and overwritten.\n    :return: Yields progress percent.\n    '
    if os.path.exists(filepath):
        os.remove(filepath)
    response = urlopen(url, context=ssl_context)
    total_size = response.info().getheader('Content-Length').strip()
    total_size = int(total_size)
    with open(filepath, 'ab') as f:
        dl = 0
        while 1:
            chunk = response.read((total_size / 100))
            if (not chunk):
                break
            dl += 1
            f.write(chunk)
            (yield (dl - 1))

def maya_process_count():
    if (SYSTEM == 'Windows'):
        try:
            a = shell_command('wmic process get description', shell=True)
            processes = []
            for line in a.split('\n'):
                processes.append(line.strip())
            return processes.count('maya.exe')
        except Exception as e:
            log.warning('Failed counting running maya processes: {}'.format(e))
            return 1
    else:
        log.debug('Counting Maya processes is currently only supported on Windows.')
        return 1

class BroApiEndpoint():
    check_delay = timedelta(seconds=300)
    if DEV_MODE:
        broapi_hosts = [API_MAIN_URL]
    else:
        broapi_hosts = [API_MAIN_URL]
    broapi_endpoint_suffix = '/v2'

    def __init__(self):
        self.endpoint = None
        self.last_update = None
        self.update_endpoint()

    def __str__(self):
        if (self.endpoint == None):
            self.update_endpoint()
        return str(self.endpoint)

    def __repr__(self):
        if (self.endpoint == None):
            self.update_endpoint()
        return self.endpoint

    def update_endpoint(self):
        if ((self.last_update is None) or ((datetime.now() - self.last_update) > self.check_delay)):
            log.debug('Looking for BroAPI endpoint...')
            self.endpoint = self.get_broapi_endpoint()
            self.last_update = datetime.now()
        else:
            log.debug('Using cached BroAPI endpoint ({}), {} until checking for update.'.format(self.endpoint, ((self.last_update + self.check_delay) - datetime.now())))

    def force_update_endpoint(self):
        self.last_update = ((datetime.now() - self.check_delay) - timedelta(seconds=10))
        self.update_endpoint()

    @catch_errors
    def get_broapi_endpoint(self):
        log.debug('Check connection...')
        broapi_endpoint = None
        for host in self.broapi_hosts:
            if (not ping(host)):
                log.warning('{:<30} {:<20} {:<20}'.format(host, 'ping test', 'FAILED'))
                continue
            endpoint = (host + self.broapi_endpoint_suffix)
            try:
                result = get(endpoint, timeout=10)
                if (result.get('status', False) == 'success'):
                    broapi_endpoint = endpoint
                    break
                else:
                    log.warning('{:<30} {:<20} {:<20}'.format(host, 'status test', 'FAILED'))
            except Exception as e:
                log.debug('{:<40} {:<20} {:<20}'.format(endpoint, 'response test', 'FAILED'))
                log.debug(e)
        if (broapi_endpoint == None):
            log.debug("Could not reach any online BroAPI endpoint. Either you don't have connection to the internet, or API server is down or unreachable from your location.\n            Things to try:\n            1. Make sure you're connected to the internet\n            2. Make sure you can reach any of the following addresses: {}\n            3. If you can't - check your network settings, firewall\n            4. Your ISP may be blocking certain websites as well\n            5. API website may be down for maintenance".format(self.broapi_hosts))
        return broapi_endpoint

    def test_connection(self):
        try:
            if (not ping(str(self))):
                log.warning('API Connection Failed: Could not ping server.', PING_FAIL_MESSAGE)
                return (False, PING_FAIL_MESSAGE, '')
            result = get(str(self))
            if (result.get('status', False) != 'success'):
                log.warning('API Connection Failed: Could ping server, but could not get proper response from API.', PING_FAIL_MESSAGE, str(result))
                return (False, PING_FAIL_MESSAGE, result)
            return (True, '', '')
        except Exception as e:
            log.warning('API Connection Failed: Unknown error.', e, exc_info=True)
            return (False, ((PING_FAIL_MESSAGE + '\n\n') + str(e)), traceback.format_exc())
broapi_endpoint = BroApiEndpoint()
______________LICENSING______________ = None

def decode_hid(hid):
    hid = base64decode(hid)
    return hid

def get_username():
    username = None
    getlogin = getattr(os, 'getlogin', None)
    if getlogin:
        try:
            username = os.getlogin()
        except:
            username = None
    if (not username):
        for var in ['USER', 'USERNAME', 'LOGNAME']:
            if (var in os.environ):
                username = os.environ[var]
    if (not username):
        cmds.error("Could not determine current user's name.")
    return username

def get_windows_uuid_legacy():
    'Try to get UUID using the legacy wmic command'
    try:
        uuid = sanitize(shell_command('wmic csproduct get uuid', shell=True).replace('UUID', '', 1))
        name = sanitize(shell_command('wmic csproduct get name', shell=True).replace('Name', '', 1))
        if (uuid.strip() and name.strip()):
            return f'{uuid}__{name}'.replace(' ', '_')
    except CalledProcessError:
        log.debug("Failed to get Windows UUID using legacy method. This is normal if you're using Windows 11. Will try to use new updated method.")
        return
    except Exception as e:
        log.warning('Failed to get Windows UUID using legacy method: {}'.format(e))
        return

def get_windows_uuid_modern():
    "Get UUID using PowerShell's Get-CimInstance"
    try:
        uuid = sanitize(shell_command('powershell -Command "Get-CimInstance -Class Win32_ComputerSystemProduct | Select-Object -ExpandProperty UUID"', shell=True))
        name = sanitize(shell_command('powershell -Command "Get-CimInstance -Class Win32_ComputerSystemProduct | Select-Object -ExpandProperty Name"', shell=True))
        if (uuid.strip() and name.strip()):
            return f'{uuid}__{name}'.replace(' ', '_')
    except CalledProcessError:
        log.warning('Failed to get Windows UUID.')
        return
    except Exception as e:
        log.error('Failed to get Windows UUID: {}'.format(e))
        return

def sanitize(txt):
    return txt.strip().replace('\n', '').replace('\r', '')

def get_uid():
    if (SYSTEM == 'Windows'):
        uid = get_windows_uuid_legacy()
        if (not uid):
            uid = get_windows_uuid_modern()
        if (not uid):
            log.warning('Failed to get Windows UUID using both legacy and modern methods')
            raise BroToolsError('Unable to get UUID on this machine. This could be due to using a VM, unsupported system language or other system related issues. Contact developer for support.')
    elif (SYSTEM == 'Darwin'):
        uid = shell_command('ioreg -rd1 -c IOPlatformExpertDevice | awk \'/IOPlatformUUID/ { split($0, line, "\\""); printf("%s\\n", line[4]); }\'', shell=True)
        uid = uid.replace('\n', '').replace('\r', '').strip()
    elif (SYSTEM == 'Linux'):
        uid = shell_command("ifconfig | grep ether | sha256sum | awk '{print $1}'", shell=True)
    return uid

def get_uid_legacy():
    address = uuid.getnode()
    h = iter(hex(address)[2:].zfill(12))
    mac = ':'.join(((i + next(h)) for i in h))
    address_check = uuid.getnode()
    if (((address >> 40) % 2) or (address != address_check)):
        cmds.warning('BroTools: Failed to acquire correct legacy UID.')
        return
    return mac

def get_uid_host(ip):
    log.debug(('arp lookup for ' + ip))
    hid = None
    if (SYSTEM == 'Windows'):
        shell_command(('ping -n 1 -w 3000 ' + ip), shell=True)
        s = str(shell_command(('arp -a ' + ip), shell=True))
        mac = re.search('{}(.*)(?P<mac>\\S\\S-\\S\\S-\\S\\S-\\S\\S-\\S\\S-\\S\\S)(.*)'.format(ip), s)
    elif (SYSTEM == 'Linux'):
        shell_command(('ping -c 1 -w 3 ' + ip), shell=True)
        s = str(shell_command(('arp -n ' + ip), shell=True))
        mac = re.search('{}(.*)(?P<mac>\\S\\S:\\S\\S:\\S\\S:\\S\\S:\\S\\S:\\S\\S)(.*)'.format(ip), s)
    elif (SYSTEM == 'Darwin'):
        shell_command(('ping -c 1 -t 3 ' + ip), shell=True)
        s = str(shell_command(('arp -n ' + ip), shell=True))
        mac = re.search('{}(.*)at (?P<mac>\\S*:\\S*:\\S*:\\S*:\\S*:\\S*)(.*)'.format(ip), s)
    if (mac is None):
        raise BroToolsError("Can't find MAC for {} in ARP table:\n {}".format(str(ip), str(s)))
    else:
        log.debug('MAC of {}: {}'.format(ip, mac.groupdict()['mac']))
        if (SYSTEM == 'Windows'):
            mac = mac.groupdict()['mac'].replace('-', ':')
        elif (SYSTEM == 'Linux'):
            mac = mac.groupdict()['mac']
        elif (SYSTEM == 'Darwin'):
            mac = mac.groupdict()['mac']
    try:
        (primary_hostname, alias_hostnames, other_ips) = socket.gethostbyaddr(ip)
        hostnames = str(primary_hostname)
    except socket.herror as e:
        hostnames = 'dns_reverse_lookup_error'
        log.warning('DNS Reverse Lookup Error: {}'.format(e))
    return (mac, hostnames)

def get_available_ip_mac_pairs():
    data = []
    if (SYSTEM == 'Windows'):
        raw = shell_command('arp -a', shell=True)
    else:
        raw = shell_command('arp -n', shell=True)
    if (raw is None):
        raise BroToolsError('arp returned None')
    for line in raw.split('\n'):
        line = line.replace(':', '-')
        arp_data = re.search('(\\s*)(?P<ip>[0-9]*.[0-9]*.[0-9]*.[0-9]*)(.*)(?P<mac>\\S\\S-\\S\\S-\\S\\S-\\S\\S-\\S\\S-\\S\\S)(.*)', line)
        if arp_data:
            arp_data = arp_data.groupdict()
            ip = arp_data['ip']
            mac = arp_data['mac']
            if ((ip.startswith('192.') or ip.startswith('10.') or ip.startswith('172.') or ip.startswith('100.')) and (mac != 'ff-ff-ff-ff-ff-ff') and (mac != '00-00-00-00-00-00')):
                data.append((ip, mac.replace('-', ':')))
    return data

def get_hid(host=None, legacy=False):
    '\n    Gets unique identifier of a machine, local if `host` is not supplied or remote if `host` is set to ip.\n    For local it uses OS specific hardware identifier + username. For IP locking it uses just the MAC address.\n    '
    uid = ''
    if (not host):
        if legacy:
            uid = get_uid_legacy()
        else:
            uid = get_uid()
        hid = '{}||{}||{}'.format(uid, get_username(), SYSTEM)
    else:
        (mac, hostnames) = get_uid_host(host)
        hid = '{}||{}||{}'.format(mac, hostnames, 'Network')
    hid = base64encode(hid)
    return hid

def date_to_string(date):
    return date.strftime('%Y-%m-%d')

def string_to_date(date):
    return datetime.strptime(date, '%Y-%m-%d').date()

def date_now():
    if broapi_endpoint:
        try:
            return string_to_date(get((broapi_endpoint + '/date/now')))
        except Exception as e:
            log.debug("Can't check date online,", e)
    return datetime.now().date()
email_regex = '(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\\.[a-zA-Z0-9-.]+$)'

class BroAPI():

    @property
    def endpoint(self):
        return str(broapi_endpoint)

    def get(self, resource, type='json', timeout=5, exceptions=False):
        '\n        :param resource: URL\n        :param type:\n        :param timeout:\n        :param exceptions:\n        :return:\n        '
        r = get((self.endpoint + resource), type, timeout)
        if (r.get('status') != 'success'):
            log.error('API GET ERROR: {}'.format(r.get('message')))
            if exceptions:
                raise BroToolsError(r.get('message'))
        return r

    def post(self, resource, data, type='json', timeout=10, exceptions=False):
        '\n        See BroAPI.get\n        :param data: Data to send in post request\n        '
        r = post((self.endpoint + resource), data, type, timeout)
        if (r.get('status') != 'success'):
            log.error('API POST ERROR: {}'.format(r.get('message')))
            if exceptions:
                raise BroToolsError(r.get('message'))
        return r

    def get_licman_intaller_data(self, email, auth_hid, networked, host):
        r = self.post('/license_manager/installer_data', {'email': email, 'hid': auth_hid, 'networked': networked, 'host': host})
        return r

    def check_email_hid_auth(self, email, hid, hostname):
        return self.post('/license_manager/email_authentication', {'email': email, 'hid': hid, 'hostname': hostname})

    def install(self, licid, auth_hid, target_hid, email, host=None):
        return self.post('/license_manager/install', {'licid': licid, 'auth_hid': auth_hid, 'target_hid': target_hid, 'email': email, 'host': host})

    def uninstall(self, licid, auth_hid, target_hid, email, **kwargs):
        return self.post('/license_manager/uninstall', {'licid': licid, 'auth_hid': auth_hid, 'target_hid': target_hid, 'email': email})

    def reinstall(self, licid, auth_hid, target_hid, email, host=None):
        return self.post('/license_manager/reinstall', {'licid': licid, 'auth_hid': auth_hid, 'target_hid': target_hid, 'email': email, 'host': host})

    def get_Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07_data(self, target_hid, email, networked=False, renew_reason='get_Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07_data None'):
        if (target_hid == None):
            log.error("Target HID can't be None", exc_info=True)
        data = {'email': email, 'target_hid': target_hid, 'renew_reason': renew_reason}
        if (renew_reason == 'Legacy HID Upgrade'):
            data['legacy_hid'] = get_hid(legacy=True)
        return self.post('/license_manager/get_key', data)

    def today_date(self):
        try:
            return datetime.strptime(self.get('/datetime')['data']['date'], '%Y.%m.%d')
        except Exception as e:
            log.debug(e)
            return

    def get_sync_data(self, auth_hid, target_hid, email, client_files_data):
        return self.post('/updater/syncdata', {'email': email, 'target_hid': target_hid, 'auth_hid': auth_hid, 'client_files_data': client_files_data, 'client_version': BroTools.__version__})

    def ping_api(self, *args, **kwargs):
        '\n        Usage statistics.\n        Ping home.\n        '
        if (broapi_endpoint == 'None'):
            log.debug('Ignoring ping, because API endpoint is not available.')
            return
        delay = kwargs.get('delay', 0)
        time.sleep(delay)
        self.post('/updater/on_startup', {'client_version': BroTools.__version__, 'auth_hid': get_hid(), 'Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07_data': (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists else 'No License Key'), 'email': (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['customer_id'] if (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists and ('customer_id' in Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data)) else None), 'maya_version_info': json.dumps(maya_version_info), 'issl': API_IGNORE_SSL})

    @property
    def is_new_version_available(self):
        global latest_version
        r = self.post('/updater/version', {'client_version': BroTools.__version__})
        log.debug('VERSION CHECK DATA: {}'.format(str(r)))
        if ('data' in r):
            latest_version = r['data'].get('latest', BroTools.__version__)
            return r['data'].get('update_required', False)
        else:
            return False

    def start_trial(self, email, auth_hid):
        return self.post('/start_trial', {'hid': auth_hid, 'email': email})

    def is_trial_available(self, email, auth_hid):
        r = self.post('/start_trial', {'hid': auth_hid, 'email': email, 'test': True})
        if (r['status'] == 'success'):
            return r['data']['trial_available']
        else:
            return False

def limitcheck_debug(name, data, result):
    pass

class LimitChecks():
    '\n    True = good\n    False = bad\n    '
    info = {'expires': 'License key needs to be verified or license expired.', 'hid_lock': 'Wrong HID.', 'max_version': 'Your installed plugin version is higher than license allows. Consider downgrading plugin version or upgrading your license.'}

    @staticmethod
    def expires(data):
        if (data is not None):
            expire_date = datetime.strptime(data, '%Y-%m-%d').date()
            now_date = datetime.now().date()
            result = (expire_date > now_date)
        else:
            result = True
        limitcheck_debug('expires', data, result)
        return result

    @staticmethod
    def hid_lock(data):
        result = (get_hid(data['host']) == data['hid'])
        limitcheck_debug('hid_lock', data, result)
        return result

    @staticmethod
    def max_version(data):
        result = (not semver_compare(BroTools.__version__, data['max_version']))
        limitcheck_debug('max_version', data, result)
        return result

    @staticmethod
    def tf_server(data):
        if (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07):
            result = False
        else:
            result = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_lease()
        limitcheck_debug('tf_server', data, result)
        return result

def json_safe_loads(s, default=None):
    try:
        return json.loads(s)
    except:
        return default

class Ox8r0_ZUp3TEVWUFhxX0NlcHFiMkN3QU82UU5m_ox07():
    fp = os.path.join(APP_DIR, 'license.key')
    filepath = fp
    tf_instance = None

    @property
    def validated_verified(self):
        if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
            if (not self.tf):
                self.__init__()
            result = self.tf.has_lease()
            if (result is False):
                self._validated_verified = False
            else:
                self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = None
                self._validated_verified = True
        if ((self.data is None) or (len(self.data.get('tools', [])) < 1)):
            self._validated_verified = False
        return self._validated_verified

    @property
    def tf(self):
        if (self.tf_instance is None):
            self.tf_instance = TurboFloat('2dm7w42xirfsw7up37eydi2batqpn6a', self.tf_leasecallback)
        return self.tf_instance

    @catch_errors
    def __init__(self):
        self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = None
        self._validated_verified = False
        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = None
        if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
            self.tf_server = os.environ.get(Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_VARNAME, DataCache().get('turbofloat_server')).split(':')
            log.message('Floating license server: {}'.format(self.tf_server))
            try:
                if ((not self.tf.has_lease()) and (FLOATING_DONT_LEASE_ON_START is False)):
                    try:
                        self.tf.request_lease()
                    except (TurboFloatServerError, TurboFloatInetError):
                        log.message('BroTools Floating: Save server and re-request lease: IP {}; PORT {}.'.format(self.tf_server[0], int(self.tf_server[1])))
                        self.tf.save_server(self.tf_server[0], int(self.tf_server[1]), TF_USER)
                        self.tf.request_lease()
                    except TurboFloatInetError as e:
                        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'Could not connect to TurboFloatServer (TurboFloatInetError)'
                        log.warning(('Could not connect to floating license server: TurboFloatInetError. ' + str(e)))
                    except TurboFloatWrongServerProductError as e:
                        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'TurboFloatWrongServerProductError'
                        log.warning(('Wrong floating server product: TurboFloatWrongServerProductError. ' + str(e)))
                    except TurboFloatServerUUIDMismatchError as e:
                        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'TurboFloatServerUUIDMismatchError'
                        log.warning(('TurboFloat UUID Mismatch Error: TurboFloatServerUUIDMismatchError. ' + str(e)))
                    except TurboFloatUsernameNotAllowedError as e:
                        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'TurboFloatUsernameNotAllowedError'
                        log.warning(('TurboFloat Username not allowed error: TurboFloatUsernameNotAllowedError. ' + str(e)))
                    log.debug('About to check lease...')
                    if self.tf_lease_check():
                        log.message('Floating license lease was requested successfully.')
                    else:
                        self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'Something went wrong while validating license from the float server.'
                        log.warning('Something went wrong while validating license from the float server.')
                else:
                    log.message('Already have lease.')
            except TurboFloatFailError as e:
                log.debug(('TurboFloatFailError. This is usually normal, just means that TurboFloat library has already been loaded.' + str(e)), exc_info=True)
            except TurboFloatError as e:
                log.warning(('TurboFloatError. ' + str(e)), exc_info=True)
                if (self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 is None):
                    self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'TurboFloatError'
                else:
                    self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 += '[TurboFloatError]'
            except Exception as e:
                log.warning(('Failed to get the floating license lease: Unhandled exception. ' + str(e)))
                if (self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 is None):
                    self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 = 'Unhandled exception'
                else:
                    self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 += '[Unhandled exception]'
        else:
            self.legacy_upgrade()
            self.self_check()
            self.start_background_checks()

    def tf_leasecallback(self, status):
        if (status == TF_CB_FEATURES_CHANGED):
            log.message('TODO: reload any features using TF_GetFeatureValue()')
        elif (status == TF_CB_LEASE_DROPPED_SLEEP):
            log.message('The lease has been dropped due to computer sleeping.')
            self._validated_verified = False
            self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = 'The lease has been dropped due to computer sleeping.'
        elif (status == TF_CB_LEASE_REGAINED):
            log.message('The lease has been successfully regained after a sleep.')
            self._validated_verified = True
            self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = None
        else:
            log.message('The lease expired or has been dropped: ', status)
            self._validated_verified = False
            self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = ('The lease expired or has been dropped: ' + str(status))

    def tf_get_feature_value(self, key):
        "\n        Wrapper require to take care of Python 3 version returning bytes. Not sure if it's only on linux and python 3 or what. \n        Does not seem to affect Windows.\n        "
        v = self.tf.get_feature_value(key)
        try:
            v = v.decode('utf-8')
        except (UnicodeDecodeError, AttributeError):
            pass
        return v

    def drop_lease(self):
        self.tf.drop_lease()
        self.tf_lease_check()

    def request_lease(self):
        self.tf.request_lease()
        self.tf_lease_check()

    def legacy_upgrade(self):
        if self.exists:
            log.debug('Legacy upgrade check...')
            if (not self.networked):
                legacy_hid = get_hid(legacy=True)
                log.debug('LEGACY HID: {}; CURRENT HID: {};'.format(legacy_hid, self.target_hid))
                if ((legacy_hid is not None) and (self.target_hid == legacy_hid)):
                    new_hid = get_hid()
                    log.info('Legacy HID detected, renewing key with new HID: {}'.format(new_hid))
                    self.renew_key(force_target_hid=new_hid, renew_reason='Legacy HID Upgrade')
            else:
                log.debug('Legacy HID upgrade skipped, no upgrade required for networked licenses.')
        else:
            log.debug('Legacy HID upgrade check skipped, no license key found.')

    def tf_lease_check(self):
        log.debug('Checking lease...')
        if self.tf.has_lease():
            self._validated_verified = True
        else:
            self._validated_verified = False
            self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = 'No lease from floating license server. '
            if (self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07 is not None):
                self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 += str(self.Ox8r0_ZUp4emw5UlFLZmVaeGF4eTU4WWtya1VGMWU4WGlPMWY4UDY2eVZkMVhic0hBTEdqRFhN_ox07)
        return self.validated_verified

    def start_tf_background_checks(self):
        t = threading.Thread(target=self.tf_background_check)
        t.start()

    def background_check(self):
        while True:
            time.sleep(3600)
            log.debug('Verifying license...')
            self.self_check(renew_on_fail=False)
            if (self.validated_verified is not True):
                log.warning('License key is invalid! {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))

    def file_watcher(self):
        while True:
            time.sleep(10)
            current_hash = (file_hash(self.fp) if self.exists else None)
            if (self.filehash != current_hash):
                log.debug('License file changed, verifying license...')
                self.filehash = current_hash
                self.self_check(renew_on_fail=False)
                if (self.validated_verified is not True):
                    log.warning('License key is invalid! {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))

    def start_background_checks(self):
        self.filehash = (file_hash(self.fp) if self.exists else None)
        t = threading.Thread(target=self.background_check)
        t.start()
        t2 = threading.Thread(target=self.file_watcher)
        t2.start()

    def self_check(self, renew_on_fail=True):
        log.debug('Checking license key...')
        self._validated_verified = False
        self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = ''
        try:
            if (not self.exists):
                self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = 'No license key.'
                log.debug('Key exists: FAIL')
                return
            log.debug('Key exists: OK')
            hard_check_test = self.hard_checks()
            if (hard_check_test is True):
                self._validated_verified = True
            else:
                log.debug('Key hard limits: FAIL')
                self._validated_verified = False
                self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = hard_check_test
                if renew_on_fail:
                    renew_result = self.renew_key(renew_reason='Key hard limits: FAIL [{}]'.format(hard_check_test))
                    if (renew_result is not True):
                        return
            soft_checks = self.soft_checks()
            if (soft_checks is not True):
                log.debug('Key soft limits: FAIL')
                if renew_on_fail:
                    self.renew_key(silent=True, renew_reason='Key soft limits: FAIL [{}]'.format(soft_checks))
        except Exception as e:
            self._validated_verified = False
            self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 = 'ERROR: {}; Details in Script Editor log.'.format(e)
            log.error(str(e), exc_info=True)

    @property
    def status_text(self):
        try:
            if (self.validated_verified is not True):
                status_text = 'Invalid: {}'.format(self.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07)
            elif (len(self.data['tools']) < 1):
                status_text = 'No licenses activated on this HID'
            else:
                status_text = 'Valid'
                if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
                    status_text += ' Floating Lease'
        except Exception as e:
            status_text = 'Invalid: Error - {}'.format(str(e))
        return status_text

    def tool_available(self, tool_id, show_err=True):
        pass

    @property
    def Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07(self):
        if (self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07 in LimitChecks.info):
            return LimitChecks.info[self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07]
        return self._Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07

    @property
    def data(self):
        if (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07):
            d = self.load_key()
            if (d is not None):
                if ('licenses' in d):
                    d['tools'] = []
                    for Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07 in d['licenses']:
                        add = True
                        for limit in Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07['hard_limits']:
                            if isinstance(limit, list):
                                print('LIMIT IS LIST 2', limit)
                            for (limit_id, limit_data) in limit.items():
                                if (not getattr(LimitChecks, limit_id)(limit_data)):
                                    add = False
                                    break
                        if add:
                            d['tools'].append(Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07['tool_id'])
                else:
                    log.debug('licenses field not found in license key file. Assuming this is an old license key.')
            else:
                self._validated_verified = False
            return d
        else:
            hlimits = {'max_version': self.tf_get_feature_value('max_version')}
            if self.tf.has_feature('is_subscription'):
                hlimits['expires'] = self.tf_get_feature_value('expiry_date').replace('.', '-')
            d = dict(customer_id=self.tf_get_feature_value('customer_id'), tools=json_safe_loads(self.tf_get_feature_value('tools'), ''), ppaths=json_safe_loads(self.tf_get_feature_value('ppaths'), None), hard_limits=hlimits)
            return d

    @property
    def next_verification(self):
        for limit in self.data['hard_limits']:
            l = limit.get('expires', None)
            if (l is not None):
                return l

    @property
    def target_hid(self):
        for limit in self.data['hard_limits']:
            if ('hid_lock' in limit):
                return limit['hid_lock'].get('hid', None)

    @property
    def target_host_ip(self):
        host = None
        for limit in self.data['hard_limits']:
            if ('hid_lock' in limit):
                if (limit['hid_lock']['host'] is not None):
                    host = limit['hid_lock']['host']
        return host

    @property
    def networked(self):
        return (self.target_host_ip is not None)

    @property
    def renewable(self):
        if (('hid' in self.data) or ('network_hid' in self.data)):
            return True
        return False

    def show_unable_to_renew_notification(self):
        Ox8r0_ZUp5cmxERlFTb2hzWjlGNS0zby14LVo2QUNrQkJnaw_ox07 = {'target_hid': self.target_hid, 'auth_hid': get_hid(), 'host': (self.data.host if (self.data.networked is True) else None), 'networked': self.data.networked, 'email': self.data.email, 'hostname': gethostname()}
        ct = [['Activation Code', 'abt.{}.sfti'.format(encode(json.dumps(Ox8r0_ZUp5cmxERlFTb2hzWjlGNS0zby14LVo2QUNrQkJnaw_ox07), nlib64.paths.cpath()))]]
        w = NotificationWindow(title='Unable to renew license key', text='BroTools is unable to reach the licensing server.\n\nIt is possible that servers are undergoing maintenance, in which case you might wait 10 minutes and try again.\n\nAlternatively you can try to manually renew license file at {}.\n\nLicense key invalid reason: {}'.format(URL_CLIENT_DASHBOARD, self.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07), copy_text=ct)
        w.show()

    def Ox8r0_ZUp5ckVUQldyUXFmeENuejRtSVRBQmp0Qktn_ox07(self, hid, email, networked=False, download_reason='None'):
        if ((broapi_endpoint == 'None') or (not ping(broapi_endpoint))):
            self.show_unable_to_renew_notification()
            return False
        data = broapi.get_Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07_data(hid, email, networked, renew_reason=download_reason)
        self.write_key_data(data)
        self.load_key()
        self.self_check()

    def renew_key(self, silent=False, force_target_hid=None, renew_reason='renew_key None'):
        log.debug('Renewing license key...')
        if ((broapi_endpoint == 'None') or (not ping(broapi_endpoint))):
            log.info('Unable to renew license key, could not reach server.')
            if (not silent):
                self.show_unable_to_renew_notification()
            return False
        try:
            data = broapi.get_Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07_data((force_target_hid if (force_target_hid is not None) else self.data['target_hid']), self.data['customer_id'], networked=self.networked, renew_reason=renew_reason)
            self.write_key_data(data)
            self.load_key()
            log.debug('License key renewed.')
            if (not silent):
                self.self_check(renew_on_fail=False)
            return True
        except Exception as e:
            log.warning('Unable to renew license key. {}'.format(e), exc_info=True)
            if (not silent):
                w = NotificationWindow(title='Unable to automatically renew license key', text="BroTools was unable to renew license key, because of a '{}' error.\n\nIf you continue to experience this error, please, contact me at support@brotools.tech.\n\nAlternatively you can try manually renewing license at {}\n\nLicense key invalid reason: {}".format(e, URL_CLIENT_DASHBOARD, self.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
                w.show()
            return False

    def write_key_data(self, data):
        '\n        Writes key data to file from data received by response from the server.\n        :param data:\n        :return:\n        '
        if (data['status'] == 'success'):
            with open(Ox8r0_ZUp3TEVWUFhxX0NlcHFiMkN3QU82UU5m_ox07.fp, 'w+') as f:
                f.write(data['data']['keydata'])
            return True
        else:
            log.error('Unable to renew license key. {}'.format((data['message'] if ('message' in data) else 'Unknown error.')))
            return False

    def load_key(self):
        if (not os.path.exists(self.fp)):
            return
        with open(self.fp, 'r') as f:
            data = f.read()
        data = json.loads(_decode(data))
        return data

    @property
    def exists(self):
        if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
            return True
        return os.path.exists(self.fp)

    def checks(self, category):
        for limit in self.data[category]:
            if isinstance(limit, list):
                print('LIMIT IS LIST', limit)
            for (limit_id, limit_data) in limit.items():
                if (not getattr(LimitChecks, limit_id)(limit_data)):
                    log.debug('{:<20} {:<20} {:<20}'.format(category, limit_id, 'FAILED'))
                    return limit_id
                else:
                    log.debug('{:<20} {:<20} {:<20}'.format(category, limit_id, 'PASSED'))
        return True

    def soft_checks(self):
        return self.checks('soft_limits')

    def hard_checks(self):
        result = self.checks('hard_limits')
        if (result is not True):
            self._validated_verified = False
        return result

def Ox8r0_ZUp3ckVWUFhxX0NlWnFUMF9zWVV1VVZWQUNwR0JpSQ_ox07(tool_id=None, namespace=None):
    if ((Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists is not True) or (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.validated_verified is not True)):
        return False
    Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07 = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['tools']
    ppaths = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['ppaths']
    if (tool_id is not None):
        if (tool_id not in ppaths):
            log.warning('Licensing error, tool_id {} missing from key.'.format(tool_id))
            return False
        for allowing_id in ppaths[tool_id]:
            if (allowing_id in Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07):
                return True
        log.warning('Invalid license, license required for {}.'.format(tool_id))
        return False
    else:
        if (namespace is None):
            namespace = sys._getframe(1).f_globals
        if ('__file__' in namespace):
            if (Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07 is None):
                log.warning('{} blocked, missing license file!'.format(namespace['__file__']))
                return False
            filepath = ntpath.normpath(namespace['__file__'])
            if any(((ppath in filepath) for ppath in ppaths)):
                blocked = True
                if isinstance(ppaths, list):
                    print('PPATHS IS LIST', ppaths)
                for (ppath, reqs) in ppaths.items():
                    if ((ntpath.normpath(ppath) in filepath) and any(((x in Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07) for x in reqs))):
                        blocked = False
                        break
                if (blocked is True):
                    log.warning('{} blocked, invalid license: {}'.format(namespace['__file__'], Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
                    return False
        else:
            log.debug("Can't determine filepath of a protected file, no __file__ in", namespace)
        return True

def clamp(value, lower, upper):
    return (lower if (value < lower) else (upper if (value > upper) else value))

def check_email_mask(email):
    if re.search(email_regex, email):
        return True
    else:
        return False

def open_web_url(*args, **kwargs):
    if ('url' in kwargs):
        url = kwargs.get('url')
    else:
        url = args[0]
    try:
        webbrowser.open(url)
    except Exception as e:
        raise BroToolsError('Tried to open web-link ({}; {}; {}), but experienced an error: {}'.format(args[0], args, kwargs, e), exc_info=True)

def get_client_files_data_generator():
    log.info('Analysing files...')
    filepaths = []
    for (root, dirs, files) in os.walk(APP_DIR, topdown=False):
        for name in files:
            filepaths.append(os.path.join(root, name))
    (yield len(filepaths))
    for filepath in filepaths:
        log.debug(filepath)
        relpath = ntpath.normpath(os.path.relpath(filepath, APP_DIR))
        checksum = file_hash(filepath)
        log.debug('File analysis data: {}'.format((relpath, checksum)))
        (yield (relpath, checksum))

def get_client_files_data():
    log.info('Gathering file analysis data...')
    return_data = {}
    for (i, data) in enumerate(get_client_files_data_generator()):
        if (i > 0):
            return_data[data[0]] = data[1]
    log.debug('File analysis data: {}'.format(data))
    return return_data
______________UI______________ = None

class ListSignal(QtCore.QObject):
    sig = QtCore.Signal(list)

class DictSignal(QtCore.QObject):
    sig = QtCore.Signal(dict)

class BoolSignal(QtCore.QObject):
    sig = QtCore.Signal(bool)

class IntSignal(QtCore.QObject):
    sig = QtCore.Signal(int)

class FloatSignal(QtCore.QObject):
    sig = QtCore.Signal(float)

class EmailEdit(QtWidgets.QLineEdit):

    def __init__(self, *args, **kwargs):
        super(EmailEdit, self).__init__(*args, **kwargs)

    def text(self, *args, **kwargs):
        return super(EmailEdit, self).text().strip()

class Overlay(QtWidgets.QWidget):

    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        palette = QtGui.QPalette(self.palette())
        palette.setColor(palette.ColorRole.Window, QtCore.Qt.transparent)
        self.setPalette(palette)
        self.text = ''
        self.completion = None

    def paintEvent(self, event):
        painter = QtGui.QPainter()
        painter.begin(self)
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        painter.fillRect(event.rect(), QtGui.QBrush(QtGui.QColor(255, 255, 255, 127)))
        painter.setPen(QtGui.QPen(QtCore.Qt.NoPen))
        count = 12
        dot_radius = 8
        radius = 30
        for i in range(count):
            if (((self.counter / 5) % count) == i):
                painter.setBrush(QtGui.QBrush(QtGui.QColor(clamp((250 - ((self.counter % 5) * 20)), 127, 255), clamp((163 - ((self.counter % 5) * 20)), 127, 255), 0)))
            else:
                painter.setBrush(QtGui.QBrush(QtGui.QColor(127, 127, 127)))
            painter.drawEllipse((((self.width() / 2) + (radius * math.cos((((2 * math.pi) * i) / float(count))))) - 10), (((self.height() / 2) + (radius * math.sin((((2 * math.pi) * i) / float(count))))) - 10), dot_radius, dot_radius)
        font = QtGui.QFont()
        font.setFamily('Tahoma')
        font.setPixelSize(20)
        font.setBold(True)
        pen = QtGui.QPen(QtCore.Qt.black)
        painter.setPen(pen)
        painter.setFont(font)
        painter.drawText(0, 50, self.width(), self.height(), QtCore.Qt.AlignCenter, self.text)
        if (self.completion is not None):
            painter.drawText(0, 0, self.width(), self.height(), QtCore.Qt.AlignCenter, (str(self.completion) + '%'))
        painter.end()

    def showEvent(self, event):
        self.timer = self.startTimer(50)
        self.counter = 0

    def timerEvent(self, event):
        if self.isVisible():
            self.counter += 1
            self.update()

class OverlayProgress(QtWidgets.QWidget):

    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        palette = QtGui.QPalette(self.palette())
        palette.setColor(palette.ColorRole.Window, QtCore.Qt.transparent)
        self.setPalette(palette)
        self.text = ''
        self.completion = None
        self.part_completion = None

    def paintEvent(self, event):
        count = 12
        dot_radius = 8
        radius = 30
        painter = QtGui.QPainter()
        painter.begin(self)
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        painter.fillRect(event.rect(), QtGui.QBrush(QtGui.QColor(40, 40, 40, 255)))
        painter.setPen(QtGui.QPen(QtCore.Qt.NoPen))
        if (self.part_completion is not None):
            color = QtGui.QColor(255, 255, 255, 20)
            painter.setBrush(QtGui.QBrush(color))
            angle = utils.remap_range(self.part_completion, 0, 100, 0, 360)
            ia = ((0 * (- 16)) + 1440)
            ib = (angle * (- 16))
            painter.drawPie((((self.width() / 2) - radius) - 6), (((self.height() / 2) - radius) - 6), (radius * 2), (radius * 2), int(ia), int(ib))
        for i in range(count):
            orange = (250, 163, 0)
            if ((self.completion is not None) and (((float(i) / float(count)) * 100) < self.completion)):
                bg_color = [0, 255, 0]
            else:
                bg_color = [80, 80, 80]
            if (((self.counter / 5) % count) == i):
                color = lerp_color(orange, bg_color, (float((self.counter % 5)) / float(5)))
            else:
                color = bg_color
            color = QtGui.QColor(*color)
            painter.setBrush(QtGui.QBrush(color))
            painter.drawEllipse((((self.width() / 2) + (radius * math.cos((((2 * math.pi) * i) / float(count))))) - 10), (((self.height() / 2) + (radius * math.sin((((2 * math.pi) * i) / float(count))))) - 10), dot_radius, dot_radius)
        font = QtGui.QFont()
        font.setFamily('Verdana')
        font.setPixelSize(14)
        pen = QtGui.QPen(QtGui.QColor(200, 200, 200))
        painter.setPen(pen)
        painter.setFont(font)
        painter.drawText(0, 70, self.width(), self.height(), QtCore.Qt.AlignCenter, self.text)
        painter.end()

    def showEvent(self, event):
        self.timer = self.startTimer(50)
        self.counter = 0

    def timerEvent(self, event):
        if self.isVisible():
            self.counter += 1
            self.update()

class Ox8r0_ZUp3TEVWUFhxX0NlcG1MLTdDWUFFeVFFTXc_ox07(QtWidgets.QTreeWidgetItem):

    def __init__(self, *args, **kwargs):
        super(Ox8r0_ZUp3TEVWUFhxX0NlcG1MLTdDWUFFeVFFTXc_ox07, self).__init__(*args, **kwargs)
        self.combo = None

    @property
    def licid(self):
        return self.text(0)
    id = licid

    @property
    def tool(self):
        return self.text(1)

    @property
    def status(self):
        return self.text(2)

    @property
    def selected_action(self):
        if (self.combo is not None):
            return self.combo.currentText()
        else:
            log.warning('Combo is not set on BroOx8r0_ZUp3TEVWUFhxX0NlcG1MLTdDWUFFeVFFTXc_ox07, but code is trying to access it: {}.'.format(self.id))
            return

class Ox8r0_ZUp3QkpBRGJfMVFXSnk1NFM1WWdJdTNWbkF5MVVtSHBtaEdpck5MbU5QVW5OU2YyT0tyRFZKYnVPeUFGRVNF_ox07(QtWidgets.QWidget):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(Ox8r0_ZUp3QkpBRGJfMVFXSnk1NFM1WWdJdTNWbkF5MVVtSHBtaEdpck5MbU5QVW5OU2YyT0tyRFZKYnVPeUFGRVNF_ox07, self).__init__(*args, **kwargs)
        self.rootWidget = rootWidget
        self.supported_combo_actions = ['Do Nothing', 'Install', 'Uninstall', 'Reinstall Here']
        self.button_actions = {'open_url': open_web_url}
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.treeView = QtWidgets.QTreeWidget()
        self.treeView.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.treeView.setFocusPolicy(QtCore.Qt.NoFocus)
        self.treeView.setColumnCount(5)
        self.treeView.setHeaderLabels(['ID', 'Tool', 'Status', 'Expires', 'Available Actions'])
        self.treeView.setObjectName('BroLicenseManagerTreeView')
        self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07 = QtWidgets.QLabel('You have no available licenses.\n Make sure you entered correct email and selected proper license type - Regular or Networked.\n\nPlease, use Start Trial button or purchase a license on Gumroad.\n\nIf you updated from BroDynamics 2.x then you will have extended Trial period, use Start Trial button to activte it, make sure to use the same email you used to purchase BroDynamics 2.x.')
        self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07.setStyleSheet('QLabel {font-size:16px;color: red;}')
        self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07.setWordWrap(True)
        self.mainLayout.addWidget(self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07)
        self.mainLayout.addWidget(self.treeView)
        self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07.hide()

    def on_go_back_button_clicked(self, *args, **kwargs):
        self.rootWidget.showFirstPage()

    def populate(self, data=None):
        self.treeView.clear()
        if ((data is None) or (len(data) < 1)):
            self.rootWidget.nextButton.setEnabled(False)
            self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07.show()
        else:
            if ((len(data) == 1) and ('Install' in data[0].get('combo_actions', []))):
                auto_mode = True
            else:
                auto_mode = False
            self.Ox8r0_ZUp3ckU1QldyNC1leG16dzdQZ1NqdFhWVmY4QU5rVUhZdw_ox07.hide()
            for lic in data:
                treeItem = Ox8r0_ZUp3TEVWUFhxX0NlcG1MLTdDWUFFeVFFTXc_ox07(self.treeView, [str(lic['lic_id'])])
                toolWidget = QtWidgets.QWidget(self)
                toolWidgetLayout = QtWidgets.QHBoxLayout()
                toolWidget.setLayout(toolWidgetLayout)
                treeItem.setData(1, QtCore.Qt.DisplayRole, lic['tool_name'])
                treeItem.setData(2, QtCore.Qt.EditRole, lic['status'])
                treeItem.setData(3, QtCore.Qt.EditRole, lic['expires'])
                combo = QtWidgets.QComboBox(self)
                treeItem.combo = combo
                toolWidgetLayout.addWidget(combo)
                if (not auto_mode):
                    combo.addItem('Do Nothing')
                if ('combo_actions' in lic):
                    for combo_action in lic['combo_actions']:
                        if (combo_action in self.supported_combo_actions):
                            combo.addItem(combo_action)
                if auto_mode:
                    combo.addItem('Do Nothing')
                if ('buttons' in lic):
                    for button_data in lic['buttons']:
                        if (button_data['action'] in self.button_actions):
                            button = QtWidgets.QPushButton(button_data['text'])
                            toolWidgetLayout.addWidget(button)
                            if (self.button_actions[button_data['action']] is not None):
                                button.clicked.connect(partial(self.button_actions[button_data['action']], *button_data['args'], **button_data['kwargs']))
                self.treeView.setItemWidget(treeItem, 4, toolWidget)
            self.rootWidget.nextButton.setEnabled(True)

class Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTBSaVQ5NnVOc1dfSC12OUJVQWdWSUw3UQ_ox07(QtWidgets.QWidget):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTBSaVQ5NnVOc1dfSC12OUJVQWdWSUw3UQ_ox07, self).__init__(*args, **kwargs)
        self.rootWidget = rootWidget
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.emailBox = QtWidgets.QGroupBox('Email')
        self.emailBoxLayout = QtWidgets.QVBoxLayout()
        self.emailBox.setLayout(self.emailBoxLayout)
        self.email = EmailEdit()
        self.email.setPlaceholderText('Enter your email...')
        self.emailBoxLayout.addWidget(self.email)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07 = QtWidgets.QGroupBox('License Type')
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07 = QtWidgets.QButtonGroup()
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout = QtWidgets.QVBoxLayout()
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWF0cTJsMWZfbWNlOVk5dldVMFZjak0xVUFzNXNOUHc_ox07 = QtWidgets.QRadioButton('Workstation (Regular license)')
        self.licenseTypeFloatingButton = QtWidgets.QRadioButton('Floating (Site or Enterprise license)')
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSlduczJaNlQtbTJDN1otXzY1SmdCXzlBdmQ_ox07 = QtWidgets.QRadioButton('Network (Site or Enterprise license) (Legacy)')
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07.setLayout(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.addButton(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWF0cTJsMWZfbWNlOVk5dldVMFZjak0xVUFzNXNOUHc_ox07, 0)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.addButton(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSlduczJaNlQtbTJDN1otXzY1SmdCXzlBdmQ_ox07, 1)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.addButton(self.licenseTypeFloatingButton, 2)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWF0cTJsMWZfbWNlOVk5dldVMFZjak0xVUFzNXNOUHc_ox07)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSlduczJaNlQtbTJDN1otXzY1SmdCXzlBdmQ_ox07)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout.addWidget(self.licenseTypeFloatingButton)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWF0cTJsMWZfbWNlOVk5dldVMFZjak0xVUFzNXNOUHc_ox07.setChecked(True)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07Layout.addStretch()
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.buttonReleased.connect(partial(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSU94bVVGLWU5bkF3QkIxUWc0_ox07))
        self.ipBox = QtWidgets.QGroupBox('Network Host IP')
        self.ipBoxLayout = QtWidgets.QVBoxLayout()
        self.ipBox.setLayout(self.ipBoxLayout)
        self.ip = QtWidgets.QComboBox()
        self.ipHint = QtWidgets.QLabel("You can bind your license to one of the IPs listed in this dropdown. Select a device that's always online like your router or a server.\nIf you can't find a device you'd like to bind you license to, please, contact me at support@brotools.tech for an alternative solution.")
        self.ipBoxLayout.addWidget(self.ip)
        self.ipBoxLayout.addWidget(self.ipHint)
        self.ipBox.hide()
        self.ipBoxFloating = QtWidgets.QGroupBox('TurboFloat Server Address')
        self.ipBoxLayoutFloating = QtWidgets.QVBoxLayout()
        self.ipBoxFloating.setLayout(self.ipBoxLayoutFloating)
        self.ipFloating = QtWidgets.QLineEdit()
        self.ipHintFloating = QtWidgets.QLabel('Provide the address of your TurboFloat server. If unsure what to enter, you might need to contact your IT department to find out the address of your TurboFloat server.')
        self.ipBoxLayoutFloating.addWidget(self.ipFloating)
        self.ipBoxLayoutFloating.addWidget(self.ipHintFloating)
        self.ipBoxFloating.hide()
        self.currentStatusBox = QtWidgets.QGroupBox('Current License')
        self.currentStatusBoxLayout = QtWidgets.QVBoxLayout()
        self.currentStatusBox.setLayout(self.currentStatusBoxLayout)
        self.resetFloatingLicenseButton = QtWidgets.QPushButton('Forget TurboFloat server\n(Use to reset, if you want to change server or switch to a different license type)')
        self.resetFloatingLicenseButton.clicked.connect(partial(self.on_reset_floating_license_button_clicked))
        self.turboFloatResetLabel = QtWidgets.QLabel('Now you need to go to BroTools - Reload BroTools or restart Maya.')
        if (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07):
            self.mainLayout.addWidget(self.emailBox)
            self.mainLayout.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk90djBBSlRrRjdn_ox07)
            self.mainLayout.addWidget(self.ipBox)
            self.mainLayout.addWidget(self.ipBoxFloating)
        self.mainLayout.addWidget(self.currentStatusBox)
        self.mainLayout.addStretch()
        if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
            self.updateLeaseButton = QtWidgets.QPushButton('Checking...')
            self.updateLeaseButton.clicked.connect(partial(self.on_lease_button_clicked))
            self.mainLayout.addWidget(self.updateLeaseButton)
            if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07_FROM_UI:
                self.mainLayout.addWidget(self.turboFloatResetLabel)
                self.mainLayout.addWidget(self.resetFloatingLicenseButton)
                self.turboFloatResetLabel.hide()
            else:
                self.turboFloatResetLabel.setText("TurboFloat server address was set up through Environment variable, and it's impossible to deactivate it from the UI.")
        if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists:
            try:
                target_host_ip = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.target_host_ip
                if (target_host_ip is not None):
                    self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSlduczJaNlQtbTJDN1otXzY1SmdCXzlBdmQ_ox07.setChecked(True)
                    self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSU94bVVGLWU5bkF3QkIxUWc0_ox07()
                else:
                    self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWF0cTJsMWZfbWNlOVk5dldVMFZjak0xVUFzNXNOUHc_ox07.setChecked(True)
                self.email.setText(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['customer_id'])
            except Exception as e:
                log.error('TODO - show more meaningful error, TLDR - something went wrong while trying to get data from existing license key.', exc_info=True)
        self.update_current_status()
        if (((not Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists) or (not Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.validated_verified) or (len(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['tools']) < 1)) and (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07)):
            self.trial_notice = BroText('Trial period of complete BroTools experience!')
            self.trial_button = QtWidgets.QPushButton(('Start Trial' if (not WelcomeBackWindow.revert_available()) else 'Start Extended Trial (Upgrade license from BroDynamics 2.x)'))
            self.check_trial_availability()
            self.trial_button.clicked.connect(partial(self.on_start_trial_button_clicked))
            self.purchase_button = QtWidgets.QPushButton('Purchase')
            self.purchase_button.clicked.connect(partial(open_web_url, url='{}/purchase'.format(API_MAIN_URL)))
            for widget in [self.trial_notice, self.trial_button, self.purchase_button]:
                self.currentStatusBoxLayout.addWidget(widget)
            self.email.textChanged.connect(partial(self.check_trial_availability))
            self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.buttonClicked.connect(partial(self.check_trial_availability))

    def on_reset_floating_license_button_clicked(self, *args, **kwargs):
        data_cache.remove('turbofloat_server')
        self.turboFloatResetLabel.show()

    def populateIpBox(self):
        self.ip.clear()
        if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists:
            selection = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.target_host_ip
        else:
            selection = None
        selection_id = None
        pairs = get_available_ip_mac_pairs()
        for (i, pair) in enumerate(pairs):
            self.ip.addItem('{}'.format(pair[0]))
            if (pair[0] == selection):
                selection_id = i
        if (selection_id is not None):
            try:
                self.ip.setCurrentIndex(selection_id)
            except Exception as e:
                log.warning('Failed to set current IP box index to {}: {}'.format(selection_id, e))

    def check_trial_availability(self, *args, **kwargs):
        if (check_email_mask(self.email.text()) and (self.selection == 0)):
            self.trial_button.setEnabled(True)
        else:
            self.trial_button.setEnabled(False)

    def on_start_trial_button_clicked(self, *args, **kwargs):
        try:
            r = broapi.start_trial(email=self.email.text(), auth_hid=get_hid())
            if (r['status'] != 'success'):
                NotificationWindow(title=('Trial Start ' + str(r['status']).title()), text=(r['message'] + '\n')).show()
            else:
                self.rootWidget.nextButtonClicked()
                self.trial_button.setEnabled(False)
        except Exception as e:
            show_error(str(e), title='Failed to start trial.')

    def show(self, *args, **kwargs):
        self.update_current_status()
        super(Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTBSaVQ5NnVOc1dfSC12OUJVQWdWSUw3UQ_ox07, self).show(*args, **kwargs)

    def on_lease_button_clicked(self, *args, **kwargs):
        if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_lease():
            LicensingAPI.turbofloat_lease_drop()
            self.update_current_status()
        else:
            LicensingAPI.turbofloat_lease_drop()
            self.update_current_status()

    def update_current_status(self, *args, **kwargs):
        for i in reversed(list(range(self.currentStatusBoxLayout.count()))):
            w = self.currentStatusBoxLayout.itemAt(i).widget()
            w.setParent(None)
            w.deleteLater()
        if ((Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists is not True) and (Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 is not True)):
            status = BroEntry('License Status', 'No License Key')
            self.currentStatusBoxLayout.addWidget(status)
        else:
            Ox8r0_ZUp3ckVWUFhxX0NleGluel9UWUFFLXNFTUE_ox07 = BroEntry('Licensed To', Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data.get('customer_id', 'Promo License'))
            status = BroEntry('License Status', Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.status_text)
            tools = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data.get('tools', [])
            available_tools = BroEntry('Available Tools', (', '.join(tools) if (len(tools) > 0) else 'No tools activated on this machine'))
            self.currentStatusBoxLayout.addWidget(Ox8r0_ZUp3ckVWUFhxX0NleGluel9UWUFFLXNFTUE_ox07)
            self.currentStatusBoxLayout.addWidget(status)
            if (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07):
                next_verification = BroEntry('Next verification', Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.next_verification)
                self.currentStatusBoxLayout.addWidget(next_verification)
            self.currentStatusBoxLayout.addWidget(available_tools)
            self.currentStatusBoxLayout.addWidget(QHLine())
            if ('licenses' in Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data):
                for Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07 in Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['licenses']:
                    entry = BroEntry('{} ({})'.format(Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07['tool_id'], Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07['type']), '')
                    self.currentStatusBoxLayout.addWidget(entry)
                    for limit in Ox8r0_ZUp3ckVWUFhxX0NlWmlSekJ3QVFHd05q_ox07['hard_limits']:
                        for (limit_id, limit_data) in limit.items():
                            passed = getattr(LimitChecks, limit_id)(limit_data)
                            passed = ('PASS' if (passed is True) else 'FAIL')
                            if ((limit_id == 'expires') and (limit_data is None)):
                                limit_data = 'Lifetime'
                            txt = '{} ({}): {}; '.format(limit_id, passed, limit_data)
                        entry = BroEntry('', txt)
                        self.currentStatusBoxLayout.addWidget(entry)
            if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
                entry = BroEntry('TurboFloat Server', str(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf_server))
                self.currentStatusBoxLayout.addWidget(entry)
                entry = BroEntry('TurboFloat Lease', str(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_lease()))
                self.currentStatusBoxLayout.addWidget(entry)
                if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_lease():
                    if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_feature('is_subscription'):
                        entry = BroEntry('TurboFloat Expiry', str(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf_get_feature_value('expiry_date')))
                        self.currentStatusBoxLayout.addWidget(entry)
                        entry = BroEntry('TurboFloat Type', 'Subscription')
                        self.currentStatusBoxLayout.addWidget(entry)
                    else:
                        entry = BroEntry('TurboFloat Max Version', str(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf_get_feature_value('max_version')))
                        self.currentStatusBoxLayout.addWidget(entry)
                        entry = BroEntry('TurboFloat Type', 'Lifetime')
                        self.currentStatusBoxLayout.addWidget(entry)
                self.updateLeaseButton.setText(('Drop lease' if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.tf.has_lease() else 'Request lease'))

    @property
    def selection(self):
        return self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSk9tYzFaNlU5MjhPN2I4eG9BYVBzSzF3_ox07.checkedId()

    @property
    def can_proceed(self):
        if (not check_email_mask(self.email.text())):
            self.rootWidget.showAlert('Invalid email.')
            return False
        if (self.selection == 1):
            ipt = self.ip.currentText()
            if (not (ipt.startswith('192.') or ipt.startswith('10.') or ipt.startswith('172.') or ipt.startswith('100.'))):
                self.rootWidget.showAlert('Entered IP does not look like private network IP. Please, use ip of a device within your LAN.')
                return False
            if (not ping(ipt)):
                self.rootWidget.showAlert('Failed pinging host {}.'.format(ipt))
                return False
        email_hid_check = broapi.check_email_hid_auth(self.email.text(), get_hid(), gethostname())
        try:
            if (email_hid_check['status'] != 'success'):
                self.rootWidget.showAlert(email_hid_check['message'])
                return False
        except Exception as e:
            raise e
        return True

    def Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSU94bVVGLWU5bkF3QkIxUWc0_ox07(self, *args, **kwargs):
        self.ipBox.hide()
        self.ipBoxFloating.hide()
        if (self.selection == 1):
            self.populateIpBox()
            self.ipBox.show()
        elif (self.selection == 2):
            self.ipBoxFloating.show()
        self.update_current_status()

class Ox8r0_ZUp3QkpRRGFfMVFXSnk1NFM1WWdJdTNWbkF5MVVtSHBtaEdpck5fbk1POHFOaXIzTDRfOVdaWHNLbmN5a0JHcg_ox07(QtWidgets.QWidget):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(Ox8r0_ZUp3QkpRRGFfMVFXSnk1NFM1WWdJdTNWbkF5MVVtSHBtaEdpck5fbk1POHFOaXIzTDRfOVdaWHNLbmN5a0JHcg_ox07, self).__init__(*args, **kwargs)
        self.rootWidget = rootWidget

class Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJ4dVUtNmVUY3NQZk5RLVpteUxnQ1piZ3di_ox07(QtWidgets.QStackedWidget):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJ4dVUtNmVUY3NQZk5RLVpteUxnQ1piZ3di_ox07, self).__init__(*args, **kwargs)
        self.mainLayout = self.layout()
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07 = Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTBSaVQ5NnVOc1dfSC12OUJVQWdWSUw3UQ_ox07(rootWidget=rootWidget)
        self.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07)
        self.Ox8r0_ZUp3ckVWUFhxX0NlcHFMNzRjQXMxdFhWVlhlblNXOWI4eDRBWTlFS1dB_ox07 = Ox8r0_ZUp3QkpBRGJfMVFXSnk1NFM1WWdJdTNWbkF5MVVtSHBtaEdpck5MbU5QVW5OU2YyT0tyRFZKYnVPeUFGRVNF_ox07(rootWidget=rootWidget)
        self.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlcHFMNzRjQXMxdFhWVlhlblNXOWI4eDRBWTlFS1dB_ox07)

class LoaderThread(QtCore.QThread):

    def __init__(self, parent=None, function=None):
        QtCore.QThread.__init__(self, parent)
        self.exiting = False
        self.signal_completed = BoolSignal()
        self.signal_failed = ListSignal()
        self.signal_rejected = BoolSignal()
        self.parent = parent
        self.function = function

    def run(self):
        if (self.function is None):
            self.signal_failed.sig.emit(['No function provided for LoaderThread (most likely bug in code, report to developer).', ''])
            return
        try:
            log.debug('Running in Thread: {}'.format(self.function.__name__))
            if (self.function() == True):
                self.signal_completed.sig.emit(True)
            else:
                self.signal_rejected.sig.emit(True)
        except Exception as e:
            self.signal_failed.sig.emit([str(e), str(traceback.format_exc())])

class CheckFileSyncThread(QtCore.QThread):

    def __init__(self, parent=None, **kwargs):
        QtCore.QThread.__init__(self, parent)
        self.exiting = False
        self.signal_error = DictSignal()
        self.signal_completed = DictSignal()
        self.signal_progress = DictSignal()
        self.parent = parent
        self.auth_hid = None
        self.delay = kwargs.get('delay', 0)

    def run(self):
        try:
            time.sleep(self.delay)
            (test, message, details) = broapi_endpoint.test_connection()
            if (not test):
                self.signal_error.sig.emit({'message': message, 'traceback': details})
                return
            self.signal_progress.sig.emit({'status': 'Preparing...', 'percent': 0})
            self.auth_hid = get_hid()
            client_files_data = {}
            log.info('Gathering file analysis data...')
            filecount = 1
            for (i, data) in enumerate(get_client_files_data_generator()):
                if (i > 0):
                    analysis_percent = int(((float((i + 1)) / filecount) * 100))
                    client_files_data[data[0]] = data[1]
                    self.signal_progress.sig.emit({'status': 'Analysing file: {} out of {}'.format((i + 1), filecount), 'percent': analysis_percent})
                else:
                    filecount = (data + 2)
                    analysis_percent = 0
                    self.signal_progress.sig.emit({'status': 'Analysing file: {} out of {}'.format((i + 1), filecount), 'percent': analysis_percent})
            log.debug('File analysis data: {}'.format(data))
            self.signal_progress.sig.emit({'status': 'Contacting server...', 'percent': ((filecount - 1) / filecount)})
            sync_data = broapi.get_sync_data(auth_hid=self.auth_hid, target_hid=Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.target_hid, email=Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['customer_id'], client_files_data=json.dumps(client_files_data))
            log.debug(sync_data)
            if ('status' not in sync_data):
                log.warning('Unable to get sync data, server error. \n{}'.format(sync_data))
                return
            if (sync_data['status'] != 'success'):
                log.warning('Unable to get sync data: {}'.format(sync_data['message']))
                return
            total = (len(sync_data['data']['download']) + len(sync_data['data']['remove']))
            self.signal_progress.sig.emit({'status': 'Finishing up...', 'percent': 100})
            self.signal_completed.sig.emit({'update_required': (total > 0), 'sync_data': sync_data['data']})
        except Exception as e:
            log.warning(traceback.format_exc())
            self.signal_error.sig.emit({'message': str(e), 'traceback': traceback.format_exc()})

class WelcomeWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(WelcomeWindow, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools Welcome Window', rootWidget=rootWidget, objectName='BroWelcomeWindow_02', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.main_layout)
        self.title = BroTitle('Thank you for purchasing BroTools!')
        self.text = BroText('Please, read the end-user license agreement')
        self.title.setWordWrap(False)
        self.title.setAlignment(QtCore.Qt.AlignHCenter)
        self.imagePath = img('logo_128')
        self.imageLabel = QtWidgets.QLabel(self)
        self.imageMap = QtGui.QPixmap(self.imagePath)
        self.imageLabel.setPixmap(self.imageMap)
        self.imageLabel.setAlignment(QtCore.Qt.AlignHCenter)
        self.headerLayout = QtWidgets.QVBoxLayout()
        self.headerLayout.addWidget(self.imageLabel)
        self.headerLayout.addWidget(self.title)
        self.headerLayout.setAlignment(QtCore.Qt.AlignCenter)
        self.headerLayout.addWidget(self.text)
        self.declineBtn = QtWidgets.QPushButton('Decline')
        self.declineBtn.clicked.connect(partial(self.on_decline))
        self.acceptBtn = QtWidgets.QPushButton('Accept and Proceed to License Manager')
        self.acceptBtn.clicked.connect(partial(self.on_accept))
        self.buttonsLayout = QtWidgets.QHBoxLayout()
        self.buttonsLayout.addWidget(self.acceptBtn)
        self.buttonsLayout.addWidget(self.declineBtn)
        self.Ox8r0_ZUp3ckVWUFhxX0NlWm1UeTZ0Wjh0ZzBBSkg0RnVB_ox07 = QtWidgets.QTextBrowser()
        with open(os.path.join(APP_DIR, 'LICENSE.html'), 'r') as f:
            self.Ox8r0_ZUp3ckVWUFhxX0NlWm1UeTZ0Wjh0ZzBBSkg0RnVB_ox07.setHtml(f.read())
        self.main_layout.addLayout(self.headerLayout)
        self.main_layout.addWidget(self.Ox8r0_ZUp3ckVWUFhxX0NlWm1UeTZ0Wjh0ZzBBSkg0RnVB_ox07)
        self.main_layout.addLayout(self.buttonsLayout)
        self.resize(800, 600)
        self.main_layout.setSpacing(10)
        self.main_layout.setContentsMargins(10, 10, 10, 10)

    def on_accept(self, *args, **kwargs):
        Ox8r0_ZUp6TEZ0ZTI4UXlaeGFmMjl2ZzhzMVgxYWE5blNDMEdBRTZwQ0t3_ox07()
        self.close()

    def on_decline(self, *args, **kwargs):
        self.close()

class WelcomeBackWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(WelcomeBackWindow, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools Welcome Back!', rootWidget=rootWidget, objectName='BroWelcomeWindow_03', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.main_layout)
        self.title = BroTitle('Welcome back!')
        self.text = QtWidgets.QTextBrowser()
        self.text.setHtml('BroDynamics 2.x to 2021.x is <strong>not a free upgrade</strong>, but as an existing customer you are eligible for a discount. Whether you choose Subscription or Lifetime license - it\'s <strong>20% off</strong>.\n\nHere\'s your discount code: <code>2to2021</code>\n\nYou can use these links to purchase BroDynamics with your discount applied:\n\n- <a href="https://nix.gumroad.com/l/BroDynamics/2to2021"><font color="#faa300">BroDynamics Subscription</font></a>\n- <a href="https://nix.gumroad.com/l/BroDynamics_Lifetime/2to2021"><font color="#faa300">BroDynamcis Lifetime</font></a>\n\n<strong>What you need to know:</strong>\n\n- BroDynamics starting with version 2021.x now offers both Lifetime and Subscription licenses. You can read more about available options and their differences and limitations <a href="https://docs.brotools.tech/licensing.html"><font color="#faa300">here</font></a>.\n- Every existing BroDynamics 2.x customer receives an extended <strong>90 day trial for the entire BroTools collection</strong> as well as a <strong>20% discount for BroDynamics</strong> which applies both to subscription and lifetime license\n- You can continue using BroDynamics 2.x indefinitely, but it will no longer receive any updates. Last supported Maya version for BroDynamics 2.x is Maya 2020.\n\nYou can view changelog for the new version <a href="https://api.brotools.tech/changelog"><font color="#faa300">here</font></a>\n\nIf you would rather continue using the old version - you can revert now by clicking "Revert" and then restarting Maya, or you can do it later - a \'Downgrade to BroDynamics 2.x\' menu item is also available under BroTools menu.\n\nThank you,\nMichael Davydov'.replace('\n', '<br>'))
        self.text.setStyleSheet('\n        QTextBrowser {font-size:16px;background-color: rgb(35,35,35);border: 0px solid black;}\n        ')
        self.text.setOpenExternalLinks(True)
        self.title.setWordWrap(False)
        self.title.setAlignment(QtCore.Qt.AlignHCenter)
        self.imagePath = img('logo_128')
        self.imageLabel = QtWidgets.QLabel(self)
        self.imageMap = QtGui.QPixmap(self.imagePath)
        self.imageLabel.setPixmap(self.imageMap)
        self.imageLabel.setAlignment(QtCore.Qt.AlignHCenter)
        self.headerLayout = QtWidgets.QVBoxLayout()
        self.headerLayout.addWidget(self.imageLabel)
        self.headerLayout.addWidget(self.title)
        self.headerLayout.setAlignment(QtCore.Qt.AlignCenter)
        self.headerLayout.addWidget(self.text)
        self.revertBtn = QtWidgets.QPushButton('Revert')
        self.revertBtn.clicked.connect(partial(self.on_revert))
        self.acceptBtn = QtWidgets.QPushButton('Accept')
        self.acceptBtn.clicked.connect(partial(self.on_accept))
        self.buttonsLayout = QtWidgets.QHBoxLayout()
        self.buttonsLayout.addWidget(self.acceptBtn)
        self.buttonsLayout.addWidget(self.revertBtn)
        self.main_layout.addLayout(self.headerLayout)
        self.main_layout.addLayout(self.buttonsLayout)
        self.resize(800, 850)
        try:
            if cmds.about(q=True, win=True):
                import ctypes
                user32 = ctypes.windll.user32
                screensize = (user32.GetSystemMetrics(0), user32.GetSystemMetrics(1), user32.GetSystemMetrics(78))
            self.resize(800, int((screensize[1] * 0.8)))
        except Exception as e:
            log.info('Failed determining screen size to adjust the welcome window height: {}'.format(str(e)))
        self.main_layout.setSpacing(10)
        self.main_layout.setContentsMargins(10, 10, 10, 10)

    def on_info(self, *args, **kwargs):
        open_web_url('https://brotools.tech/blog/brotools_release.html')

    def on_accept(self, *args, **kwargs):
        show_welcome_window()
        self.close()
    legacy_update_filepath = os.path.join(APP_DIR, 'BroTools_BroDynamics_2.x.zip')

    @staticmethod
    def revert_version(*args, **kwargs):
        confirm = cmds.confirmDialog(title='Confirm downgrade', message='This action will downgrade your current BroTools and BroDynamics installation to BroDynamics 2.x. Would you like to proceed?', button=['Yes', 'No'], defaultButton='Yes', cancelButton='No', dismissString='No')
        if (confirm != 'Yes'):
            log.info('User canceled reverting.')
            return
        if (not DEV_MODE):
            zip_file = zipfile.ZipFile(WelcomeBackWindow.legacy_update_filepath)
            zip_file.extractall(APP_DIR)
            zip_file.close()
        else:
            print("Would extract BroTools_BroDynamics_2.x.zip, but we're in dev mode. Skip.")
            time.sleep(3)
        NotificationWindow(title='Version reverted to BroDynamics 2.x', text='Please, restart Maya to complete the downgrade.').show()

    @staticmethod
    def revert_available(*args, **kwargs):
        return os.path.exists(WelcomeBackWindow.legacy_update_filepath)

    @generic_function_wrap
    def on_revert(self):
        self.revertBtn.setText('Extracting...')
        self.revertBtn.setEnabled(False)
        self.acceptBtn.setEnabled(False)
        self.repaint()
        self.revert_version()
        self.revertBtn.setText('Done!')
        self.close()

class NotificationWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        text = kwargs.pop('text', '')
        title = kwargs.pop('title', 'BroTools Notification')
        copy_text = kwargs.pop('copy_text', None)
        super(NotificationWindow, self).__init__(*args, qss=bro_qss, title=title, **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.title = BroTitle(title)
        self.text = BroText(text)
        self.text.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
        self.mainLayout.addWidget(self.title)
        self.mainLayout.addWidget(self.text)
        if (copy_text is not None):
            log.info('({}): {}'.format(len(copy_text), copy_text))
            for (c_title, c_text) in copy_text:
                if (c_title is not None):
                    c_title = BroTitle(c_title)
                    self.mainLayout.addWidget(c_title)
                ct = QtWidgets.QLineEdit()
                ct.setText(c_text)
                ct.setReadOnly(True)
                self.mainLayout.addWidget(ct)
        self.mainLayout.addStretch()
        self.declineBtn = QtWidgets.QPushButton('Decline')
        self.declineBtn.clicked.connect(partial(self.on_decline))
        self.acceptBtn = QtWidgets.QPushButton('Accept and Proceed to License Manager')
        self.acceptBtn.clicked.connect(partial(self.on_accept))
        self.buttonsLayout = QtWidgets.QHBoxLayout()
        self.buttonsLayout.addWidget(self.acceptBtn)
        self.buttonsLayout.addWidget(self.declineBtn)
        self.mainLayout.setSpacing(10)
        self.mainLayout.setContentsMargins(10, 10, 10, 10)
        self.resize(640, 100)

    def add_action(self, text, func):
        btn = QtWidgets.QPushButton(text)
        self.mainLayout.addWidget(btn)
        btn.clicked.connect(func)

    def closeEvent(self, evt):
        'Hide the QDockWidget and trigger the closeEventTriggered signal\n        '
        if evt.isAccepted():
            log.info('NotificationWindow closeEvent: {}'.format(evt))
            self.deleteLater()

    def show_delayed(self, delay=1500):
        timer = QtCore.QTimer(self)
        timer.timeout.connect(self.show)
        timer.start(delay)

    def on_accept(self, *args, **kwargs):
        Ox8r0_ZUp6TEZ0ZTI4UXlaeGFmMjl2ZzhzMVgxYWE5blNDMEdBRTZwQ0t3_ox07()
        self.close()

    def on_decline(self, *args, **kwargs):
        self.close()

def smart_remove(filepath):
    "\n    Use deleteme tag and rename file instead of deleting if it's a dll. Otherwise just delete a file.\n    If file does not exist, ignore.\n    "
    if os.path.exists(filepath):
        (no_ext, ext) = os.path.splitext(filepath)
        if (ext in ['.pyd', '.so']):
            i = 0
            destpath = ((((no_ext + '_deleteme_') + ext) + '.') + str(i))
            while os.path.exists(destpath):
                i += 1
                destpath = ((((no_ext + '_deleteme_') + ext) + '.') + str(i))
            os.rename(filepath, destpath)
        else:
            os.remove(filepath)

def zipFolder(folder, archive, path_root, comment=None):
    "\n    'fileList' is a list of file names - full path each name\n    'archive' is the file name for the archive with a full path\n    "
    fileList = []
    for (root, dirs, files) in os.walk(folder, topdown=False):
        for name in files:
            filepath = os.path.join(root, name)
            if ('backups' not in filepath):
                fileList.append(filepath)
    if os.path.exists(archive):
        os.remove(archive)
    a = zipfile.ZipFile(archive, 'w', zipfile.ZIP_DEFLATED)
    (yield 0)
    for (i, f) in enumerate(fileList):
        a.write(f, os.path.relpath(f, path_root))
        (yield int(((i / len(fileList)) * 100)))
    if comment:
        a.comment = comment.encode('utf-8')
    a.close()
    (yield 100)

class DownloaderThread(QtCore.QThread):

    def __init__(self, parent=None, overlay=None):
        QtCore.QThread.__init__(self, parent)
        self.exiting = False
        self.signal_completed = BoolSignal()
        self.signal_error = DictSignal()
        self.signal_progress = DictSignal()
        self.parent = parent
        self.overlay = overlay
        self.auth_hid = None
        self.total = None
        self.current = 0

    def percent(self, string=False):
        p = int(((float(self.current) / float(self.total)) * 100))
        self.current += 1
        if string:
            return '{}%'.format(p)
        else:
            return p

    def clear_empty_folders(self, *args, **kwargs):
        log.debug('Action run: clear_empty_folders')
        for (root, dirs, files) in os.walk(APP_DIR, topdown=False):
            for name in dirs:
                dirpath = os.path.join(root, name)
                if (len(os.listdir(dirpath)) == 0):
                    if (not DEV_MODE):
                        log.debug('REMOVE EMPTY FOLDER: {}'.format(dirpath))
                    else:
                        log.debug('WOULD REMOVE EMPTY FOLDER (DEV LOCK): {}'.format(dirpath))

    def run(self):
        try:
            self.overlay.text = 'Contacting server...'
            self.auth_hid = get_hid()
            sync_data = broapi.get_sync_data(auth_hid=self.auth_hid, target_hid=Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.target_hid, email=Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['customer_id'], client_files_data=json.dumps(get_client_files_data()))
            log.debug(sync_data)
            if ('status' not in sync_data):
                self.signal_error.sig.emit({'message': 'Unable to get sync data: {}'.format(sync_data['message']), 'traceback': ''})
                log.warning('Unable to get sync data, server error. \n{}'.format(sync_data))
                return
            if (sync_data['status'] != 'success'):
                self.signal_error.sig.emit({'message': 'Unable to get sync data: {}'.format(sync_data['message']), 'traceback': ''})
                log.warning('Unable to get sync data: {}'.format(sync_data['message']))
                return
            self.total = ((len(sync_data['data']['download']) + len(sync_data['data']['remove'])) + len(sync_data['data']['extra_actions']))
            zipname = 'BroTools_Backup_v{}_d{}.zip'.format(BroTools.__version__, datetime.now().strftime('%Y.%m.%d.%H.%m.%S'))
            zippath = os.path.join(backups_dir, zipname)
            self.overlay.text = 'Backing up current version...\n{}'.format(zippath)
            for p in zipFolder(APP_DIR, zippath, os.path.abspath(os.path.join(APP_DIR, '..')), comment='Backup created at {}\nTo recover from a broken update, please, completely remove your existing BroTools folder and replace with the contents of this archive.'.format(str(datetime.now()))):
                self.overlay.completion = p
            self.current = 0
            self.overlay.completion = 0
            if ('pre_update_script' in sync_data['data']):
                self.overlay.text = 'Running pre-update script...'
                try:
                    exec(sync_data['data']['pre_update_script'])
                except Exception as e:
                    self.signal_error.sig.emit({'message': 'Failed to run pre-update script: {}'.format(str(e)), 'traceback': traceback.format_exc()})
            if DEV_MODE:
                sync_data['data']['download'].extend(['fake_file_1', 'fake_file_2', 'fake_file_3', 'fake_file_4'])
            for relpath in sync_data['data']['download']:
                filepath = os.path.normpath(os.path.join(APP_DIR, relpath))
                try:
                    if (not DEV_MODE):
                        log.debug('DOWNLOADING: {} into {}'.format(relpath, filepath))
                        self.overlay.text = 'DOWNLOADING:\n {}'.format(filepath)
                        self.overlay.completion = self.percent()
                        for file_download_percent in post_stream((broapi.endpoint + '/updater/download'), {'relpath': relpath, 'auth_hid': self.auth_hid, 'target_hid': Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.target_hid, 'email': Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['customer_id']}, filepath):
                            self.overlay.part_completion = file_download_percent
                    else:
                        log.debug('WOULD DOWNLOAD (DEV LOCK): {} into {}'.format(relpath, filepath))
                        self.overlay.text = 'FAKE DOWNLOADING:\n {}'.format(filepath)
                        self.overlay.completion = self.percent()
                        for i in range(100):
                            self.overlay.part_completion = i
                            time.sleep(0.1)
                except Exception as e:
                    print(('EMIT ERROR', e))
                    self.signal_error.sig.emit({'message': str(e), 'traceback': traceback.format_exc()})
            try:
                for relpath in sync_data['data']['remove']:
                    filepath = crossplatform_normpath(os.path.join(APP_DIR, relpath))
                    self.overlay.text = 'REMOVING:\n {}'.format(filepath)
                    self.overlay.completion = self.percent()
                    if (not DEV_MODE):
                        log.debug('REMOVE: {}'.format(filepath))
                        smart_remove(filepath)
                    else:
                        log.debug('WOULD REMOVE (DEV LOCK): {}'.format(filepath))
                        time.sleep(0.001)
                for funcname in sync_data['data']['extra_actions']:
                    action = getattr(self, funcname)
                    action()
            except Exception as e:
                self.signal_error.sig.emit({'message': str(e), 'traceback': traceback.format_exc()})
            if ('post_update_script' in sync_data['data']):
                self.overlay.text = 'Running post-update script...'
                try:
                    exec(sync_data['data']['post_update_script'])
                except Exception as e:
                    self.signal_error.sig.emit({'message': 'Failed to run post-update script: {}'.format(str(e)), 'traceback': traceback.format_exc()})
            self.signal_completed.sig.emit(True)
        except Exception as e:
            self.signal_error.sig.emit({'message': str(e), 'traceback': traceback.format_exc()})

class UpdaterWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(UpdaterWindow, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools Updater', rootWidget=rootWidget, objectName='BroUpdaterWindow', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.mainLayout.setSpacing(10)
        self.mainLayout.setContentsMargins(10, 10, 10, 10)
        self.syncAlertTitle = BroTitle('')
        self.syncAlertTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.syncAlert = BroText('...')
        self.syncAlert.setAlignment((QtCore.Qt.AlignCenter | QtCore.Qt.AlignTop))
        self.syncAlert.setMaximumHeight(800)
        self.syncButton = QtWidgets.QPushButton(('Update' if (DEV_MODE is False) else 'Update (Dev Mode)'))
        self.syncButton.clicked.connect(partial(self.perform_sync))
        self.closeButton = QtWidgets.QPushButton('Close updater')
        self.closeButton.clicked.connect(partial(self.close))
        self.closeButton.hide()
        self.mainLayout.addStretch()
        self.mainLayout.addWidget(self.syncAlertTitle)
        self.mainLayout.addWidget(self.syncAlert)
        self.mainLayout.addWidget(self.syncButton)
        self.mainLayout.addWidget(self.closeButton)
        self.mainLayout.addStretch()
        self.overlay = OverlayProgress(self)
        self.overlay.hide()
        self.current = 0
        self.total = 0
        self.sync_errors = []
        self.resize(640, 200)

    def close(self, *args, **kwargs):
        super(UpdaterWindow, self).close()

    def on_update_check(self, data, *args, **kwargs):
        update_required = data['update_required']
        if (update_required == True):
            self.syncButton.show()
            self.syncAlert.show()
            self.syncAlertTitle.setText('There is an update for you!')
            self.syncAlert.setText('Your BroTools installation is missing files for some of the installed tools and needs to be updated.')
        else:
            self.syncAlertTitle.setText('You are up to date.')
            self.syncAlert.setText('No further actions are required, your BroTools installation is up to date.')
            self.closeButton.show()

    def on_update_check_error(self, data, *args, **kwargs):
        self.syncAlertTitle.setText('Unable to check for updates :(')
        self.syncAlert.setText(data['message'])
        self.syncAlert.show()
        log.warning(data['message'], data['traceback'])

    def on_progress(self, data, *args, **kwargs):
        self.syncAlert.show()
        self.syncAlert.setText('{}%, {}'.format(nice_number_string(int(data['percent'])), data['status']))

    def integrity_check(self, text='Please, wait, checking if you need an update...', on_complete=None):
        self.syncAlertTitle.setText(text)
        self.syncButton.hide()
        self.syncAlert.hide()
        self.t = CheckFileSyncThread()
        self.t.signal_progress.sig.connect(partial(self.on_progress))
        self.t.signal_completed.sig.connect(partial(on_complete))
        self.t.signal_error.sig.connect(partial(self.on_update_check_error))
        self.t.start()

    def show(self, *args, **kwargs):
        broapi_endpoint.force_update_endpoint()
        if ((Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists is True) and (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.validated_verified is True)):
            self.integrity_check(on_complete=self.on_update_check)
        elif (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists is not True):
            self.syncAlertTitle.setText('License key does not exist. Please, use License Manager first.')
            self.syncButton.hide()
        elif (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.validated_verified is not True):
            self.syncAlertTitle.setText('Invalid license key. {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
            self.syncButton.hide()
        super(UpdaterWindow, self).show(*args, **kwargs)

    def resizeEvent(self, event):
        self.overlay.resize(event.size())
        event.accept()

    def perform_sync(self, *args, **kwargs):
        self.overlay.show()
        self.sync_errors = []
        self.loaderThread = DownloaderThread(self, overlay=self.overlay)
        self.loaderThread.signal_progress.sig.connect(partial(self.on_progress))
        self.loaderThread.signal_completed.sig.connect(partial(self.on_completed))
        self.loaderThread.signal_error.sig.connect(partial(self.on_record_error))
        self.loaderThread.start()

    def on_completed(self, *args, **kwargs):
        data = args[0]
        self.overlay.hide()
        if (len(self.sync_errors) > 0):
            it = ''
            for (i, se) in enumerate(self.sync_errors):
                it += ((('# ' + str(i)) + '. ') + str(se['message']))
                if ('traceback' in se):
                    it += (('\n\n```\n ' + str(se['traceback']).strip()) + '\n```')
                it += '\n'
            show_error('Sync completed with errors!', informativeText=it, icn='Warning')
            self.syncAlertTitle.setText('Something went wrong during an update!')
            self.syncButton.hide()
            self.syncAlert.show()
            self.syncAlert.setText('Please, try again later, or try a manual update.')
        else:
            log.info('Sync completed.')
            if (len(self.sync_errors) == 0):
                self.integrity_check(text='Checking if update went smoothly...', on_complete=self.please_restart_message)

    @catch_errors
    def please_restart_message(self, data, *args, **kwargs):
        update_required = data['update_required']
        if (update_required is False):
            self.syncAlertTitle.setText('Update was successful. Please, reload BroTools from the menu or restart Maya to apply it.')
            self.syncButton.hide()
            self.syncAlert.hide()
        else:
            self.syncAlertTitle.setText('It looks like we were unable to complete the update.')
            self.syncButton.hide()
            self.syncAlert.show()
            self.syncAlert.setText("This means that some files are still not as they should be. This can happen if:\n            \n            - There was an error during the update\n            - There was a network error during the update\n            - Some files were corrupted during download\n            - There was permission error while trying to replace some files\n            - There was a new update released while we were downloading previous one (gotta be very lucky!)\n            \n            Please, try again later. If this keeps happening, try a manual update or contact developer.\n            \n            Here's a list of changes we were unable to perform:\n            \n            Replace {} files\n            Delete {} files".format(len(data['sync_data']['download']), len(data['sync_data']['remove'])))
            show_error('It looks like we were unable to complete the update.', informativeText="This means that some files are still not as they should be. This can happen if:\n            \n            - There was an error during the update\n            - There was a network error during the update\n            - Some files were corrupted during download\n            - There was permission error while trying to replace some files\n            \n            Please, try again in a few minutes. If this keeps happening, try a manual update or contact developer.\n            \n            Here's a list of changes we were unable to perform:\n            \n            Replace files: {}\n            Delete files: {}".format(data['sync_data']['download'], data['sync_data']['remove']), icon='Warning')

    def on_record_error(self, *args, **kwargs):
        print(('on_record_error ERROR!!!!', args, kwargs))
        data = args[0]
        self.sync_errors.append(data)

    def on_error(self, *args, **kwargs):
        print(('on_error ERROR!!!!', args, kwargs))
        data = args[0]
        self.overlay.hide()
        self.sync_errors.append(data)
        self.syncAlertTitle.setText('Something went wrong during an update!')
        self.syncButton.hide()
        self.syncAlert.show()
        it = ''
        for (i, se) in enumerate(self.sync_errors):
            it += ((('# ' + str(i)) + '. ') + str(se['message']))
            if ('traceback' in se):
                it += (('\n\n```\n ' + str(se['traceback']).strip()) + '\n```')
            it += '\n'
        self.syncAlert.setText(it)
        show_error('Sync completed with errors!', informativeText=it, icon='Warning')

class Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJlaVU5bUN5d0RBRXgxQ0Zr_ox07(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        try:
            super(Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJlaVU5bUN5d0RBRXgxQ0Zr_ox07, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools License Manager', rootWidget=rootWidget, objectName='BroLicenseManager', **kwargs)
            self.tasks = {'Install': broapi.install, 'Uninstall': broapi.uninstall, 'Reinstall Here': broapi.reinstall}
            self.setWindowFlags(QtCore.Qt.Dialog)
            self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
            self.mainLayout = QtWidgets.QVBoxLayout()
            self.setLayout(self.mainLayout)
            self.mainLayout.setSpacing(0)
            self.mainLayout.setContentsMargins(0, 0, 0, 0)
            self.headerWidget = QtWidgets.QWidget()
            self.headerWidget.setObjectName('LicenseHeaderWidget')
            self.headerWidget.setMinimumHeight(50)
            self.headerWidget.setMaximumHeight(350)
            self.headerWidget.setStyleSheet('QWidget {background: rgba(0,0,0,0.1);margin: -1;padding: 10px;}')
            self.headerLayout = QtWidgets.QVBoxLayout()
            self.headerLayout.setSpacing(0)
            self.headerLayout.setContentsMargins(0, 0, 0, 0)
            self.headerWidget.setLayout(self.headerLayout)
            self.midWidget = Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJ4dVUtNmVUY3NQZk5RLVpteUxnQ1piZ3di_ox07(rootWidget=self)
            self.centralLayout = QtWidgets.QVBoxLayout()
            self.midWidget.setLayout(self.centralLayout)
            self.navigationWidget = QtWidgets.QWidget()
            self.navigationLayout = QtWidgets.QHBoxLayout()
            self.navigationWidget.setFixedHeight(100)
            self.navigationWidget.setLayout(self.navigationLayout)
            self.mainLayout.addWidget(self.headerWidget)
            self.mainLayout.addWidget(self.midWidget)
            self.mainLayout.addWidget(self.navigationWidget)
            self.headerTitle = QtWidgets.QLabel('Loading...')
            self.headerTitle.setObjectName('LicenseHeader')
            self.headerDescription = BroText('Loading...')
            self.headerAlert = BroTitle('')
            self.headerAlert.setStyleSheet('QLabel {color: red;font-size:15px;}')
            self.headerAlert.hide()
            self.headerLayout.addWidget(self.headerTitle)
            self.headerLayout.addWidget(self.headerDescription)
            self.headerLayout.addWidget(self.headerAlert)
            self.nextButton = QtWidgets.QPushButton('Next')
            self.nextButton.clicked.connect(partial(self.nextButtonClicked))
            self.nextButton.setFixedWidth(200)
            self.navigationLayout.addStretch()
            self.navigationLayout.setContentsMargins(0, 0, 20, 0)
            if (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07):
                self.navigationLayout.addWidget(self.nextButton)
            self.overlay = Overlay(self)
            self.overlay.hide()
            self.installer_data = None
            self.resize(800, 600)
            self.showFirstPage()
        except Exception as e:
            traceback.print_exc()

    def showFirstPage(self, *args, **kwargs):
        self.midWidget.setCurrentIndex(0)
        self.nextButton.setText('Next')
        self.headerTitle.setText(('Select license type' if (Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 is False) else 'Floating license info'))
        self.headerDescription.setText(('Please, select type of your license. Workstation is best suited for individual installations. Network license is best suited for studios.' if (Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07 is False) else 'Here you can see information about current status of TurboFloat license server and lease.'))
        self.nextButton.setEnabled(True)

    def resizeEvent(self, event):
        self.overlay.resize(event.size())
        event.accept()

    def showAlert(self, text, style='critical'):
        styles = {'critical': 'QLabel {color: red;font-size:15px;}', 'warning': 'QLabel {color: yellow;font-size:15px;}', 'info': 'QLabel {color: green;font-size:15px;}', 'success': 'QLabel {color: green;font-size:15px;}'}
        self.headerAlert.setStyleSheet(styles[style])
        self.headerAlert.setText(text)
        self.headerAlert.show()

    def hideAlert(self):
        self.headerAlert.setText('')
        self.headerAlert.hide()

    @property
    def email(self):
        return self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.email.text()

    @property
    def networked(self):
        return (self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.selection == 1)

    @property
    def floating(self):
        return (self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.selection == 2)

    @property
    def host(self):
        return self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.ip.currentText()

    @property
    def floating_server_address(self):
        return self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.ipFloating.text()

    @property
    def auth_hid(self):
        return get_hid()

    @property
    def target_hid(self):
        if (self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.selection == 1):
            target_hid = get_hid(self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.ip.currentText())
        else:
            target_hid = get_hid()
        return target_hid

    def nextButtonClicked(self, *args, **kwargs):
        try:
            self.hideAlert()
            if (self.midWidget.currentIndex() == 0):
                if self.floating:
                    data_cache.set('turbofloat_server', self.floating_server_address)
                    self.showAlert('Floating license server address is set. Please Restart BroTools to apply changes.', style='success')
                    return False
                self.overlay.show()
                broapi_endpoint.force_update_endpoint()
                (test, message, details) = broapi_endpoint.test_connection()
                if (not test):
                    self.overlay.hide()
                    Ox8r0_ZUp5cmxERlFTb2hzWjlGNS0zby14LVo2QUNrQkJnaw_ox07 = {'target_hid': self.target_hid, 'auth_hid': get_hid(), 'host': (self.host if (self.networked is True) else None), 'networked': self.networked, 'email': self.email, 'hostname': gethostname()}
                    ct = [['Activation Code', 'abt.{}.sfti'.format(encode(json.dumps(Ox8r0_ZUp5cmxERlFTb2hzWjlGNS0zby14LVo2QUNrQkJnaw_ox07), nlib64.paths.cpath()))]]
                    w = NotificationWindow(title='Unable to contact licensing server', text='BroTools is unable to reach the licensing server.\n\nPlease, make sure that Maya has access to the internet and is not blocked by your firewall, and that you can open https://api.brotools.tech in your browser.    \n\nIt is possible that servers are undergoing maintenance, in which case you might wait 10 minutes and try again.\nAlternatively you can try offline activation, generate license file by visiting {}.'.format(URL_CLIENT_DASHBOARD), copy_text=ct)
                    w.show()
                    return False
                self.loaderThread = LoaderThread(self, function=self.Ox8r0_ZUp3ckVWRFZqd3Fjd0tINzRlSW05aTFabWM4bVN5MEdBRTFoQ0lB_ox07)
                self.loaderThread.signal_completed.sig.connect(partial(self.Ox8r0_ZUp6TEZ0ZTJpUXFjd0tINzRlSW05aTFabWM4bVN5MEdBRTU3Q0pF_ox07))
                self.loaderThread.signal_failed.sig.connect(partial(self.Ox8r0_ZUp3ckYtUlFLWS1aSmlfLS10Zzhqc1VSbVk4blMyMEZBRXVwQ0ZR_ox07))
                self.loaderThread.signal_rejected.sig.connect(partial(self.hide_overlay))
                self.loaderThread.start()
            elif (self.midWidget.currentIndex() == 1):
                self.overlay.show()
                self.loaderThread = LoaderThread(self, function=self.Ox8r0_ZUp3ckVWUFhxX0NlcHFEMDl1b2NucTEtdVQ5bUNpeGVjdjZseVN0VEFKRllERWc_ox07)
                self.loaderThread.signal_completed.sig.connect(partial(self.Ox8r0_ZUp6TEZ0ZTJDUXB2WWRaX2MzVS16MVlBSWRjRmFn_ox07))
                self.loaderThread.signal_failed.sig.connect(partial(self.Ox8r0_ZUp3ckYtUlFLWS1aSmlfLS10Zzhqc1VSbVk4blMyMEZBRXVwQ0ZR_ox07))
                self.loaderThread.signal_rejected.sig.connect(partial(self.hide_overlay))
                self.loaderThread.start()
        except Exception as e:
            self.overlay.hide()
            self.showAlert(str(e))
            log.error(e, exc_info=True)

    def hide_overlay(self, *args, **kwargs):
        self.overlay.hide()

    def Ox8r0_ZUp3ckYtUlFLWS1aSmlfLS10Zzhqc1VSbVk4blMyMEZBRXVwQ0ZR_ox07(self, message, *args, **kwargs):
        if message:
            text = message[0]
            traceback_text = message[1]
            self.overlay.hide()
            self.showAlert(str(text))
            log.error(text, exc_info=True, traceback_text=traceback_text)

    def Ox8r0_ZUp3ckVWRFZqd3Fjd0tINzRlSW05aTFabWM4bVN5MEdBRTFoQ0lB_ox07(self):
        if self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlWm1uMS1lSWFoc1dWdWY4QU9Xb0hvdw_ox07.can_proceed:
            installer_data = broapi.get_licman_intaller_data(self.email, self.auth_hid, self.networked, self.host)
            if (installer_data['status'] != 'success'):
                self.showAlert(installer_data['message'])
                return False
            self.installer_data = installer_data['data']
            return True
        return False

    def Ox8r0_ZUp6TEZ0ZTJpUXFjd0tINzRlSW05aTFabWM4bVN5MEdBRTU3Q0pF_ox07(self, *args, **kwargs):
        log.debug(('Ox8r0_ZUp6TEZ0ZTJpUXFjd0tINzRlSW05aTFabWM4bVN5MEdBRTU3Q0pF_ox07', args, kwargs))
        self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlcHFMNzRjQXMxdFhWVlhlblNXOWI4eDRBWTlFS1dB_ox07.populate(self.installer_data)
        self.midWidget.setCurrentIndex(1)
        self.nextButton.setText('Next')
        self.headerTitle.setText('Manage')
        self.headerDescription.setText('Here you can manage your Tools and licenses - Install and Uninstall them, see how many licenses of each tool you have available.')
        self.overlay.hide()

    def Ox8r0_ZUp3ckVWUFhxX0NlcHFEMDl1b2NucTEtdVQ5bUNpeGVjdjZseVN0VEFKRllERWc_ox07(self):
        root = self.midWidget.Ox8r0_ZUp3ckVWUFhxX0NlcHFMNzRjQXMxdFhWVlhlblNXOWI4eDRBWTlFS1dB_ox07.treeView
        child_count = root.topLevelItemCount()
        for i in range(child_count):
            item = root.topLevelItem(i)
            log.debug('{}. Task: {}, {};'.format(i, item.selected_action, item.licid))
            if (item.selected_action is not None):
                if (item.selected_action in self.tasks):
                    r = self.tasks[item.selected_action](licid=item.licid, auth_hid=self.auth_hid, target_hid=self.target_hid, email=self.email, host=(self.host if self.networked else None))
                    log.debug(r)
        log.debug('Updating license key...')
        Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp5ckVUQldyUXFmeENuejRtSVRBQmp0Qktn_ox07(self.target_hid, self.email, self.networked, download_reason='License manager performed tasks')
        return True

    def Ox8r0_ZUp6TEZ0ZTJDUXB2WWRaX2MzVS16MVlBSWRjRmFn_ox07(self, *args, **kwargs):
        log.debug('Ox8r0_ZUp6TEZ0ZTJDUXB2WWRaX2MzVS16MVlBSWRjRmFn_ox07')
        self.overlay.hide()
        show_updater()
        self.close()

class AboutWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(AboutWindow, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='About BroTools', rootWidget=rootWidget, objectName='BroAboutWindow', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.setMinimumWidth(450)
        self.setMinimumHeight(300)
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.mainLayout.setContentsMargins(20, 20, 20, 20)
        self.mainLayout.setSpacing(15)
        self.headerLayout = QtWidgets.QHBoxLayout()
        self.directory = os.path.dirname(os.path.abspath(__file__))
        self.imagePath = img('logo_128')
        self.logoLabel = QtWidgets.QLabel(self)
        self.logoMap = QtGui.QPixmap(self.imagePath)
        self.logoLabel.setPixmap(self.logoMap)
        self.logoLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.titleLayout = QtWidgets.QVBoxLayout()
        self.titleLayout.setSpacing(5)
        self.titleLabel = QtWidgets.QLabel('BroTools')
        self.titleLabel.setObjectName('LicenseHeader')
        self.titleLabel.setAlignment((QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter))
        self.versionLabel = QtWidgets.QLabel(f'Version {BroTools.__version__}')
        self.versionLabel.setStyleSheet('font-size: 14px; color: #c99126;')
        self.versionLabel.setAlignment((QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter))
        self.titleLayout.addWidget(self.titleLabel)
        self.titleLayout.addWidget(self.versionLabel)
        self.titleLayout.addStretch()
        self.headerLayout.addWidget(self.logoLabel)
        self.headerLayout.addLayout(self.titleLayout)
        self.headerLayout.addStretch()
        self.infoContainer = QtWidgets.QWidget()
        self.infoContainer.setObjectName('LicenseHeaderWidget')
        self.infoLayout = QtWidgets.QVBoxLayout(self.infoContainer)
        self.infoLayout.setSpacing(10)
        self.copyrightLabel = QtWidgets.QLabel(f'© {datetime.now().year} Michael Davydov')
        self.copyrightLabel.setStyleSheet('font-size: 13px; background-color: #1A1A1A;')
        self.systemLabel = QtWidgets.QLabel(f'System library version: {__version__}')
        self.systemLabel.setStyleSheet('font-size: 12px; color: #a0a0a0; background-color: #1A1A1A;')
        self.websiteLabel = QtWidgets.QLabel('<a href="https://brotools.tech" style="color: #c99126; text-decoration: none;">brotools.tech</a>')
        self.websiteLabel.setOpenExternalLinks(True)
        self.websiteLabel.setStyleSheet('font-size: 13px; background-color: #1A1A1A;')
        self.portfolioLabel = QtWidgets.QLabel('<a href="https://michaeldavydov.com" style="color: #c99126; text-decoration: none;">michaeldavydov.com</a>')
        self.portfolioLabel.setOpenExternalLinks(True)
        self.portfolioLabel.setStyleSheet('font-size: 13px; background-color: #1A1A1A;')
        self.infoLayout.addWidget(self.copyrightLabel)
        self.infoLayout.addWidget(self.websiteLabel)
        self.infoLayout.addWidget(self.portfolioLabel)
        self.infoLayout.addWidget(self.systemLabel)
        self.buttonLayout = QtWidgets.QHBoxLayout()
        self.buttonLayout.setContentsMargins(0, 10, 0, 0)
        self.closeButton = QtWidgets.QPushButton('Close')
        self.closeButton.setFixedWidth(100)
        self.closeButton.clicked.connect(self.close)
        self.buttonLayout.addStretch()
        self.buttonLayout.addWidget(self.closeButton)
        self.mainLayout.addLayout(self.headerLayout)
        self.mainLayout.addWidget(self.infoContainer)
        self.mainLayout.addStretch()
        self.mainLayout.addLayout(self.buttonLayout)
        self.setLayout(self.mainLayout)

class BugReportWindow(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(BugReportWindow, self).__init__(*args, icon=icon('bug', size=32), qss=bro_qss, title='BroTools Bug Report', rootWidget=rootWidget, objectName='BroBugReportWindow', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.stackedLayout = QtWidgets.QStackedLayout()
        self.loaderWidget = QtWidgets.QWidget()
        self.reportWidget = QtWidgets.QWidget()
        self.loaderLayout = QtWidgets.QVBoxLayout()
        self.reportLayout = QtWidgets.QVBoxLayout()
        self.loaderWidget.setLayout(self.loaderLayout)
        self.reportWidget.setLayout(self.reportLayout)
        self.loaderLabel = BroTitle('Communicating with the server... \n This may take a minute.')
        self.loaderLayout.addWidget(self.loaderLabel)
        self.stackedLayout.addWidget(self.reportWidget)
        self.stackedLayout.addWidget(self.loaderWidget)
        self.formLayout = QtWidgets.QFormLayout()
        self.label = QtWidgets.QLabel('If, for any reason, this form does not work, feel free to contact me via my email: support@brotools.tech.')
        self.description = QtWidgets.QTextEdit()
        self.description.setText('Describe your bug.')
        self.logs = QtWidgets.QTextEdit()
        self.email = QtWidgets.QLineEdit()
        self.email.setPlaceholderText("Leave blank if it's the same email you used for purchase.")
        self.sendBtn = QtWidgets.QPushButton('Send')
        self.sendBtn.clicked.connect(partial(self.send))
        self.reportLayout.addWidget(self.label)
        self.reportLayout.addLayout(self.formLayout)
        self.reportLayout.addWidget(self.sendBtn)
        self.formLayout.addRow('Your email (for response)', self.email)
        self.formLayout.addRow('Description', self.description)
        self.formLayout.addRow('Maya Logs', self.logs)
        if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists:
            self.email.setText(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data.get('customer_id', ''))
        self.setLayout(self.stackedLayout)

    def showEvent(self, *args, **kwargs):
        self.stackedLayout.setCurrentIndex(0)
        if os.path.exists(log_path):
            with open(log_path, 'r') as f:
                logs = f.read()
            if (brotools_config.Debug.write_log_file is False):
                logs = ("Logging to file was disabled. These logs may be outdated. Please, include as much log information as you can from the Script Editor window. Without it it's going to be much harder to locate the problem. n\n Alternatively you can enable logging to file in BroTools - Preferences - Write log to file, and repeat your actions to recreate the problem. \n\n=== LOGS ===\n\n" + str(logs))
            self.logs.setText(logs)
        else:
            self.logs.setText("Logging file does not exist, logging to file was disabled. Please, include as much log information as you can from the Script Editor window. Without it it's going to be much harder to locate the problem. \n\n Alternatively you can enable logging to file in BroTools - Preferences - Write log to file, and repeat your actions to recreate the problem.")

    @catch_errors
    def send(self, *args, **kwargs):
        try:
            self.stackedLayout.setCurrentIndex(1)
            cmds.refresh()
            data = {'Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07': (json.dumps(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data) if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists else 'does not exist'), 'email': self.email.text(), 'description': self.description.toPlainText(), 'log': self.logs.toPlainText(), 'auth_hid': get_hid()}
            self.sendBtn.setEnabled(False)
            response = broapi.post('/bug_report', data).get('message')
            cmds.confirmDialog(title='Bug Report Response', message=str(response), button=['OK'], defaultButton='OK', cancelButton='OK', dismissString='OK')
        except Exception as e:
            raise BroToolsError('Failed to send bug report: {}.\nPlease, send it directly to support@brotools.tech'.format(e))
        finally:
            self.sendBtn.setEnabled(True)
            self.stackedLayout.setCurrentIndex(0)

def nice_number_string(n):
    if (n == 69):
        s = (str(n) + ' (nice)')
    else:
        s = str(n)
    return s

class NewVersionAlert(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(NewVersionAlert, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools Updater', rootWidget=rootWidget, objectName='NewVersionAlert', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.title = BroTitle('New version available!')
        self.text = BroText()
        self.yes_btn = QtWidgets.QPushButton('Yes')
        self.yes_btn.clicked.connect(partial(self.open_updater))
        self.changelog_btn = QtWidgets.QPushButton("View What's New\n(Opens in Browser)")
        self.changelog_btn.clicked.connect(partial(self.changelog))
        self.later_btn = QtWidgets.QPushButton('Later')
        self.later_btn.clicked.connect(partial(self.close))
        self.skip_btn = QtWidgets.QPushButton('Skip Version')
        self.skip_btn.clicked.connect(partial(self.skip))
        for i in [self.title, self.text, self.yes_btn, self.changelog_btn, self.later_btn]:
            self.mainLayout.addWidget(i)
        if Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists:
            self.check_thread = LoaderThread(function=self.check_version)
            self.check_thread.signal_completed.sig.connect(partial(self.on_new_version))
            self.check_thread.signal_rejected.sig.connect(partial(self.on_up_to_date))
            self.check_thread.signal_failed.sig.connect(partial(self.on_error))
            self.check_thread.start()

    def skip(self, *args, **kwargs):
        data_cache.set('skip_update_version', latest_version)
        self.close()

    def on_sync_check_completed(self, data):
        if (data['update_required'] is True):
            print('\n\n# BroTools sync check details:')
            print('Files to download:\n     - ', '\n     - '.join(data['sync_data']['download']))
            print('Files to remove:\n     - ', '\n     - '.join(data['sync_data']['remove']))
            print('\n\n')
            download = len(data['sync_data']['download'])
            remove = len(data['sync_data']['remove'])
            t = ''
            if (download > 0):
                t += 'There are {} files that need to be downloaded. '.format(nice_number_string(len(data['sync_data']['download'])))
            if ((download > 0) and (remove > 0)):
                t += 'And there are {} redundant files that should be cleaned up.'.format(len(data['sync_data']['remove']))
            elif (remove > 0):
                t += 'There are {} redundant files that should be cleaned up.'.format(nice_number_string(len(data['sync_data']['remove'])))
            t += '\nWould you like to proceed?\n\nMore details are available in Script Editor log.\n\nIf you have multiple Mayas open - it may cause issues during the update.\nPlease, close all other Maya instances before starting the update.'
            title = 'BroTools needs to '
            if (download > 0):
                title += 'download '
            if ((download > 0) and (remove > 0)):
                title += 'and '
            if (remove > 0):
                title += 'remove '
            title += 'some files!'
            self.title.setText(title)
            self.text.setText(t)
            self.show()

    def on_sync_check_error(self, data):
        log.debug('on_sync_check_error: {}'.format(str(data)))

    def check_version(self):
        time.sleep(random.randint(5, 10))
        return broapi.is_new_version_available

    def on_error(self, data):
        raise BroToolsError(data[0], traceback_text=data[1])

    def on_new_version(self, *args, **kwargs):
        if (latest_version != data_cache.get('skip_update_version')):
            log.debug('New version available!')
            self.text.setText("Your version: {}\nNew version: {}\n\nWould you like to update?\n\nIf you have multiple Mayas open - it may cause issues during the update.\nPlease, close all other Maya instances before starting the update.\n\n(If you're using a Lifetime license, please note, that they are limited to only receiving updates within 1 year after purchase. If that time has passed you will only be updated to the latest version allowed by your license.)".format(BroTools.__version__, latest_version))
            self.show()
        else:
            log.info('Not showing update notification, because new version is marked for skipping: {}'.format(latest_version))
            self.close()

    def on_up_to_date(self, *args, **kwargs):
        log.debug('BroTools is up to date.')
        self.sync_check_thread = CheckFileSyncThread(delay=random.randint(3, 5))
        self.sync_check_thread.signal_completed.sig.connect(partial(self.on_sync_check_completed))
        self.sync_check_thread.signal_error.sig.connect(partial(self.on_sync_check_error))
        self.sync_check_thread.start()

    def changelog(self, *args, **kwargs):
        open_web_url((API_MAIN_URL + '/changelog'))

    def open_updater(self, *args, **kwargs):
        show_updater()
        self.close()

class ServerNotificationThread(QtCore.QThread):

    def __init__(self, parent=None, function=None):
        QtCore.QThread.__init__(self, parent)
        self.exiting = False
        self.signal_completed = DictSignal()
        self.signal_failed = ListSignal()
        self.signal_rejected = BoolSignal()
        self.parent = parent

    def run(self):
        try:
            time.sleep(5)
            r = broapi.get('/alert')
            if (r.get('status') == 'success'):
                if (r['data'] is not None):
                    if (r['data']['title'] != data_cache.get('last_alert_title')):
                        if ((r['data'].get('version', None) is None) or (r['data']['version'] == BroTools.__version__)):
                            self.signal_completed.sig.emit(r)
                            return
            self.signal_rejected.sig.emit(True)
            return
        except Exception as e:
            self.signal_failed.sig.emit([str(e), str(traceback.format_exc())])
            return

class ServerNotificationAlert(BroSimpleWindow):

    def __init__(self, rootWidget=None, *args, **kwargs):
        super(ServerNotificationAlert, self).__init__(*args, icon=icon('logo', size=32), qss=bro_qss, title='BroTools Important Notification!', rootWidget=rootWidget, objectName='ServerNotificationAlert', **kwargs)
        self.setWindowFlags(QtCore.Qt.Dialog)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self.mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(self.mainLayout)
        self.title = BroTitle('')
        self.text = BroText('')
        self.mainLayout.addWidget(self.title)
        self.mainLayout.addWidget(self.text)
        self.check_thread = ServerNotificationThread()
        self.check_thread.signal_completed.sig.connect(partial(self.on_alert))
        self.check_thread.signal_failed.sig.connect(partial(self.on_error))
        self.check_thread.signal_rejected.sig.connect(partial(self.close))
        self.check_thread.start()

    def close(self, *args, **kwargs):
        super(ServerNotificationAlert, self).close()

    def on_alert(self, r):
        self.title.setText(r['data']['title'])
        self.text.setText(r['data']['text'])
        data_cache.set('last_alert_title', self.title.text())
        self.resize(640, 360)
        self.show()

    def on_error(self, data):
        log.warning(str(data))
        self.close()
MOD_NAME = 'BroTools'
MOD_Prefix = 'Bro'
plugin_path = cmds.pluginInfo((MOD_NAME + '.py'), query=True, path=True)
plugin_dir = os.path.dirname(plugin_path)
MOD_dir_relative = os.path.join(plugin_dir, '..', '..')
MOD_PATH = os.path.abspath(MOD_dir_relative)
import importlib

def splitall(path):
    allparts = []
    while 1:
        parts = os.path.split(path)
        if (parts[0] == path):
            allparts.insert(0, parts[0])
            break
        elif (parts[1] == path):
            allparts.insert(0, parts[1])
            break
        else:
            path = parts[0]
            allparts.insert(0, parts[1])
    return_parts = []
    for part in allparts:
        if ((part != os.path.sep) and (part not in ['\\\\', '\\', '/', '//'])):
            return_parts.append(part)
    return return_parts

@catch_errors
def bro_menu_build(*args, **kwargs):
    '\n    This function builds the base menu for this module, and then walks through all subfolders of mod_dir, looking for\n      "menu.py" files, and runs their "build" function, allowing each subfolder to have it\'s own menu.\n\n    :param mod_dir: Folder to walk through and find menu.py files.\n    '
    log.debug('{} Root Menu build...'.format(MOD_Prefix))
    try:
        mod_dir = MOD_PATH
        if ((mod_dir != None) and (mod_dir != '')):
            mod_dir = os.path.join(mod_dir, MOD_NAME)
            if cmds.menu('{}MenuRoot'.format(MOD_Prefix), q=True, exists=True, aob=True):
                log.debug('{}MenuRoot exists, remove.'.format(MOD_Prefix))
                cmds.deleteUI('{}MenuRoot'.format(MOD_Prefix))
            if cmds.window('MayaWindow', exists=True):
                log.debug('MayaWindow exists, create menu root.'.format(MOD_Prefix))
                cmds.menu('{}MenuRoot'.format(MOD_Prefix), l=MOD_NAME, p='MayaWindow', to=True)
            log.debug('Scan for menu files inside {}'.format(mod_dir))
            for (root, dirs, files) in os.walk(mod_dir, topdown=True):
                for name in files:
                    fullpath = os.path.join(root, name)
                    if (name.lower() == 'menu.py'):
                        try:
                            log.debug('   Building menu from file:', fullpath, debug=True)
                            rel_path = os.path.relpath(fullpath, mod_dir)
                            path_chunks = splitall(rel_path)
                            modpath = ((MOD_NAME + '.') + os.path.join(*path_chunks).replace('.py', '').replace(os.path.sep, '.'))
                            log.debug('       Menu module, import and build:', modpath, debug=True)
                            m = importlib.import_module(modpath)
                            m.build()
                            log.debug('      ', modpath, 'menu built', debug=True)
                        except:
                            log.warning('       x Unable to build menu from file:', modpath)
                            traceback.print_exc()
            from BroTools.configuration import optional_menu_ids
            log.debug('Cleaning up empty menus...', debug=True)
            for menu_id in optional_menu_ids:
                try:
                    menu_item_array = cmds.menu(menu_id, q=True, itemArray=True)
                    do_cleanup = (True if ((menu_item_array == None) or (len(menu_item_array) < 1)) else False)
                    log.debug(menu_id, 'clean up:', do_cleanup, debug=True)
                    if do_cleanup:
                        cmds.deleteUI(menu_id)
                except Exception as e:
                    log.info("Can't clean up menu", menu_id, str(e))
        else:
            log.debug('mod_dir not supplied for bro_menu_build. Skip.')
            log.debug(args, kwargs)
    except:
        log.warning('Unknown error during menu build.', traceback=True)

def bro_menu_build_remove():
    '\n    Removes menu\n    '
    log.debug((MOD_NAME + ' menu removing...'))
    delete_items = ['{}MenuRoot'.format(MOD_Prefix)]
    for item in delete_items:
        log.debug('Looking for {} menu item...'.format(item))
        if cmds.menu(item, q=True, exists=True):
            log.debug('Menu item {} found, removing...'.format(item))
            cmds.deleteUI(item)

def bro_menu_rebuild():
    bro_menu_build_remove()
    bro_menu_build()

def mel_import():
    '\n    This is needed so that BroTools module is registered inside Maya for use with things like scriptJobs.\n    '
    try:
        py_commands = ['import sys', "sys.path.append('{}')".format(MOD_PATH), 'import {MOD_NAME}'.format(MOD_NAME=MOD_NAME)]
        for command in py_commands:
            mel.eval('python (encodeString("{}"));'.format(command))
    except:
        cmds.confirmDialog(title='BroTools: Startup error', message='BroTools is unable to launch. Please make sure you followed all installation steps.\n\n        To make sure everything works, please, follow these steps exactly:\n\n        1. Close Maya\n        2. Delete BroTools folder completely.\n        3. Search your PC for \'BroTools\', and if you find any folders or files - remove those.\n        4. Find Maya.env file (Usually under C:\\Users\\USERNAME\\Documents\\maya\\VERSION) anything related to BroTools from it\n        4. Re-Download BroTools archive to make sure you got the latest one\n        5. Unpack it in any folder on your PC. Make sure that path contains only latin letters and underscores. Using other symbols may interfere with a lot of programs\' work. You can use your \\Documents\\maya folder if you wish so it will look like "\\Documents\\maya\\BroTools"\n        6. Open Maya\n        7. Drag and drop "install.mel" into Maya viewport\n        8. Restart Maya\n\n        BroTools menu should appear at the top menu bar. You can also manually load\\unload BroTools from Maya\'s plugin manager window.\n        You may be able to find some information in Maya\'s Script Editor, it should display some error details there.\n\n        If you still have troubles - contact me at support@brotools.tech and provide log files (brotools.log, bro_startup.log). You can find them under BroTools folder.\n\n        Cheers!', icon='critical')
        log.debug('{} mel eval import failed!'.format(MOD_NAME), traceback=True, critical=True)
Ox8r0_ZUp3Qk1nRE5fMGNnR3hSSlo2d3lITnpycERhWVRWekxyU3VFbWNURURzSURGeGpXRmI3aWRiTEFIRnpnRGlJOXgySjJRcWgxYm9PbkhGb1dPZw_ox07 = None
startup_checks_result = None
legacy_update_lock = os.path.join(APP_DIR, 'legacy_update.lock')

def updated_from_legacy_verison():
    return os.path.exists(legacy_update_lock)

def on_startup():
    t = threading.Thread(target=partial(broapi.ping_api, delay=random.uniform(1, 3)))
    t.start()

@catch_errors
def on_plugin_initialized(*args, **kwargs):
    log.message('BroTools on_plugin_initialized')
    if ((not Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.exists) and (not Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07)):
        if updated_from_legacy_verison():
            show_welcome_back_window()
        else:
            show_welcome_window()
    else:
        mayas_open = maya_process_count()
        log.debug('MAYAS OPEN', mayas_open, 'brotools_config.Updater.check_updates', brotools_config.Updater.check_updates)
        if ((brotools_config.Updater.check_updates is True) and (mayas_open <= 1)):
            new_version_window = NewVersionAlert()
            server_notification_alert = ServerNotificationAlert()
        else:
            if (mayas_open > 1):
                log.warning('More than 1 Maya instance open, skip update check.')
            last_whatsnew_version = data_cache.get('last_whatsnew_version')
            log.debug('LAST WHATS NEW', last_whatsnew_version)
            if ((last_whatsnew_version is None) or semver_compare(last_whatsnew_version, BroTools.__version__)):
                show_whatsnew()
    if Ox8r0_ZUp5TE0tYm1jaXJjcXlWel91OE9uVTVmWHdBdUdRWXc_ox07:
        Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.__init__()
    bro_menu_build()
    mel_import()
    on_startup()

def on_plugin_uninitialized(*args, **kwargs):
    log.debug('BroTools on_plugin_uninitialized')
    bro_menu_build_remove()
    callbacks.remove_all_callbacks()

def show_whatsnew(*args, **kwargs):
    log.info('show_whatsnew', args, kwargs)
    whatsnew_window = NotificationWindow(title="BroTools {}: What's new".format(BroTools.__version__), text="You've just updated BroTools to {}! You can read about what's new in this release by clicking on the button below.".format(BroTools.__version__))
    whatsnew_window.add_action('View recent changes (Opens web browser)', partial(open_web_url, (API_MAIN_URL + '/changelog?start={}&end={}'.format(data_cache.get('last_whatsnew_version'), BroTools.__version__))))
    whatsnew_window.show()
    data_cache.set('last_whatsnew_version', BroTools.__version__)

def Ox8r0_ZUp6TEZ0ZTI4UXlaeGFmMjl2ZzhzMVgxYWE5blNDMEdBRTZwQ0t3_ox07(*args, **kwargs):
    log.debug('Ox8r0_ZUp6TEZ0ZTI4UXlaeGFmMjl2ZzhzMVgxYWE5blNDMEdBRTZwQ0t3_ox07', args, kwargs)
    Ox8r0_ZUp3TEVWUFhxX0NlcHFEMDl1b2NucTJlaVU5bUN5d0RBRXgxQ0Zr_ox07().show()

def show_updater(*args, **kwargs):
    log.debug('show_updater', args, kwargs)
    UpdaterWindow().show()

def show_about_window(*args, **kwargs):
    log.debug('show_about_window', *args, **kwargs)
    AboutWindow().show()

def show_bugreport_window(*args, **kwargs):
    log.debug('show_bugreport_window', *args, **kwargs)
    BugReportWindow().show()

def restore_backup(*args, **kwargs):
    log.debug('restore_backup', *args, **kwargs)
    from BroTools.configuration import APP_DIR, crossplatform_normpath
    from maya import mel
    import os
    mel.eval('source "{}";'.format(crossplatform_normpath(os.path.join(APP_DIR, 'restore_backup.mel'))).replace('\\', '/'))

def show_welcome_window(*args, **kwargs):
    log.debug('show_welcome_window', *args, **kwargs)
    welcome_window = WelcomeWindow()
    welcome_window.show()

def show_welcome_back_window(*args, **kwargs):
    log.debug('show_welcome_back_window', *args, **kwargs)
    welcome_back_window = WelcomeBackWindow()
    welcome_back_window.show()
import json, traceback
from maya.api import OpenMaya as om
from maya import OpenMaya as om1
from maya import cmds
from BroTools.core.debug import default_logger as log
from BroTools.core.errors import BroToolsError
from BroTools.core import utils

def maAddAttrValueType(value):
    '\n    This function returns a value type suitable for maya setAttr\x07ddAttr\\getAttr commands.\n    Returns dict that can be **passed to a function.\n    :param value: Value\n    :return:\n    '
    type_values = {str: {'dt': 'string'}, bool: {'at': 'bool'}, int: {'at': 'long'}, float: {'at': 'float'}, unicode: {'dt': 'string'}}
    value_type = type(value)
    if (value_type in type_values):
        return type_values[value_type]
    else:
        return {}

def maSetAttrValueType(value):
    '\n    This function returns a value type suitable for maya setAttr\x07ddAttr\\getAttr commands.\n    Returns dict that can be **passed to a function.\n    :param value: Value\n    :return:\n    '
    type_values = {str: {'type': 'string'}, unicode: {'type': 'string'}}
    value_type = type(value)
    if (value_type in type_values):
        log.debug('maSetAttrValueType', value, type_values[value_type])
        return type_values[value_type]
    else:
        log.debug('maSetAttrValueType', value, '{}')
        return {}

def parse_attr(a):
    '\n    Ensures that passed attribute is in maya\'s format "name.attr".\n    Can take in pairs [BroNode, \'attr\'] or string \'name.attr\', or pair [\'name\', \'attr\']\n\n    TODO Make it possible to pass BroNode.attr with no quotes.\n    '
    if isinstance(a, list):
        if isinstance(a[0], BroNode):
            a_name = a[0].longName
        else:
            a_name = a[0]
        a = ((a_name + '.') + a[1])
    return a

def connect_nodes(*args, **kwargs):
    '\n    Connects the first attribute to the second.\n    TODO Improve to remove rudimentary parse_attr, it only adds overhead, or make it work with bronodes properly\n    '
    if (len(args) == 2):
        a = args[0]
        b = args[1]
        if (isinstance(a, list) and isinstance(b, list)):
            return cmds.connectAttr(parse_attr(a), parse_attr(b), **kwargs)
    if (len(args) == 4):
        a = [args[0], args[1]]
        b = [args[2], args[3]]
        return cmds.connectAttr(parse_attr(a), parse_attr(b), **kwargs)
    cmds.error('Cant connect nodes, wrong number of parameters?', exc_info=True)

def ensureBroNode(node):
    '\n    This function makes sure that passed object is a BroNode.\n    If it is, returns it, if not, converts.\n    :param obj:\n    :return:\n    '
    if (not issubclass(type(node), BroNode)):
        log.debug(node, 'is not a BroNode, convert.')
        node = BroNode(node)
    return node

def ensureClass(node, cls):
    if (not issubclass(type(node), cls)):
        log.debug(node, 'is not of specified class, convert.')
        node = cls(node)
    return node
BRONODES_CACHE = []
L_LAST_CHECK_TIME = datetime.now()
L_CHECK_DELTA = timedelta(minutes=5)

def MSpace(space):
    if isinstance(space, str):
        spaces = {'object': om.MSpace.kObject, 'world': om.MSpace.kWorld, 'transform': om.MSpace.kTransform, 'preTransform': om.MSpace.kPreTransform, 'postTransform': om.MSpace.kPostTransform, 'last': om.MSpace.kLast}
        return spaces[space]
    elif isinstance(space, int):
        return space
    else:
        log.warning('Unhandled space type passed to MSpace: {}'.format(space))
        return space

class BroNode(object):
    '\n    Custom universal OOP node wrapper\\Meta node.\n    It behaves as a regular string, but always returns current correct full dag path,\n    based on OpenMaya.MDagPath. of an object.\n    So you can just pass this node to any maya command.\n\n    Can also be used to get and set attributes with ease:\n    node.tx - will return current tx\n    node.tx = 10 will set tx to 10\n\n    It can also handle connections between nods, using message attribute.\n\n    Some of it is inspired by Red9\'s meta node, but I tried to keep it simplified and adjusted for my own workflows.\n    In this kind of base classes and functions I also prefer to use pythonic philosophy of \'explicit is better than\n    implicit\', when it comes to using cmds.commmands\' attribute names.\n\n    TODO: What did I want to use _bro_node_type for?\n\n    TODO: Tried subclassing str, but it had some weird behaviour where node would sometimes return [u\'name\'] as a string,\n    which is very odd. Need to research it more.\n\n\n    # Adding multiple connections one by one, iteratively\n\n    ```from BroTools.core.system.utilities import BroNode\n\n        a = BroNode.create("spaceLocator")\n\n        for i in range(10):\n            b = BroNode.create("spaceLocator")\n            if a.test is None:\n                a.test = [b]\n            else:\n                a.test += [b]\n    ```\n    '
    _mDagPath = None
    _mObject = None
    _node = None
    _bro_node_type = 'BroNode'
    _is_bro_node = True

    @staticmethod
    def get_or_create(passed_name, type='group'):
        if cmds.objExists(passed_name):
            return BroNode(passed_name)
        else:
            return BroNode.create(type, name=passed_name)

    @classmethod
    def create(cls, type, **kwargs):
        name = utils.brokwargs('name', 'n', None, kwargs).get()
        types = {'group': {'cmd': cmds.group, 'attrs': {'em': True}}, 'joint': {'cmd': cmds.joint, 'attrs': {}}, 'spaceLocator': {'cmd': cmds.spaceLocator, 'attrs': {}}, 'circle': {'cmd': cmds.circle, 'attrs': {}}, 'sphere': {'cmd': cmds.sphere, 'attrs': {}}, 'cone': {'cmd': cmds.cone, 'attrs': {}}, 'cylinder': {'cmd': cmds.cylinder, 'attrs': {}}}
        attrs = {}
        selection = cmds.ls(sl=True, l=True)
        if name:
            attrs['n'] = name
        if (type in types):
            attrs = utils.merge_two_dicts(types[type]['attrs'], attrs)
            s = cmds.ls(sl=True, long=True)
            node = cls(types[type]['cmd'](**attrs))
            cmds.select(s, r=True)
        else:
            node = cls(cmds.createNode(type, skipSelect=True, **attrs))
        cmds.select(selection, r=True)
        return node

    def __new__(cls, passed_name, *args, **kwargs):
        if isinstance(passed_name, BroNode):
            return passed_name
        else:
            return super(BroNode, cls).__new__(cls, *args, **kwargs)

    def __init__(self, passed_name):
        global L_LAST_CHECK_TIME
        if ((datetime.now() - L_LAST_CHECK_TIME) > L_CHECK_DELTA):
            L_LAST_CHECK_TIME = datetime.now()
            if (Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.validated_verified is not True):
                raise BroToolsError('Tool blocked by license. {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
            Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07 = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['tools']
            ppaths = Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.data['ppaths']
            namespace = sys._getframe(1).f_globals
            if ('__file__' in namespace):
                if (Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07 is None):
                    cmds.warning('{} blocked, missing license file!'.format(namespace['__file__']))
                    raise BroToolsError('Tool blocked by license. {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
                filepath = ntpath.normpath(namespace['__file__'])
                if any(((ppath in filepath) for ppath in ppaths)):
                    blocked = True
                    if isinstance(ppaths, list):
                        log.warning('PPATHS IS LIST!!!', ppaths)
                    for (ppath, reqs) in ppaths.items():
                        if ((ntpath.normpath(ppath) in filepath) and any(((x in Ox8r0_ZUp3ckVWUFhxX0NleGluel9mWVUxaG5sT1Q4Qk5qSUhQUQ_ox07) for x in reqs))):
                            blocked = False
                            break
                    if (blocked is True):
                        cmds.warning('{} blocked, invalid license: {}'.format(namespace['__file__'], Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
                        raise BroToolsError('Tool blocked by license. {}'.format(Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.Ox8r0_ZUp3ckZEVFNxZ3FjYm1UNDdHb0gyMG9BSkRjRmxn_ox07))
            else:
                log.debug("Can't determine filepath of a protected file, no __file__ in", namespace)
        log.debug('Initializing BroNode (type {}) for {}; {}'.format(type(self), passed_name, type(passed_name)))
        if (passed_name == None):
            raise BroToolsError('Unable to initialize BroNode - received None argument.')
        if issubclass(type(passed_name), list):
            log.debug('List passed to BroNode, using first object from the list - [', passed_name, ']')
            passed_name = passed_name[0]
        if (not cmds.objExists(passed_name)):
            raise BroToolsError('Unable to initialize BroNode for', passed_name, "object does not exist in Maya's scene. It should be a string name of an existing maya node as well, not Python object.")
        passed_name = str(passed_name)
        self._passed_name = str(passed_name)
        self._node = om.MFnDependencyNode()
        self._mObject = om.MObject()
        self._mDagPath = om.MDagPath()
        try:
            selection = om.MSelectionList()
            selection.add(passed_name)
            self._mObject = selection.getDependNode(0)
            self._node = om.MFnDependencyNode(self._mObject)
            self._mDagPath = selection.getDagPath(0)
            self._mFnDagNode = om.MFnDagNode(self._mObject)
        except:
            log.debug('Unable to get _mDagPath for {}'.format(passed_name), exc_info=True)
        try:
            selection_legacy = om1.MSelectionList()
            selection_legacy.add(passed_name)
            self._mObjectLegacy = om1.MObject()
            self._mDagPathLegacy = om1.MDagPath()
            selection_legacy.getDependNode(0, self._mObjectLegacy)
            self._nodeLegacy = om1.MFnDependencyNode(self._mObjectLegacy)
            selection_legacy.getDagPath(0, self._mDagPathLegacy, om1.MObject())
            self._mFnDagNodeLegacy = om1.MFnDagNode(self._mObjectLegacy)
        except:
            log.debug('Unable to get legacy _mDagPath for {}'.format(passed_name), exc_info=True)
        '\n        # Old implementation\n        # get a MDagPath for our node:\n        selList = om.MSelectionList()  # make a sel list # MSelectionList\n        selList.add(str(passed_name))  # add our node by name\n        _obj = om.MObject()\n        selList.getDependNode(0, _obj)\n        _dag = om.MDagPath()  # create an empty dag path # MDagPath\n        selList.getDagPath(0, _dag)  # fill the dag path with our node # Does not work with non transform nodes\n\n        self._mDagPath = _dag\n        self._mObject = _obj\n        '
        BRONODES_CACHE.append(self)
        log.debug('BroNode initialized for: {}, longName: {};'.format(passed_name, self.longName))

    def __repr__(self):
        "\n        :return: Returns machine-readable string required to recreate this object.\n        In case of BroNode it's just calling BroNode with the correct longName again.\n        This can be used for serialization, but only in the case if longName will not be changed between saving and loading\n        of data.\n        "
        ln = self.longName
        return str("BroTools.core.system.utilities.BroNode('{}')".format(ln))

    def __str__(self):
        "\n        :return: Returns human readable string representing this object. In case of BroNode it's simply a longName of\n        this object.\n\n        From my tests it looks like Maya's CMDS commands convert objects to __str__ when they need a string name.\n        "
        ln = self.longName
        return str(ln)

    def __unicode__(self):
        return self.__str__()

    def __hash__(self):
        return hash(self.longName)

    def __add__(self, other):
        return (str(self.longName) + str(other))

    def __getattr__(self, attr):
        if (attr == 'longName'):
            raise AttributeError('longName called before BroNode initialization... I think?')
        if str(attr).startswith('_'):
            return self.__dict__[attr]
        attr_path = ((self.longName + '.') + attr)
        if cmds.objExists(attr_path):
            try:
                if cmds.attributeQuery(attr, n=self.longName, message=True):
                    if cmds.attributeQuery(attr, n=self.longName, multi=True):
                        return self.mGetConnectedObjects(attr)
                    else:
                        return self.mGetConnection(attr)
            except RuntimeError as e:
                if ('attribute name not recognized' not in str(e).lower()):
                    raise e
            value = cmds.getAttr(attr_path)
            if (issubclass(type(value), str) or issubclass(type(value), unicode)):
                if (('[' in value) or ('{' in value)):
                    try:
                        json_data = json.loads(value)
                        is_json = True
                    except ValueError:
                        is_json = False
                    if ((not (issubclass(type(json_data), list) == False)) and (not (issubclass(type(json_data), dict) == False))):
                        is_json = False
                else:
                    is_json = False
                if is_json:
                    log.debug('JSON: ({})'.format(str(value)))
                    return json_data
                else:
                    log.debug('Not JSON: ({})'.format(str(value)))
                    return value
            else:
                return value
        else:
            return

    def __setattr__(self, key, value):
        if key.startswith('_'):
            self.__dict__[key] = value
        if (key not in dir(self)):
            if (value == None):
                if cmds.objExists(((self.longName + '.') + key)):
                    if (key in cmds.deleteAttr(self.longName, q=True)):
                        cmds.deleteAttr(self.longName, key)
                    else:
                        log.warning("Cannot delete attribute {}.{} as it's not dynamic.".format(self.longName, key))
                else:
                    log.warning('Attribute {}.{} was not created as it would be set to None'.format(self.longName, key))
                return
            log.debug(value, type(value), isBroNodeInstance(value))
            if isBroNodeInstance(value):
                log.debug('{} is instance of BroNode, make connection.'.format(value))
                if cmds.objExists(((self.longName + '.') + key)):
                    self.mDisconnect(self, key)
                self.mConnectChild(value, sourceAttribute=key, destinationAttribute=('parent_' + value.longName.replace('|', '_')))
                return
            elif (isinstance(value, list) or isinstance(value, tuple)):
                if listIsBroNodes(value):
                    log.debug('{} is a list of instances of BroNode, make connection.'.format(value))
                    if cmds.objExists(((self.longName + '.') + key)):
                        self.mDisconnect(self, key)
                    self.mConnectChildren(value, destinationAttribute=key, sourceAttribute=('parent_' + key))
                    return
            attr_path = ((self.longName + '.') + key)
            if (not cmds.objExists(attr_path)):
                log.debug('Attribute', attr_path, 'does not exist. Creating.')
                if (issubclass(type(value), list) or issubclass(type(value), dict)):
                    cmds.addAttr(self.longName, longName=key, keyable=True, dt='string')
                else:
                    cmds.addAttr(self.longName, longName=key, keyable=True, **maAddAttrValueType(value))
            if (issubclass(type(value), list) or issubclass(type(value), dict)):
                value = json.dumps(value)
            cmds.setAttr(attr_path, value, **maSetAttrValueType(value))
        elif (key == 'parent'):
            if ((self.parent != None) or (value != None)):
                if (value == None):
                    log.debug('BroNode {} parent to None.'.format(self.longName))
                    cmds.parent(self.longName, world=True)
                else:
                    new_parent = BroNode(value)
                    log.debug('Parenting {} to {}'.format(self, new_parent))
                    if (self.parent == None):
                        cmds.parent(self, new_parent.longName)
                    elif (self.parent.longName != new_parent.longName):
                        cmds.parent(self, new_parent.longName)
                    else:
                        log.debug(self.longName, 'is already child of', new_parent)
        elif (key == 'name'):
            cmds.rename(self, value)
        else:
            self.__dict__[key] = value

    def __get_comparison_pair__(self, other):
        this = self.longName
        _other = other
        if isBroNodeInstance(other):
            other = other.longName
        elif isinstance(other, str):
            other = other
        else:
            try:
                other = BroNode(other).longName
            except:
                raise TypeError("Unable to compare between {} ({}) and {} ({}), can't create BroNode for {}".format(str(type(self)), self, str(type(_other)), _other, other))
        return (this, other)

    def __eq__(self, other):
        if (other is None):
            return False
        (this, other) = self.__get_comparison_pair__(other)
        return (this == other)

    def __ne__(self, other):
        if (other is None):
            return True
        (this, other) = self.__get_comparison_pair__(other)
        return (this != other)

    def __lt__(self, other):
        (this, other) = self.__get_comparison_pair__(other)
        return (this < other)

    def __gt__(self, other):
        (this, other) = self.__get_comparison_pair__(other)
        return (this > other)

    def __le__(self, other):
        (this, other) = self.__get_comparison_pair__(other)
        return (this <= other)

    def __ge__(self, other):
        (this, other) = self.__get_comparison_pair__(other)
        return (this >= other)

    def __apiobject__(self):
        return self._mObjectLegacy

    @property
    def fullName(self):
        '\n        Returns short name of an object without path\\wildcards but with namespace\n        '
        if self._mDagPath.isValid():
            nm = self._mDagPath.partialPathName()
            if issubclass(type(nm), list):
                nm = nm[0]
            return nm.rpartition('|')[(- 1)]
        else:
            if (not self._mObject.isNull()):
                return self._node.name()
            return
        '\n        try:\n            nm = self._mDagPath.partialPathName()\n            if issubclass(type(nm), list):\n                nm = nm[0]\n            # log.debug ("nm", nm)\n            return nm.rpartition(\'|\')[-1]\n        except:\n            traceback.print_exc()\n        '

    @property
    def name(self):
        '\n        Returns short name of the object, with no namespaces and no wildcards, even if it is not unique.\n        '
        try:
            if self._mDagPath.isValid():
                nm = self._mDagPath.partialPathName()
                if issubclass(type(nm), list):
                    nm = nm[0]
                return nm.rpartition(':')[(- 1)].rpartition('|')[(- 1)]
            else:
                if (not self._mObject.isNull()):
                    return self._node.name()
                log.warning("BroNode - tried accessing name, but it's None on object: {}".format(self._passed_name))
                return self._passed_name
        except:
            traceback.print_exc()

    @property
    def longName(self):
        "\n        Returns fullPathName of the object\n        try:\n            ln = self._mDagPath.fullPathName()\n            if issubclass(type(ln), list):\n                ln = ln[0]\n            # log.debug ('ln', ln)\n            return ln\n        except:\n            traceback.print_exc()\n        "
        if self._mDagPath.isValid():
            ln = self._mDagPath.fullPathName()
            if issubclass(type(ln), list):
                ln = ln[0]
            return ln
        else:
            if (not self._mObject.isNull()):
                return self._node.name()
            log.warning("BroNode - tried accessing longName, but it's None on object: {}".format(self._passed_name), exc_info=True)
            return self._passed_name
    long_name = longName

    @property
    def longNameNoNamespaces(self):
        '\n        Returns fullPathName of the object without namespaces\n        '
        try:
            if self._mDagPath.isValid():
                ln = self._mDagPath.fullPathName()
                if issubclass(type(ln), list):
                    ln = ln[0]
                newLongName = ''
                objects = ln.split('|')
                for (i, obj) in enumerate(objects):
                    if (obj != ''):
                        newLongName += ('|' + obj.rpartition(':')[(- 1)])
                return newLongName
            else:
                if (not self._mObject.isNull()):
                    return self._node.name()
                return
        except:
            traceback.print_exc()

    @property
    def parent(self):
        try:
            relatives = cmds.listRelatives(str(self), parent=True, fullPath=True)
            if (not issubclass(type(relatives), list)):
                return
            else:
                return BroNode(relatives[0])
        except:
            log.debug('Unable to get parent of object', str(self))
            traceback.print_exc()

    @property
    def children(self):
        try:
            return convertListToBroNodes(cmds.listRelatives(self, children=True, fullPath=True))
        except:
            log.debug('Unable to get children of object', str(self))
            traceback.print_exc()

    @property
    def namespace(self):
        if (':' in self.fullName):
            return self.fullName.rpartition(':')[0]
        else:
            return ''

    @property
    def transform(self):
        return self.getTransform()

    @property
    def objectType(self):
        return cmds.objectType(self.longName)

    def hasAttribute(self, attr):
        '\n        Wrapper around cmds.objExists\n        :param attr: attr\n        :return: True or False\n        '
        return cmds.objExists('{}.{}'.format(self, attr))

    def listRelatives(self, *args, **kwargs):
        return cmds.listRelatives(self, **kwargs)

    def isChildNode(self, node, attr=None, srcAttr=None):
        "\n        Test if a node is already connected to the BroNode via a given attr link.\n\n        cmds.isConnected('node.attr[0]','other.attr[0]')\n        fails if simply asked:\n        cmds.isConnected('node.attr',other.attr')\n        "
        if (not issubclass(type(node), BroNode)):
            node = BroNode(node)
        if attr:
            cons = cmds.ls(cmds.listConnections('{}.{}'.format(self, attr), source=False, destination=True, plugs=True), long=True)
        else:
            cons = cmds.ls(cmds.listConnections(self, source=False, destination=True, plugs=True), long=True)
        if cons:
            for con in cons:
                if srcAttr:
                    if ('{}.{}'.format(cmds.ls(node, long=True)[0], srcAttr) in con):
                        return True
                elif ('{}.'.format(cmds.ls(node, long=True)[0]) in con):
                    return True
        return False

    def ensureBroNode(self, node):
        '\n        This function makes sure that passed object is a BroNode.\n        If it is, returns it, if not, converts.\n        :param obj:\n        :return:\n        '
        if (not issubclass(type(node), BroNode)):
            log.debug(node, 'is not a BroNode, convert.')
            node = BroNode(node)
        return node

    def addAttr(self, *args, **kwargs):
        cmds.addAttr(self.longName, **kwargs)
    add_attr = addAttr

    def addClampedFloatAttr(self, name, min=0, max=1, default=0, keyable=True, **kwargs):
        log.debug('Add clamped attr to {}'.format(self.longName))
        if (not cmds.objExists(((self.longName + '.') + name))):
            cmds.addAttr(self.longName, longName=name, minValue=min, maxValue=max, defaultValue=default, keyable=keyable, **kwargs)

    def connect_to(self, target_node, target_attribute, attribute='message', multi=False):
        '\n        Connect an attribute to a target attribute.\n        :param target_node: Node to connect to.\n        :param target_attribute: Attribute on target node to connect to.\n        :param attribute: Attribute on this node to connect. By default is a message attribute.\n        :return:\n        '
        target_node = ensureBroNode(target_node)
        log.debug(f'Connecting {self.longName}.{attribute} to {target_node.longName}.{target_attribute} (multi: {multi})')
        connections = cmds.listConnections(f'{target_node.longName}.{target_attribute}', source=True, destination=False, plugs=True)
        attr_size = (len(connections) if connections else 0)
        log.debug(f'    Attribute size: {attr_size}')
        if (multi is False):
            result = cmds.connectAttr(f'{self.longName}.{attribute}', f'{target_node.longName}.{target_attribute}')
        elif (multi is True):
            next_index = (attr_size + 1)
            result = cmds.connectAttr(f'{self.longName}.{attribute}', f'{target_node.longName}.{target_attribute}[{next_index}]')
        log.debug(f'    Result: {result}')
        return result

    def connect(self, otherobj, attr, this_input):
        connect_nodes(otherobj, attr, self, this_input)
        return self

    def matchTransform(self, other, **kwargs):
        cmds.matchTransform(self, other, **kwargs)
        return self

    def xform(self, **kwargs):
        cmds.xform(self, **kwargs)
        return self

    def mConnect(self, source, destination, destinationAttribute=None, sourceAttribute=None, multi=True, force=True):
        "\n        Connect two objects together using 'message' attribute.\n        Connection goes in direction parent.message -> child.attr\n        Args:\n            fromObj(str): Object to connect FROM\n            destination(str): Object to connect TO\n            destinationAttribute(str): Name of connection on the child node. If left empty or 'none' - turns into parent name.\n            sourceAttribute(str): Name of connection on the parent node. If left empty or 'none' - turns into child name.\n            multi(bool): if true - multiple connections can be made to one attribute.\n            force(bool): if true, will delete existing connection and overwrite it with new one.\n        "
        log.debug('mConnect: {}; {}; {}; {}; {}'.format(source, destination, destinationAttribute, multi, force))
        source = self.ensureBroNode(source)
        destination = self.ensureBroNode(destination)
        if (destinationAttribute == None):
            destinationAttribute = source.name
        if (sourceAttribute == None):
            sourceAttribute = destination.name
        if ((not force) and (not multi)):
            if cmds.objExists('{}.{}'.format(destination, destinationAttribute)):
                log.debug("You need to pass 'force' or 'multi' flags to make this connection.")
                raise BroToolsError('metaConnect {}.{}: Plug already exists.'.format(destination, destinationAttribute))
        if (cmds.objExists(((destination + '.') + destinationAttribute)) == False):
            if multi:
                cmds.addAttr(destination, ln=destinationAttribute, at='message', multi=True, indexMatters=False)
            else:
                cmds.addAttr(destination, ln=destinationAttribute, at='message', multi=False)
        if (cmds.objExists(((source + '.') + sourceAttribute)) == False):
            cmds.addAttr(source, ln=sourceAttribute, at='message', multi=False)
        sourcePlug = '{}.{}'.format(source, sourceAttribute)
        destinationPlug = '{}.{}'.format(destination, destinationAttribute)
        if multi:
            try:
                cmds.connectAttr(sourcePlug, destinationPlug, na=True, force=force)
            except RuntimeError as e:
                if ('already connected' in str(e).lower()):
                    log.debug('mConnect warning:', e)
                else:
                    log.warning('mConnect warning:', e)
        else:
            cmds.connectAttr(sourcePlug, destinationPlug, force=force)
        log.debug('>', sourcePlug, 'connected to', destinationPlug)

    def mConnectChild(self, child, **kwargs):
        '\n        Create outgoing connection from this node to child node.\n        There can be multiple outgoing connections, so multiple children can be connected one after another.\n        sourceAttribute is the name of attribute on the PARENT\n        destinationAttribute is the name of the attribute on the CHILD\n\n        Check mConnect for available arguments.\n        '
        kwargs.pop('multi', None)
        self.mConnect(self, child, multi=False, **kwargs)

    def mConnectChildren(self, *args, **kwargs):
        '\n        Create multiple connections from children to this node.\n\n        Check mConnect for available arguments.\n\n        NOTE: source and destination need to be reversed for this type of connection as "children" are connected TO the\n        parent as incoming connections\n\n        NOTE 2: Using obj.connection_name to query a connection with multiple children will return only the last child connected.\n        If you need to query multiple children use mGetConnectedObjects("connection_name") instead.\n        '
        log.debug('mConnectChildren: {}; {}'.format(args, kwargs))
        if ('destinationAttribute' not in kwargs):
            kwargs['destinationAttribute'] = self.name
        children = []
        for arg in args:
            if issubclass(type(arg), list):
                children.extend(arg)
            else:
                children.append(arg)
        kwargs.pop('multi', None)
        for child in children:
            self.mConnect(child, self, multi=True, **kwargs)

    def mConnectParent(self, parent, **kwargs):
        "\n        Create incoming connection from given `parent` node to this node. Current node is considered 'child' in this case.\n\n        :param child:\n        :param attributeName:\n        :param multi:\n        :param force:\n        :return:\n        "
        kwargs.pop('multi', None)
        ca = utils.brokwargs('childAttribute', 'ca', None, kwargs).get()
        pa = utils.brokwargs('parentAttribute', 'pa', None, kwargs).get()
        if ca:
            kwargs['destinationAttribute'] = ca
            if ('ca' in kwargs):
                kwargs.pop('ca')
            if ('childAttribute' in kwargs):
                kwargs.pop('childAttribute')
        if pa:
            kwargs['sourceAttribute'] = pa
            if ('pa' in kwargs):
                kwargs.pop('pa')
            if ('parentAttribute' in kwargs):
                kwargs.pop('childAttribute')
        self.mConnect(parent, self, multi=False, **kwargs)

    def mDisconnect(self, node, attr=None, deleteSourcePlug=True, deleteDestPlug=True):
        sPlug = None
        dPlug = None
        sPlugMeta = None
        returnData = []
        searchConnection = ('%s.' % self.name)
        if attr:
            searchConnection = ('%s.%s' % (self.name, attr))
        if (not issubclass(type(node), BroNode)):
            node = BroNode(node)
        cons = cmds.listConnections(node, s=True, d=False, p=True, c=True)
        if cons:
            for (sPlug, dPlug) in zip(cons[0::2], cons[1::2]):
                log.debug(('attr Connection inspected : %s << %s' % (sPlug, dPlug)))
                if ((attr and (searchConnection == dPlug.split('[')[0])) or ((not attr) and (searchConnection in dPlug))):
                    log.debug(('Disconnecting %s >> %s as %s found in dPlug' % (dPlug, sPlug, searchConnection)))
                    cmds.disconnectAttr(dPlug, sPlug)
                    returnData.append((dPlug, sPlug))
        if (deleteSourcePlug and (sPlug is not None)):
            try:
                allowDelete = True
                attr = sPlug.split('[')[0]
                if cmds.listConnections(attr):
                    allowDelete = False
                    log.debug(('sourceAttr connections remaining: %s' % ','.join(cmds.listConnections(attr))))
                if allowDelete:
                    log.debug(('Deleting deleteSourcePlug Attr %s' % attr))
                    if sPlugMeta:
                        delattr(sPlugMeta, attr.split('.')[(- 1)])
                    else:
                        cmds.deleteAttr(attr)
                else:
                    log.debug('deleteSourcePlug attr aborted as node still has connections')
            except Exception as error:
                log.warning('Failed to Remove mNode Connection Attr: {}'.format(error))
                log.debug(error)
        if (deleteDestPlug and (dPlug is not None)):
            try:
                allowDelete = True
                attr = dPlug.split('[')[0]
                if cmds.listConnections(attr):
                    allowDelete = False
                    log.debug(('sourceAttr connections remaining: %s' % ','.join(cmds.listConnections(attr))))
                if (attr.split('.')[(- 1)] == 'message'):
                    allowDelete = False
                    log.debug("Attribute is message attribute, can't delete this one")
                if allowDelete:
                    log.debug(('Deleting deleteDestPlug Attr %s' % attr))
                    delattr(self, attr.split('.')[(- 1)])
                else:
                    log.debug('deleteDestPlug attr aborted as node still has connections or attribute is a message attribute')
            except Exception as error:
                log.warning('Failed to Remove mNode Connection Attr: {}'.format(error))
                log.debug(error, exc_info=True)
        return returnData

    def mGetConnections(self, attribute=None, source=True, destination=True):
        connections = []
        cons = cmds.listConnections(self, source=source, destination=destination, connections=True)
        if cons:
            for (plug, node) in zip(cons[::2], cons[1::2]):
                if (cmds.getAttr(plug, type=True) == 'message'):
                    if (not (node in connections)):
                        plug_attrName = plug.split('.')[(- 1)]
                        log.debug(self, 'is connected with', node, 'through', plug_attrName)
                        if ((attribute == None) or (attribute == plug_attrName.split('[')[0])):
                            connections.append([plug_attrName, BroNode(node)])
        return connections

    def mGetConnectedObjects(self, *args, **kwargs):
        connections = self.mGetConnections(*args, **kwargs)
        objects = []
        for i in connections:
            objects.append(BroNode(i[1]))
        return objects

    def mGetConnection(self, attribute):
        connections = self.mGetConnections(attribute)
        for con in connections:
            log.debug(con, (con[0] == attribute))
            if (con[0] == attribute):
                return BroNode(con[1])

    def getTransform(self, t=False, r=False):
        '\n        Function to get different transforms of this object using MFnTransform.\n        :param t: Return translation only\n        :param r: Return rotation only. If both t and r set will return translation and rotation as a tuple. If none set,\n        will return MFnTransform\n        :param quat: If return rotaion - will return it as a quaternion\n        '
        try:
            mfn_transform = om.MFnTransform(self._mDagPath)
        except Exception as e:
            raise BroToolsError('Object {} does not have a transform. (Maya error text: {})'.format(self.name, str(e)))
        if ((not t) and (not r)):
            return mfn_transform
        elif (t and (not r)):
            tr = mfn_transform.translation(om.MSpace.kWorld)
            return (tr.x, tr.y, tr.z)
        elif (r and (not t)):
            Quat = om.MQuaternion()
            mfn_transform.rotation(Quat, om.MSpace.kWorld)
            return Quat

    def position(self):
        '\n        Return position as om.Vector\n        '
        t = self.getTransform(t=True)
        return om.MVector(t[0], t[1], t[2])

    def getMatrix(self):
        return self._mFnDagNode.transformationMatrix()

    def setTranslation(self, *args, **kwargs):
        t = args[0]
        space = kwargs.get('space', 'object')
        if (not isinstance(t, om.MVector)):
            t = om.MVector(t[0], t[1], t[2])
        self.getTransform().setTranslation(t, MSpace(space))

    def getTranslation(self, space='object', **kwargs):
        '\n        :param space: object, world, transform, preTransform, postTransform, last\n        :return:\n        '
        return om.MVector(self.getTransform().rotatePivot(MSpace(space)))

    def setRotation(self, *args, **kwargs):
        r = args[0]
        space = kwargs.get('space', 'object')
        if (not isinstance(r, list)):
            if (len(r) == 3):
                r = om.MEulerRotation(r[0], r[1], r[2])
            elif (len(r) == 4):
                r = om.MQuaternion(r[0], r[1], r[2], r[3])
        if isinstance(r, om.MEulerRotation):
            self.getTransform().setRotation(r)
        elif isinstance(r, om.MQuaternion):
            self.getTransform().setRotation(r, MSpace(space))

    def getRotation(self, space='object', quaternion=False):
        '\n        :param space: object, world, transform, preTransform, postTransform, last\n        :return:\n        '
        if quaternion:
            return self.getTransform().rotation(space=MSpace(space), asQuaternion=quaternion)
        else:
            return self.getTransform().rotation(asQuaternion=quaternion)
    rotation = getRotation
    translation = getTranslation

    def setAttributesFromDict(self, dict):
        '\n        Set object attributes from dictionary.\n        :param dict: Dictionary, {attributeName: value}. attributeName should be a valid name for cmds.setAttr(objName.attributeName, value) command.\n        '
        log.debug('setAttributesFromDict', dict)
        if isinstance(dict, list):
            print('setAttributesFromDict IS LIST', dict)
        for (simProperty, value) in dict.items():
            self.__setattr__(simProperty, value)

    def setAttr(self, attr, value, **kwargs):
        cmds.setAttr(((self.longName + '.') + attr), value, **kwargs)

    @catch_errors
    def getShapes(self, index=None):
        shapes = convertListToBroNodes(cmds.listRelatives(self.longName, c=True, s=True, fullPath=True))
        if (index is not None):
            if (len(shapes) <= 0):
                return
            else:
                return shapes[index]
        else:
            return shapes

    def delete(self):
        if self.exists:
            cmds.delete(self.longName)

    @catch_errors
    def deleteAttr(self, attr):
        log.debug('deleteAttr: {}'.format(((self.longName + '.') + str(attr))))
        cmds.deleteAttr(((self.longName + '.') + str(attr)))

    @property
    def exists(self):
        return cmds.objExists(self.longName)

    def attrCallbackTemplate(self, msg, m_plug, otherMplug, clientData, func=None, attr=None):
        if (msg == 2056):
            if (m_plug.partialName() == attr):
                func()

    def addAttributeChangedCallback(self, func, attr):
        return om.MNodeMessage.addAttributeChangedCallback(self._mObject, partial(self.attrCallbackTemplate, func=func, attr=attr))

    def this_is_not_ctrl(self, *args, **kwargs):
        raise BroToolsError('Tried running a function on the wrong type of node. You probably wanted to use BroCtrl class, but {} object is BroNode.'.format(self.longName))
    replace_shape = this_is_not_ctrl

def convertBroNodeListToNames(lst, **kwargs):
    long = kwargs.get('long', True)
    data = []
    for node in lst:
        node = ensureBroNode(node)
        if long:
            data.append(node.longName)
        else:
            data.append(node.name)
    return data

def convertListToBroNodes(lst):
    "\n    Convert a list of maya object names as strings into BroNodes.\n    :param lst: A list of strings representing Maya's objects\n    :return: A list of BroNodes\n    "
    nodes = []
    if lst:
        for i in lst:
            nodes.append(ensureBroNode(i))
    return nodes

def isBroNodeInstance(obj):
    '\n    Determine if the object is an instance of BroNode.\n    This is abstracted to be able to work around the limitations of isinstance, and rework it later if required.\n\n    :param obj: Object to check\n    :return: True or False\n    '
    if isinstance(obj, BroNode):
        return True
    if (obj.__class__.__name__ == 'BroNode'):
        return True
    if hasattr(obj, '_is_bro_node'):
        return True
    log.debug(f"Not bronode: {isinstance(obj, BroNode)}|{obj.__class__.__name__}: {(obj.__class__.__name__ == 'BroNode')}|{hasattr(obj, '_is_bro_node')}")
    return False

def convertListToClass(list, cls):
    "\n    Convert a list of maya object names as strings into BroNodes.\n    :param list: A list of strings representing Maya's objects\n    :return: A list of BroNodes\n    "
    nodes = []
    if list:
        for i in list:
            nodes.append(ensureClass(i, cls))
    return nodes

def listIsBroNodes(list):
    '\n    Check if every member of the list is BroNodes.\n    :return: True or False\n    '
    log.debug(f'Checking if list is bronodes: {list}')
    if (not list):
        log.debug(' List is none')
        return False
    if (len(list) == 0):
        log.debug(' List is empty')
        return False
    for i in list:
        if (not isBroNodeInstance(i)):
            log.debug(f'Not bronode: {i}')
            return False
    return True

def getActiveBroNodes():
    return_list = []
    for node in BRONODES_CACHE:
        if cmds.objExists(node):
            return_list.append(node)
    return return_list

class Spring1D():

    def __init__(self, **kwargs):
        self.value = kwargs.get('value', 0)
        self.target_value = kwargs.get('target_value', 0)
        self.spring = kwargs.get('spring', 1)
        self.damping = kwargs.get('damping', 0.1)
        self.force = kwargs.get('force', 0)
        self.velocity = kwargs.get('velocity', 0)

    def step(self):
        self.displacement = (self.value - self.target_value)
        self.force = ((- self.spring) * self.displacement)
        self.velocity += self.force
        self.value += self.velocity
        self.velocity *= (1 - self.damping)
        return self.value
NOISE_LINEAR = 1
NOISE_COSINE = 2
NOISE_CUBIC = 3

class PerlinNoise():

    def __init__(self, **kwargs):
        self.seed = random.Random(kwargs.get('seed')).random()
        self.amplitude = kwargs.get('amplitude', 1)
        self.frequency = kwargs.get('frequency', 1)
        self.octaves = kwargs.get('octaves', 1)
        self.interp = kwargs.get('interp', NOISE_COSINE)
        self.use_fade = kwargs.get('use_fade', False)
        self.mem_x = dict()

    def set_seed(self, new_seed):
        self.seed = random.Random(new_seed).random()

    def __noise(self, x):
        if (x not in self.mem_x):
            self.mem_x[x] = random.Random((self.seed + x)).uniform((- 1), 1)
        return self.mem_x[x]

    def __interpolated_noise(self, x):
        prev_x = int(x)
        next_x = (prev_x + 1)
        frac_x = (x - prev_x)
        if self.use_fade:
            frac_x = self.__fade(frac_x)
        if (self.interp is NOISE_LINEAR):
            res = self.__linear_interp(self.__noise(prev_x), self.__noise(next_x), frac_x)
        elif (self.interp is NOISE_COSINE):
            res = self.__cosine_interp(self.__noise(prev_x), self.__noise(next_x), frac_x)
        else:
            res = self.__cubic_interp(self.__noise((prev_x - 1)), self.__noise(prev_x), self.__noise(next_x), self.__noise((next_x + 1)), frac_x)
        return res

    def get(self, x):
        frequency = self.frequency
        amplitude = self.amplitude
        result = 0
        for _ in range(self.octaves):
            result += (self.__interpolated_noise((x * frequency)) * amplitude)
            frequency *= 2
            amplitude /= 2
        return result

    def __linear_interp(self, a, b, x):
        return (a + (x * (b - a)))

    def __cosine_interp(self, a, b, x):
        x2 = ((1 - math.cos((x * math.pi))) / 2)
        return ((a * (1 - x2)) + (b * x2))

    def __cubic_interp(self, v0, v1, v2, v3, x):
        p = ((v3 - v2) - (v0 - v1))
        q = ((v0 - v1) - p)
        r = (v2 - v0)
        s = v1
        return ((((p * (x ** 3)) + (q * (x ** 2))) + (r * x)) + s)

    def __fade(self, x):
        return (((6 * (x ** 5)) - (15 * (x ** 4))) + (10 * (x ** 3)))
maya_version_info = {'maya_version': cmds.about(q=True, v=True), 'maya_api_version': cmds.about(q=True, api=True), 'maya_qt_version': cmds.about(q=True, qt=True), 'maya_installed_version': cmds.about(q=True, iv=True)}
log.debug('Init BroAPI')
broapi = BroAPI()
log.debug('Init Ox8r0_ZUp3TEVWUFhxX0NlcHFiMkN3QU82UU5m_ox07')
Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07 = Ox8r0_ZUp3TEVWUFhxX0NlcHFiMkN3QU82UU5m_ox07()
log.debug('Init DataCache')
data_cache = DataCache()

class LicensingAPI():

    @staticmethod
    def turbofloat_lease_drop():
        Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.drop_lease()
        bro_menu_rebuild()
        log.info('Turbofloat lease dropped.')

    @staticmethod
    def turbofloat_lease_request():
        Ox8r0_ZUp3ckVWUFhxX0NlWnFUeDdDd0FGSU1FUmc_ox07.request_lease()
        bro_menu_rebuild()
        log.info('Turbofloat lease requested.')
